Package: MSCquartets
Type: Package
Title: Analyzing Gene Tree Quartets under the Multi-Species Coalescent
Version: 3.3
Authors@R: c(
     person("Elizabeth", "Allman", email = "esallman@alaska.edu",
                  role = "aut"),
     person("Hector", "Banos", email = "hbassnos@gmail.com",
                  role =  "aut"),
     person("Jonathan", "Mitchell", email = "jonathanmitchell88@gmail.com",
                  role =  "aut"),
     person("Kristina", "Wicke", email = "kristina.wicke@njit.edu",
                  role =  "aut"),              
     person("John", "Rhodes", email = "j.rhodes@alaska.edu",
                  role = c("aut", "cre"),comment=c(ORCID = "0000-0001-9921-1091"))  )
Description: Methods for analyzing and using quartets displayed on a collection 
 of gene trees, primarily to make inferences about the species tree or network 
 under the multi-species coalescent model. These include quartet hypothesis tests
 for the model, as developed by Mitchell et al. (2019) <doi:10.1214/19-EJS1576>, 
 simplex plots of quartet concordance factors as presented by Allman et al. (2020) 
 <doi:10.1101/2020.02.13.948083>, species tree inference methods based on quartet 
 distances of Rhodes (2019) <doi:10.1109/TCBB.2019.2917204> and Yourdkhani and 
 Rhodes (2019) <doi:10.1007/s11538-020-00773-4>, the NANUQ algorithm for inference 
 of level-1 species networks of Allman et al. (2019) <doi:10.1186/s13015-019-0159-2>, 
 the TINNIK algorithm for inference of the tree of blobs of an arbitrary 
 network of Allman et al.(2022) <doi:10.1007/s00285-022-01838-9>,
 NANUQ+ routines for resolving multifurcations in the tree of blobs to cycles as in
 Rhodes et al.(2024) <doi:10.1186/s13015-025-00274-w>, and the ECToBlob algorithm for inference of a 
 network with no anomalous quartets of Allman et al. (2026) (forthcoming).
 Software announcement by Rhodes et al. (2020) <doi:10.1093/bioinformatics/btaa868>. 
License: MIT + file LICENSE
Imports: zipfR, graphics, stats, Rdpack, foreach, doParallel, methods,
        Rcpp (>= 1.0.10), igraph (>= 2.0.0)
RdMacros: Rdpack
Encoding: UTF-8
LazyData: true
NeedsCompilation: yes
LinkingTo: Rcpp, RcppProgress
Depends: R (>= 3.5.0), ape (>= 5.0), phangorn
Packaged: 2026-05-27 15:50:22 UTC; jarhodes2
Author: Elizabeth Allman [aut],
  Hector Banos [aut],
  Jonathan Mitchell [aut],
  Kristina Wicke [aut],
  John Rhodes [aut, cre] (ORCID: <https://orcid.org/0000-0001-9921-1091>)
Maintainer: John Rhodes <j.rhodes@alaska.edu>
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Repository: CRAN
Date/Publication: 2026-05-28 07:20:02 UTC

# Generated by roxygen2: do not edit by hand

export(BQinference)
export(ECToBlob)
export(ECToBlob_bi)
export(ECToBlob_mul)
export(ECToBlob_quad)
export(HolmBonferroni)
export(M0)
export(NANUQ)
export(NANUQdist)
export(QDC)
export(QDS)
export(T1density)
export(T3density)
export(TINNIK)
export(TINNIKdist)
export(WQDC)
export(WQDCrecursive)
export(WQDS)
export(WQDSAdjustLengths)
export(allPerms)
export(blobDistance)
export(circHybOrders)
export(collapseEdges)
export(combineCycleResolutions)
export(combineP)
export(compatibleSplits)
export(cutDensity)
export(estimateEdgeLengths)
export(expNANUQCycleDist)
export(expectedCFs)
export(expmodNANUQCycleDist)
export(fitCycleOrders)
export(initBquartets)
export(labelIntNodes)
export(nexusDist)
export(nodeGroups)
export(ordersHeuristicmodNANUQ)
export(powerDivStat)
export(pvalHist)
export(quartetBTinferencePlot)
export(quartetCutMLE)
export(quartetCutTest)
export(quartetCutTestInd)
export(quartetDist)
export(quartetDistTree)
export(quartetECtestInd)
export(quartetNetworkDist)
export(quartetStarTest)
export(quartetStarTestInd)
export(quartetTable)
export(quartetTableCollapse)
export(quartetTableDominant)
export(quartetTableParallel)
export(quartetTablePrint)
export(quartetTableResolved)
export(quartetTallyCpp)
export(quartetTestPlot)
export(quartetTreeErrorProb)
export(quartetTreeTest)
export(quartetTreeTestInd)
export(quartetWeightedDist)
export(resolveCycle)
export(resolveLevel1)
export(simplexCoords)
export(simplexLabels)
export(simplexPoint)
export(simplexPrepare)
export(simplexSegment)
export(simplexText)
export(sortQuartetTableRows)
export(suppressNetBinaryNodes)
export(taxonNames)
export(topDist)
export(treeFromSplits)
export(treeOfBlobs)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
importFrom(ape,.PlotPhyloEnv)
importFrom(ape,Ntip)
importFrom(ape,bind.tree)
importFrom(ape,compute.brlen)
importFrom(ape,cophenetic.phylo)
importFrom(ape,di2multi)
importFrom(ape,drop.tip)
importFrom(ape,evonet)
importFrom(ape,fastme.bal)
importFrom(ape,is.rooted)
importFrom(ape,keep.tip)
importFrom(ape,nj)
importFrom(ape,nodelabels)
importFrom(ape,read.evonet)
importFrom(ape,read.tree)
importFrom(ape,root)
importFrom(ape,unroot)
importFrom(ape,write.evonet)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,all_simple_paths)
importFrom(igraph,as.igraph)
importFrom(igraph,degree)
importFrom(igraph,difference)
importFrom(igraph,distances)
importFrom(igraph,edge_attr)
importFrom(igraph,intersection)
importFrom(igraph,is_directed)
importFrom(methods,is)
importFrom(phangorn,Descendants)
importFrom(phangorn,as.networx)
importFrom(phangorn,compatible)
importFrom(phangorn,mrca.phylo)
importFrom(stats,chisq.test)
importFrom(stats,cophenetic)
importFrom(stats,dmultinom)
importFrom(stats,integrate)
importFrom(stats,na.omit)
importFrom(stats,pbinom)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rmultinom)
importFrom(stats,runif)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(zipfR,Ibeta)
useDynLib(MSCquartets)

#' TINNIK algorithm to infer species tree of blobs
#'
#' Apply the TINNIK algorithm of \insertCite{ABMR24;textual}{MSCquartets} (see also \insertCite{ABMR22;textual}{MSCquartets})
#' to infer a tree of blobs for the species network from a collection of gene trees,
#' under the network multispecies coalescent (NMSC) model.
#'
#' @details  This function
#' \enumerate{
#' \item counts displayed quartets across gene trees to form quartet count concordance factors (qcCFs),
#' \item applies appropriate hypothesis tests to judge qcCFs as representing putative hybridization,
#' resolved trees, or unresolved (star) trees using \code{alpha} and \code{beta} as significance levels,
#' \item produces a simplex plot showing results of the hypothesis tests for all qcCFs
#' \item computes the appropriate TINNIK distance table, and infers the tree of blobs from the distance.
#' }
#'
#' A call of \code{TINNIK} with \code{genedata} given as a table previously output from \code{TINNIK} is
#' equivalent to a call of \code{TINNIKdist} followed by tree construction from the distance table.
#' If \code{genedata} is a
#' table previously output from \code{quartetTableResolved}
#' which lacks columns of p-values for hypothesis tests, these will be appended to the table output by \code{TINNIK}.
#' This table must contain a row with quartet counts for every 4 taxon set.
#'
#' If plots are produced, there are 2 simplex plots: The first shows the hypothesis test results,
#' and the second shows
#' inferred B-quartets and T-quartets. In both,
#' each point in the simplex plot corresponds to an empirical quartet concordance factor,
#' color-coded to represent test or inference results.
#'
#' In general, \code{alpha} should be chosen to be small and \code{beta}
#' to be large so that most quartets are interpreted as resolved trees. More quartets judges to have
#' either blob or unresolved relationships will lead to a less resolved blob tree.
#'
#' Usually, an initial call to \code{TINNIK} will not give a good analysis, as values
#' of \code{alpha} and \code{beta} are likely to need some adjustment based on inspecting the data. Saving the returned
#' table of test results from \code{TINNIK} will allow for the results of the time-consuming computation of qcCFs to be
#' saved, along with p-values,
#' for input to further calls of \code{TINNIK} with new choices of \code{alpha} and \code{beta}.
#'
#' See the documentation for \code{\link{TINNIKdist}} for an explanation of a small, rarely noticeable,
#' stochastic element of the algorithm.
#'
#' For data sets of many gene trees, user time may be reduced by using parallel code for
#' counting displayed quartets. See \code{\link{quartetTableParallel}}.
#'
#'
#' @references
#' \insertRef{ABMR22}{MSCquartets}
#'
#' \insertRef{ABMR24}{MSCquartets}
#'
#'
#' @param genedata gene tree data that may be supplied in any of 3 forms:
#' \enumerate{
#' \item as a character string giving the name of a file containing Newick gene trees,
#' \item as a multiPhylo object containing the gene trees, or
#' \item as a table of quartets on the gene trees, as produced by a previous call to
#' \code{TINNIK} or \code{quartetTableResolved}, which has columns only for taxa, resolved quartet counts,
#' and possibly p_T3, p_cut, and p_star
#' }
#' @param omit \code{FALSE} to treat unresolved quartets as 1/3 of each resolution;
#' \code{TRUE} to discard unresolved quartet data; ignored if gene tree data given as quartet table
#' @param epsilon minimum for branch lengths to be treated as non-zero; ignored if gene tree data given as quartet table
#' @param test a hypothesis test to perform, either "cut" or "T3" (default)
#' @param alpha a value or vector of significance levels for judging p-values for test specified by "test";
#' testing a null hypothesis of no hybridization vs. an alternative of hybridization, for each quartet;  a smaller value applies
#' a less conservative test for a tree (more trees), hence a stricter requirement for deciding in favor of hybridization (fewer reticulations)
#' @param beta a value or vector of significance levels for judging p-values testing
#' a null hypothesis of a star tree (polytomy) for each quartet vs. an alternative of anything else; a smaller value applies a less conservative
#' test for a star tree (more polytomies), hence a stricter requirement for deciding in favor of a resolved tree or network;
#' if vectors, \code{alpha} and \code{beta} must have the same length
#' @param treemethod a function implementing a method of tree inference from a distance table,
#' e.g. the ape package's fastme.bal or nj
#' @param delta a minimum edge length to retain in tree of blobs (see \insertCite{ABMR24}{MSCquartets} for related theory); shorter edges are collapsed
#' @param taxonnames if \code{genedata} is a file or a multiPhylo object, a vector of a subset
#' of the taxa names on the gene trees
#' to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genedata}
#' is a quartet table, this argument is ignored and all taxa in the table are used
#' @param plot \code{TRUE} produces simplex plots of hypothesis test results and plots the tree of blobs \code{FALSE} omits plots
#'
#' @return \code{output} (returned invisibly), with \code{output$ToB} the TINNIK tree of blobs, \code{output$pTable}
#' the table of quartets and p-values for judging fit to the MSC on quartet
#' trees, and \code{output$Bquartets} a TRUE/FALSE indicator vector of B-quartets; if \code{alpha, beta} are vectors, \code{output$ToB} is a vector of trees;
#' the table can be used as input to \code{TINNIK} or \code{TINNIKdist} with new choices of \code{alpha, beta}, without re-tallying quartets on
#' gene trees
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableParallel}}, \code{\link{quartetTableDominant}}, \code{\link{quartetCutTestInd}},\code{\link{quartetTreeTestInd}},
#' \code{\link{quartetStarTestInd}}, \code{\link{TINNIKdist}}, \code{\link{quartetTestPlot}}, \code{\link{pvalHist}}
#'
#' @examples
#' data(pTableYeastRokas)
#' out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)
#'
#' @importFrom ape di2multi nodelabels
#' @importFrom Rcpp evalCpp
#'
#' @export
TINNIK = function(genedata,
                  omit = FALSE,
                  epsilon = 0,
                  test = "T3",
                  alpha = .05,
                  beta = .80,
                  treemethod = fastme.bal,
                  delta = 2,
                  taxonnames = NULL,
                  plot = TRUE) {
  if (!(test %in% c("cut", "T3"))) {
    stop("Argument 'test' must be 'cut' or 'T3'.")
  }
  pcol = paste0("p_", test)

  if (!(is.numeric(alpha) && is.numeric(beta))) {
    stop("Arguments 'alpha' and 'beta' must be numeric.")
  }
  if (length(alpha)!=length(beta)){
    stop("Arguments 'alpha' and 'beta' must have same number of entries.")
  }

  if ("matrix" %in% class(genedata)) {
    pTable = genedata
    momit=missing(omit)
    mepsilon=missing(epsilon)
    mtaxonnames=missing(taxonnames)
    if (!momit | !mepsilon | !mtaxonnames){
      message(
        "Since genedata supplied as quartet table, ignoring arguments 'omit', 'epsilon', 'taxonnames'."
      )}
    
    cols=colnames(pTable)
    n=match("12|34",cols) -1 # number of taxa
    taxonnames = cols[1:n]
  } else {
    if ("multiPhylo" %in% class(genedata))  {
      genetrees = genedata
    } else {
      if ("character" %in% class(genedata)) {
        genetrees <- read.tree(genedata) #read gene trees
        message(paste("Read", length(genetrees), "gene trees from file."))
      } else {
        stop("Data must be supplied as an object of type multiPhylo, character, or matrix.")
      }
    }

    if (is.null(taxonnames)) {
      # if no taxa names specified,
      taxonnames = genetrees[[1]]$tip.label   # ... get them from first tree
    }
    taxonnames = sort(taxonnames)
    if (length(taxonnames) <= 25) {
      namelist = paste0(taxonnames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxonnames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxonnames), " taxa: ", namelist)

    pTable = quartetTable(genetrees, taxonnames, epsilon = epsilon)   # tally quartets on gene trees
    pTable = quartetTableResolved(pTable, omit)   # treat unresolved quartets
  }



  if (!(pcol %in% colnames(pTable))) {
    if (test == "cut") {
      pTable = quartetCutTestInd(pTable, lambda = 0) #compute p-values for cut test
    }
    if (test == "T3") {
      pTable = quartetTreeTestInd(pTable, model = "T3", lambda = 0)  #compute p-values for cut test
    }
  }



  if (!("p_star" %in% colnames(pTable))) {
    pTable = quartetStarTestInd(pTable)# compute p-values for each quartet CFs fit to star tree
  }

  ToBs=c() #empty list for ToBs

  for  (i in 1:length(alpha) ){
  ToBdist = TINNIKdist(pTable,
                       test = test,
                       alpha = alpha[i],
                       beta = beta[i]) # construct TINNIK distance table
  ToB = treemethod(ToBdist$dist) # infer tree of blobs
  ToB=collapseEdges(ToB,delta) #collapse short edges
  ToB = di2multi(ToB) # suppress edges of length 0

  #ToBdist$Bquartets encodes B-quartets T/F

  if (plot == TRUE) {
    if (test == "cut") {
      quartetTestPlot(pTable, "cut", alpha[i], beta[i]) # plot quartets in simplex, indicating which are interpretted as hybridization
      titletext = bquote("2-cut test," ~ alpha * "=" * .(alpha[i]) * "," ~ beta *
                           "=" * .(beta[i]))
    } else {
      if (test == "T3") {
        quartetTestPlot(pTable, "T3", alpha[i], beta[i]) # plot quartets in simplex, indicating which are interpreted as hybridization
        titletext = bquote("T3 test," ~ alpha * "=" * .(alpha[i]) * "," ~ beta *
                             "=" * .(beta[i]))
      }
    }


    quartetBTinferencePlot(pTable,ToBdist$Bquartets,test,alpha[i], beta[i])

    plot(ToB, type = "unrooted", main = "TINNIK Tree of Blobs")
    mtext(eval(bquote(.(titletext))),
          side = 3,
          line = 0,
          cex = 1) # add rest of title text

    ntaxa = length(taxonnames)
    intnodes = (ntaxa + 1):(ntaxa + ToB$Nnode)

    nodelabels(
      node = intnodes,
      pch = 19,
      col = "red",
      cex = 1
    )
  }

  if (is.null(ToBs)) {
      ToBs=ToB
      } else {
    ToBs=c(ToBs,ToB) #append new ToB
      }
  }

  if (length(ToBs)==1) {ToBs=ToBs[[1]]} #if 1-element list, make just the element

  output = list(ToBs, pTable,ToBdist$Bquartets)
  names(output) = c("ToB", "pTable", "Bquartets")
  invisible(output)
}

###########################################################







# Cotangent function
#
#
#@param x argument
#@return function value
#
cot <- function(x) {
  1 / tan(x)
}
#########################################################

# Error function
#
#@param x argument
#@return function value
#
erf <- function(x) {
  2 * pnorm(x * sqrt(2)) - 1
}

#########################################################

# Inverse error function
#
#@param x argument
#@return function value
#
erf.inv <- function(x) {
  qnorm((x + 1) / 2) / sqrt(2)
}


###############################################################

# Log function allowing zero argument
#
# Log function, adjusted for use on zero argument: returns 0 when applied to 0
#
#@param x argument
#@return function value
#
logZ <- function(x) {
  output <- ifelse(x == 0, 0, log(x))
  return(output)
}
######################################################################

#' Probability density function for Cut Model
#'
#' Value of probability density function for Cut Model of \insertCite{ABMR24;textual}{MSCquartets}, Section 3.
#'
#' @references
#' \insertRef{ABMR24}{MSCquartets}
#'
#' @param lambda  statistic value (e.g., likelihood ratio statistic, or other power divergence statistic)
#' @param mu0 parameter of density function
#' @param alpha0 parameter of density function
#' @param beta0 parameter of density function
#'
#' @return  value of density function
#'
#' @seealso \code{\link{T1density},\link{T3density}}
#'
#' @export
cutDensity <- function(lambda, mu0, alpha0, beta0) {
  (1 / (2 * sqrt(2 * pi * lambda))) * (
    exp(-(1 / 2) * lambda) * (2 - erf((
      sqrt(lambda) * tan(alpha0 + beta0) + mu0
    ) / sqrt(2)) - erf((
      sqrt(lambda) * tan(alpha0 + beta0) - mu0
    ) / sqrt(2))) + exp(-(1 / 2) * (sqrt(lambda) + mu0 * cos(alpha0)) ^ 2) *
      (2 - erf((
        sqrt(lambda) * cot(alpha0) - mu0 * sin(alpha0)
      ) / sqrt(2)) - erf((
        sqrt(lambda) * cot(beta0) + mu0 * sin(alpha0)
      ) / sqrt(2))) + exp(-(1 / 2) * (sqrt(lambda) - mu0 * cos(alpha0)) ^ 2) *
      (2 - erf((
        sqrt(lambda) * cot(alpha0) + mu0 * sin(alpha0)
      ) / sqrt(2)) - erf((
        sqrt(lambda) * cot(beta0) - mu0 * sin(alpha0)
      ) / sqrt(2)))
  )
}



######################################################################

#' Maximum likelihood estimate of quartet tree of blobs topology and CF under Cut model
#'
#' Computes Maximum likelihood estimate of quartet tree of blobs topology and CF under the Cut model of
#' \insertCite{ABMR24;textual}{MSCquartets}, Section 3. In case of ties, the topology and CF estimate are chosen randomly among those maximizing
#' the likelihood. In particular, a resolved tree of blobs is always returned.
#'
#'@references
#'\insertRef{ABMR24}{MSCquartets}
#'
#'@param qcCF  a quartet count Concordance Factor (a 3-element vector)
#'
#'@return output with \code{output$topology} = 1, 2, or 3 indicating topology of
#'tree of blobs in accord with ordering of qcCF entries,
#' and \code{output$CFhat} the ML estimate for the CF
#'
#'@examples
#'  quartetCutMLE(c(17,72,11))
#'  quartetCutMLE(c(48,11,41))
#'  quartetCutMLE(c(11,48,41))
#'  quartetCutMLE(c(48,41,11))
#'
#'@export
#'
quartetCutMLE <- function(qcCF) {
  n = sum(qcCF)
  s1 = qcCF[2] + qcCF[3]
  L1 <- qcCF[1] * logZ(2 * qcCF[1]) + s1 * logZ(s1)
  s2 = qcCF[1] + qcCF[3]
  L2 <- qcCF[2] * logZ(2 * qcCF[2]) + s2 * logZ(s2)
  s3 = qcCF[1] + qcCF[2]
  L3 <- qcCF[3] * logZ(2 * qcCF[3]) + s3 * logZ(s3)
  L = c(L1, L2, L3)
  maxs = which(L == max(L))

  if (length(maxs) == 1) {
    top = maxs
  }
  else {
    top = sample(maxs, 1)
  }

  if (top == 1) {
    MLE <- c(qcCF[1], s1 / 2, s1 / 2) / n
  } else if (top == 2) {
    MLE <- c(s2 / 2, qcCF[2], s2 / 2) / n
  } else {
    MLE <- c(s3 / 2, s3 / 2, qcCF[3]) / n
  }
  output = list(top, MLE)
  names(output) = list("topology", "CFhat")
  return(output)
}


#############################################################


#' Hypothesis test for quartet counts fitting a resolved quartet tree of blobs under NMSC
#'
#' Test the hypothesis H_0=Cut model of \insertCite{ABMR24;textual}{MSCquartets},
#' Section 3., vs. H_1= everything else.
#' Returns p-value and estimate of tree of blobs topology.
#'
#' @details
#'
#' The Cut model for quartet CFs is the NMSC combined with the quartet species network having
#' a cut edge separating two of the taxa from the other two.
#'
#' This function implements the test described in \insertCite{ABMR24;textual}{MSCquartets}
#' as well as parametric bootstrapping,
#' with other procedures for when some expected counts are small.
#' These are more accurate tests than, say, a Chi-square with one degree of freedom,
#' which is not theoretically
#' justified near the singularity of the model, nor for small counts.
#'
#' If \code{method="MLtest"}, this uses the test for the Cut model described
#' in Section 3 of \insertCite{ABMR24;textual}{MSCquartets}, using the ML
#' estimate of the generating parameter.
#' As shown in simulations in that paper, the test is conservative when small
#' levels are used for rejection.
#' Although the test generally performs well in practice, it lacks a uniform
#' asymptotic guarantee over the full parameter space.
#'
#' If \code{method="conservative"}, the test uses the Chi-square distribution with 1 degree of freedom
#' (the "least favorable" approach). This is asymptotically guaranteed to reject the null
#' hypothesis at most at a specified level, but at the expense of increased type II errors.
#'
#' If \code{method="bootstrap"}, then parametric bootstrapping is performed,
#' based on ML estimates of the CF. The bootstrap sample size is given by the \code{bootstrap} argument.
#'
#' When some expected topology counts are small, the methods \code{"MLest"} and \code{"conservative"}
#' are not appropriate.
#' The argument \code{smallcounts} determines whether bootstrapping or a faster approximate method is used.
#' These use ML estimates of the CF under the Cut model.
#'
#' If two of the three counts are small (so the estimated CF is near a vertex of the simplex),
#' The approximate approach
#' returns a precomputed p-value, found by replacing the largest observed count
#' with 1e+6 and performing 1e+8 bootstraps. When n is sufficiently large (at least 30) and
#' some expected counts are small, the probability of topological error  is small and the bootstrap p-value is
#' approximately independent of the largest observed count.
#'
#' If one of the three counts is small (so the estimated CF is near an edge of the simplex),
#' a chi-squared test using the binomial model for the larger counts is used, as described
#' by \insertCite{ABMR24;textual}{MSCquartets}.
#'
#' The returned p-value should be taken with caution when there is a small sample size, e.g. less than 30 gene trees.
#'
#' @param obs  vector of 3 counts of resolved quartet frequencies
#' @param lambda  parameter for power-divergence statistic (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)
#' @param method \code{"MLtest"},\code{"conservative"}, or \code{"bootstrap"}
#' @param smallcounts \code{"bootstrap"} or \code{"approximate"}, method of obtaining p-value when some counts are small
#' @param bootstraps  number of samples for bootstrapping
#'
#' @return output where \code{output$p.value} is a p-value and \code{output$topology} = 1, 2, or 3
#' indicates the ML estimate of the topology of the quartet tree of blobs in accord with ordering of qcCF entries.
#'
#'@examples
#'  quartetCutTest(c(17,72,11))
#'  quartetCutTest(c(48,11,41))
#'  quartetCutTest(c(11,48,41))
#'  quartetCutTest(c(48,41,11))
#'
#' @references
#' \insertRef{ABMR22}{MSCquartets}
#'
#' \insertRef{ABMR24}{MSCquartets}
#'
#' \insertRef{ABRW25}{MSCquartets}
#'
#' \insertRef{MAR19}{MSCquartets}
#'
#'
#' @seealso \code{\link{quartetCutTestInd}}
#'
#' @importFrom stats pbinom rmultinom integrate pnorm uniroot
#'
#' @export
#'
quartetCutTest <-
  function (obs,
            lambda = 0,
            method = "MLest",
            smallcounts = "approximate",
            bootstraps = 10 ^ 4) {
    if (is.numeric(obs) == FALSE || is.vector(obs) == FALSE ||
        length(obs) != 3 || min(obs) < 0 || sum(obs) < 1 || all.equal(sum(obs),
                                                                      round(sum(obs))) != TRUE || is.numeric(lambda) == FALSE ||
        is.vector(lambda) == FALSE || length(lambda) != 1 ||
        smallcounts %in% c("bootstrap", "approximate") ==
        FALSE || is.numeric(bootstraps) == FALSE || is.vector(bootstraps) ==
        FALSE || length(bootstraps) != 1 || bootstraps <= 0 ||
        bootstraps%%1 != 0 || method %in% c("MLest", "conservative",
                                            "bootstrap") == FALSE) {
      stop("Invalid arguments: obs must be a numeric non-negative vector of length 3 summing to a positive integer;\n      lambda must be a real number;\n      method must be \"MLest\", \"conservative\", or \"bootstrap\";\n      smallcounts must be \"bootstrap\" or \"approximate\";\n      bootstraps must be a positive integer.")
    }
    else {
      n <- round(sum(obs))
      if (n < 30) {
        warning("The number of gene quartets is <30; p-value may be inaccurate.")
      }
    }
    obs <- as.numeric(obs)
    cutMLE = quartetCutMLE(obs)
    top = cutMLE[[1]]
    expd <- n * (cutMLE[[2]])
    if (top == 2) {
      obs = obs[c(2, 1, 3)]
      expd = expd[c(2, 1, 3)]
    }
    else {
      if (top == 3) {
        obs = obs[c(3, 2, 1)]
        expd = expd[c(3, 2, 1)]
      }
    }
    if (all.equal(sum(abs(obs - expd)), 0) == TRUE) {
      p <- 1
    }
    else {
      stat <- powerDivStat(obs, expd, lambda)
      if (method == "bootstrap" || (method != "bootstrap" &&
                                    min(expd) < 5 && smallcounts == "bootstrap") ||
          (method != "bootstrap" && min(expd) < 5 &&
           expd[1] > expd[2] && smallcounts != "bootstrap" &&
           ((all.equal(sum(abs(obs * 3 - round(obs * 3))),
                       0) != TRUE) || (all.equal(sum(abs(obs * 3 -
                                                         round(obs * 3))), 0) == TRUE && var(obs - trunc(obs)) !=
                                       0)))) {
        message("Bootstrapping has been selected. p-values are approximate. Bootstrapping is only necessary when some expectations are small and approximate bootstrap p-values not available.")
        if (method == "bootstrap" && min(expd) < 5 &&
            smallcounts == "approximate") {
          message("Method is prioritized over smallcounts. Bootstrap has been chosen.")
        }
        if (method != "bootstrap" && min(expd) < 5 &&
            max(expd) > n - 10 && smallcounts != "bootstrap" &&
            ((all.equal(sum(abs(obs * 3 - round(obs * 3))),
                        0) != TRUE) || (all.equal(sum(abs(obs * 3 -
                                                          round(obs * 3))), 0) == TRUE && var(obs - trunc(obs)) !=
                                        0))) {
          warning("Approximate bootstrap p-values not available when two expected counts are below 5 and observed counts are not all integers, all integers + 1/3 or all integers + 2/3.")
          }
        if (bootstraps == 0) {
          bootstraps <- 10^4
        }
        count <- 0


        sims = rmultinom(bootstraps, n, prob = expd/n)

        for (i in 1:bootstraps) {
          dat <- sims[, i]
          expected <- n * quartetCutMLE(dat)[[2]]
          simstat <- powerDivStat(dat, expected, lambda)
          count <- ifelse(simstat >= stat, count + 1, count)
        }
        p <- count/bootstraps
      }
      else if (method != "bootstrap" && min(expd) < 5 &&
               expd[1] >= expd[2] && smallcounts == "approximate" &&
               (all.equal(sum(abs(obs * 3 - round(obs * 3))), 0) ==
                TRUE) && var(obs - trunc(obs)) == 0) {
        sortobs23times3 = sort(round(3 * obs[2:3]))
        if (sortobs23times3[1]%%3 == 0) {
          if (identical(sortobs23times3, c(0, 27))) {
            p <- 0.000919
          }
          else if (identical(sortobs23times3, c(3, 24))) {
            p <- 0.0209
          }
          else if (identical(sortobs23times3, c(6, 21))) {
            p <- 0.103
          }
          else if (identical(sortobs23times3, c(9, 18))) {
            p <- 0.359
          }
          else if (identical(sortobs23times3, c(12, 15))) {
            p <- 0.79
          }
          else if (identical(sortobs23times3, c(0, 24))) {
            p <- 0.00191
          }
          else if (identical(sortobs23times3, c(3, 21))) {
            p <- 0.0397
          }
          else if (identical(sortobs23times3, c(6, 18))) {
            p <- 0.175
          }
          else if (identical(sortobs23times3, c(9, 15))) {
            p <- 0.523
          }
          else if (identical(sortobs23times3, c(0, 21))) {
            p <- 0.00416
          }
          else if (identical(sortobs23times3, c(3, 18))) {
            p <- 0.0763
          }
          else if (identical(sortobs23times3, c(6, 15))) {
            p <- 0.297
          }
          else if (identical(sortobs23times3, c(9, 12))) {
            p <- 0.768
          }
          else if (identical(sortobs23times3, c(0, 18))) {
            p <- 0.00871
          }
          else if (identical(sortobs23times3, c(3, 15))) {
            p <- 0.129
          }
          else if (identical(sortobs23times3, c(6, 12))) {
            p <- 0.476
          }
          else if (identical(sortobs23times3, c(0, 15))) {
            p <- 0.0183
          }
          else if (identical(sortobs23times3, c(3, 12))) {
            p <- 0.24
          }
          else if (identical(sortobs23times3, c(6, 9))) {
            p <- 0.737
          }
          else if (identical(sortobs23times3, c(0, 12))) {
            p <- 0.0393
          }
          else if (identical(sortobs23times3, c(3, 9))) {
            p <- 0.439
          }
          else if (identical(sortobs23times3, c(0, 9))) {
            p <- 0.086
          }
          else if (identical(sortobs23times3, c(3, 6))) {
            p <- 0.681
          }
          else if (identical(sortobs23times3, c(0, 6))) {
            p <- 0.197
          }
          else if (identical(sortobs23times3, c(0, 3))) {
            p <- 0.478
          }
        }
        else if (sortobs23times3[1]%%3 == 1) {
          if (identical(sortobs23times3, c(1, 28))) {
            p <- 0.00222
          }
          else if (identical(sortobs23times3, c(4, 25))) {
            p <- 0.0237
          }
          else if (identical(sortobs23times3, c(7, 22))) {
            p <- 0.119
          }
          else if (identical(sortobs23times3, c(10, 19))) {
            p <- 0.358
          }
          else if (identical(sortobs23times3, c(13, 16))) {
            p <- 0.777
          }
          else if (identical(sortobs23times3, c(1, 25))) {
            p <- 0.00452
          }
          else if (identical(sortobs23times3, c(4, 22))) {
            p <- 0.0436
          }
          else if (identical(sortobs23times3, c(7, 19))) {
            p <- 0.203
          }
          else if (identical(sortobs23times3, c(10, 16))) {
            p <- 0.52
          }
          else if (identical(sortobs23times3, c(1, 22))) {
            p <- 0.0094
          }
          else if (identical(sortobs23times3, c(4, 19))) {
            p <- 0.0794
          }
          else if (identical(sortobs23times3, c(7, 16))) {
            p <- 0.295
          }
          else if (identical(sortobs23times3, c(10, 13))) {
            p <- 0.754
          }
          else if (identical(sortobs23times3, c(1, 19))) {
            p <- 0.0194
          }
          else if (identical(sortobs23times3, c(4, 16))) {
            p <- 0.143
          }
          else if (identical(sortobs23times3, c(7, 13))) {
            p <- 0.47
          }
          else if (identical(sortobs23times3, c(1, 16))) {
            p <- 0.0404
          }
          else if (identical(sortobs23times3, c(4, 13))) {
            p <- 0.233
          }
          else if (identical(sortobs23times3, c(7, 10))) {
            p <- 0.72
          }
          else if (identical(sortobs23times3, c(1, 13))) {
            p <- 0.085
          }
          else if (identical(sortobs23times3, c(4, 10))) {
            p <- 0.422
          }
          else if (identical(sortobs23times3, c(1, 10))) {
            p <- 0.113
          }
          else if (identical(sortobs23times3, c(4, 7))) {
            p <- 0.665
          }
          else if (identical(sortobs23times3, c(1, 7))) {
            p <- 0.237
          }
          else if (identical(sortobs23times3, c(1, 4))) {
            p <- 0.532
          }
        }
        else {
          if (identical(sortobs23times3, c(2, 26))) {
            p <- 0.00827
          }
          else if (identical(sortobs23times3, c(5, 23))) {
            p <- 0.0436
          }
          else if (identical(sortobs23times3, c(8, 20))) {
            p <- 0.199
          }
          else if (identical(sortobs23times3, c(11, 17))) {
            p <- 0.516
          }
          else if (identical(sortobs23times3, c(2, 23))) {
            p <- 0.0164
          }
          else if (identical(sortobs23times3, c(5, 20))) {
            p <- 0.0777
          }
          else if (identical(sortobs23times3, c(8, 17))) {
            p <- 0.298
          }
          else if (identical(sortobs23times3, c(11, 14))) {
            p <- 0.739
          }
          else if (identical(sortobs23times3, c(2, 20))) {
            p <- 0.0237
          }
          else if (identical(sortobs23times3, c(5, 17))) {
            p <- 0.146
          }
          else if (identical(sortobs23times3, c(8, 14))) {
            p <- 0.466
          }
          else if (identical(sortobs23times3, c(2, 17))) {
            p <- 0.0468
          }
          else if (identical(sortobs23times3, c(5, 14))) {
            p <- 0.238
          }
          else if (identical(sortobs23times3, c(8, 11))) {
            p <- 0.703
          }
          else if (identical(sortobs23times3, c(2, 14))) {
            p <- 0.0926
          }
          else if (identical(sortobs23times3, c(5, 11))) {
            p <- 0.409
          }
          else if (identical(sortobs23times3, c(2, 11))) {
            p <- 0.185
          }
          else if (identical(sortobs23times3, c(5, 8))) {
            p <- 0.645
          }
          else if (identical(sortobs23times3, c(2, 8))) {
            p <- 0.377
          }
          else if (identical(sortobs23times3, c(2, 5))) {
            p <- 0.525
          }
        }
      }
      else if (method != "bootstrap" && min(expd) < 5 &&
               expd[1] < expd[2] && smallcounts == "approximate" &&
               all.equal(sum(abs(obs - round(obs))), 0) == TRUE) {
        p <- 2 * pbinom(min(obs[2:3]), sum(obs[2:3]), 1/2)
      }
      else if (method != "bootstrap" && min(expd) < 5 &&
               expd[1] < expd[2] && smallcounts == "approximate" &&
               all.equal(sum(abs(obs - round(obs))), 0) != TRUE) {
        p <- pchisq(stat, 1, lower.tail = FALSE)
      }
      else if (method != "bootstrap" && min(expd) >=
               5) {
        if (method == "MLest") {
          phinhat <- 3 * (n - expd[1])/(2 * n)
          munhat <- sqrt(2 * n) * (1 - phinhat)/sqrt(phinhat *
                                                       (3 - 2 * phinhat))
          alphanhat <- atan(1/sqrt(3 * (3 - 2 * phinhat)))
          betanhat <- (1/2) * (pi/2 - phinhat)
          p <- ifelse(stat == 0, 1, min(1, max(0, integrate(cutDensity,
                                                            lower = stat, upper = Inf, mu0 = munhat, alpha0 = alphanhat,
                                                            beta0 = betanhat)$value)))
        }
        else if (method == "conservative") {
          p <- pchisq(stat, 1, lower.tail = FALSE)
        }
      }
    }
    ptop = list(p, top)
    names(ptop) = c("p.value", "topology")
    return(ptop)
  }

#############################################################

#' Multiple independent hypothesis tests for quartet counts fitting the Cut model under the NMSC
#'
#' Perform a hypothesis test for all quartet counts in an input table, as if the counts for different choices of 4 taxa
#' are independent.
#'
#' @details This function assumes all quartets are resolved.  The test performed and the arguments
#' are described more fully in \code{quartetCutTest}.
#'
#' @references
#' \insertRef{ABMR24}{MSCquartets}
#'
#' @param rqt  table of resolved quartet counts, as produced by \code{quartetTableResolved}, or \code{quartetStarTestInd}
#' @param lambda power divergence statistic parameter (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)
#' @param method \code{"MLest"}, \code{"conservative"}, or \code{"bootstrap"}; see \code{quartetCutTest} for explanation
#' @param smallcounts \code{"bootstrap"} or \code{"approximate"}, method of obtaining p-value when some counts are small, so
#' the chosen \code{method} is inappropriate
#' @param bootstraps  number of samples for bootstrapping
#'
#' @return
#'    a copy of \code{rqt} with two columns appended: \code{"p_cut"} with p-values and \code{"cutindex"}
#'    giving index 1,2, or 3 of ML estimate of quartet tree of blobs (1 if 12|34, 2 if 13|24, 3 if 14|23)
#'    under Cut model.
#'
#' @seealso \code{\link{quartetCutTest}}, \code{\link{quartetTestPlot}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetTableResolved}}
#'
#' @examples
#' data(pTableYeastRokas)
#' pT=pTableYeastRokas[1:10,1:11]
#' pTable=quartetCutTestInd(pT)
#'
#' @importFrom ape unroot read.tree cophenetic.phylo
#' @export
quartetCutTestInd <-
  function(rqt,
           lambda = 0,
           method = "MLest",
           smallcounts = "approximate",
           bootstraps = 10 ^ 4) {
    if (is.vector(lambda) == FALSE ||
        length(lambda) != 1 ||
        smallcounts %in% c("bootstrap", "approximate") == FALSE ||
        bootstraps <= 0 || bootstraps %% 1 != 0 ||
        method %in% c("MLest", "conservative", "bootstrap") == FALSE) {
      # Return an error and stop if arguments are misspecified.
      stop(
        'Invalid arguments: lambda must be real; smallcounts must be "bootstrap" or "approximate";
      bootstraps must be a positive integer; method must be "MLest","conservative", or "bootstrap".'
      )
    }
    colnames = colnames(rqt)
    if (("p_cut" %in% colnames) | ("cutindex" %in% colnames)) {
      stop('Input table already has column to be appended.')
    }
    n=match("12|34",colnames)-1 # number of taxa
    taxonnames = colnames[1:n]
    M = dim(rqt)[1] # number of quartets

    pTable = cbind(rqt, p_cut = 0, cutindex = 0) # tack on columns for p-values, and cutindices
    message("Applying hypothesis test for Cut model to ", M, " quartets.")
    for (m in 1:M) {
      # consider each quartet
      obs = unname(rqt[m, c("12|34", "13|24", "14|23")])
      ptop = quartetCutTest(
        obs,
        lambda = lambda,
        smallcounts = smallcounts,
        bootstraps = bootstraps,
        method = method
      )  #  compute p-value
      pTable[m, "p_cut"] = ptop$p.value  # store p-value
      pTable[m, "cutindex"] = ptop$topology # and topology
    }
    return(pTable)

  }


#################################################################
#' Compute TINNIK distance from quartets and hypothesis test p-values
#'
#' Apply the B-quartet inference algorithm of \insertCite{ABMR22;textual}{MSCquartets}, \insertCite{ABMR24;textual}{MSCquartets} to
#' infer all B-quartets from results of hypothesis tests, and then compute an estimate of an intertaxon distance
#' fitting the topological tree of blobs of the species network.
#'
#' @details This function assumes \code{pTable} has columns for taxa and resolved
#' quartet counts as originally produced by \code{quartetTable},
#' and hypothesis test results as produced by
#' \code{quartetStarTestInd}, and either \code{quartetTreeTestInd} for the \code{T3} test or \code{quartetCutTestInd}.
#' Rows must be present for every 4-taxon subset.
#' (Note: Of functions in this package, only \code{HolmBonferroni} might modify the row order from the required one.)
#'
#' This function uses the Rcpp package for significant speed up in computation time.
#'
#' @references
#' \insertRef{ABMR22}{MSCquartets}
#'
#' \insertRef{ABMR24}{MSCquartets}
#'
#' @param pTable  table of resolved quartet counts, as produced by
#' \code{quartetTableResolved}, with extra columns from
#' both star hypothesis test, and either cut or T3 hypothesis tests
#' @param test either "cut" or "T3"
#' @param alpha level for cut or T3 test
#' @param beta level for star test
#'
#' @return a distance table \code{output$dist} and
#'    a vector \code{output$Bquartets} with TRUE/FALSE entries indicating B-quartets
#'    ordered as rows of \code{pTable}.
#'
#' @seealso \code{\link{quartetTable},\link{quartetTableResolved},\link{quartetStarTest}},
#' \code{\link{quartetCutTest}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetCutTestInd}}
#'
#' @examples
#' data(pTableYeastRokas)
#' out=TINNIKdist(pTableYeastRokas,test="T3",alpha=.05,beta=.05)
#'
#' @export
TINNIKdist <- function(pTable,
                       test = "T3",
                       alpha = .05,
                       beta = .05) {
  if (!(test %in% c("cut", "T3")) |
      ((test == "cut") &
       !all(c("p_cut", "cutindex", "p_star") %in% colnames(pTable))) |
      ((test == "T3") &
       !all(c("p_T3", "p_star") %in% colnames(pTable)))) {
    stop('Invalid test argument, or input table missing columns from tests.')
  }

  
  cols=colnames(pTable)
  n=match("12|34",cols) -1 # number of taxa
  taxonnames = cols[1:n]
  m = dim(pTable)[1]# number of 4-taxon sets in table
  if (m != choose(n, 4)) {
    stop("Dimensions of pTable not correct; must have row for each choice of 4 taxa.")# check number of rows is correct
  }

  L1 = c() #empty vectors for storing indices (i.e., row numbers) of B-quartets in pTable
  L2 = L1

  if (test == "cut") {
    cuttops = pTable[, "cutindex"] #already have topology for MLE for tree of blobs
  } else {
    #here if test="T3"
    cuttops = max.col(pTable[, c("12|34", "13|24", "14|23")]) # Use dominant topology (MLE) appropriate for T3 model
  }

  # Initialization of B-quartets from hypothesis tests
  Bquartets = rep(FALSE, m) #create initial indicator and index vectors for B-quartets inferred from CFs
  p_test = paste0("p_", test) #create p-value column names for test
  colptest= which(colnames(pTable) ==p_test)
  colpstar= which(colnames(pTable) =="p_star")
  ret=initBquartets(pTable,m,alpha,beta,colptest,colpstar,Bquartets)
  L1=(ret$Lout)[(ret$Lout)!=0]
  Bquartets=as.logical(ret$Bout)

  lenL1 = length(L1) # number of new B-quartets found
  Binit = lenL1  # save initial number of B-quartets

  if (Binit == 0) {
    message(
      c(
        "No support for hybridizations/gene flow in any quartet using ",
        test,
        " test with  alpha=",
        alpha,
        ", beta=",
        beta,
        "."
      )
    )
  }

  np1 = n + 1 #number of taxa + 1
  C = matrix(nrow = n, ncol = 4) #cache binomial coefficients for efficiency in computing indices
  for (i in 0:(n - 1)) {
    for (j in 1:4) {
      C[i + 1, j] = choose(i, j)
    }
  }
  Cn4 = choose(n, 4) # need one extra value

  Nrule1 = 0 #counters of uses of inference rules
  Nrule2 = 0

  # Perform B-quartet inference procedure using C++ code
  Bquartets=BQinference(pTable,C,Cn4,n,Bquartets,L1,lenL1,Nrule1,Nrule2,cuttops)

  #After all B-quartets are inferred, compute TINNiK distance
  taxa = colnames(pTable)[1:n]# get names of taxa
  D = matrix(0, n, n)# create blank distance matrix
  colnames(D) = taxa
  rownames(D) = taxa

  for (i in 1:m) {
    taxa = which(pTable[i, 1:n] == 1)
    #if not a B-quartet
    if (Bquartets[i] == FALSE) {
      ones = taxa[c(1, cuttops[i] + 1)]
      nones = setdiff(taxa, ones)
      D[ones, nones] = D[ones, nones] + 1 # increment 4 entries of D
      D[nones, ones] = D[nones, ones] + 1 # and another 4
    } else {
      #if a B-quartet
      D[taxa, taxa] = D[taxa, taxa] + 1 #increment 16 entries of D (diagonal will be reset to 0 later) to treat as unresolved
    }
  }

  D = 2 * D + 2 * n - 4 # adjust to appropriate distance formula
  for (i in 1:n) {
    D[i, i] = 0 # and set diagonal back to 0
  }

  #print(paste("B-quartets from initial hypothesis tests:", Binit))
  #print(paste("B-quartets from inference rules:", sum(Bquartets) - Binit))
  #print(paste("Rule 1 used", Nrule1, "times."))
  #print(paste("Rule 2 used", Nrule2, "times."))

  output = list(dist = D, Bquartets = Bquartets) # return B-quartet indicators and distance
  return(output)
}



##############################################

#' Groups taxa by deleting a node in a tree
#'
#' Finds groups of taxa determined by the connected components of the graph resulting from deleting an internal node in a tree.
#'
#' @details
#' When applied to a rooted tree, the last group returned is the set of tips that are
#' non-descendants of the node (provided any exist).
#'
#'@param tree  a tree, of class "phylo"
#'@param nodeNum a node number, representing an internal node in the phylo representation
#'
#'@return a list of lists of tree tip numbers for each group.
#'The union of the groups is the set of all tips.
#'
#'@examples
#'  tree=read.tree(text="((a,b),((c,d,e),(f,g)));")
#'  nodeGroups(tree,8)
#'  nodeGroups(tree,10)
#'  nodeGroups(tree,11)
#'
#'@export
#'
nodeGroups <- function(tree, nodeNum) {
  ntaxa = length(tree$tip.label)

  if ((nodeNum <= ntaxa) || (nodeNum > (ntaxa + tree$Nnode))) {
    stop("Argument 'nodeNum' not in valid range.")
  }

  children = Descendants(tree, nodeNum, "children") #find children of node
  groups = list() #begin list
  alld = c() #and set of all descendents
  for (c in children) {
    d = Descendants(tree, c, 'tips')
    groups = append(groups, d)
    alld = union(alld, d[[1]])
  }

  nonDesc = setdiff(1:ntaxa, alld)

  if (length(nonDesc) > 0) {
    groups = append(groups, list(nonDesc))
  }

  return(groups)
}



################################

#' Produce simplex plot with  results of B/T-quartet inference
#'
#' Plot a 2-d probability simplex, with points for all normalized quartet count
#' vectors. Colors indicate B- or T-quartets from TINNIK algorithm, at specified
#' test levels.
#'
#' @details The first argument of this function is a table of quartets and p-values. The
#' plot may show results using either the T3, or 2-cut
#' test, and a star tree test.
#' The p-values must be computed by or before previous calls to
#' \code{TINNIK}. The second argument is the indicator vector for B/T quartets produced by \code{TINNIK}.
#'
#' @param pTable table of quartets and p-values
#' @param Bquartets indicator vector for B-quartets (1=B, 0=T), ordered as in pTable
#' @param test  test model used for tree null hypothesis; options are \code{"cut"}, \code{"T3"}
#' @param alpha  significance level used by TINNIK for test \code{test}
#' @param beta  significance level used by TINNIK for star tree test
#' @param cex scaling factor for size of plotted symbols
#'
#' @return NULL
#'
#' @seealso \code{TINNIK}, \code{\link{quartetTestPlot}}
#'
#' @examples
#' data(pTableYeastRokas)
#' out=TINNIKdist(pTableYeastRokas,test="T3",alpha=.05,beta=.05)
#' quartetBTinferencePlot(pTableYeastRokas,out$B,test="T3",alpha=.05,beta=.05)
#'
#' @export
quartetBTinferencePlot <- function(pTable,
                            Bquartets,
                            test,
                            alpha,
                            beta,
                            cex=1) {
  if (!(is.numeric(alpha) && is.numeric(beta))) {
    stop("Test levels alpha and beta must be numeric.")
  }

  if ( !(test %in% c("T3","cut") ) ){ stop("Invalid test name.")}

  lineWidth = 1.5
  redColor = "goldenrod"
  blueColor = "darkgreen"

  counts = c("12|34", "13|24", "14|23")
  M = dim(pTable)[1]


  if (test == "T3")  {
    #T3 and star
    titletext = bquote("T3," ~ alpha * "=" * .(alpha) * "," ~ beta *
                         "=" * .(beta))
  } else {
      #cut and star
      titletext = bquote("2-cut," ~ alpha * "=" * .(alpha) * "," ~ beta *
                           "=" * .(beta))
  }

    legtext = c("B-quartet",
                "T-quartet")
    legpch = c(5, 1)
    legcol = c(redColor, blueColor)
    model = test

simplexPrepare(model, maintitle = "TINNIK Inferred B-,T-quartets ", titletext =
                   titletext) # draw outline of simplex and lines for model

  reds = Bquartets
  blues = !Bquartets

  bluepoints = pTable[which(blues), counts, drop = FALSE] # points to plot blue
  nblue = dim(bluepoints)[1]
  redpoints = pTable[which(reds), counts, drop = FALSE] #      red
  nred = dim(redpoints)[1]

  if (nblue > 0) {
    for (i in 1:nblue) {
      simplexPoint(
        bluepoints[i,],
        type = "o",
        pch = 1,
        col = blueColor,
        lwd = lineWidth,
        cex = cex
      )
    }
  }
  if (nred > 0) {
    for (i in 1:nred) {
      simplexPoint(
        redpoints[i,],
        type = "o",
        pch = 5,
        col = redColor,
        lwd = lineWidth,
        cex = cex
      )
    }
  }

  legend(
    'topleft',
    legend = legtext,
    pch = legpch,
    col = legcol,
    cex = 0.8,
    inset = 0.01,
    pt.cex = 1.2,
    lty = 0,
    lwd = 2
  )
}

##################################################################
#' Tree of blobs for a network
#'
#' Given extended newick, an evonet object, or an igraph object for a network, return its reduced, unrooted
#' tree of blobs. Here 'reduced' means all nodes resulting from 2-blobs are suppressed, as are edges above
#' the network's LSA.
#'
#' @param net A network, supplied as an extended Newick string, an evonet object, or an igraph object
#' @param plot if TRUE (default), plot the tree of blobs.
#'
#'
#' @return An object of class \code{phylo} containing the unrooted topological tree
#' derived from the network by contracting all blobs.  All edge lengths are 1.
#'
#' @seealso \code{\link{TINNIK}}
#'
#' @examples network = "(((a:1,d:1):1,(b:1)#H1:1):1,(#H1,c:1):2);"
#' plot(read.evonet(text=network))
#' treeOfBlobs(network, plot=TRUE)
#'
#' @importFrom igraph as.igraph is_directed degree V E all_simple_paths difference intersection
#' @importFrom igraph edge_attr distances
#' @importFrom ape read.evonet .PlotPhyloEnv
#' @importFrom utils combn
#'
#' @export
#'
treeOfBlobs = function (net, plot = FALSE)  {
  # convert Newick string to evonet if needed
  if (is(net, "character")) {
    # read as tree if no reticulations
    if ((regexpr('#', net)==-1)) {
      net = read.tree(text = net)
      print("Newick argument is for a tree, returning tree of blobs.")
      return(net)
    } else {
      net = read.evonet(text=net)
    }
  }

  if (!any(class(net) %in% c("evonet", "phylo", "igraph")))   stop("Network argument must be a newick string, evonet, or igraph.")

  # return tree if net is tree.  Otherwise convert to igraph object and label internal nodes
  if (any( class(net) %in% c("evonet", "phylo")) ) {
    if (is.null(net$reticulation)) {
      print("Network argument is a tree, so returning it as tree of blobs.")
      return(net)
    }
  }

  tmp <- nchar(net$tip.label)
  if (any(tmp == 0)) {
    newLabel <- net$tip.label
    newLabel[tmp == 0] <- paste0("ROOT", 1:sum(tmp==0))
    net$tip.label[tmp == 0] <- newLabel[tmp == 0]
  }

  # add internal labels if some are null.
  # Ape's function read.tree did not read all labels, possible because of
  # 3-cycle, possibly because of root node

  # to add node.label attr
  ## ESA
  # net$node.label = NULL
  #
  # label hybrids if label missing
  if (is.null(net$node.label)) {
    net$node.label <- character(net$Nnode)

    # avoid adding NAs by looping
    nTax = Ntip(net)
    for (nodeNum in net$reticulation[,2]) {
      net$node.label[nodeNum-nTax] <- paste0("#H", as.character(nodeNum))
    }
  }

  # try adding all internal node labels too since problem seems to be root
  # if (!is.null(net$node.label)) {
  #   tmp <- nchar(net$node.label)
  #   if (any(tmp == 0)) {
  #     newLabel <- paste0("n", (nTax+1):(nTax+net$Nnode))
  #     net$node.label[tmp == 0] <- newLabel[tmp == 0]
  #   }
  # }

  # read as igraph
  net = igraph::as.igraph(net)

  if (!(igraph::is_directed(net))) {
    stop("Network must be rooted.")
  }

  leaves = which(igraph::degree(net, v = igraph::V(net), mode = "out")==0, useNames = TRUE)
  root = which(igraph::degree(net, v = igraph::V(net), mode = "in")==0, useNames = TRUE)

  blob_edges = c()

  # obtain blob edges, by getting all paths from the root to taxa
  for (cur_leaf in 1:length(leaves)) {
    S = igraph::all_simple_paths(net, from = root, to = leaves[cur_leaf])
    if (length(S)>1) {

      # get number of pairs of paths in S
      nc2 = combn(1:length(S), 2, simplify = FALSE)
      for (j in 1:length(nc2)) {
        v_path1 = S[[nc2[[j]][1]]]
        v_path2 = S[[nc2[[j]][2]]]
        path1 = igraph::E(net,path=v_path1)
        path2 = igraph::E(net,path=v_path2)
        common_edges = igraph::intersection(path1,path2)
        blob_edges = unique(c(igraph::difference(path1,common_edges),igraph::difference(path2,common_edges),blob_edges))
      }
    }
  }

  igraph::edge_attr(net,"weight") = 1
  igraph::edge_attr(net,"weight",index = blob_edges) <- 0

  dTable = igraph::distances(net,leaves,leaves)
  tob = nj(dTable) #construct tree
  tob = di2multi(unroot(tob), tol = 1e-10) # unroot and suppress degree 2 nodes
  tob$edge.length=rep(1,length(tob$edge.length)) # again make all edges length 1

  # plot tob
  if (plot) {
    plot(tob,main="Tree of blobs",type="unrooted",cex=0.7,lab4ut="axial")
    lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
    internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
    XX <- lastPP$xx[internal_nodes]
    YY <- lastPP$yy[internal_nodes]
    points(XX,YY,pch=16,col="blue",cex=1)
  }
  return(tob)
}


##################################################################
#' Extract compatible splits
#'
#' Given an object of class splits, first discards any with weight less than a tolerance, and
#' then further removes all remaining splits that are incompatible with any other remaining one.
#
#'
#'@param sp an object of class splits
#'@param tol splits with weights below tol are dropped
#'@param plot a logical; if TRUE plots tree displaying remaining spilts
#'
#'@return splits objects containing only those that are compatible and high weight
#'
#'@examples
#' data(pTableYeastRokas)
#' dist=NANUQdist(pTableYeastRokas, alpha=.05, beta=.80,outfile=NULL)
#' nn=neighborNet(dist)
#' plot(nn,"2D")
#' tob=treeFromSplits(compatibleSplits(nn$splits),plot=TRUE) #produce tree of blobs of splits graph
#'
#'@seealso \code{\link{treeFromSplits}, \link{TINNIK}}
#'
#'@export
compatibleSplits = function(sp, tol = 0, plot = FALSE) {
  if (!is(sp,"splits")) {
    stop("First argument must be of class 'splits'.")
  }

  # compute number of splits
  num_splits <- length(attr(sp,"weights"))

  # filter low support splits using value of tol by setting
  # any split of weight < tol to have weight 0
  low_support_splits <- which(attr(sp,"weights") < tol)
  high_support_splits <- setdiff(1:num_splits,low_support_splits)
  sp <- sp[high_support_splits]

  # determine which remaining splits are compatible
  cmat = as.matrix(compatible(sp))

  # get indices of incompatible splits
  nrows = dim(cmat)[1]
  incompatible_splits = c()
  for (rnum in 1:nrows) {
    if (any(cmat[rnum,]==1)) {
      incompatible_splits <- c(incompatible_splits,rnum)
    }
  }

  # eliminate incompatible splits
  num_splits <- length(attr(sp,"weights"))
  comp_splits <- setdiff(1:num_splits,incompatible_splits)
  sp <- sp[comp_splits]

  if (plot) {
    plot(as.networx(sp))
  }

  # return the splits system with only the compatible splits
  invisible(sp)
}

################################################
#'Produce tree from compatible splits
#'
#'@param sp a compatible split system, as produced by compatibleSplits
#'@param plot, a logical, if TRUE, plot tree
#'
#'@return a phylo object for tree displaying splits
#'
#'@examples
#' data(pTableYeastRokas)
#' dist=NANUQdist(pTableYeastRokas, alpha=.05, beta=.80,outfile=NULL)
#' nn=neighborNet(dist)
#' plot(nn,"2D")
#' tob=treeFromSplits(compatibleSplits(nn$splits),plot=TRUE) #produce tree of blobs of splits graph
#'
#' @seealso \code{\link{compatibleSplits}, \link{TINNIK}}
#'
#'@importFrom phangorn compatible as.networx
#'@importFrom ape nj
#'@importFrom stats cophenetic
#'
#'@export
treeFromSplits = function(sp, plot = FALSE) {

  if (length(compatibleSplits(sp))<length(sp) ) {
    stop("Splits are not all compatible.")
  }

  nn = as.networx(sp)

  D = cophenetic(nn)
  tree = nj(D)
  tree = di2multi(unroot(tree), tol = 1e-10)

  if (plot) {
    plot(tree,type = "unrooted",
         main = "Tree from compatible splits system")
  }
  return(tree)
}

############################################################
#' Collapse short tree edges
#'
#' Set all edges of a tree with short lengths to be zero.
#'
#'@param tree a phylo object
#'@param delta minimum edge length to retain
#'
#'@return a phylo object
#'
#'@examples
#' tree=read.tree(text="((a:1,b:1):.5,(c:.5,d:2):1);")
#' newtree=collapseEdges(tree,delta=1)
#' write.tree(newtree)
#'@export

collapseEdges=function(tree,delta) {
  ntree=tree
  ntree$edge.length=ntree$edge.length*(ntree$edge.length>=delta)
  ntree = di2multi(ntree) # suppress edges of length 0
  return(ntree)
}

#' Multiple independent hypothesis tests for gene quartet counts fitting 
#' a species quartet star tree under the MSC
#'
#' Perform hypothesis tests for a species quartet star tree vs. any alternative for all quartet counts in an input table, 
#' as if the quartets are independent. 
#' 
#' @details This function assumes all quartets are resolved. 
#' The test performed is described in \code{quartetStarTest}.
#'
#' @param rqt  Table of resolved quartet counts, as produced by \code{quartetTableResolved}, or \code{quartetTreeTestInd}
#'
#' @return the same table as the input \code{rqt} with column \code{"p_star"} appended, containing p-values for 
#' judging fit to MSC on a star tree
#'
#' @seealso \code{\link{quartetStarTest}}, \code{\link{quartetTreeTest}}, \code{\link{quartetTreeTestInd}},
#' \code{\link{quartetTableResolved}}, \code{\link{quartetTestPlot}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' pTable=quartetStarTestInd(RQT)
#' quartetTablePrint(pTable[1:6,])
#'
#' @export
quartetStarTestInd = function(rqt) {
  M = dim(rqt)[1]
  qcols = c("12|34", "13|24", "14|23")
  rqt = cbind(rqt, p_star = 0)
  
  message("Applying hypothesis test for star tree model to ",M," quartets.")
  for (i in 1:M) {
    obs = rqt[i, qcols] # get quartet counts
    rqt[i, "p_star"] = quartetStarTest(obs) # store p-value for H0=star tree
  }
  return(rqt)
}

#' Hypothesis test for quartet counts fitting a star tree under the MSC
#'
#' Perform hypothesis test for star tree for a vector of  quartet counts to fit expected frequencies of (1/3,1/3,1/3).
#' The test performed is a standard Chi-square with 2 degrees of freedom.
#'
#' @param obs  vector of 3 counts of resolved quartet frequencies
#' @return p-value
#'
#' @examples
#' obs=c(16,72,12)
#' quartetStarTest(obs)
#'
#' @export
quartetStarTest = function(obs) {
  z = chisq.test(obs) #test agaist uniform expectated values
  p = z$p.value
  return(p)
}

#' Plot histogram of log p-values in table
#'
#' Graphical exploration of extreme p-values from quartet hypothesis tests, to aid in choosing critical values
#' for hypothesis tests. Log base 10 of p-values exceeding some minimum are plotted, to explore gaps in
#' the tail of the distribution.
#'
#' @details
#' Since logarithms are plotted, p-values close to 0 will appear as negative numbers of large magnitude, putting the tail of the distribution
#' to the left in the histogram.
#'
#' When exploring possible critical values for the hypothesis tests in the NANUQ algorithm, use \code{model= "T3"} to
#' choose values for \code{alpha} which distinguish treelikeness from hybridization, and \code{model= "star"} to
#' choose values for \code{beta} which distinguish polytomies from resolved trees.
#' In general, \code{alpha} should be chosen to be small and \code{beta}
#' to be large so that most quartets are interpreted as resolved trees.
#'
#'@param pTable a quartet table with p-values, such as from \code{NANUQ}, \code{quartetTreeTestInd},
#'or \code{quartetStarTestInd}
#'@param model  one of \code{"T1"}, \code{"T3"}, or \code{"star"}, where \code{pTable}
#'contains a column \code{p_model} of p-values
#'@param pmin include only p-values above \code{pmin} in the histogram
#'
#'@examples
#' data(pTableYeastRokas)
#' pvalHist(pTableYeastRokas,"T3")
#'
#'@seealso \code{\link{NANUQ}}, \code{\link{NANUQdist}}
#'
#'@export
pvalHist = function(pTable,
                    model,
                    pmin = 0) {
  if (!(model %in% c("T1","T3","star"))) {
    stop('Argument model must be one of "T1", "T3", or "star".')
  }
  pcol=paste0("p_",model)
  if (!(pcol %in% colnames(pTable))) {
    stop(c('Argument pTable has no column ',pcol,'.'))
  }
  pvals = pTable[, pcol]
  vals = pvals[which(pvals >= pmin)]

  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  par(mar = c(5.1, 4.1, 4.1, 2.1))# set margin
  hist(log10(vals),
       main = bquote("Histogram of log"[10] * "(" * .(pcol) * "), " * .(pcol) >
                       .(pmin)))
}

############################################
#' Topological distances on a tree
#'
#' Compute a pairwise table of topological distances from a tree, after contracting short edges
#'
#'@param tree a phylo object, with or without edge lengths
#'@param epsilon a tolerance, so all edges shorter than epsilon are contracted
#'
#'@return a distance table, with rows and columns named by taxa
#'
#'@export
topDist = function(tree, epsilon=0) {
  #compute topological distance matrix for tree (after contracting branches<=epsilon)
  if (is.null(tree$edge.length)) {
    tree$edge.length =rep(1,dim(tree$edge)[1])
  }
  if (sum(tree$edge.length < 0) > 0) {
    stop("Error: Negative branch length in tree") #if any negative lengths abort
  } else {
    zeros = which(tree$edge.length <= epsilon) # find locations of any zero branch lengths
    tree$edge.length = rep(1,length(tree$edge.length)) # set all lengths to 1
    tree$edge.length[zeros] = 0  # reset zeros to 0
    d = cophenetic.phylo(tree) # compute distances
    snames=sort(colnames(d))# put taxa in order
    d=d[snames,snames]

    return(d)
  }
}


#########################################################################
#' Produce table of counts of quartets displayed on trees
#'
#' Compiles table of quartet count concordance factors (qcCFs) for topological quartets displayed on a
#' collection of trees.
#'
#'
#' @details
#' The names in \code{taxonnames} may be any subset of those on the trees.
#' Branch lengths of non-negative size less than or equal to \code{epsilon}
#' are treated as zero, giving polytomies.
#'
#' In the returned table, columns are labeled by taxon names and quartet names ("12|34", etc.).
#' 1s and 0s in taxon columns indicate the taxa in a quartet. Quartet 12|34
#' means the first and second indicated taxa form a cherry, 13|24 means the first and third form a cherry, 14|23 means
#' the first and fourth form a cherry, and 1234 means the quartet is unresolved.
#'
#' An error occurs if any branch length is negative.
#' Warnings are given if some of \code{taxonnames} are missing on some trees, or
#' if some 4-taxon set is not on any tree.
#'
#' If \code{random}>0, then for efficiency \code{random} should be much smaller then
#' the number of possible 4 taxon subsets.
#'
#' This function calls an Rcpp function for tallying quartets, for much shorter computational
#' time than can be achieved in R alone.
#'
#' @param trees multiphylo object containing un/rooted metric/topological trees
#' @param taxonnames vector of \code{n} names of taxa of interest; if \code{NULL} then taken from taxa on \code{trees[[1]]}
#' @param epsilon minimum for branch lengths to be treated as non-zero (default 0)
#' @param random number of random subsets of 4 taxa to consider; if 0, use all \code{n} choose 4 subsets
#' @param progressbar FALSE, set to TRUE if want to see tally progress
#' @return
#'     an (\code{n} choose 4)x(\code{n}+4) matrix (or (\code{random})x(\code{n}+4) matrix) encoding
#'     4 taxon subsets of \code{taxonnames} and counts of each of the
#'     quartets 12|34, 13|24, 14|23, 1234 across the trees
#'
#'
#' @seealso \code{\link{quartetTableParallel}}, \code{\link{quartetTableResolved}}, \code{\link{quartetTableDominant}}, \code{\link{taxonNames}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT)
#'
#' @importFrom ape cophenetic.phylo compute.brlen
#' @importFrom Rcpp evalCpp
#'
#' @export
quartetTable = function(trees,
                        taxonnames=NULL,
                        epsilon = 0,
                        random = 0,
                        progressbar = FALSE) {
  if (random < 0)
    stop("Parameter 'random' must be non-negative.")
  random = round(random)

  if (is.null(taxonnames)) {
    taxonnames=trees[[1]]$tip.label #get taxa names from 1st tree
    message("Using taxa that appear on first gene tree.")
  }

  taxonnames = sort(taxonnames) #put taxa in fixed order
  nt = length(trees) # number of gene trees
  N = length(taxonnames) # number of taxa
  if (random > 0)
    M = random  # number of 4-taxon sets to consider
  else
    M = choose(N, 4)
  Q = matrix(0, M, N + 4) #allocate space for table
  qnames = c("12|34", "13|24", "14|23", "1234")
  colnames(Q) = c(taxonnames, qnames) #create column names

  message(
    'Counting occurrences of displayed quartets for ',
    M,' four-taxon subsets of ',
    N,' taxa across ',
    nt,' gene trees.'
  )

  dList=lapply(trees,topDist,epsilon=epsilon) # compute all distance tables

  out=quartetTallyCpp(dList,M,nt,Q,random,progressbar=progressbar) #do the rest in C++ for speed
  Q=out$table

  if (out$missingFlag == 1)
    warning("Some taxa missing from some trees.")
  if ((N > 4) && (sum(rowSums(Q[, qnames, drop = FALSE]) == 0) > 0))
    warning("Some 4-taxon set not present on any tree.")
  if (sum(Q[, "1234"]) > 0)
    warning("Some quartets unresolved.")

  return(Q)
}


#################################################

#' Modify quartet table to show only resolved quartets
#'
#' Converts table of all quartet counts, including unresolved ones,
#' by either dropping unresolved ones, or distributing them uniformly
#' among the three resolved counts.
#'
#' @param qt table, as produced by \code{quartetTable} for \code{n} taxa, with \code{n+4} columns
#' @param omit \code{TRUE} deletes unresolved quartets column;
#'            \code{FALSE} deletes the column but redistributes unresolved counts as (1/3,1/3,1/3) to resolved counts
#' @return
#'      a table of quartet counts similar to \code{qt}, but with columns showing only resolved quartet counts
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' QT[1:6,]
#' RQT=quartetTableResolved(QT)
#' RQT[1:6,]
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableDominant}}
#' @export
quartetTableResolved = function(qt,
                                omit = FALSE) {
  if ("1234" %in% colnames(qt)) { # if unresolved column present
    RT = qt[,-which(colnames(qt) == "1234"), drop = FALSE]  #copy table without unresolved column
    if (omit == FALSE) {
      qs = c("12|34", "13|24", "14|23") #columns to be adjusted
      RT[, qs] = RT[, qs, drop = FALSE] + qt[, "1234", drop = FALSE] %*% matrix(1 / 3, 1, 3) #redistribute counts
    }
  } else { #no unresolved column present
    RT = qt
  }
  return(RT)
}

#################################################


#' Produce table of dominant quartets, with estimates of internal edge lengths
#'
#' Converts table of counts of resolved quartets on \code{n} taxa to show only dominant one, with
#' maximum likelihood estimate of internal edge weight under the MSC.
#'
#' @details
#' If \code{bigweights="finite"}, when for a set of 4 taxa the quartet counts are (m,0,0) then
#' the edge weight is computed as if the relative frequency of the dominant topology were m/(m+1).
#'
#' @param rqt a table, as produced by \code{quartetTableResolved} of size (n choose 4)x(n+3);
#' @param bigweights \code{"infinite"} or \code{"finite"}, to indicate whether the weight (internal edge length)
#' of a quartet for which only one
#' topology appears is given as \code{Inf} or a finite, but large, numerical value
#'
#' @return
#'   an (n choose 4)x(n+1) array with dominant quartet topology encoded by 1,1,-1,-1 in
#' taxon columns, with signs indicating cherries; the (n+1)th column \code{"weight"} contains the maximum likelihood estimates,
#' under MSC on a 4-taxon tree, of the quartets' central edge lengths, in coalescent units
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' RQT[1:6,]
#' DQT=quartetTableDominant(RQT)
#' DQT[1:6,]
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableResolved}}
#' @importFrom stats runif
#' @export
quartetTableDominant = function(rqt,
                                bigweights="infinite") {
  M = dim(rqt)[1] # number of 4-taxon sets
  N = dim(rqt)[2] - 3 # number of taxa
  taxonnames = colnames(rqt)[1:N] # names of taxaR


  DQT = rqt[, 1:(N + 1)] # allocate space for most frequent quartet topologies and weight
  if (is.vector(DQT)){ # if only 1 row,
    dim(DQT)=c(1,N+1) # make it a matrix,
    colnames(DQT)=colnames(rqt)[1:(N+1)] #with column names
    }
  for (m in 1:M) {
    # for each 4-taxon set
    taxa = which(rqt[m, 1:N] == 1) # taxa numbers
    z = rqt[m, (N + 1):(N + 3)] # frequencies of quartets
    tz = sum(z) # total number of quartets
    if (tz > 0) {
      # if some quartets were found
      mq = which.max(z + runif(3)) # which is biggest? (random tie-breaking)
      if (mq == 1) {
        DQT[m, c(taxa[3], taxa[4])] = -1
      } else {
        if (mq == 2) {
          DQT[m, c(taxa[2], taxa[4])] = -1
        } else {
          DQT[m, c(taxa[2], taxa[3])] = -1
        }
      }
      f = z[mq] / tz #relative frequency of most frequent quartet
      if (f<1) { # if some ILS
        DQT[m, N + 1] = -log((1.5 * (1 - f))) # save weight
      } else { # if no ILS
      if (bigweights =="infinite"){
        DQT[m, N + 1] = Inf
      } else { #despite no ILS give finite weight
      f = tz/(tz+1) # pretend f is a bit less than 1
      DQT[m, N + 1] = -log((1.5 * (1 - f))) # save weight
    }
  }

    }
  }
  colnames(DQT) = c(taxonnames, "weight") #add column names
  return(DQT)
}

######################################################################################

#' Get all taxon names from a collection of trees
#'
#' Create a vector of all taxon names appearing on a collection of trees, with no repeats.
#'
#' @param trees a multiPhylo object containing a collection of trees
#'
#' @return a vector of unique names of taxa appearing on the trees
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#'
#' @export
taxonNames = function(trees) {
  taxa = c()
  for (i in 1:length(trees)) {
    taxa = union(taxa, trees[[1]]$tip.label)
  }
  return(taxa)
}

###################################################################################
#' Print a quartet table with nice formatting
#'
#' Print a quartet table with the taxa in each quartet shown by name.
#'
#' @param qt a table such as returned by \code{quartetTable}, \code{quartetTableResolved},
#' or \code{quartetTableDominant}, possibly with extra columns added by other functions
#'
#' @return NULL
#'
#' @examples
#'  gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#'  tnames=taxonNames(gtrees)
#'  QT=quartetTable(gtrees,tnames[1:6])
#'  QT[1:6,]
#'  quartetTablePrint(QT[1:6,])
#'  RQT=quartetTableResolved(QT)
#'  RQT[1:6,]
#'  quartetTablePrint(RQT[1:6,])
#'  pTable=quartetTreeTestInd(RQT,"T3")
#'  pTable[1:6,]
#'  quartetTablePrint(pTable[1:6,])
#'  DQT=quartetTableDominant(RQT)
#'  DQT[1:6,]
#'  quartetTablePrint(DQT[1:6,])
#'
#'@export
quartetTablePrint <- function (qt) {

  # If only send one row, convert to matrix
  if (!("matrix" %in% class(qt)))
  {
    qt = t(as.matrix(qt))
  }

  nrows = dim(qt)[1]
  ncols = dim(qt)[2]
  colsToDisplay = rep(0, nrows)
  for (i in 1:nrows) {
    colsToDisplay[i] = which(qt[i, ] != 0)[5]
  }
  minColNumber = min(colsToDisplay)
  colsToDisplay = minColNumber:ncols
  if (-1 %in% qt[1, 1:(minColNumber - 1)]) {
    prettyDisplay = matrix(0, nrows, 1 + length(colsToDisplay))
    for (i in 1:nrows) {
      cherry1 = which(qt[i, 1:(minColNumber - 1)] == 1)
      cherry2 = which(qt[i, 1:(minColNumber - 1)] == -1)
      qtree = paste0(colnames(qt)[cherry1[1]], " ", colnames(qt)[cherry1[2]],
                     " | ", colnames(qt)[cherry2[1]], " ", colnames(qt)[cherry2[2]],
                     " ")
      prettyDisplay[i, ] = c(qtree, qt[i, colsToDisplay])
    }
    colnames(prettyDisplay) <- c(" ", colnames(qt)[colsToDisplay])
  }
  else {
    prettyDisplay = matrix(0, nrows, 4 + length(colsToDisplay))
    for (i in 1:nrows) {
      prettyDisplay[i, ] = c(colnames(qt)[which(qt[i, 1:(minColNumber -
                                                           1)] != 0)], qt[i, colsToDisplay])
    }
    colnames(prettyDisplay) <- c(rep(" ", 4), colnames(qt)[colsToDisplay])
  }
  rownames(prettyDisplay) <- rep("", nrows)
  print(as.table(prettyDisplay))
}

###############################################################
#'Sort quartet table rows by lex order
#'
#'Sort the rows of a quartet table so they are in MSCquartet's standard lex order.
#'This is the order produced by the \code{quartetTable} function. The only exceptions
#'to this order produced in the package are when \code{quartetTable} is called with the
#'\code{random} argument non-zero, or when the \code{HolmBonferoni} function is called.
#'However, for tables made outside this package, it can be useful.
#'
#' @param qT a quartet Table to be sorted
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{HolmBonferroni}}
#'
#' @return sorted table
#'
#' @export
sortQuartetTableRows <- function(qT) {
  l = as.list(as.data.frame(qT))
  inds <- do.call("order", c(l, list(decreasing = TRUE)))
  qT = qT[inds,]

  return(qT)
}

#' Multispecies Coalescent Model Quartet Package
#'
#' A package for analyzing quartets displayed on gene trees, under the
#' multispecies coalescent (MSC) model and network multispecies coalescet model (NMSC).
#'
#' @details
#' This package contains routines to analyze a collection of gene trees through the displayed
#' quartets on them.
#'
#' A quartet count concordance factor (qcCF) for a set of 4 taxa is the triple of counts of the three
#' possible resolved quartet trees on those taxa across some set of gene trees. The major routines in this package can:
#' \enumerate{
#' \item Tabulate all qcCFs for a collection of gene trees.
#' \item Perform hypothesis tests of whether one or more qcCFs are consistent with the MSC model on
#' a species tree \insertCite{MAR19}{MSCquartets}.
#' \item Produce simplex plots showing all estimated CFs as well as results of hypothesis tests \insertCite{AMR2020}{MSCquartets}.
#' \item Infer a species tree using the qcCFs via the QDC and WQDC methods \insertCite{Rho19,YR19}{MSCquartets}.
#' \item Infer a level-1 species network via the NANUQ method \insertCite{ABR19}{MSCquartets}.
#' \item Infer the tree of blobs for a species network via the TINNiK or ECToBlob methods \insertCite{ABMR22}{MSCquartets},\insertCite{ABMR24}{MSCquartets}\insertCite{ECTo2026}{MSCquartets}.
#' \item Resolove multifurcations in a tree of blobs to cycles via NANUQ+ routines \insertCite{ABRW25}{MSCquartets}.
#' }
#'As discussed in the cited works, the inference methods for species trees and networks are
#'statistically consistent under the MSC and Network MSC respectively.
#'
#'This package, and the theory on which it is based, allows gene trees to have
#'missing taxa (i.e., not all gene trees display all the taxa). It does require
#'that each subset of 4 taxa is displayed on at least one of the gene trees.
#'
#'
#' Several gene tree data sets, simulated and empirical, are included.
#'
#' In publications please cite the software announcement \insertCite{RBMA2020}{MSCquartets}, as well as the
#' appropriate paper(s) above developing the theory behind the routines you used.
#'
#' @references
#'
#' \insertRef{RBMA2020}{MSCquartets}
#'
#' \insertRef{MAR19}{MSCquartets}
#'
#' \insertRef{AMR2020}{MSCquartets}
#'
#' \insertRef{Rho19}{MSCquartets}
#'
#' \insertRef{YR19}{MSCquartets}
#'
#' \insertRef{ABR19}{MSCquartets}
#'
#' \insertRef{ABMR22}{MSCquartets}
#'
#' \insertRef{ABMR24}{MSCquartets}
#'
#' \insertRef{ABRW25}{MSCquartets}
#'
#' \insertRef{ECTo2026}{MSCquartets}
#'
#' @importFrom graphics hist legend mtext par plot points segments text
#' @importFrom stats chisq.test dmultinom pchisq qnorm var na.omit pcauchy
#' @importFrom Rdpack reprompt
#' @importFrom Rcpp sourceCpp
#'
#' @useDynLib MSCquartets
#'
"_PACKAGE"
NULL





#' Yeast gene tree dataset
#'
#' A text file dataset for Yeast containing 106 gene trees on 8 taxa  (7 Saccharomyces and 1 Candida outgroup). This is a subset of the data
#' of \insertCite{Rokas03;textual}{MSCquartets}.
#'
#' @docType data
#'
#' @name dataYeastRokas
#'
#' @references \insertRef{Rokas03}{MSCquartets}
#'
#' @details File is accessed as \code{system.file("extdata","dataYeastRokas",package="MSCquartets")}, for example
#' via the \code{ape} command:
#'
#' \code{ gts=read.tree(file = system.file("extdata","dataYeastRokas",package="MSCquartets"))}
#'
#' @format A text file with 106 topological Newick gene trees on the taxa:
#'  Sbay, Scas, Scer, Sklu, Skud, Smik, Spar, and Calb (outgroup).
#'
#' @source from L. Nakhleh's website at Rice U., link no longer available as of 2026
NULL

#' pTable for Yeast dataset
#'
#' An .rda file dataset for the "dataYeastRokas" dataset. This is a subset of the data
#' of \insertCite{Rokas03;textual}{MSCquartets}.
#'
#' @docType data
#'
#' @name pTableYeastRokas
#'
#' @usage data(pTableYeastRokas)
#'
#' @references \insertRef{Rokas03}{MSCquartets}
#'
#' @details This is provided primarily so that examples of other functions run more quickly. It can be reproduced by the following
#' example code below.
#'
#' @examples
#' \donttest{
#' gtrees=read.tree(file = system.file("extdata","dataYeastRokas",package="MSCquartets"))
#' QT=quartetTable(gtrees)
#' RQT=quartetTableResolved(QT)
#' pTable=quartetCutTestInd(RQT)
#' pTable=quartetTreeTestInd(pTable)
#' pTableYeastRokas=quartetStarTestInd(pTable)
#' }
#' @format an R data file
#'
#' @source from L. Nakhleh's website at Rice U., link no longer available as of 2026
"pTableYeastRokas"



#' Papionini gene tree dataset
#'
#' A text file dataset for Papionini containing 1730 gene trees on 7 taxa. This is a subset of the data of \insertCite{Vanderpool2020;textual}{MSCquartets}.
#'
#' @references \insertRef{Vanderpool2020}{MSCquartets}
#'
#' @docType data
#'
#' @name dataPapioniniVanderpool
#'
#' @details File is accessed as \code{system.file("extdata","dataPapioniniVanderpool",package="MSCquartets")}, for
#' example via the \code{ape} command:
#'
#' \code{gts = read.tree(file=system.file("extdata","dataPapioniniVanderpool",package="MSCquartets"))}
#'
#' @format A text file with 1703 metric Newick gene trees each with 7 leaves labelled:\cr
#' Cercocebus_atys, Mandrillus_leucophaeus, Papio_anubis, Theropithecus_gelada,
#' Macaca_fascicularis, Macaca_mulatta, Macaca_nemestrina
#'
#' @source \doi{10.5061/dryad.rfj6q577d}
#'
NULL

#' Simulated gene tree dataset from species tree
#'
#' A text file dataset containing 1000 gene trees on 9 taxa simulated under the MSC on a species tree
#'
#' @details This simulated dataset was produced by SimPhy \insertCite{simphy}{MSCquartets}, using the species tree
#'
#' ((((t5:5000,t6:5000):5000,t4:10000):2500,t7:12500):7500,((t8:3000,t9:3000):5000,\cr
#' ((t1:4000,t2:4000):2500,t3:6500):1500):12000);
#'
#' with a population size of 10,000 throughout the tree.
#'
#' @docType data
#'
#' @name dataGeneTreeSample
#'
#' @details File is accessed as  \code{system.file("extdata","dataGeneTreeSample",package="MSCquartets")}, for example
#' via the ape command:
#'
#' \code{gts=read.tree(file = system.file("extdata","dataGeneTreeSample",package="MSCquartets") )}
#'
#' @format A text file with 1000 metric Newick gene trees on the taxa t1-t9
#'
#' @references
#' \insertRef{simphy}{MSCquartets}
#'
NULL



#' Quartet table for Heliconius gene tree dataset
#'
#' An .rda file containing a quartet table summarizing the "Heliconius" gene trees
#' of \insertCite{Martin2013;textual}{MSCquartets}. This table contains quartet counts
#' from 2909 gene trees on 7 taxa, with 4 individuals sampled for each of 3 of the taxa,
#' for a total of 16 leaves per gene tree.
#'
#' @docType data
#'
#' @name tableHeliconiusMartin
#'
#' @usage data(tableHeliconiusMartin)
#'
#' @references \insertRef{Martin2013}{MSCquartets}
#'
#' @details This table is provided rather than the original gene trees to save storage space.
#' If used, please cite \insertCite{Martin2013;textual}{MSCquartets}.
#'
#' @source \doi{10.5061/dryad.dk712}
#'
#' @format an R data file
"tableHeliconiusMartin"


#' Quartet table for Leopardus dataset
#'
#' An .rda file containing a quartet table summarizing the "Leopardus" gene trees
#' of \insertCite{Lescroart2023;textual}{MSCquartets}. This table contains quartet counts
#' for 16,338 "gene" trees for 16 taxa, inferred from sliding widows across
#' genomic sequences.
#'
#' @docType data
#'
#' @name tableLeopardusLescroart
#'
#' @usage data(tableLeopardusLescroart)
#'
#' @references \insertRef{Lescroart2023}{MSCquartets}
#'
#' @details This table is provided rather than the original gene trees to save storage space.
#' If used, please cite \insertRef{Lescroart2023}{MSCquartets}.
#'
#' @format an R data file
"tableLeopardusLescroart"

#' ASTRAL tree for Leopardus dataset
#'
#' An .rda file containing the tree produced by ASTRAL from the "Leopardus" gene trees
#' of \insertCite{Lescroart2023;textual}{MSCquartets}. By the theory of \insertCite{ECTo2026}{MSCquartets}, 
#' this is a putative resolution of the tree of blobs for the unknown network, 
#' and can be used by \code{ECToBlob} to infer the network's tree of blobs.
#'
#' @docType data
#'
#' @name ASTRALtreeLeopardusLescroart
#'
#' @usage data(ASTRALtreeLeopardusLescroart)
#'
#' @references \insertRef{Lescroart2023}{MSCquartets}, \insertRef{ECTo2026}{MSCquartets}
#'
#' @details If used for purposes other than learning software, please cite \insertRef{Lescroart2023}{MSCquartets}.
#'
#' @format an R data file
"ASTRALtreeLeopardusLescroart"

#'Label internal nodes on tree
#'
#'Label all internal nodes of tree, as "Node NN" where NN is the node number, and
#' plot tree with labels
#'
#'@param tree a rooted or unrooted tree (phylo object)
#'@param plot TRUE for plot, FALSE for no plot
#'@param type plot type (e.g.,"unrooted") to be passed to ape plot command
#'
#'@return a phylo object with internal node labels
#'
#'@examples
#' data(pTableYeastRokas)
#' out=TINNIK(pTableYeastRokas,test="T3",alpha=.01, beta=.05)
#' labelIntNodes(out$ToB)
#'
#' @seealso \code{\link{resolveCycle}}, \code{\link{combineCycleResolutions}},
#'   \code{\link{resolveLevel1}}
#'
#'@export
#'
labelIntNodes = function(tree, plot = TRUE, type = "unrooted")
{
  #Label internal nodes and plot ToB
  treeLab = tree
  treeLab$node.label = paste0("Node ", 1:treeLab$Nnode + length(treeLab$tip.label))

  if (plot > 0)
  {
    plot(
      treeLab,
      show.node.label = T,
      type = "unrooted",
      main = "Internal Node Labels"
    ) #plot tree with node numbers
  }
  return(treeLab)
}



####################################################
#' Resolve a node on a tree of blobs as a cycle
#'
#' Given a Tree of Blobs and quartet Concordance Factor data, resolve a specific
#' polytomy to a cycle. Resolution is performed by finding a least-squares
#' best-fit of an empirical distance to an expected distance related to the cycle,
#' as described in \insertCite{ABRW25;textual}{MSCquartets}.
#'
#' Possible distances to use are the NANUQ distance and a modified NANUQ distance
#' that weights quartet trees differently, but from which the cycle structure is
#' still identifiable.
#'
#' For multifucations of degree less than a designated cutoff, all possible
#' circular orders and choices of hybrid nodes are considered in choosing the
#' best. Above that cutoff, a heuristic method built on the modified NANUQ
#' distance is used to obtain a small number of orders likely to be good fits,
#' with the least-squares fitting applied only to those.
#'
#'
#'@references
#' \insertRef{ABRW25}{MSCquartets}
#'
#'@param ToB an unrooted tree of blobs (phylo object), as determined by ECToBlob, TINNIK
#'or another method, with multifurcations labelled by \code{labelIntNodes}
#'@param node number of an internal node to be resolved
#'@param pTable a table of qcCFs, with columns p_star and p_test
#'@param test either "T3" or "cut", indicating the test to use for determining what
#'qcCFs indicate hybridization
#'@param alpha test level for p_test
#'@param beta test level for p_star
#'@param distance cycle resolution distance to be used; one of "NANUQ" or "modNANUQ"
#'@param hdegree resolve a multifurcation of this degree or larger by a heuristic
#'method; must be at least 5
#'@param plot if FALSE (0), no plots; if TRUE (>0), make plots of resolved cycle(s) considered
#'best and histogram of measure of fit for all hybrid/orders considered
#'@param delta cutoff for relative difference in squared residuals and smallest,
#'(RSS-minRSS)/minRSS, for determining near ties as "best" fit resolutions
#'
#'@return a list of resolution information, given as a list of:
#'\itemize{
#'\item \code{$node} node number,
#'\item \code{$cycleRes} list [[1]]-[[k]] of  best resolutions,
#'\item \code{$RSSs} RSSs from all cycle resolutions considered in choosing best.
#'}
#'Each resolution is itself a 5-element list with entries:
#'\itemize{
#'\item \code{$cycleNet} Newick network with 1 cycle (with all edge lengths 1)
#'\item \code{$cycleRSS} RSS for cycle,
#'\item \code{$taxonGroups} taxon groups for cycle,
#'\item \code{$order} order of groups around cycle,
#'\item \code{$nonRootEdges} logical vector indicating edges of \code{ToB}
#' where root cannot be.
#' }
#' (Items \code{$taxonGroups,$order,$nonRootEdges} are
#' needed to combine resolutions to form networks with multiple cycles using
#' \code{\link{combineCycleResolutions}}, and otherwise may not be of interest to users).
#'
#'
#'@examples
#' data(pTableYeastRokas)
#' out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)
#' ToB=labelIntNodes(out$ToB)
#' resolveCycle(ToB, node=9, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")
#'
#'@seealso \code{\link{ECToBlob}} \code{\link{TINNIK}}, \code{\link{labelIntNodes}}, \code{\link{combineCycleResolutions}},
#' \code{\link{resolveLevel1}}
#'
#'@importFrom ape keep.tip evonet write.evonet root bind.tree
#'@importFrom phangorn mrca.phylo Descendants
#'@importFrom graphics hist
#'
#'@export
resolveCycle = function(ToB,
                        node,
                        pTable,
                        test = "T3",
                        alpha,
                        beta,
                        distance = "NANUQ",
                        hdegree = 10,
                        plot = TRUE,
                        delta = 10 ^ -6)
{
  if (is.rooted(ToB)) {
    stop("Argument 'ToB' must be unrooted.")
  }
  if (is.null(ToB$node.label)) {
      stop("Argument 'ToB' must have internal nodes labelled by 'labelIntNodes'.")
    }
  if (!(distance %in% c("NANUQ", "modNANUQ"))) {
    stop("Argument 'distance' is not valid; must be 'NANUQ' or 'modNANUQ'.")
  }

  #Initialize
  nodeResolutions = list(node = node)# storage for Newick and order info for this node's resolutions
  cycleRes = list()# storage for all best cycle resolutions of this node we find

  taxa = ToB$tip.label
  ntaxa = length(taxa)

  groups = nodeGroups(ToB, node) #Determine groups of taxa around node to be resolved
  nGroups = length(groups) #degree of multifurcation
  if (nGroups <= 3) {
    stop(paste0("Node degree is ", nGroups, "; no need to resolve."))
  }
  hdegree = max(c(hdegree, 5)) #Heuristic not used for 4-blobs
  heurflag = (nGroups >= hdegree) # set flag, TRUE means use heuristic method

  outgroup = groups[[nGroups]] # make a copy of tree rerooted so last group is the outgroup
  ToBr = ape::root(ToB, outgroup = outgroup)

  #convert groups to a vector encoding
  groupvec = rep(0, ntaxa) #make it easy to look up group by taxon name
  names(groupvec) = taxa
  for (j in 1:nGroups) {
    groupvec[groups[[j]]] = j # put group number in list by taxon name
  }

  if (heurflag) {
    D = blobDistance(pTable,
                     taxa,
                     groupvec,
                     test = 'T3',
                     alpha,
                     beta,
                     dist = "modNANUQ") #compute modNANUQ distance for these blob groups

    orders = ordersHeuristicmodNANUQ(D, delta) #  use Min modNANUQ distance approach to find orders
  } else {
    orders = circHybOrders(nGroups) # consider all orders
  }

  #Find measures of fit
  D = blobDistance(pTable,
                   taxa,
                   groupvec,
                   test = 'T3',
                   alpha,
                   beta,
                   dist = distance) #compute empirical distance for these blob groups

  if (distance == 'NANUQ') {
    #Find expected sunlet distance for the distance
    E = expNANUQCycleDist(dim(D)[1])
  } else {
    E = expmodNANUQCycleDist(dim(D)[1])
  }


  fitMeasure = fitCycleOrders(D, E, orders) # measure fit with orders

  minfit = min(fitMeasure) #find best fit measure
  bestorders = which((fitMeasure - minfit) <= minfit * delta)

  if (length(bestorders) > 1) {
    message(paste0("For node ", node, ", ", length(bestorders)," resolutions found."))
    bestorders = bestorders[order(fitMeasure[bestorders])] #sort so best are first
  }

  for (k in bestorders) {
    # consider all best fit orders
    nonRootEdges = rep(F, dim(ToB$edge)[1]) #indicator vector for edges we know can't contain root

    order = orders[k, ]
    RSS = fitMeasure[k]
    if (nGroups > 4) {
      # for large cycles, flag edges below hybrid
      hybGroup = which(groupvec == match(1, order))
      if (length(hybGroup) == 1) {
        hybDesc = hybGroup
      }
      else{
        #first make sure ToB root is outside of hybrid group subtree
        mrca = phangorn::mrca.phylo(ToB, hybGroup)
        hybDesc = c(mrca, phangorn::Descendants(ToB, mrca, "all"))
      }

      if (length(hybDesc) != ToB$Nnode + ntaxa)
        #if (arbitrary) root of ToB was not in hybrid group subtree
      {
        nonRootEdges[match(hybDesc, ToB$edge[, 2])] = TRUE # flag edges ending at these nodes as below a hybrid
      } else
        #root of ToB is in hybrid group subtree (due to unfortunate rooting of ToB)
      {
        hyfitMeasureonDesc = phangorn::Descendants(ToB, node, "all")
        nonRootEdges[which(!(ToB$edge[, 2] %in% hyfitMeasureonDesc))] = TRUE # flag all edges in hybrid group subtree as below a hybrid
      }
    }

    #generate tree that will represent cycle once reticulation edge added
    groupt = read.tree(text = paste0("(", -nGroups, ",-1);")) # generate phylo object, mostly to be written over, using negative numbers as taxa names
    groupt$edge = matrix(0, nGroups + 2, 2)
    groupt$edge[, 1] = c(3, 3:(3 + nGroups))
    groupt$edge[, 2] = c(1, 4:(3 + nGroups), 2)
    groupt$edge.length = rep(1, nGroups + 2)
    groupt$Nnode = nGroups + 1
    groupt$node.label = c("r", "TH1", -(nGroups - 1):-2, "#H1")

    for (i in 1:nGroups) {
      #attach subtrees for each group
      gtaxa = which(groupvec == i)
      subtree = ape::keep.tip(ToBr, gtaxa)
      subtree$root.edge = 1 #add a root edge for resolution when attached
      attachlabel = -order[i]# attach at this label in cycle tree just created
      if (attachlabel %in% c(-1, -nGroups)) {
        attachPoint = which(groupt$tip.label == attachlabel) #will attach at a tip
      } else {
        attachPoint = length(groupt$tip.label) + which(groupt$node.label == attachlabel) #will attach at an internal node
      }
      groupt = ape::bind.tree(groupt, subtree, where = attachPoint)# attach the subtree
    }

    # add reticulation edge, and clean up internal node labels, edge lengths
    rt = which(groupt$node.label == "rt")
    H1 = which(groupt$node.label == "#H1")
    TH1 = which(groupt$node.label == "TH1")
    groupt$node.label = rep("", length(groupt$node.label))
    groupt$node.label[c(H1, TH1)] = c("#H1", "TH1")
    nTips = length(groupt$tip.label)
    groupn = ape::evonet(groupt, from = nTips + TH1, to = nTips + H1)
    groupn$edge.length = rep(1, length(groupn$edge.length))
    groupn=suppressNetBinaryNodes(groupn,addlength=FALSE)

    if (plot > 0)
    {
      titletext = bquote("distance: " * .(distance) * "," * ~ alpha * "=" * .(alpha) * "," ~ beta * "=" * .(beta))

      subtitle = "Network should be semidirected; Rooting is arbitrary."
      if (nGroups == 4)
        subtitle = "Network should be semidirected; Rooting is arbitrary.\n Hybrid node on 4-cycle is not identified."
      plot(
        groupn,
        show.node.label = T,
        type = "unrooted",
        main = paste0(
          "Resolution ",
          which(bestorders == k),
          " of Node ",
          node,
          "; RSS=",
          signif(RSS, digits = 6)
        ),
        sub = subtitle
      )
      mtext(eval(bquote(.(titletext))),
            side = 3,
            line = 0,
            cex = 1) # add rest of title text
    }

    netnwk = ape::write.evonet(groupn)
    netnwk=gsub(":-?[0123456789]+",":1",netnwk) #replace ape's odd branch lengths by 1
    cycleRes = append(cycleRes, list(
      list(
        cycleNet = netnwk,
        cycleRSS = fitMeasure[k],
        taxonGroups = groupvec,
        order = order,
        nonRootEdges = nonRootEdges
      )
    ))
    #save this resolution and all info
  }

  nodeResolutions = list(node = node,
                         cycleRes = cycleRes,
                         RSSs = list(as.double(fitMeasure))) #save for output

  if (plot > 0) {
    #
    graphics::hist(fitMeasure,
                   main = paste0(
                     "Node ",
                     node,
                     "; RSSs for ",
                     length(fitMeasure),
                     " orders considered"
                   ))
    mtext(eval(bquote(.(titletext))),
          side = 3,
          line = 0,
          cex = 1) # add rest of title text
  }
  return(nodeResolutions)
}

####################################################
#' Resolve Tree of Blobs to Level-1 network
#'
#' Given a Tree of Blobs and qcCF information, resolve all multifurcations to
#' cycles. Resolution is performed by finding a least-squares best-fit of an
#' empirical distance to an expected distance related to the cycle, as described
#' in \insertCite{ABRW25;textual}{MSCquartets}.
#'
#' Possible distances to use are the NANUQ distance and a modified NANUQ distance
#' that weights quartet trees differently, but from which the cycle structure is
#' still identifiable.
#'
#' For multifucations of degree less than a designated cutoff, all possible
#' circular orders and choices of hybrid nodes are considered in choosing the
#' best. Above that cutoff, a heuristic method  is used to obtain
#' a small number of orders likely to be good fits, with the least-squares
#' fitting applied only to those.
#'
#'@references
#' \insertRef{ABRW25}{MSCquartets}
#'
#'@param ToB an unrooted tree of blobs (phylo object) as determined by ECToBlob, TINNIK or
#'another method
#'@param pTable a table of qcCFs, with columns p_star and p_test
#'@param test either "T3" or "cut", indicating test to use for determining what
#'qcCFs indicate hybridization
#'@param alpha test level for p_test
#'@param beta test level value for p_star
#'@param distance cycle resolution distance to be used ("NANUQ" or "modNANUQ")
#'@param hdegree resolve a multifurcation of this degree or larger by a heuristic
#'method; must be at least 5
#'@param plot if 0, no plots; if 1, plot only possible root locations on ToB and
#'full resolution; if 2 (default), include plots of each individual blob resolution, if 3
#'include histograms of measure of fit for all hybrid/orders considered in choosing
#'best
#'@param delta cutoff for relative difference in squared residuals and smallest,
#'(RSS-minRSS)/minRSS, for determining near ties as "best" fit resolutions
#'@param fullResMax maximum number of full resolutions (all multifurcations at once)
#' to form; if the product of the number of resolutions of individual multifurcations
#' exceeds this, no full resolutions are produced, although \code{\link{combineCycleResolutions}}
#' can be applied to produce them.
#'
#'
#'@return a list of resolutions and squared residuals:
#'\itemize{
#'\item [[1]] is a list of Newick
#'resolutions of entire network, with all edge lengths 1 (NULL if one cannot be produced or \code{fullResMax} is exceeded),
#'\item [[2]]-[[n]] are individual resolutions of each  multifurcation on \code{ToB},
#'each given as a list as output from \code{\link{resolveCycle}}.
#'}
#'
#'@examples
#' data(pTableYeastRokas)
#' out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)
#' resolveLevel1(out$ToB, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")
#'
#'@seealso \code{\link{ECToBlob}}, \code{\link{TINNIK}}, \code{\link{labelIntNodes}}, \code{\link{resolveCycle}},
#' \code{\link{combineCycleResolutions}}
#'
#'@importFrom ape keep.tip evonet write.evonet root bind.tree
#'@importFrom phangorn mrca.phylo Descendants
#'@importFrom graphics hist
#'
#'@export
resolveLevel1 = function(ToB,
                         pTable,
                         test = "T3",
                         alpha,
                         beta,
                         distance = "NANUQ",
                         hdegree = 10,
                         plot = 2,
                         delta = 10 ^ -6,
                         fullResMax = 10)
{
  if (is.rooted(ToB)) {
    stop("Argument 'ToB' must be unrooted.")
  }

  #Label internal nodes and plot ToB
  ToBLab = labelIntNodes(ToB, plot, type = "unrooted") #Label internal nodes, and plot

  taxa = ToB$tip.label
  ntaxa = length(taxa)
  resolutions = c() #list for all resolutions

  #Determine multifurcations on ToB to be resolved into cycles
  polyList = c()
  if (length(Descendants(ToB, ntaxa + 1, 'children')) > 3) {
    polyList = c(polyList, ntaxa + 1)
  }
  if (ToB$Nnode >= 2) {
    for (i in (ntaxa + 2):(ntaxa + ToB$Nnode)) {
      #loop over internal nodes
      if (length(Descendants(ToB, i, 'children')) > 2) {
        polyList = c(polyList, i)
      }
    }
  }
  npoly = length(polyList)

  if (npoly == 0) {
    message("No multifurcations on tree.")
    resolutions=list(list(ToB))
  } else {
    if (npoly == 1) {
      message("1 multifurcation on tree, at node ",
              as.character(polyList))
    } else {
      message(
        npoly,
        " multifurcations on tree, at nodes ",
        paste(as.character(polyList), collapse = ", "),
        "."
      )
    }

    #Initialize
    nonRootEdges = rep(FALSE, dim(ToB$edge)[1]) #indicator vector for edges that can't contain root
    #resolveAll = TRUE # flag to produce one resolved network at end

    # Resolve each multifurcation
    numFullRes = 1 #compute number of Full resolutions
    for (node in polyList) {
      resCyc = resolveCycle(ToBLab,
                            node,
                            pTable,
                            test,
                            alpha,
                            beta,
                            distance,
                            hdegree,
                            plot - 1,
                            delta)
      numFullRes = numFullRes * length(resCyc$cycleRes)
      resolutions = append(resolutions, list(resCyc)) #add these resolutions to list for all polytmoies
    }
    # combine resolutions of polytomies
    if (numFullRes > fullResMax)
    {
      warning("Exceeded fullResMax; use 'combineCycleResolutions' to see full resolutions")
      nets = list("fullResMax exceeded; full resolutions not produced")
    }
    else {
      titletext = bquote("distance: " * .(distance) * "," * ~ alpha * "=" * .(alpha) * "," ~ beta * "=" * .(beta))
      nets = (combineCycleResolutions(ToBLab, resolutions, plot, titletext))
    }
    resolutions = append(list(nets), resolutions)
  }
  return(resolutions)
}


#############################################################
#' Combine several cycle resolutions on a tree of blobs to create a network
#'
#' Given a list of resolutions of different multifurcations on a tree of blobs
#' (each as produced by \code{\link{resolveCycle}}), combine these with the tree
#' of blobs to form a network.
#'
#' This function is useful for forming near-optimal networks when there are
#' several resolutions that have similar fit for some of the multifurcations.
#'
#' @param ToB an unrooted tree of blobs for the network, with multifurcating nodes
#' labelled by \code{\link{labelIntNodes}}
#' @param resolutions a list of resolutions (each of which may be a list)
#' for different nodes with elements in format described in output of \code{\link{resolveCycle}}
#' @param plot if FALSE (0), no plots; if TRUE (>0) plot networks
#' @param titletext a string of text for plot
#'
#' @return a list of Newick strings for the networks, with all edge lengths 1
#'
#' @examples
#' data(pTableYeastRokas)
#' out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)
#' ToB=labelIntNodes(out$ToB)
#' R9=resolveCycle(ToB, node=9, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")
#' R10=resolveCycle(ToB, node=10, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")
#' combineCycleResolutions(ToB, resolutions=list(R9,R10),plot=TRUE)
#'
#'@seealso \code{\link{ECToBlob}}, \code{\link{TINNIK}}, \code{\link{labelIntNodes}}, \code{\link{resolveCycle}},
#'  \code{\link{resolveLevel1}}
#'
#' @export
combineCycleResolutions = function(ToB,
                                   resolutions,
                                   plot = 1,
                                   titletext = NULL)
{
  if (is.null(ToB$node.label)) {
    stop("Argument 'ToB' must have internal nodes labelled by 'labelIntNodes'.")
  }
  savedToB = ToB
  nEdgeToB = dim(ToB$edge)[1] #save number of edges on original ToB

  #Check for duplicate nodes among resolutions, 4-cycles
  cycle4 = FALSE
  nodes = c()
  nRes = length(resolutions) # number of nodes resolved
  resols = vector(mode = "list", length = nRes)
  for (i in 1:nRes)
    #check for repeated nodes and expand resolution lists
  {
    if (resolutions[[i]]$node %in% nodes) {
      stop("Input error: Node is repeated in given resolutions.")
    }
    else {
      nodes = c(nodes, resolutions[[i]]$node)
    }

    if (length(resolutions[[i]]$cycleRes[[1]]$order) == 4) {
      cycle4 = TRUE
    }

    # expand resolutions of node into list of individual resolutions
    curNodeRes = resolutions[[i]] # focus on resolutions of single node
    k = length(curNodeRes$cycleRes) # number of resolutions of this node
    curNodeResList = vector(mode = "list", length = k) # list for these
    for (j in 1:k) {
      curNodeResList[[j]] = list(
        node = resolutions[[i]]$node,
        cycleRes = resolutions[[i]]$cycleRes[[j]],
        RSSs = resolutions[[i]]$RSSs
      )
    }
    resols[[i]] = curNodeResList
  }
  if (nRes==1) { #if only 1 node resolved, expand.grid doesn't work}
    numFullRes=length(resols[[1]])
    allFullRes=array(resols[[1]],c(numFullRes,1))
  } else {
   allFullRes = do.call(expand.grid, resols) #create big table with rows giving all full resolutions
   numFullRes = nrow(allFullRes)
  }

  Nets = vector(mode = "list", length = numFullRes)
  for (ri in 1:numFullRes) {
    # loop through the full resolutions

    resols = allFullRes[ri, ] #pick up infor for one full resolultion
    if (length(resols)==1) { resols=list(resols)} #if only 1 node resolution, need to make list
    rest=resols

    nonRootEdges = rep(FALSE, nEdgeToB) #indicator vector for edges that can't contain root

    for (j in 1:length(resols))
      # check for rootability
    {
      nonRootEdges = nonRootEdges |
        resols[[j]][[1]]$cycleRes$nonRootEdges
    }

    if (all(nonRootEdges))
      #if not rootable
    {
      Nets[[ri]] = NULL
      message(paste0("No root location is consistent for resolution ", ri))
    } else
    {
      if (plot > 0)
        # plot allowable root locations
      {
        #plot possible root locations
        edgecolor = rep("black", length(nonRootEdges))
        edgecolor[which(nonRootEdges == T)] = "red"

        if (numFullRes > 1)
          maintitle = paste0("Possible root locations; resolution ", ri)
        else
          maintitle = "Possible root locations"

        plot(
          savedToB,
          edge.color = edgecolor,
          type = "unrooted",
          main = maintitle,
          sub = "May be rooted on black edges; within cycles also possible"
        )
        mtext(eval(bquote(.(titletext))),
              side = 3,
              line = 0,
              cex = 1) # add rest of title text
      }

      ToB = savedToB #restore original ToB
      pendantRootEdges = (!nonRootEdges) &
        (ToB$edge[, 2] < length(ToB$tip.label)) #find possible pendant edges for rooting
      possibleOutTaxa = ToB$edge[which(pendantRootEdges), 2] # pick taxon to be outgroup for rooting
      outTaxon = possibleOutTaxa[1]
      outTaxonLabel = ToB$tip.label[outTaxon]

      ToB = root(ToB, outgroup = outTaxon, resolve.root = T)
      ToB$edge.length = rep(1, length(ToB$edge.length))

      Net = ToB
      TaxaFromToB = Net$tip.label # need original order to interpret multifurcation data

      for (ii in 1:length(resols))
        # loop through multifurcations to resolve
      {
        res = resols[[ii]][[1]] #get info for this multifurcation resolution
        nodeLab = res$node
        groupvec = res$cycleRes$taxonGroups
        order = res$cycleRes$order
        n = length(order)

        if (n == 4) {
          # Since hybrid node not determined for 4-cycles,
          # may have to switch order if outTaxon descended from
          # hybrid group

          if (order[groupvec[outTaxon]] == 1) {
            #check if need to reorder to avoid outTaxon as hybrid descendant
            order = order[c(2:length(order), 1)] # if so, shift order
            if (order[2] > order[4])
            {
              order[2:4] = order[4:2]
            }
          }
        }



        #generate tree that will represent cycle once reticulation edge added
        groupt = read.tree(text = paste0("(", -n, ",-1);")) # generate phylo object, mostly to be written over (negatives for taxa names)
        groupt$edge = matrix(0, n + 2, 2)
        groupt$edge[, 1] = c(3, 3:(3 + n))
        groupt$edge[, 2] = c(1, 4:(3 + n), 2)
        groupt$edge.length = rep(1, n + 2)
        groupt$Nnode = n + 1
        groupt$node.label = c("r", paste0("TH", ii), -(n - 1):-2, paste0("#H", ii))

        Net = ape::root(Net,
                        node = length(Net$tip.label) + which(Net$node.label == paste0("Node ", nodeLab))) #temporarily root at multifurcation

        for (i in 1:n)
          #loop through groups
        {
          gtaxa = TaxaFromToB[which(groupvec == i)] #Get taxa in this group
          subtree = drop.tip(Net,
                             setdiff(Net$tip.label, gtaxa),
                             collapse.singles =
                               F)
          subtree$node.label[1] = ""
          subtree$root.edge = 1 #add a root edge for resolution when attached
          attachlabel = -order[i]# attach at this label in cycle tree just created

          if (attachlabel %in% c(-1, -n)) {
            attachPoint = which(groupt$tip.label == attachlabel) #will attach at a tip
          } else {
            attachPoint = length(groupt$tip.label) + which(groupt$node.label == attachlabel) #will attach at an internal node
          }
          groupt = ape::bind.tree(groupt, subtree, where = attachPoint)# attach the subtree
          groupt$node.label[which(groupt$node.label == attachlabel)] = "" #wipe out this label
        }
        Net = groupt
      }

      Net$node.label[which(!(substr(Net$node.label, 1, 2) %in% c("TH", "#H")))] =
        "" # clean up irrelevent node labels

      Net = ape::root(Net, outgroup = outTaxonLabel, resolve.root = TRUE) # root properly
      Net$node.label[which(Net$node.label == "Root")] = "" #remove label for root
      if (Net$node.label[1] != "")
        #if root is labeled (THn), need to move root. Ape requires a hack for this
      {
        newLeaf <- list(
          edge = matrix(c(2, 1), 1, 2),
          #creat edge to extra taxon
          tip.label = c("***"),
          edge.length = 1,
          Nnode = 1
        )
        class(newLeaf) <- "phylo"

        Net$root.edge = 1
        Net = bind.tree(newLeaf, Net) #Attach an extra taxon above root
        Net = drop.tip(Net, outTaxonLabel, collapse.singles = F) # drop old outtaxon
        Net$tip.label[which(Net$tip.label == "***")] = outTaxonLabel #and rename new one
        Net$node.label[1] = ""
      }

      Net$edge.length = rep(1, length(Net$edge.length))

      fromNode = match(paste0("TH", 1:length(resols)), Net$node.label) + length(Net$tip.label)
      toNode = match(paste0("#H", 1:length(resols)), Net$node.label) + length(Net$tip.label)

      Net = ape::evonet(Net, from = fromNode, to = toNode)
      Net=suppressNetBinaryNodes(Net,addlength=FALSE)

      if (plot > 0) {
        subtitle = "Network should be semidirected; Rooting is arbitrary"
        if (cycle4)
          subtitle = "Network should be semidirected; Rooting is arbitrary\nHybrid node on 4-cycle is arbitrary"

        if (numFullRes > 1)
          maintitle = paste0("Inferred Level-1 Network, resolution ", ri)
        else
          maintitle = "Inferred Level-1 Network"

        plot(
          Net,
          type = "unrooted",
          show.node.label = T,
          main = maintitle,
          sub = subtitle
        )
        mtext(eval(bquote(.(titletext))),
              side = 3,
              line = 0,
              cex = 1) # add rest of title text
      }

      Nets[[ri]] = gsub(":-?[0123456789]+",":1",ape::write.evonet(Net))# write Newick for net, making sure all branch lengths are 1
    }
  }

  return(Nets)
}



###################################################
#'Compute empirical distance between taxon groups.
#'
#'From gene quartet counts, computes NANUQ or modNANUQ distances between groups
#'of taxa (which should be those around a multifurcation in a tree of blobs.
#'If these groups are not singletons, averaging is done over group elements.
#'
#'@param pTable table of giving empirical gene quartet counts for the taxa on tree, with columns p_star and p_test
#'@param taxa a list of taxon names, who positions are used in 'groups'
#'@param groupvec taxon groups encoded in vector
#'@param test to be used for detecting hybridizations in quartete ("T3" or "cut")
#'@param alpha test level for p_test
#'@param beta test level for p_star
#'@param dist the distance to compute, either "NANUQ" or "modNANUQ"
#'
#'@return the distance matrix, ordered by taxon group number
#'
#'@export
blobDistance <-
  function(pTable,
           taxa,
           groupvec,
           test = 'T3',
           alpha,
           beta,
           dist = "NANUQ") {
    if (!(is.numeric(alpha) &&
          is.numeric(beta))) {
      stop("Test levels alpha and beta must be numeric.")
    }
    if (!(dist %in% c("NANUQ", "modNANUQ"))) {
      stop('Argument "dist" must be either "NANUQ" or "modNANUQ".')
    }

    pcol = paste0("p_", test)
    output = c()

    M = dim(pTable)[1] # number of quartet concordance factors
    ntaxa = length(taxa) #number of taxa
    if (!(M == choose(ntaxa, 4))) {
      stop("Improper number of rows in pTable.")
    }

    qnames = c("12|34", "13|24", "14|23")

    nGroups = length(unique(groupvec))
    groupSizes = as.vector(table(groupvec))

    Dist = matrix(0, nGroups, nGroups) #initialize distance matrix

    for (m in 1:M) {
      # consider each set of 4 taxa
      taxanames = colnames(pTable)[which(pTable[m, 1:ntaxa] == 1)] #determine taxa
      groupNums = groupvec[taxanames]    #get groups these are in

      if (length(unique(groupNums)) == 4)  {
        #if all 4 in different groups...
        factor = 1 / prod(groupSizes[groupNums]) #compute factor to downweight dist entry by
        qcounts = pTable[m, qnames] # get counts

        if (pTable[m, "p_star"] > beta) {
          # if quartet judged as star tree
          Dist[groupNums, groupNums] = Dist[groupNums, groupNums] + 1 *
            factor #  it separates all taxa
        } else {
          qcountspr = qcounts + runif(3) * .0001 # introduce random tie breaking
          majorq = which.max(qcountspr) #determine quartet with largest count
          if (pTable[m, pcol] > alpha) {
            # if high p-value indicates tree
            if (majorq == 1) {
              Dist[groupNums[1:2], groupNums[3:4]] = Dist[groupNums[1:2], groupNums[3:4]] +
                1 * factor
              if (dist == 'modNANUQ') {
                Dist[groupNums[1], groupNums[2]] = Dist[groupNums[1], groupNums[2]] + .5 *
                  factor
                Dist[groupNums[3], groupNums[4]] = Dist[groupNums[3], groupNums[4]] +
                  .5 * factor
              }
            } else {
              if (majorq == 2) {
                Dist[groupNums[c(1, 3)], groupNums[c(2, 4)]] = Dist[groupNums[c(1, 3)], groupNums[c(2, 4)]] +
                  1 * factor
                if (dist == 'modNANUQ') {
                  Dist[groupNums[1], groupNums[3]] = Dist[groupNums[1], groupNums[3]] + .5 *
                    factor
                  Dist[groupNums[2], groupNums[4]] = Dist[groupNums[2], groupNums[4]] +
                    .5 * factor
                }
              } else {
                Dist[groupNums[c(1, 4)], groupNums[c(2, 3)]] = Dist[groupNums[c(1, 4)], groupNums[c(2, 3)]] +
                  1 * factor
                if (dist == 'modNANUQ') {
                  Dist[groupNums[1], groupNums[4]] = Dist[groupNums[1], groupNums[4]] + .5 *
                    factor
                  Dist[groupNums[2], groupNums[3]] = Dist[groupNums[2], groupNums[3]] +
                    .5 * factor
                }
              }
            }
          } else {
            # if low p-value indicates cycle
            minorq = which.min(qcountspr)
            middleq = setdiff(1:3, c(majorq, minorq))
            if (majorq == 1 || middleq == 1) {
              Dist[groupNums[1:2], groupNums[3:4]] = Dist[groupNums[1:2], groupNums[3:4]] +
                .5 * factor
            }
            if (majorq == 2 || middleq == 2) {
              Dist[groupNums[c(1, 3)], groupNums[c(2, 4)]] = Dist[groupNums[c(1, 3)], groupNums[c(2, 4)]] + .5 *
                factor
            }
            if (majorq == 3 || middleq == 3) {
              Dist[groupNums[c(1, 4)], groupNums[c(2, 3)]] = Dist[groupNums[c(1, 4)], groupNums[c(2, 3)]] + .5 *
                factor
            }
          }
        }
      }
    }
    Dist = Dist + t(Dist)
    Dist = 2 * Dist + 2 * nGroups - 4 # adjust to get appropriate distances off-diagonal
    Dist = Dist - diag(diag(Dist)) # but reset diagonal to 0

    return(Dist)
  }

########################################
#'Expected NANUQ cycle distance
#'
#'Compute expected NANUQ distance for a sunlet network with 4 or more taxa. This is used
#' to resolve multifurcations in a tree of blobs by NANUQ+ functions
#'
#'@param n number of edges in cycle
#'
#'@return an nxn distance matrix with rows/columns ordered from hybrid following circular order
#'
#'@examples
#' expNANUQCycleDist(5)
#'
#'@seealso \code{\link{resolveCycle}}
#'
#'@export
expNANUQCycleDist = function(n) {
  if (n < 4)
    stop("Cycle size argument must be >3.")
  d = matrix(0, n, n)
  c = n * (n - 5) / 2
  for (j in 2:n) {
    #compute distances from hybrid to all others
    d[1, j] = c + j * n - (j - 1) ^ 2
  }
  e = n ^ 2 - 3 * n + 2
  for (i in 2:(n - 1)) {
    #compute distances between non-hybrids
    for (j in (i + 1):n) {
      d[i, j] = e - (i - 2) ^ 2 - (n - j) ^ 2
    }
  }
  return(d + t(d))
}

####################################################
#' Compute fit of circular orders to distance with least squares
#'
#' Compute residual sum of squares (RSS) comparing empirical distance for a blob
#' to an expected one for a cycle with each given order/designated hybrid. This is used
#' in NANAUQ+ commands for resolving multifurcations in a tree of blobs to a cycle
#'
#'@param D an empirical distance table
#'@param E an expected distance table, to be reordered
#'@param orders a vector indicating an order, or matrix whose rows give orders, to fit
#'
#'@return vector of RSSs, one for each order
#'
#'@seealso \code{\link{resolveCycle}}, \code{\link{resolveLevel1}}
#'
#'@export
fitCycleOrders = function(D, E, orders) {
  if (is.vector(orders)) {
    # if only 1 order...
    orders = matrix(orders, nrow = 1) # make it a row of a matrix
  }
  m = dim(orders)[1]# number of orders
  res2 = rep(0, m) # storage for residuals


  for (i in 1:m) {
    res2[i] = sum((D - E[orders[i, ], orders[i, ]]) ^ 2)
  }
  return(res2)
}

#################################################
#' Generate permutations
#'
#' Generate all permutations of 1 to n, as rows of a matrix
#'
#'@param n size of permutations
#'
#'@return an n!xn matrix whose rows give permutations
#'
#'@examples
#' allPerms(4)
#'
#'@export
allPerms = function(n) {
  if (n == 2) {
    return(matrix(c(1, 2, 2, 1), 2, 2))
  }
  else {
    a = allPerms(n - 1)
    asize = factorial(n - 1)
    b = matrix(0, factorial(n), n)
    ncol = matrix(n, asize, 1)
    b[1:asize, 1:(n - 1)] = a # first block
    b[1:asize, n] = ncol
    for (i in 1:(n - 1)) {
      b[(1 + i * asize):((i + 1) * asize), 1:(n - i - 1)] = a[, 1:(n - i - 1)]
      b[(1 + i * asize):((i + 1) * asize), n - i] = ncol
      b[(1 + i * asize):((i + 1) * asize), (n - i + 1):n] = a[, (n - i):(n - 1)]
    }
  }
  return(b)
}

#################################################
#' Generate all circular orders with designated hybrid
#'
#' Generate a matrix whose rows give all circular orders with a designated hybrid.
#' The order is encoded as a vector with entries from 1 to n, where the position
#' corresponds to a node/group of taxa.  The location in the vector of the 1
#' indicates the hybrid, the positions of 2, n its neighbors, etc.
#'
#'@details To avoid duplication of circular orders, the entry 2 in each vector
#'always occurs before the entry n.
#'
#'Since in using first-order quartet-based methods to infer 4-cycles the hybrid
#'node is not identifiable, for n=4 only 3 orders are given, with 1 as hybrid for each
#'
#'@param n size of order, with n>3
#'
#'@return an (n!/2)xn  (or 3xn if n=4) matrix whose rows give all circular orders.
#'
#'@examples
#' circHybOrders(4)
#' circHybOrders(5)
#'
#'@export
circHybOrders = function(n) {
  if (n < 4) {
    stop("Circular orders produced only for cycle size of at least 4.")
  }
  if (n == 4) {
    a = matrix(c(1, 1, 1, 2, 2, 3, 3, 4, 2, 4, 3, 4), 3, 4)
  } else {
    a = allPerms(n) # generate twice what we need
    for (i in (dim(a)[1]:2)) {
      loc1 = which(a[i, ] == 2)
      locn = which(a[i, ] == n)
      if (loc1 > locn)  {
        a = a[-i, ] #delete second representation of circular order w hybrid
      }
    }
  }
  return(a)
}

###########################################################
#'Expected modNANUQ cycle distance
#'
#'Compute expected modNANUQ distance for a sunlet network with 4 or more taxa.
#'This is used in a hueristic method for resolving multifurcation in a tree
#'of blobs to a cycle by NANUQ+ commands.
#'
#'@param n number of edges in cycle (at least 4)
#'
#'@return an nxn distance matrix with rows/columns ordered from hybrid following circular order
#'
#'@examples
#' expmodNANUQCycleDist(5)
#'
#'#'@seealso \code{\link{resolveCycle}} \code{\link{ordersHeuristicmodNANUQ}}
#'
#'@export
expmodNANUQCycleDist = function(n) {
  if (n < 4)
    stop('modNANUQ cycle distance requires at least 4 taxa.')
  D = matrix(0, n, n)

  # get row 1 first, d(x_1,x_k)
  for (k in 2:n) {
    D[1, k] = 2 * ((k - 2) * (n - k) + 1 / 2 * (choose(n - k, 2) +
                                                  choose(k - 2, 2))) + 2 * n - 4
  }
  # get other rows next
  for (k in 2:(n - 1)) {
    for (p in (k + 1):n) {
      D[k, p] =  2 * ((p - k - 1) * (n - p + k - 2) + choose(p - k - 1, 2) + (p -
                                                                                k - 1) + 1 / 2 * (n - p + k - 2) + (n - p) * (k - 2) + 1 / 2 *
                        (choose(n - p, 2) + choose(k - 2, 2))
      ) + 2 * n - 4
    }
  }
  return(D + t(D))
}

##############################################
#' Choose cycle orders heuristically from  empirical modNANUQ distance
#'
#' Find candidates for best hybrid node and circular order fitting the modNANUQ
#' distance.
#'
#' Candidadte orders are obtained  by first picking the hybrid node (from the
#' minimum column sum of the distance matrix), then ordering nodes by distance
#' from the hybrid, and for each consecutive pair picking  nodes in the cycle
#' closest to the previous node. This constructs one or more orders since ties
#' may occur. For more details, see \insertCite{ABRW25;textual}{MSCquartets}.
#'
#' This function is used by NANUQ+ commands
#' to resolve multifurcations in a tree of blobs of high degree.
#'
#'@references
#' \insertRef{ABMR24}{MSCquartets}
#'
#'@param M an empirical modNANUQ distance table
#'@param delta cutoff for relative difference in distances for determining near ties for "best" orders
#'
#'@return a list of circular orders
#'
#' @seealso \code{\link{expmodNANUQCycleDist}} \code{\link{resolveCycle}} \code{\link{resolveLevel1}}
#'
#'@export
ordersHeuristicmodNANUQ = function(M, delta = 10 ^ -6) {
  m = dim(M)[1] #size of matrix = size of cycle
  n = colSums(M)

  minn = min(n)
  hybrids = which((n - minn) <= delta * minn) # find all possible hybrid node
  allords = matrix(0, length(hybrids), m) #allocate space for orders
  allords[, 1] = hybrids # store hybrid groups first

  for (i in 2:m)
  {
    #fill out rest of entries in orders
    numords = dim(allords)[1] # get number of orders being filled
    for (j in 1:numords)
      # for each order
    {
      v = allords[j, ] # copy order
      lastentry = v[i - 1] # get last filled entry in current order
      currentrow = M[lastentry, ] #get row of distances to see what's closest to last filled entry
      currentrow[lastentry] = Inf #make 0 entry large to ignore for min
      vals = unique(sort(currentrow)) # get sorted values
      closest = NULL
      while (length(closest) == 0) {
        #look for closest entries not yet used
        closest = which((currentrow - vals[1]) <= delta * vals[1]) # find closest groups
        closest = closest[which(!(closest %in% v))] # removed any already used
        vals = vals[2:length(vals)] #and remove this value in case we need to look again
      }
      allords[j, i] = closest[1] #fill in one
      if (length(closest) > 1) {
        # if more than 1...
        v = allords[j, ] # copy order
        for (k in 2:length(closest))
        {
          v[i] = closest[k] # add new entry
          allords = rbind(allords, v) # and tack onto order array
        }
      }
    }
  }
  rownames(allords) = NULL

  #put circular orders in canonical form,  and remove duplicates
  if (m == 4) {
    # if only 4 groups, make group 1 hybrid for canonical order
    for (k in 1:dim(allords)[1]) {
      while (allords[k, 1] > 1) {
        allords[k, ] = c(allords[k, 2:4], allords[k, 1])
      }
    }
  }
  for (j in 1:dim(allords)[1]) {
    #make sure 2nd entry smaller than last
    if (allords[j, 2] > allords[j, m]) {
      allords[j, ] = allords[j, c(1, m:2)]
    }
  }
  allords = unique(allords) #remove duplicates

  #convert orders to format where kth entry give position of kth group
  for (i in 1:dim(allords)[1])
  {
    v = allords[i, ]
    for (j in 1:m)
      allords[i, j] = which(v == j)
  }

  return(allords)
}

#' Produce table of counts of quartets displayed on trees, in parallel for large data sets
#'
#' Compiles table of quartet count concordance factors (qcCFs) for topological quartets displayed on a
#' collection of trees. Gives the same output as \code{quartetTable}, but operates in parallel.
#'
#' @details
#'
#' The number of available cores can be determined by \code{parallel::detectCores()}.
#' With overhead, tabulating quartets for a large data set (many taxa and/or many gene trees) on a 4-core
#' computer using \code{numCores=4} may require less than half the elapsed time of the sequential \code{quartetTable}.
#'
#' The names in \code{taxonnames} may be any subset of those on the trees.
#' Branch lengths of non-negative size less than or equal to \code{epsilon}
#' are treated as zero, giving polytomies.
#'
#' In the returned table, columns are labeled by taxon names and quartet names ("12|34", etc.).
#' 1s and 0s in taxon columns indicate the taxa in a quartet. Quartet 12|34
#' means the first and second indicated taxa form a cherry, 13|24 means the first and third form a cherry, 14|23 means
#' the first and fourth form a cherry, and 1234 means the quartet is unresolved.
#'
#' An error occurs if any branch length is negative.
#' Warnings are given if some of \code{taxonnames} are missing on some trees, or
#' if some 4-taxon set is not on any tree.
#'
#' If \code{random}>0, then for efficiency \code{random} should be much smaller then
#' the number of possible 4 taxon subsets.
#'
#' If the quartet counts are to be used for NANUQ, or any other routines requiring resolved quartet counts,
#' \code{\link{quartetTableResolved}} must be run following \code{quartetTableParallel}. See example below.
#'
#' @param trees multiphylo object containing un/rooted metric/topological trees
#' @param taxonnames vector of \code{n} names of taxa of interest; if \code{NULL} then taken from taxa on \code{trees[[1]]}
#' @param epsilon minimum for branch lengths to be treated as non-zero
#' @param numCores number of cores to use for parallel calls
#' @return
#'     an (\code{n} choose 4)x(\code{n}+4) matrix (or (\code{random})x(\code{n}+4) matrix) encoding
#'     4 taxon subsets of \code{taxonnames} and counts of each of the
#'     quartets 12|34, 13|24, 14|23, 1234 across the trees
#'
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableResolved}}, \code{\link{quartetTableDominant}}, \code{\link{taxonNames}}
#'
#' @examples \donttest{
#' gtrees=read.tree(file=system.file("extdata","dataHeliconiusMartin",package="MSCquartets"))
#' QT=quartetTableParallel(gtrees,numCores=2)
#' RQT=quartetTableResolved(QT)
#' pTable=NANUQ(RQT,alpha=1e-40, beta=1e-30, outfile = file.path(tempdir(), "NANUQdist"))}
#'
#' @importFrom ape cophenetic.phylo compute.brlen
#' @importFrom foreach foreach %dopar%
#' @importFrom doParallel registerDoParallel
#'
#' @export
quartetTableParallel <- function(trees,
                                 taxonnames = NULL,
                                 epsilon = 0,
                                 numCores) {

  if ( !(numCores>=2) ) stop("Argument 'numCores' must be at least 2.")
  numcores=floor(numCores)

  # function for combining parallel call results
  combQuartets <-function(quartets1,quartets2){
    quartets1[,c("12|34","13|24","14|23","1234")]=
      quartets1[,c("12|34","13|24","14|23","1234")] +
      quartets2[,c("12|34","13|24","14|23","1234")]
    return(quartets1)
  }

  # main code for parallel calls
  start_time = Sys.time()

  if (is.null(taxonnames)) {
    taxonnames = trees[[1]]$tip.label
    message("Using taxa that appear on first gene tree.")
  }
  taxonnames = sort(taxonnames)
  nt = length(trees)
  N = length(taxonnames)

  M = choose(N, 4)
  Q = matrix(0, M, N)
  colnames(Q) = c(taxonnames)

  registerDoParallel(numCores)  # set number of cores to be used

  numTrees=length(trees)
  numTs=ceiling(numTrees/numCores) #number of trees for each core (except last)

  i=0 #to avoid a CRAN check Note

  qTable <- foreach::foreach (i = 1:numCores, .combine = combQuartets ) %dopar% {
   Ts=trees[(1+(i-1)*numTs):min((i*numTs),numTrees)]
   suppressWarnings(quartetTable(Ts, taxonnames=taxonnames, epsilon = epsilon))
  }

  rSums=rowSums(qTable)
  if (sum(!(rSums=rSums[1])) >0)
    warning("Some taxa missing from some trees.")
  if ((N > 4) && (sum(rSums == 0) > 0))
    warning("Some 4-taxon set(s) not present on any tree.")
  if (sum(qTable[, "1234"]) > 0)
    warning("Some quartets unresolved.")

  current_time = Sys.time()
  elapsedTime = as.numeric(difftime(current_time, start_time,
                                    units = "mins"))
  if (elapsedTime > 15) {
    message("Time to process quartets on gene trees was ",
            elapsedTime," minutes.")
  }

  return(qTable)
}

#' Apply NANUQ network inference algorithm to gene tree data
#'
#' Apply the NANUQ algorithm of \insertCite{ABR19;textual}{MSCquartets} to infer a hybridization network from a collection of gene trees,
#' under the level-1 network multispecies coalescent (NMSC) model.
#'
#' @details  This function
#' \enumerate{
#' \item counts displayed quartets across gene trees to form quartet count concordance factors (qcCFs),
#' \item applies appropriate hypothesis tests to judge qcCFs as representing putative hybridization,
#' resolved trees, or unresolved (star) trees using \code{alpha} and \code{beta} as significance levels,
#' \item produces a simplex plot showing results of the hypothesis tests for all qcCFs
#' \item computes the appropriate NANUQ distance table, writing it to a file.
#' }
#' The distance table file
#' can then be opened in the external software SplitsTree \insertCite{SplitsTree}{MSCquartets} (recommended) or within R using the package \code{phangorn} to
#' obtain a circular split system under the Neighbor-Net algorithm, which is then depicted as a splits graph.
#' The splits graph should be interpreted via
#' the theory of \insertCite{ABR19;textual}{MSCquartets} to infer the level-1 species network, or to conclude the data does
#' not arise from the NMSC on such a network.
#'
#' If \code{alpha} and \code{beta} are vectors, they must have the same length k. Then the i-th entries are paired to
#' produce k plots and k output files. This is equivalent to k calls to \code{NANUQ} with scalar values of \code{alpha} and \code{beta}.
#'
#' A call of \code{NANUQ} with \code{genedata} given as a table previously output from \code{NANUQ} is
#' equivalent to a call of \code{NANUQdist}. If \code{genedata} is a table previously output from \code{quartetTableResolved}
#' which lacks columns of p-values for hypothesis tests, these will be appended to the table output by \code{NANUQ}.
#'
#' If plots are produced, each point represents an empirical quartet concordance factor,
#' color-coded to represent test results.
#'
#' In general, \code{alpha} should be chosen to be small and \code{beta}
#' to be large so that most quartets are interpreted as resolved trees.
#'
#' Usually, an initial call to \code{NANUQ} will not give a good analysis, as values
#' of \code{alpha} and \code{beta} are likely to need some adjustment based on inspecting the data. Saving the returned
#' table from \code{NANUQ} will allow for the results of the time-consuming computation of qcCFs to be
#' saved, along with p-values,
#' for input to further calls of \code{NANUQ} with new choices of \code{alpha} and \code{beta}.
#'
#' See the documentation for \code{\link{quartetNetworkDist}} for an explanation of a small, rarely noticeable,
#' stochastic element of the algorithm.
#'
#' For data sets of many gene trees, user time may be reduced by using parallel code for
#' counting displayed quartets. See \code{\link{quartetTableParallel}}, where example commands are given.
#'
#'
#' @references
#' \insertRef{ABR19}{MSCquartets}
#'
#' \insertRef{SplitsTree}{MSCquartets}
#'
#' @param genedata gene tree data that may be supplied in any of 3 forms:
#' \enumerate{
#' \item as a character string giving the name of a file containing Newick gene trees,
#' \item as a multiPhylo object containing the gene trees, or
#' \item as a table of quartets on the gene trees, as produced by a previous call to
#' \code{NANUQ} or \code{quartetTableResolved}, which has columns only for taxa, resolved quartet counts,
#' and possibly p_T3 and p_star
#' }
#' @param outfile  a character string giving an output file name stub for
#' saving a \code{NANUQ} distance matrix in nexus format; to the stub \code{outfile}
#' will be appended an \code{alpha} and \code{beta} value and ".nex";
#' if \code{NULL} then then no file is written
#' @param omit \code{FALSE} to treat unresolved quartets as 1/3 of each resolution;
#' \code{TRUE} to discard unresolved quartet data; ignored if gene tree data given as quartet table
#' @param epsilon minimum for branch lengths to be treated as non-zero; ignored if gene tree data given as quartet table
#' @param alpha a value or vector of significance levels for judging p-values
#' testing a null hypothesis of no hybridization vs. an alternative of hybridization, for each quartet;  a smaller value applies
#' a less conservative test for a tree (more trees), hence a stricter requirement for desciding in favor of hybridization (fewer reticulations)
#' @param beta a value or vector of significance levels for judging p-values testing
#' a null hypothesis of a star tree (polytomy) for each quartet vs. an alternative of anything else; a smaller value applies a less conservative
#' test for a star tree (more polytomies), hence a stricter requirement for deciding in favor of a resolved tree or network;
#' if vectors, \code{alpha} and \code{beta} must have the same length
#' @param taxonnames if \code{genedata} is a file or a multiPhylo object, a vector of a subset
#' of the taxa names on the gene trees
#' to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genedata}
#' is a quartet table, this argument is ignored and all taxa in the table are used
#' @param plot \code{TRUE} produces simplex plots of hypothesis test results, \code{FALSE} omits plots
#'
#' @return a table \code{$pTable} of quartets and p-values for judging fit to the MSC on quartet
#' trees, and a distance table \code{$dist}, or list of distance tables, giving NANUQ distance (returned invisibly);
#' the table can be used as input to \code{NANUQ} or \code{NANUQdist} with new choices of alpha and beta, without re-tallying quartets on
#' gene trees; the distance table is to be used as input to NeighborNet.
#'
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableParallel}}, \code{\link{quartetTableDominant}}, \code{\link{quartetTreeTestInd}},
#' \code{\link{quartetStarTestInd}}, \code{\link{NANUQdist}}, \code{\link{quartetTestPlot}}, \code{\link{pvalHist}},
#' \code{\link{quartetNetworkDist}}
#'
#' @examples
#' data(pTableYeastRokas)
#' out=NANUQ(pTableYeastRokas, alpha=.05, beta=.80, outfile = NULL)
#' # Specifying an outfile would write the distance table to it for opening in SplitsTree.
#' # Alternately, to use the phangorn implementation of NeighborNet
#' # within R, enter the following additional lines:
#' nn=neighborNet(out$dist)
#' plot(nn,"2D")
#'
#' @export
NANUQ = function( genedata,
                  outfile = "NANUQdist",
                  omit = FALSE,
                  epsilon=0,
                  alpha = .05,
                  beta = .80,
                  taxonnames = NULL,
                  plot = TRUE) {

  if (!(is.numeric(alpha) && is.numeric(beta))) {
    stop("Arguments alpha and beta must be numeric.")
  }


  if ("matrix" %in% class(genedata)) {
    pTable = genedata
    momit=missing(omit)
    mepsilon=missing(epsilon)
    mtaxonnames=missing(taxonnames)
    if (!momit | !mepsilon | !mtaxonnames){
      message(
        "Since genedata supplied as quartet table, ignoring arguments 'omit', 'epsilon', 'taxonnames'."
      )}
      taxonnames=NULL
    } else {
    if ("multiPhylo" %in% class(genedata))  {
      genetrees = genedata
    } else {
      if ("character" %in% class(genedata)) {
        genetrees <- read.tree(genedata) #read gene trees
        message(paste("Read", length(genetrees), "gene trees from file."))
      } else {
      stop("Data must be supplied as an object of type multiPhylo, character, or matrix.")
      }
    }

    if (is.null(taxonnames)) {
      # if no taxa names specified,
      taxonnames = genetrees[[1]]$tip.label   # ... get them from first tree
    }
    taxonnames = sort(taxonnames)
    if (length(taxonnames) <= 25) {
      namelist = paste0(taxonnames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxonnames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxonnames), " taxa: ", namelist)

    pTable = quartetTable(genetrees, taxonnames, epsilon = epsilon)   # tally quartets on gene trees
    pTable = quartetTableResolved(pTable, omit)   # treat unresolved quartets
  }

  if (!("p_T3"%in% colnames(pTable))){
    pTable = quartetTreeTestInd(pTable, model = "T3", lambda = 0) # compute p-values for quartet CFs fit to MSC on a tree
  }

  if (!("p_star"%in% colnames(pTable))) {
  pTable = quartetStarTestInd(pTable)# compute p-values for each quartet CFs fit to star tree
  }

  dist=NANUQdist(pTable, outfile, alpha, beta, plot) # construct distance tables and put in files

  invisible(list(pTable=pTable, dist=dist))
}

####################################################################################################

#' Compute NANUQ distance and write to file
#'
#' Computes the quartet distance tables for the NANUQ algorithm of \insertCite{ABR19;textual}{MSCquartets}, using precomputed p-values for quartets,
#' for each of several levels specified. Distance tables are written to files, in nexus format.
#'
#' @details
#' If plots are produced, each point represents an empirical quartet concordance factor,
#' color-coded to represent test results giving interpretation as network, resolved tree, or star tree.
#'
#' If \code{alpha} and \code{beta} are vectors, they must be of the same length k. Then the i-th entries are
#' paired to produce k plots and k distance tables/output files. This is equivalent to k
#' calls to \code{NANUQdist} with paired scalar values from the vectors of \code{alpha} and \code{beta}.
#'
#' See the documentation for \code{\link{quartetNetworkDist}} for an explanation of a small, rarely noticeable,
#' stochastic element of the algorithm.
#'
#' @references
#' \insertRef{ABR19}{MSCquartets}
#'
#' @param pTable a table of resolved quartets and p-values, as previously computed by \code{NANUQ}, or by both \code{quartetTreeTestInd} and
#' \code{quartetStarTestInd}, with columns \code{"p_T3"} and \code{"p_star"}
#' @param outfile  a character string giving an output file name stub for
#' saving a \code{NANUQ} distance matrix in nexus format; to the stub \code{outfile}
#' will be appended an \code{alpha} and \code{beta} value and ".nex";
#' if \code{NULL} then not written to file
#' @param alpha a value or vector of significance levels for judging p-values
#' testing a null hypothesis of no hybridization for each quartet;  a smaller value applies
#' a more liberal test for a tree (more trees), hence a stricter requirement for suspecting hybridization (fewer reticulations)
#' @param beta a value or vector of significance levels for judging p-values testing
#' a null hypothesis of a star tree for each quartet; a smaller value applies a more liberal
#' test for a star tree (more polytomies), hence a stricter requirment for suspecting a resolved tree;
#' if vectors, \code{alpha} and \code{beta} must have the same length
#'
#' @param plot \code{TRUE} produces simplex plots of hypothesis tests, \code{FALSE} omits plots
#'
#' @return a NANUQ distance table, or a list of such tables if \code{alpha} and \code{beta}
#' are vectors (returned invisibly)
#'
#' @seealso \code{\link{NANUQ}}, \code{\link{quartetTreeTestInd}}, \code{\link{quartetStarTestInd}}
#'
#' @examples
#' data(pTableYeastRokas)
#' dist=NANUQdist(pTableYeastRokas, alpha=.05, beta=.80, outfile = NULL)
#'
#' @export
NANUQdist = function (pTable,
                      outfile = "NANUQdist",
                      alpha=.05,
                      beta=.80,
                      plot = TRUE) {
  if (!(is.numeric(alpha) && is.numeric(beta))) {
    stop("Critical values alpha and beta must be numeric.")
  }
  n = length(alpha)
  if (length(beta) != n)
    stop("Arguments alpha and beta must have same length")

  allQD=vector("list",n)

  for (i in 1:n) {
    if (plot == TRUE) {
      quartetTestPlot(pTable, "NANUQ", alpha[i], beta[i]) # plot quartets in simplex, indicating which are interpretted as hybridization
    }

    QD = quartetNetworkDist(pTable, alpha[i], beta[i])           # compute the quartet distance table

    if (!is.null(outfile)) {
    outfile1 = paste0(outfile, "_alpha", alpha[i], "_beta", beta[i], ".nex")  # name of the nexus output file
    nexusDist(QD, outfile1)           # Produce the file to be read by SplitsTree
    }
    allQD[[i]]=QD
  }
  if (length(allQD)==1) { #if one table, make it not a list
    allQD=allQD[[1]]
  }

  invisible(allQD)
}

##################################################################################

#' Write a distance table to a file in nexus format
#'
#' Write a distance table to a file in nexus format.
#'
#' @param distMatrix a square matrix giving a distance table, with rows and columns labeled by taxon names
#' @param outfilename the name of an output file
#'
#' @return NULL
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT)
#' Dist=quartetDist(DQT)
#' nexusDist(Dist,outfile = file.path(tempdir(), "NANUQdist"))
#'
#' @export
nexusDist = function(distMatrix,
                     outfilename) {
  taxonnames = colnames(distMatrix)
  ntaxa = length(taxonnames)
  taxonnames = matrix(taxonnames, ntaxa, 1)
  sink(outfilename)
  cat("#nexus\n BEGIN Taxa;\n DIMENSIONS ntax=",
      ntaxa,
      ";\n TAXLABELS\n")
  cat(paste(taxonnames, "\n"))
  cat(
    ";\n END;\n \n BEGIN Distances; \n DIMENSIONS ntax=",
    ntaxa,
    "; \n FORMAT labels=left diagonal triangle=both;\n MATRIX\n"
  )
  cat(t(cbind(taxonnames, distMatrix)))
  cat("; \n END;")
  sink()
 message("Distance table written to file: ", outfilename)
}

############################################################################

#' Compute network quartet distance between taxa
#'
#' Produce network quartet distance table for the NANUQ algorithm, from a table of quartets and p-values,
#' and specified levels of quartet hypothesis tests. The network quartet distance, which
#' is described more fully by \insertCite{ABR19;textual}{MSCquartets}, generalizes
#' the quartet distance of \insertCite{Rho19;textual}{MSCquartets}.
#'
#' @details In case of a triple of quartet counts with the two largest equal and the third slighltly smaller,
#' along with \code{alpha} and \code{beta} leading to a star quartet being rejected and a tree not being rejected,
#' this function chooses a resolved quartet topology uniformly at random from the two largest counts. This is the only
#' stochastic element of the code, and its impact is usually negligible.
#'
#' @references
#' \insertRef{ABR19}{MSCquartets}
#'
#' \insertRef{Rho19}{MSCquartets}
#'
#' @param pTable a table of quartets and p-values, as computed by NANUQ, or
#' \code{quartetTreeTestInd} and \code{quartetStarTestInd}
#' @param alpha a scalar significance level for judging p-values \code{p_T3} indicating hybridization on quartet;
#'      smaller value gives fewer hybridization decisions
#' @param beta a scalar significance level for judging p-values \code{p_star} indicating quartet star tree;
#'      smaller value gives fewer resolved tree decisions
#'
#' @return a distance table
#'
#' @examples
#' data(pTableYeastRokas)
#' dist=quartetNetworkDist(pTableYeastRokas, alpha=.05, beta=.80)
#'
#' @seealso \code{\link{NANUQ}}, \code{\link{NANUQdist}}
#'
#' @export
quartetNetworkDist = function(pTable,
                              alpha,
                              beta) {
  if (!(is.numeric(alpha) && is.numeric(beta))) {
    stop("Critical values alpha and beta must be numeric.")
  }
  M = dim(pTable)[1] # number of quartet concordance factors
  ntaxa = which(colnames(pTable)=="12|34")-1 #number of taxa

  taxonnames = colnames(pTable)[1:ntaxa]  #names of taxa
  qnames = c("12|34", "13|24", "14|23")

  quartetDist = matrix(0, ntaxa, ntaxa)
  #alocates space for dissimilarity matrix
  rownames(quartetDist) = taxonnames
  colnames(quartetDist) = taxonnames

  for (m in 1:M) {
    # consider each set of 4 taxa

    taxanums = which(pTable[m, 1:ntaxa] == 1) #determine taxa
    qcounts = pTable[m, qnames] # get counts

    if (pTable[m, "p_star"] > beta) {
      # if quartet judged as star tree
      quartetDist[taxanums, taxanums] = quartetDist[taxanums, taxanums] + 1 #  it separates all taxa
    } else {
      qcountspr = qcounts + runif(3) * .0001 # introduce random tie breaking
      majorq = which.max(qcountspr) #determine quartet with largest count
      if (pTable[m, "p_T3"] > alpha) {
        # if high p-value indicates tree
        if (majorq == 1) {
          quartetDist[taxanums[1:2], taxanums[3:4]] = quartetDist[taxanums[1:2], taxanums[3:4]] +
            1
        } else {
          if (majorq == 2) {
            quartetDist[taxanums[c(1, 3)], taxanums[c(2, 4)]] = quartetDist[taxanums[c(1, 3)], taxanums[c(2, 4)]] +
              1
          } else {
            quartetDist[taxanums[c(1, 4)], taxanums[c(2, 3)]] = quartetDist[taxanums[c(1, 4)], taxanums[c(2, 3)]] +
              1
          }
        }
      } else {
        # if low p-value indicates cycle
        minorq = which.min(qcountspr)
        middleq = setdiff(1:3, c(majorq, minorq))
        if (majorq == 1 || middleq == 1) {
          quartetDist[taxanums[1:2], taxanums[3:4]] = quartetDist[taxanums[1:2], taxanums[3:4]] +
            .5
        }
        if (majorq == 2 || middleq == 2) {
          quartetDist[taxanums[c(1, 3)], taxanums[c(2, 4)]] = quartetDist[taxanums[c(1, 3)], taxanums[c(2, 4)]] + .5
        }
        if (majorq == 3 || middleq == 3) {
          quartetDist[taxanums[c(1, 4)], taxanums[c(2, 3)]] = quartetDist[taxanums[c(1, 4)], taxanums[c(2, 3)]] + .5
        }
      }
    }
  }
  quartetDist = quartetDist + t(quartetDist)
  quartetDist = 2 * quartetDist + 2 * ntaxa - 4 # adjust to get appropriate distances off-diagonal
  quartetDist = quartetDist - diag(diag(quartetDist)) # but reset diagonal to 0

  return(quartetDist)
}

#' Build a Species Tree from Gene Trees Using the Quartet Distance
#'
#' This function takes a collection of gene trees and uses a
#' species tree by first computing the quartet distance of 
#' \insertCite{Rho19;textual}{MSCquartets}
#' from the gene trees and then applying a distance method.
#'
#' @references 
#' \insertRef{Rho19}{MSCquartets}
#' 
#' @param genedata gene tree data, supplied in any of 3 forms:
#' \enumerate{
#' \item as a character string giving the name of a file containing Newick gene trees,
#' \item as a \code{multiPhylo} object containing the gene trees, or
#' \item as a table of quartet counts on the gene trees, as produced by
#' \code{quartetTable} or \code{quartetTableResolved}
#' }
#' @param method a function for constructing a tree from a distance matrix, such as \code{NJ} or
#' \code{fastme.bal}
#' @param taxanames used only if \code{genedata} is a filneame or multiphylo object;
#' restrict gene trees to these taxa; if \code{NULL}, use taxa on first tree
#' @param omit used only if \code{genedata} is a filneame,\code{multiPhylo} object, 
#' or a quartet table with column \code{'1234'};
#' if TRUE omit unresolved quartets, if \code{FALSE} treat them as 1/3 of each resolution
#'
#' @return A \code{phylo} object representing the inferred tree.
#'
#' @seealso
#' \code{quartetDist}, \code{QDS}, \code{quartetTableDominant}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tree<-quartetDistTree(gtrees)
#' write.tree(tree)
#' plot(tree)
#'
#' @export
quartetDistTree = function(genedata,
                           method = ape::fastme.bal,
                           taxanames = NULL,
                           omit = FALSE) 
  {
  if ("matrix" %in% class(genedata)) {
    QT = genedata
    if (!(is.null(taxanames))) {
      message(
        "Since genedata supplied as quartet table, ignoring argument 'taxanames'."
      )
    }
  } else  {
    if ("multiPhylo" %in% class(genedata))  {
      gtrees = genedata
    } else {
      if ("character" %in% class(genedata)) {
        gtrees <- read.tree(genedata) #read gene trees
        message(paste("Read", length(gtrees), "gene trees from file."))
      } else {
        stop("Data must be supplied as an object of type multiPhylo, character, or matrix.")
      }
    }
    if (is.null(taxanames)) {
      # if no taxa names specified,
      taxanames = gtrees[[1]]$tip.label   # ... get them from first tree
    }
    taxanames = sort(taxanames)
    if (length(taxanames) <= 25) {
      namelist = paste0(taxanames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxanames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxanames), " taxa: ", namelist)
    QT <- quartetTable(gtrees, taxanames)
    
  }
  QT <- quartetTableResolved(QT, omit = omit)
  QT <- quartetTableDominant(QT)
  dist <- quartetDist(QT)
  tree <- method(dist)
  return(tree)
}


  
  
  

#' Produce table of expected quartet concordance factors for a species tree
#'
#' Compiles table of expected quartet concordance factors (CFs) for gene trees under the MSC model 
#' on a metric species tree.
#'
#' @details
#' The species tree may be rooted or unrooted, but must have edge lengths given in coalescent units. Note that while the MSC
#' requires a rooted metric species tree parameter, as shown in \insertCite{Allman2011}{MSCquartets} the quartet CFs are independent 
#' of the root.
#'
#' In the returned table, columns are labeled by taxon names and quartet names ("12|34", etc.).
#' 1s and 0s in taxon columns indicate the taxa in a quartet. Quartet 12|34 means the first and second 
#' indicated taxa form a cherry, 13|24 means the first and third form a cherry, and 14|23 means
#' the first and fourth form a cherry. Unresolved quartets always have expectation 0 under the MSC.
#'
#' If a simplex plot is produced, for the \code{T1} model all CFs will lie on the vertical model line, 
#' and many choices of 4 taxa will give the same CFs. For model \code{T3} the model lines the CFs are 
#' plotted on depend on taxon names and are essentially arbitrary.
#'
#' @param speciestree phylo or character object specifying un/rooted metric species tree
#' @param plot \code{TRUE} (default) to produce simplex plot of CFs, or \code{FALSE} to omit plot
#' @param model "T1" or "T3" specifying model for plot
#' @param cex scaling factor for size of plotted symbols
#'
#' @return
#'     an (\code{n} choose 4)x(\code{n}+3) matrix encoding
#'     4 taxon subsets of taxa and expectation of each of the
#'     quartets 12|34, 13|24, 14|23 across gene trees
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableResolved}}
#'
#' @examples
#' stree="((((t5:5000,t6:5000):5000,t4:10000):2500,t7:12500):7500,((t8:3000,t9:3000):5000,
#' ((t1:4000,t2:4000):2500,t3:6500):1500):12000);"
#' st=read.tree(text=stree)
#' st$edge.length=st$edge.length/10000
#' expectedCFs(st)
#'
#' @references
#' 
#' \insertRef{Allman2011}{MSCquartets}
#' 
#' @importFrom methods is
#' @importFrom ape unroot read.tree cophenetic.phylo
#'
#' 
#' @export
expectedCFs <- function(speciestree,
                        plot = TRUE,
                        model = "T1",
                        cex = 1) {
  if (methods::is(speciestree,"phylo")) {
    stree = ape::unroot(speciestree)
  } else {
    if (methods::is(speciestree,"character")) {
      stree = ape::unroot(read.tree(text = speciestree)) #unrooted species tree
    } else {
      stop('Species tree must be of class "phylo"" or class "character".')
    }
  }
  if (is.null(stree$edge.length)) {
    stop('Species tree must have branch lengths.')
  }
  if (plot == TRUE & isFALSE(model %in% c("T1", "T3"))) {
    stop('Argument "model" must be "T1" or "T3" for plotting.')
  }
  
  D = ape::cophenetic.phylo(stree) # create distance table, for determining displayed quartets
  D = D[order(rownames(D)), order(colnames(D))]# put taxa in sorted order
  
  
  taxonnames = row.names(D)
  N = length(taxonnames)
  
  # set up table
  M = choose(N, 4)
  Q = matrix(0, M, N + 3) #allocate space for table
  qnames = c("12|34", "13|24", "14|23")
  colnames(Q) = c(taxonnames, qnames) #create column names
  
  # fill out table
  m = 0
  for (i in 1:(N - 3)) {
    # for each 4-taxon set
    for (j in (i + 1):(N - 2)) {
      for (k in (j + 1):(N - 1)) {
        for (l in (k + 1):N) {
          m = m + 1
          Q[m, c(i, j, k, l)] = 1 #encode set
          a = D[i, j] + D[k, l]
          b = D[i, k] + D[j, l]
          c = D[i, l] + D[j, k]
          
          if ((a <= b) & (a <= c)) {
            x = ((b + c) - 2 * a) / 2
            y = (1 / 3) * exp(-x)
            Q[m, (N + 1):(N + 3)] = c(1 - 2 * y, y, y)
          } else {
            if ((b <= a) & (b <= c)) {
              x = ((a + c) - 2 * b) / 2
              y = (1 / 3) * exp(-x)
              Q[m, (N + 1):(N + 3)] = c(y, 1 - 2 * y, y)
            } else {
              x = ((a + b) - 2 * c) / 2
              y = (1 / 3) * exp(-x)
              Q[m, (N + 1):(N + 3)] = c(y, y, 1 - 2 * y)
            }
          }
        }
      }
    }
  }
  
  
  
  if (plot == TRUE) {
    if (model == "T3") {
      simplexPrepare(model = model,
                     maintitle = "Expected Quartet Concordance Factors",
                     titletext = "Model T3")
      for (m in 1:M) {
        simplexPoint(Q[m, (N + 1):(N + 3)],
                     cex = cex,
                     pch = 8,
                     col = "blue")
      }
    } else {
      simplexPrepare(model = model,
                     maintitle = "Expected Quartet Concordance Factors",
                     titletext = "Model T1")
      for (m in 1:M) {
        simplexPoint(
          sort(Q[m, (N + 1):(N + 3)], decreasing = TRUE),
          cex = cex,
          pch = 8,
          col = "blue"
        )
      }
    }
  }
  return(Q)
}

#' Compute quartet distance between taxa
#'
#' Compute the Quartet Distance of \insertCite{Rho19;textual}{MSCquartets} from a table specifying a collection of quartets on
#' \code{n} taxa.
#'
#' @references 
#' \insertRef{Rho19}{MSCquartets}
#' 
#' @param dqt an (\code{n} choose 4) x \code{n} (or \code{n+1}) matrix of form output by \code{quartetTableDominant};
#'  (Note: If present, the \code{n+1}th column of \code{dqt} is ignored.)
#'
#' @return a pairwise distance matrix on \code{n} taxa
#'
#' @seealso 
#' \code{\link{quartetTableDominant}},
#' \code{\link{QDS}},
#' \code{\link{QDC}},
#' \code{\link{quartetWeightedDist}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT)
#' Dist=quartetDist(DQT)
#' tree=NJ(Dist)
#' write.tree(tree)
#' plot(tree)
#'
#' @export
quartetDist = function(dqt) {
  Qcolnames = colnames(dqt)# get column names
  n = length(Qcolnames) - 1# number of taxa
  taxa = Qcolnames[1:n]# names of taxa
  m = dim(dqt)[1]# number of quartets
  
  D = matrix(0, n, n)# create distance matrix
  colnames(D) = taxa
  rownames(D) = taxa
  
  for (i in 1:m) {
    ones = which(dqt[i, ] == 1)
    if (length(ones) == 2) {
      # skip null quartets
      nones = which(dqt[i, ] == -1)
      D[ones, nones] = D[ones, nones] + 1 # increment 4 entries of D
      D[nones, ones] = D[nones, ones] + 1 # and another 4
    }
  }
  
  D = 2 * D + 2 * n - 4 # adjust to appropriate Q distance
  for (i in 1:n) {
    D[i, i] = 0 # and unadjust diagonal back to 0
  }
  return(D)
}

###################################################################

#' Compute Quartet Distance Supertree
#'
#' Apply the Quartet Distance Supertree method of \insertCite{Rho19;textual}{MSCquartets} to a table specifying a
#' collection of quartets on \code{n} taxa.
#'
#' @references 
#' \insertRef{Rho19}{MSCquartets}
#' 
#' @details This function is a wrapper which runs \code{quartetDist} and then builds a tree.
#'
#' @param dqt an (\code{n} choose 4) x \code{n} (or \code{n+1}) matrix of form output by \code{quartetTableDominant};
#'  (Note: If present, the \code{n+1}th column of \code{dqt} is ignored)
#' 
#' @param method tree building function (e.g., fastme.bal, nj)
#' 
#' @return
#'     an unrooted metric tree of type phylo. Edge lengths are not in interpretable units
#' @seealso 
#' \code{\link{quartetTableDominant}},
#' \code{\link{quartetDist}},
#' \code{\link{QDC}},
#' \code{\link{WQDS}}, 
#' \code{\link{WQDC}}, 
#' \code{\link{WQDCrecursive}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT)
#' tree=QDS(DQT)
#' write.tree(tree)
#' plot(tree)
#' 
#' @importFrom ape fastme.bal
#'
#' @export
QDS = function(dqt,
               method = fastme.bal) {
  D = quartetDist(dqt)
  QDS = unroot(method(D))# build tree
  return(QDS)
}

#################################################

#' Compute the Weighted Quartet Distance between taxa
#'
#' Compute the Weighted Quartet Distance between taxa of \insertCite{YR19;textual}{MSCquartets} from a table specifying a collection of quartets on
#' \code{n} taxa and the quartets' internal branch lengths.
#' 
#' @references 
#' \insertRef{YR19}{MSCquartets}
#' 
#' @param dqt an (\code{n} choose 4) x (\code{n+1}) matrix of the form output by \code{quartetTableDominant}
#'
#' @return
#'     a pairwise distance matrix on \code{n} taxa
#'
#' @seealso \code{\link{quartetTableDominant}},
#'           \code{\link{WQDSAdjustLengths}},
#'           \code{\link{WQDS}},
#'           \code{\link{WQDC}}, 
#'           \code{\link{WQDCrecursive}},
#'           \code{\link{quartetWeightedDist}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT,bigweights="finite")
#' D=quartetWeightedDist(DQT)
#' tree=NJ(D)
#' stree=WQDSAdjustLengths(tree)
#' write.tree(stree)
#'
#' @export
quartetWeightedDist = function(dqt) {
  Qcolnames = colnames(dqt)# get column names
  n = length(Qcolnames) - 1# number of taxa
  taxa = Qcolnames[1:n]# names of taxa
  m = dim(dqt)[1]# number of quartets
  w = dqt[, n + 1]# weights
  
  D = matrix(2, n, n)# create distance matrix
  colnames(D) = taxa
  rownames(D) = taxa
  
  for (i in 1:m) {
    ones = which(dqt[i,] == 1)
    if (length(ones) == 2) {
      # skip null quartets
      nones = which(dqt[i,] == -1)
      D[ones, nones] = D[ones, nones] + w[i] # increment 4 entries of D
      D[nones, ones] = D[nones, ones] + w[i] # and another 4
    }
  }
  
  for (i in 1:n) {
    D[i, i] = 0 # unadjust diagonal back to 0
  }
  return(D)
}


#################################################

#' Compute the Weighted Quartet Distance Supertree 
#'
#' Apply the Weighted Quartet Distance Supertree method of \insertCite{YR19;textual}{MSCquartets} to
#' a collection of quartets on \code{n} taxa together with internal 
#' quartet branch lengths, specified by a table.
#' 
#' @references 
#' \insertRef{YR19}{MSCquartets}
#'
#' @details This function is a wrapper which runs \code{quartetWeightedDist}, builds a tree, and then adjusts edge lengths
#' with \code{WQDSAdjustLengths}.
#'
#' @param dqt an (\code{n} choose 4) x \code{n+1}) matrix of form output by \code{quartetTableDominant}
#' @param method a distance-based tree building function (e.g., fastme.bal, NJ, etc.)
#'
#' @return an unrooted metric tree, of type phylo
#'          
#' @seealso \code{\link{quartetTableDominant}}, 
#'          \code{\link{quartetWeightedDist}},
#'          \code{\link{WQDSAdjustLengths}},
#'          \code{\link{WQDC}}, 
#'          \code{\link{WQDCrecursive}},
#'          \code{\link{QDS}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT,bigweights= "finite")
#' tree=WQDS(DQT)
#' write.tree(tree)
#' plot(tree)
#'
#' @importFrom ape fastme.bal
#' @export
WQDS = function(dqt,
                method = fastme.bal) {
  D = quartetWeightedDist(dqt)# compute distance
  WQDS = unroot(method(D))# build tree
  WQDS=WQDSAdjustLengths(WQDS)
  return(WQDS)
}

#################################################

#' Adjust edge lengths on tree built from Weighted Quartet distance 
#' to estimate metric tree
#'
#' Modify edge lengths of a tree built from a distance table produced by \code{quartetWeightedDist},
#' to remove scaling factors related to the size of the split associated to the edge.
#'
#' @details As explained by \insertCite{YR19;textual}{MSCquartets}, a metric tree produced from
#' the weighted quartet distance has edge lengths
#' inflated by a factor dependent on the associated split size. Removing these
#' factors 
#' yields a consistent estimate of the metric species tree displaying the weighted
#' quartets, if such a tree exists.
#'
#' This function should not be used on trees output from \code{WQDS}, \code{WQDC},
#' or \code{WQDCrecursive}, as 
#' their edges are already adjusted. It can be used on trees built from the distance
#' computed by \code{quartetWeightedDist}.
#' 
#' @references 
#' \insertRef{YR19}{MSCquartets}
#'  
#' @param tree an unrooted metric tree, of type phylo
#' 
#' @return an unrooted metric tree, of type phylo
#' 
#' @seealso 
#' \code{\link{WQDS}},
#' \code{\link{WQDC}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT,bigweights="finite")
#' D=quartetWeightedDist(DQT)
#' tree=NJ(D)
#' write.tree(tree)
#' plot(tree)
#' stree=WQDSAdjustLengths(tree)
#' write.tree(stree)
#' plot(stree)
#'
#' @importFrom ape is.rooted unroot root
#' @importFrom phangorn Descendants
#' @export
WQDSAdjustLengths = function(tree) {
  if (is.rooted(tree) == TRUE) {
    message("Tree is rooted; unrooting it.")
    tree = unroot(tree)
  }
  ntree = root(tree, outgroup = tree$tip.label[1])
  ntaxa = length(ntree$tip.label)
  nedges = dim(ntree$edge)[1]
  for (i in 1:nedges) {
    node = ntree$edge[i, 2] # get child of edge
    ndesc = length(Descendants(ntree, node, "tips")[[1]])# count descendents, using phangorn function
    if (ndesc == 1) {
      ntree$edge.length[i] = 1
    }
    else if (ndesc == ntaxa - 1) {
      ntree$edge.length[i] = 0
    }
    else {
      ntree$edge.length[i] = ntree$edge.length[i] / ((ndesc - 1) * (ntaxa - ndesc - 1)) # adjust edge length
    }
  }
  
  WQDSAdjustLengths = unroot(ntree)
  return(WQDSAdjustLengths)
}


#################################################


#' Compute Quartet Distance Consensus tree from gene tree data
#'
#' Compute the Quartet Distance Consensus \insertCite{Rho19}{MSCquartets} estimate of an unrooted 
#' topological species tree from gene tree data.
#'
#' @details This function is a wrapper which performs the steps of reading in a collection
#' of gene trees, tallying quartets, computing the quartet distance between taxa, building
#' a tree which consistently estimates the unrooted species tree topology under the MSC, and then possibly estimating edge
#' lengths using the "simpleML" method.
#'
#' @references 
#' \insertRef{Rho19}{MSCquartets}
#' 
#' @param genetreedata  gene tree data that may be supplied in any of 3 forms: 
#' \enumerate{
#' \item a character string giving the name of a file containing gene trees in Newick,
#' \item a multiPhylo object containing gene trees, or
#' \item a resolved quartet table, such as produced by \code{quartetTableResolved}
#' } 
#' @param taxanames if \code{genetreedata} is a file or a multiPhylo object, a vector of a subset  
#' of the taxa names on the gene trees 
#' to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genetreedata} 
#' is a quartet table, this argument is ignored and all taxa in the table are used
#' @param method a distance-based tree building function, such as \code{fastme.bal} or \code{nj}
#' @param omit \code{TRUE} ignores unresolved quartets; \code{FALSE} treats them as 1/3 of each resolution; 
#' ignored if \code{genetreedata} is supplied as a quartet table
#' @param metric if \code{FALSE} return topological tree; if \code{TRUE} return metric tree with
#' internal edge lengths estimated by \code{estimateEdgeLengths} with \code{lambda=0}, and terminal branches of length 1
#' 
#' 
#' @return an unrooted tree of type phylo
#' 
#' @seealso \code{\link{quartetTable}},
#'          \code{\link{quartetTableResolved}},
#'          \code{\link{quartetTableDominant}},
#'          \code{\link{quartetDist}},
#'          \code{\link{QDS}},
#'          \code{\link{WQDC}}, 
#'          \code{\link{WQDCrecursive}}
#'          \code{\link{estimateEdgeLengths}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' stree=QDC(gtrees,tnames[1:6])
#' write.tree(stree)
#' plot(stree)
#' streeMetric=QDC(gtrees, tnames[1:6],metric=TRUE)
#' write.tree(streeMetric)
#' plot(streeMetric)
#' 
#' @export
QDC = function(genetreedata,
               taxanames = NULL,
               method=fastme.bal,
               omit = FALSE, 
               metric=FALSE) {
  if ("matrix" %in% class(genetreedata)) {
    RQT = genetreedata
    if (!is.null(taxanames)) {
      message("Ignoring argument taxanames since genetreedata supplied as quartet table.")
    }
  } else {
    if ("multiPhylo" %in% class(genetreedata))  {
      genetrees = genetreedata
    } else {
      if ("character" %in% class(genetreedata)) {
        genetrees <- read.tree(genetreedata) #read gene trees
        message(paste("Read", length(genetrees), "gene trees from file."))
      } else {
        stop("Data must be supplied as an object of class multiPhylo, character, or matrix.")
      }
    }
    
    if (is.null(taxanames)) {
      # if no taxa names specified,
      taxanames = genetrees[[1]]$tip.label   # ... get them from first tree
    }
    taxanames = sort(taxanames)
    if (length(taxanames) <= 25) {
      namelist = paste0(taxanames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxanames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxanames), " taxa: ", namelist)
    
    # tally quartets on gene trees
    RQT = quartetTableResolved(quartetTable(genetrees, taxanames), omit = omit)         # treat unresolved quartets as 1/3 of each resolution
  }
  
  Q = quartetTableDominant(RQT)
  tree = QDS(Q,method=method)
  if (metric==FALSE) {
    tree$edge.length = NULL
  } else {
    tree = estimateEdgeLengths(tree, RQT, lambda = 0)
  }
  return(tree)
}


#############################################################

#' Compute Weighted Quartet Distance Consensus tree from gene tree data
#'
#' Compute the Weighted Quartet Distance Consensus \insertCite{YR19}{MSCquartets} estimate of a 
#' species tree from gene tree data. This is a consistent estimator of the unrooted 
#' species tree topology and all internal branch lengths.
#'
#' @details This function is a wrapper which performs the steps of reading in a collection
#' of gene trees, tallying quartets, estimating quartet internal branch lengths, computing the weighted
#' quartet distance between taxa, building
#' a tree, and adjusting edge lengths, to give a consistent estimate of the metric species tree in coalescent units
#' under the MSC.
#' 
#' If the gene tree data indicates some quartets experienced little to no incomplete lineage 
#' sorting, this algorithm tends to be less topologically accurate than \code{QDC} 
#' (which infers no metric information) or \code{WQDCrecursive} (which gives better topologies,
#' and reasonably accurate lengths for short edges, though long edge lengths may still be unreliable).
#' 
#' @references 
#' \insertRef{YR19}{MSCquartets}
#' 
#' @param genetreedata  gene tree data that may be supplied in any of 3 forms:
#' \enumerate{
#' \item a character string giving the name of a file containing gene trees in Newick
#' \item a multiPhylo object containing gene trees
#' \item a resolved quartet table, as produced by \code{quartetTableResolved}
#' }
#' @param taxanames if \code{genetreedata} is a file or a multiPhylo object, a vector of a subset  
#' of the taxa names on the gene trees 
#' to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genetreedata} 
#' is a quartet table, this argument is ignored and all taxa in the table are used
#' @param method a distance-based tree building function, such as \code{fastme.bal} or \code{nj}
#' @param omit \code{TRUE} leaves out unresolved quartets, \code{FALSE} treats them as 1/3 of each resolution; ignored if 
#' \code{genetreedata} is given as  a resolved quartet table
#' @param terminal non-negative branch length to supply for terminal branches 
#' whose length cannot be inferred by \code{WQDC}
#' @return an unrooted metric tree of type phylo
#' @seealso \code{\link{quartetTable}}, 
#'          \code{\link{quartetTableResolved}}, 
#'          \code{\link{quartetTableDominant}}, 
#'          \code{\link{quartetWeightedDist}}, 
#'          \code{\link{WQDCrecursive}}, 
#'          \code{\link{WQDS}}, 
#'          \code{\link{QDC}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' stree=WQDC(gtrees,tnames[1:6])
#' write.tree(stree)
#' plot(stree)
#'
#' @export
WQDC = function(genetreedata,
                taxanames = NULL,
                method=fastme.bal,
                omit = FALSE,
                terminal = 1) {
  
  if ("matrix" %in% class(genetreedata)) {
    RQT = genetreedata
    if (!is.null(taxanames)) {
      message("Ignoring argument taxanames since genetreedata supplied as quartet table.")
    }
  } else {
    if ("multiPhylo" %in% class(genetreedata))  {
      genetrees = genetreedata
    } else {
      if ("character" %in% class(genetreedata)) {
        genetrees <- read.tree(genetreedata) #read gene trees
        message(paste("Read", length(genetrees), "gene trees from file."))
      } else {
        stop("Data must be supplied as an object of class multiPhylo, character, or matrix.")
      }
    }
    
    if (is.null(taxanames)) {
      # if no taxa names specified,
      taxanames = genetrees[[1]]$tip.label   # ... get them from first tree
    }
    taxanames = sort(taxanames)
    if (length(taxanames) <= 25) {
      namelist = paste0(taxanames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxanames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxanames), " taxa: ", namelist)
    
    # tally quartets on gene trees
    RQT = quartetTableResolved(quartetTable(genetrees, taxanames), omit = omit)         # treat unresolved quartets as 1/3 of each resolution
  }
  
  Q=quartetTableDominant(RQT, bigweights ="finite")
  
  WQDC = WQDS(Q,method=method)
  term_edges = which(WQDC$edge[, 2] <= length(taxanames)) #modify terminal edge lengths
  WQDC$edge.length[term_edges] = terminal
  return(WQDC)
}

#' Suppress Network Binary Nodes
#'
#' Suppress any non-root binary nodes in a phylogenetic network.
#'
#' @details
#' This function is similar to ape's \code{collapse.singles} which only works
#' on phylo objects that are trees.
#'
#'@param net a phylogenetic network, of class "evonet"
#'@param addlength if TRUE (default), add lengths of two incident edges for new edge.
#'
#'@export
suppressNetBinaryNodes = function(net, addlength = TRUE) {
  ntaxa = length(net$tip.label)
  edge = net$edge
  reticEdge = net$reticulation
  edgeLength = net$edge.length
  nIntNode = net$Nnode #number of internal nodes
  nodeLabels = net$node.label

  nodeDegrees = tabulate(c(unlist(list(edge)), unlist(list(reticEdge))))

  binNodes = which(nodeDegrees == 2)
  binNodes = binNodes[which(binNodes > (ntaxa + 1))]# degree 2 nodes other than root

  if (length(binNodes) > 0) {
    # if any nodes to be suppressed, do it
    for (node in binNodes) {
      parentEdge = which(edge[, 2] == node)
      childEdge = which(edge[, 1] == node)
      childNode = edge[childEdge, 2]
      edge[parentEdge, 2] = childNode
      edge = edge[-childEdge, ]
      if (addlength == TRUE) {
        edgeLength[parentEdge] = edgeLength[parentEdge] + edgeLength[childEdge]
      }
      edgeLength = edgeLength[-childEdge]
    }
    nIntNode = nIntNode - length(binNodes)
    nodeLabels = nodeLabels[-(binNodes - ntaxa)]

    # renumber nodes
    newNodeNums = 1:(ntaxa + nIntNode)
    oldNodeNums = sort(unique(unlist(as.list(edge))))
    for (i in 1:2) {
      for (j in 1:dim(edge)[1]) {
        edge[j, i] = newNodeNums[which(oldNodeNums == edge[j, i])]
      }
      for (j in 1:dim(reticEdge)[1]) {
        reticEdge[j, i] = newNodeNums[which(oldNodeNums == reticEdge[j, i])]
      }
    }

    #prepare output
    net$edge = edge
    net$edge.length = edgeLength
    net$Nnode = nIntNode
    net$node.label = nodeLabels
    net$reticulation = reticEdge
  }
  return(net)
}

#' Compute the Recursive Weighted Quartet Distance Consensus tree from gene tree data
#'
#' Infer a metric species tree from counts
#' of quartets displayed on a collection of gene trees, as described by 
#' \insertCite{YR19;textual}{MSCquartets}. Edge lengths are in coalescent units.
#'
#'
#' @details The algorithm counts quartets displayed on the gene trees, builds a tree using \code{WQDS},
#' determines the split corresponding to the longest edge in that tree,
#' and then recursively builds trees
#' on the taxa in each split set together with a `composite taxon' formed by all
#' taxa in the other split set.
#' This approach is slower than non-recursive \code{WQDC}, but increases topological accuracy. Shorter branch 
#' lengths tend to be more accurately estimated.
#'
#' This function must be called with its argument a resolved quartet
#' table of size (n choose 4)x(n+3). Its recursive nature
#' requires building smaller resolved quartet tables on split sets with an additional
#' composite taxon.
#'
#' @references
#' \insertRef{YR19}{MSCquartets}
#'
#' @param rqt a resolved quartet table as produced by \code{quartetTableResolved}
#' @param method a distance-based tree building function, such as \code{fastme.bal} or \code{nj}
#' @param stopAt a non-negative branch length in coalescent units; recursive calls stop when the longest 
#' branch in a recursively examined subtree is smaller than this value
#' @param terminal non-negative branch length to supply for terminal branches,
#' whose lengths cannot be inferred by \code{WQDCrecursive}
#'
#' @return an unrooted metric tree, of type phylo
#'
#' @seealso \code{\link{quartetTableResolved}},\code{\link{quartetTable}},
#' \code{\link{QDC}}, \code{\link{QDS}}, \code{\link{quartetTableCollapse}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' stree=WQDCrecursive(RQT)
#' write.tree(stree)
#' plot(stree)
#'
#' @importFrom ape Ntip drop.tip bind.tree
#'
#' @export
WQDCrecursive = function(rqt,
                         method = fastme.bal,
                         stopAt = 2,
                         terminal = 1) {
  nX = dim(rqt)[2] - 3 # number of taxa
  if (nX < 4) {
    stop("Fewer than 4 taxa.")
  }
  
  taxanames = colnames(rqt)[1:nX]# get taxa names
  DQT = quartetTableDominant(rqt, bigweights = "finite")
  roughtree <-
    root(WQDS(DQT,method=method), outgroup = taxanames[1], resolve.root = TRUE)# construct initial estimate of tree, with arbitrary root
  
  internaledgeindices = which(roughtree$edge[, 2] > Ntip(roughtree)) # determine internal edges
  length = max(roughtree$edge.length[internaledgeindices]) #get longest internal edge length
  if (length <= stopAt) {
    return(unroot(roughtree))
  }
  
  longestedges = which(roughtree$edge.length[internaledgeindices] == length)# and longest edges
  if (length(longestedges) > 1) {
    longestedge = sample(longestedges, 1)# random tie breaking
  } else {
    longestedge = longestedges
  }
  
  longestinternaledgeindex = internaledgeindices[longestedge]# ...to pick edge
  node = roughtree$edge[longestinternaledgeindex, 2] # get child of edge
  taxaAind = phangorn::Descendants(roughtree, node, "tips")[[1]] # determine split sets for that edge
  taxaA = roughtree$tip.label[taxaAind]
  nA = length(taxaAind)
  taxaB = setdiff(taxanames, taxaA)
  nB = nX - nA
  
  if (nA > 2) {
    # at least 3 taxa in A
    RQTA = quartetTableCollapse(rqt, taxaA, taxaB) #create smaller tables for set A \cup {"setB"}
    Ap1tree = WQDCrecursive(RQTA) #recursively build tree
    Bname = paste0(sort(taxaB), collapse = "")
    Atree = drop.tip(root(Ap1tree, Bname, resolve.root = TRUE), Bname)
  } else {
    Atree = drop.tip(root(roughtree, taxaB[1], resolve.root = TRUE), taxaB) #keep only cherry for A
  }
  
  
  if (nB > 2) {
    # at least 3 taxa in B
    RQTB = quartetTableCollapse(rqt, taxaB, taxaA) #create smaller tables for set B \cup {"setA"}
    Bp1tree = WQDCrecursive(RQTB) #recursively build tree
    Aname = paste0(sort(taxaA), collapse = "")
    Btree = drop.tip(root(Bp1tree, Aname, resolve.root = TRUE), Aname)
  } else {
    # only 2 taxa in B
    Btree = drop.tip(root(roughtree, taxaA[1], resolve.root = TRUE), taxaA) #keep only cherry for A
  }
  
  Atree$root.edge = length
  Xtree = unroot(bind.tree(Atree, Btree, position = length))
  term_edges = which(Xtree$edge[, 2] <= length(taxanames)) #modify terminal edge lengths
  Xtree$edge.length[term_edges] = terminal
  return(Xtree)
}


#' Reduce quartet table by combining some taxa 
#'
#' Form a smaller resolved quartet table by lumping some taxa into a composite taxon.
#'
#' @details This function is needed for the recursive calls in \code{WQDSrec}.
#' It should only be applied to a resolved quartet table which includes counts
#' for all possible quartets on the taxa (though counts can be zero).
#' 
#' The sets \code{taxaA} and \code{taxaB} must be disjoint. Their union need not be all taxa in \code{rqt}.
#'
#' @param rqt a resolved quartet table, as from \code{quartetTableResolved}
#' @param taxaA a vector of taxon names in \code{rqt} to be included in the output table
#' @param taxaB a vector of taxon names in \code{rqt} to form new composite taxon in the output table
#'
#' @return a resolved quartet table with \code{length(taxaA)+1} taxa; the
#'  composite taxon is named as the concatenation of the sorted
#'  names in \code{taxaB}
#'
#' @seealso \code{\link{WQDCrecursive}}
#'
#' @export
quartetTableCollapse = function(rqt, 
                                taxaA, 
                                taxaB) {
  taxaAB = union(taxaA, taxaB)
  if (!setequal(intersect(taxaAB, colnames(rqt)), taxaAB)) {
    stop("Not all taxa in given sets in table.")
  } else {
    if (length(intersect(taxaA, taxaB)) != 0)
      stop("Given taxon sets must be disjoont.")
  }
  
  M = dim(rqt)[1]
  ntaxa = dim(rqt)[2] - 3
  coltaxa = colnames(rqt)[1:ntaxa]
  nA = length(taxaA)
  nB = length(taxaB)
  taxaA = coltaxa[sort(match(taxaA, coltaxa))] #order taxa A as in rqt
  
  #set up table for subset A + "setB"
  nAp1 = nA + 1
  mRQTA = choose(nAp1, 4)
  RQTA = matrix(0, mRQTA, nAp1 + 3)
  qnames = c("12|34", "13|24", "14|23")
  Bname = paste0(sort(taxaB), collapse = "") #create name for composite taxon for set B
  colnames(RQTA) = c(taxaA, Bname, qnames) #create column names
  
  # encode 4-taxon sets in table
  m = 0
  for (i in 1:(nAp1 - 3)) {
    # for each 4-taxon set
    for (j in (i + 1):(nAp1 - 2)) {
      for (k in (j + 1):(nAp1 - 1)) {
        for (l in (k + 1):nAp1) {
          m = m + 1
          RQTA[m, c(i, j, k, l)] = 1 #encode set
        }
      }
    }
  }
  
  # fill counts in new table, by running through old table
  for (m in 1:M) {
    memA = taxaA[which(rqt[m, taxaA] == 1)]
    numA = length(memA)
    
    if (numA == 4) {
      mA = which(rowSums(RQTA[, memA, drop = FALSE]) == 4)
      RQTA[mA, qnames] = rqt[m, qnames]
    } else {
      if (numA == 3)  {
        mA = which(rowSums(RQTA[, c(memA, Bname), drop = FALSE]) == 4)
        taxonB = which(is.element(coltaxa[which(rqt[m, coltaxa] == 1)], taxaB))
        z = rqt[m, qnames]
        if (taxonB == 1) {
          RQTA[mA, qnames] = RQTA[mA, qnames] + z[c(3, 2, 1)]
        } else{
          if (taxonB == 2) {
            RQTA[mA, qnames] = RQTA[mA, qnames] + z[c(2, 3, 1)]
          } else{
            if (taxonB == 3) {
              RQTA[mA, qnames] = RQTA[mA, qnames] + z[c(1, 3, 2)]
            } else{
              RQTA[mA, qnames] = RQTA[mA, qnames] + z
            }
          }
        }
      }
    }
  }
  return(RQTA)
}

#' Estimate edge lengths on a species tree from gene tree quartet counts
#'
#' Estimate edge lengths, in coalescent units, on an unrooted species tree 
#' from a table of resolved quartet counts from a collection of gene trees.
#'
#' @details While the argument \code{tree} may be supplied as rooted or unrooted, metric or topological,
#' only its unrooted topology will be used for determining new metric estimates.
#' 
#' Counts of quartets for all those quartets which define a single edge
#' on the tree (i.e., whose internal edge is the
#' single edge on the unrooted input tree) are summed, and from this an
#' estimate of the branch length is computed. If \code{method= "simpleML"} this is the maximum likelihood estimate.
#' If \code{method="simpleBayes"} this is the Bayesian estimate of Theorem 2
#' of \insertCite{SayMir16;textual}{MSCquartets}, using parameter \code{lambda}.
#' Using \code{lambda=1/2}
#' gives a flat prior on [1/3,1] for the probability of the quartet displayed on the species tree.
#' 
#' These methods are referred to as `simple' since they use only the quartets defining a single edge of the species tree.
#' Quartets with central edges composed of several edges in the species tree are ignored.
#'
#' Note that branch length estimates may be 0 (if the count for the quartet
#' displayed on the input tree is not dominant),
#' positive, or \code{Inf} (if
#' the counts for quartet topologies not displayed on the input tree are all 0, and \code{method="simpleML"}).
#'
#' @references
#'  \insertRef{SayMir16}{MSCquartets}
#'
#' @param tree a phylo object, giving a resolved tree on which to estimate edge lengths
#' @param rqt a resolved quartet table, as from \code{quartetTableResolved},
#'  in which all taxa on \code{tree} appear
#' @param terminal an edge length to assign to terminal edges, whose lengths cannot be estimated
#' @param method \code{"simpleML"} or \code{"simpleBayes"}
#' @param lambda a positive parameter for the \code{"simpleBayes"} method
#'
#' @return  an unrooted metric tree with the same topology as \code{tree}, of type phylo
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' taxanames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,taxanames[1:6])
#' RQT=quartetTableResolved(QT)
#' DQT=quartetTableDominant(RQT)
#' tree=QDS(DQT)
#' write.tree(tree)
#' plot(tree)
#' metricMTree=estimateEdgeLengths(tree,RQT,method="simpleML")
#' write.tree(metricMTree)
#' plot(metricMTree)
#' metricBTree=estimateEdgeLengths(tree,RQT,method="simpleBayes")
#' write.tree(metricBTree)
#' plot(metricBTree)
#' 
#' @importFrom phangorn Descendants
#' @export
estimateEdgeLengths = function(tree,
                               rqt,
                               terminal = 1,
                               method = "simpleML",
                               lambda = 1/2) {
  if (!(method %in% c("simpleML","simpleBayes"))) {
    stop('Argument method must be "simpleML" or "simpleBayes".')
  }
  
  if (method=="simpleBayes") {
    if (lambda<=0) {
      stop("Argument lambda must be positive.")
    }
  } else { # for "simpleML"
      lambda=0 
    }
  
  newroot = tree$tip.label[1]
  tree = root(tree, outgroup = newroot, resolve.root = TRUE)
  
  nedges = dim(tree$edge)[1]
  tree$edge.length = rep(0, nedges)
  taxanames = tree$tip.label
  ntaxa = length(taxanames)
  
  tableTaxa = colnames(rqt)[1:ntaxa]
  qlabels = c("12|34", "13|24", "14|23")
  
  for (i in 1:nedges) {
    if (tree$edge[i, 2] <= ntaxa) {
      tree$edge.length[i] = terminal # pendant edge
    } else {
      edgeParent = tree$edge[i, 1]
      if (edgeParent == (ntaxa + 1)) {
        tree$edge.length[i] = 0 #root edge
      } else {
        edgeChild = tree$edge[i, 2]
        edgesBelowChild = which(tree$edge[, 1] == edgeChild)
        setA = phangorn::Descendants(tree, tree$edge[edgesBelowChild[1], 2], "tips")[[1]]
        setA = taxanames[setA]
        setB = phangorn::Descendants(tree, tree$edge[edgesBelowChild[2], 2], "tips")[[1]]
        setB = taxanames[setB]
        edgesBelowParent = which(tree$edge[, 1] == edgeParent)
        if (length(edgesBelowParent) != 2) {
          stop("Something's wrong...")
        } else {
          otherEdgeBelowParent = setdiff(edgesBelowParent, i)
          setC = phangorn::Descendants(tree, tree$edge[otherEdgeBelowParent, 2], "tips")[[1]]
          setC = taxanames[setC]
          setD = setdiff(taxanames, union(setA, union(setB, setC)))
          CF = rep(0, 3)
          
          for (a in setA) {
            for (b in setB) {
              for (c in setC) {
                for (d in setD) {
                  kk = 1
                  k = 0
                  while (k == 0) {
                    # find row for this quartet
                    taxind = rqt[kk, a] * rqt[kk, b] * rqt[kk, c] * rqt[kk, d]
                    if (taxind == 1) {
                      k = kk
                    }
                    kk = kk + 1
                  }
                  
                  
                  acol = which(tableTaxa == a)
                  bcol = which(tableTaxa == b)
                  ccol = which(tableTaxa == c)
                  dcol = which(tableTaxa == d)
                  r = rank(c(acol, bcol, ccol, dcol))
                 
                  if (identical(r, c(1, 2, 3, 4)) |
                      identical(r, c(2, 1, 4, 3)) |
                      identical(r, c(3, 4, 1, 2)) |
                      identical(r, c(4, 3, 2, 1))) {
                    qorder = c(1, 2, 3)
                  } else {
                    if (identical(r, c(1, 2, 4, 3)) |
                        identical(r, c(2, 1, 3, 4)) |
                        identical(r, c(3, 4, 2, 1)) |
                        identical(r, c(4, 3, 1, 2))) {
                      qorder = c(1, 3, 2)
                    } else {
                      if (identical(r, c(1, 3, 2, 4)) |
                          identical(r, c(2, 4, 1, 3)) |
                          identical(r, c(3, 1, 4, 2)) |
                          identical(r, c(4, 2, 3, 1))) {
                        qorder = c(2, 1, 3)
                      } else{
                        if (identical(r, c(1, 3, 4, 2)) |
                            identical(r, c(2, 4, 3, 1)) |
                            identical(r, c(3, 1, 2, 4)) |
                            identical(r, c(4, 2, 1, 3))) {
                          qorder = c(2, 3, 1)
                        } else {
                          if (identical(r, c(1, 4, 2, 3)) |
                              identical(r, c(2, 3, 1, 4)) |
                              identical(r, c(3, 2, 4, 1)) |
                              identical(r, c(4, 1, 3, 2))) {
                            qorder = c(3, 1, 2)
                          } else {
                            if (identical(r, c(1, 4, 3, 2)) |
                                identical(r, c(2, 3, 4, 1)) |
                                identical(r, c(3, 2, 1, 4)) |
                                identical(r, c(4, 1, 2, 3))) {
                              qorder = c(3, 2, 1)
                            }
                          }
                        }
                      }
                    }
                  }
                  CF = CF + rqt[k, qlabels[qorder]]
                }
              }
            }
          }
          
          avCF = CF / (length(setA) * length(setB) * length(setC) * length(setD))
          f = avCF[1] / (sum(avCF) + 2 * lambda)
          if (f >= 1) {
            edgeweight = Inf
          } else {
            if (f >= (1 / 3)) {
              edgeweight = -log((1.5 * (1 - f)))
            } else {
              edgeweight = 0
            }
          }
          tree$edge.length[i] = edgeweight
        }
      }
    }
  }
  tree = unroot(tree)
  return(tree)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Build quartet table from distances
#'
#' This is a C++ function, called from quartetTable, to fill in the quartet counts.
#' From a list of topological distance matrices (1 for each gene tree) it determines all
#' gene quartets. It is not intended to be used as a stand-alone function, and hence not fully
#' documented. The faster looping in C++ over R gives substantial time improvements
#'
#' @param dList a list of distance matrices
#' @param M number of sets of 4 taxa
#' @param nt number of gene trees/distance matrices
#' @param Q matrix to fill out as table of quartet counts
#' @param random if 0 compute for all sets of 4 taxa, otherwise for M random ones
#' @param progressbar if TRUE, display progress bar
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableParallel}}
#'
#' @export
quartetTallyCpp <- function(dList, M, nt, Q, random, progressbar = FALSE) {
    .Call('_MSCquartets_quartetTallyCpp', PACKAGE = 'MSCquartets', dList, M, nt, Q, random, progressbar)
}

#' Initialize vector of B quartets
#'
#' This is a C++ function, called from \code{TINNIKdist}, to initialize for
#' inference of B and T quartets.
#'
#' @param pTable a quartet table with p-values
#' @param m number of rows in pTable
#' @param alpha critical value for tree test
#' @param beta critical value for star tree test
#' @param colptest column with p value for tree test
#' @param colpstar column with p value for star tree test
#' @param Bquartets 0/1 vector encoding initial B quartets
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableParallel}}
#'
#' @export
initBquartets <- function(pTable, m, alpha, beta, colptest, colpstar, Bquartets) {
    .Call('_MSCquartets_initBquartets', PACKAGE = 'MSCquartets', pTable, m, alpha, beta, colptest, colpstar, Bquartets)
}

#' Main loop of B-quartet inference
#'
#' This is a C++ function, called from \code{TINNIKdist}, to
#' infer B and T quartets.
#'
#' @param pTable a quartet table with p-values
#' @param C precomputed binomial coefficients
#' @param Cn4 precomputed binomial coefficient
#' @param n number of taxa
#' @param Bquartets 0/1 vector of initial Bquartets
#' @param L1 vector of recently inferred B quartets
#' @param lenL1 lnegth of L1
#' @param Nrule1 count of inference from rule 1
#' @param Nrule2 count of inference from rule 2
#' @param cuttops inferred cut topologies
#'
#' @seealso \code{\link{quartetTable}}, \code{\link{quartetTableParallel}}
#'
#' @export
BQinference <- function(pTable, C, Cn4, n, Bquartets, L1, lenL1, Nrule1, Nrule2, cuttops) {
    .Call('_MSCquartets_BQinference', PACKAGE = 'MSCquartets', pTable, C, Cn4, n, Bquartets, L1, lenL1, Nrule1, Nrule2, cuttops)
}

#' Combine p-values of tests
#'
#' Given a vector of p-values from not-necessarily-independent tests, compute a combined p-value
#' using the Cauchy combination method Liu and Xie (2020), the Bonferroni method (also called MinP, the minimum times the number of tests),
#' or the combined Cauchy/Bon variants BBC and CBC as proposed by Chen (2022).
#'
#' All \code{NA}a in vector are ignored.
#'
#' @references
#' \insertRef{LiuXie2020}{MSCquartets}
#'
#' \insertRef{Chen2022}{MSCquartets}
#' 
#' @param pvec a vector of p-values to be combined
#' @param method the combination method to be used, one of \code{"Cauchy"}, \code{"Bon"} (default), \code{"BBC"}, \code{"CBC"}, or \code{"all"}
#'
#' @returns If the argument \code{method} is \code{"Cauchy"}, \code{"Bon"}, \code{"BBC"}, or \code{"CBC"} a single p-value obtained from that method is returned.
#' If \code{method} is "All", a vector of all 4 are returned, with entries names "p_Cauchy", "p_Bon", "p_BBC", and "p_CBC"
#'
#' @examples
#' combineP(c(.01,.04,.11))
#' combineP(c(.01,.04,.11),"all")
#'
#'@export
combineP = function(pvec, method = "Bon")
{
  if (!(method %in% c("Cauchy", "Bon", "BBC", "CBC", "all")))  {
    stop("Invalid argument: 'method' ")
  }
  
  pvec=pvec[which(!is.na(pvec))] #remove NAs
  
  if (length(pvec) == 0)
    pvec = 1

  #Cauchy
  if (method != "Bon") {
    is.small = (pvec < 1e-15)
    tvec = rep(NULL, length(pvec))
    tvec[!is.small] = tan((0.5 - pvec[!is.small]) * pi)
    tvec[is.small] = 1 / (pvec[is.small] * pi)
    tstat = mean(tvec)
    pCauchy = pcauchy(tstat, lower.tail = F)
  }

  #Bon
  if (method != "Cauchy")
    pBon = min(1, length(pvec) * min(pvec))


  #BBC
  if (method %in% c("BBC", "all"))
    pBBC = min(1, 2 * min(pCauchy, pBon))

  #CBC
  if (method %in% c("CBC", "all")) {
    pvec2 = c(pCauchy, pBon)
    is.small = (pvec2 < 1e-15)
    tvec = rep(NULL, 2)
    tvec[!is.small] = tan((0.5 - pvec2[!is.small]) * pi)
    tvec[is.small] = 1 / (pvec2[is.small] * pi)
    tstat = mean(tvec)
    pCBC = pcauchy(tstat, lower.tail = F)
  }

  if (method == "all")
    pall = c(
      p_Cauchy = pCauchy,
      p_Bon = pBon,
      p_BBC = pBBC,
      p_CBC = pCBC
    )

  p = get(paste0("p", method))
  return(p)
}



##########################################

#' Hypothesis testing of quartet counts for ECToBlob
#'
#' Append p-values for the star and T1 tests of displayed quartets on a given
#' tree to resolved quartet count table. This is used in \code{ECToBlob}
#' to obtain an estimated Tree of Blobs for a data set from a supplied tree that
#' is assumed to be a resolution of the Tree of Blobs. Three columns are appended
#' to the quartet count table: "p_T1-D" holds the T1 test p-values, "qindexD" holds
#' the index (1,2,or 3) of the quartet topology displayed on the tree, and "p_star"
#' holds the star tree test p-value.
#'
#' If on the tree a quartet is unresolved, p_T1 and qindex column entry will be NA
#'
#' \code{tree} must be supplied as phylo object, but may have multifurcations.
#'
#' If either p_star or p_T1-D and qindex-D values are already in table, they are
#' not recomputed.
#'
#' @param rqt either a table of resolved quartet counts from a collection of gene trees,
#' or that table with additional columns of quartet p-values.
#' @param tree an unrooted tree determining the topologies of the displayed quartet trees to test
#' @param lambda parameter for power-divergence statistic (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)
#' @param method "MLest", "conservative", or "bootstrap"
#' @param smallsample	"precomputed" or "bootstrap", method of obtaining p-value when sample is small (<30)
#' @param smallcounts	"precomputed" or "bootstrap", method of obtaining p-value when some (but not all) counts are small
#' @param bootstraps	number of samples for bootstrapping
#'
#' @returns the same table as \code{rqt} with three columns appended:
#' \code{qindex-D} containing the indicies (1,2,or 3)
#' of the quartet topology displayed on \code{tree}, \code{p_T1-D}
#' containing the p-value for the T1 test of the displayed quartet topologies.
#' \code{p_star} containing p-values for the star tree test.
#'
#' @examples
#' data(ASTRALtreeLeopardusLescroart)
#' Atree=read.tree(text=ASTRALtreeLeopardusLescroart)
#' data(tableLeopardusLescroart)
#' pTable=quartetECtestInd(tableLeopardusLescroart,Atree)
#'
#' @export
quartetECtestInd <- function (rqt,
                              tree,
                              lambda = 0,
                              method = "MLest",
                              smallsample = "precomputed",
                              smallcounts = "precomputed",
                              bootstraps = 10 ^ 4)
{
  if (is.null(tree) ||
      is.numeric(lambda) == FALSE || is.vector(lambda) == FALSE ||
      length(lambda) != 1 ||
      smallsample %in% c("bootstrap", "precomputed") == FALSE ||
      smallcounts %in% c("bootstrap", "precomputed") == FALSE ||
      is.numeric(bootstraps) ==  FALSE ||
      is.vector(bootstraps) == FALSE ||
      length(bootstraps) != 1 ||
      bootstraps <= 0 || bootstraps %% 1 != 0 ||
      method %in% c("MLest", "conservative", "bootstrap") == FALSE)
  {
    stop("Invalid argument(s)")
  }

  colnames = colnames(rqt)
  taxanames = tree$tip.label

  n = length(taxanames)
  if (!(all.equal(sort(colnames[1:n]), sort(taxanames))))
    stop("Taxa in table do not match those on tree.")

  M = dim(rqt)[1]
  tree = di2multi(unroot(tree)) #allow multifurcating tree
  nedges = dim(tree$edge)[1]
  tree$edge.length = rep(1, nedges) #to compute displayed quartets by 4pt, make all edges length 1
  D = cophenetic.phylo(tree)
  D = D[order(rownames(D)), order(colnames(D))] #order distance table alphabetically

  pTable = rqt
  if (all(c("p_T1-D", "qindex-D") %in%  colnames(pTable)))
    message("Not recomputing T1 p-values for displayed trees.")
  else
  {
    pTable = cbind(pTable, ct = 0, qi = 0)
    colnames(pTable)[which(colnames(pTable) == "ct")] = "p_T1-D"
    colnames(pTable)[which(colnames(pTable) == "qi")] = "qindex-D"

    message("Applying hypothesis tests for T1 model to ", M, " quartets.")
    for (m in 1:M) {
      #run through sets of 4 taxa; determine from 4pt condition what topology is displayed on tree
      qnames = which(rqt[m, 1:n] == 1)
      a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
      b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
      c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
      if (min(c(a, b, c)) == max(c(a, b, c))) {
        #unresolved quartet
        pTable[m, "qindex-D"] = NA
        pTable[m, "p_T1-D"] = NA
      }
      else {
        treeQuartet = which.min(c(a, b, c)) #for test, put count corresponding to tree topology first
        qcounts = rqt[m, c("12|34", "13|24", "14|23")]
        temp = qcounts[1]
        qcounts[1] = qcounts[treeQuartet]
        qcounts[treeQuartet] = temp
        pTable[m, "qindex-D"] = treeQuartet
      

      pvec = quartetTreeTest(
        unname(qcounts),
        model = "T1",
        lambda = lambda,
        method = method,
        smallsample = smallsample,
        smallcounts = smallcounts,
        bootstraps = bootstraps
      )
      pTable[m, "p_T1-D"] = pvec$p.value
      }
    }
  }
  if ("p_star" %in% colnames(pTable))
    # compute p_star if necessary
    message("Not recomputing p_star values.")
  else
    pTable = quartetStarTestInd(pTable)

  return(pTable)
}

####################################################################

#' Edge Contraction to infer a Tree of Blobs
#'
#' Infer the tree of blobs using edge contractions
#'
#' Given quartet gene tree count data from a network, and a starting tree that is 
#' a resolution of its tree of blobs, contract edges for which there is poor 
#' support due to, first, star-like quartet signals and then, successively the 
#' most extreme non-tree-like signals.
#'
#' p-values for star and T1 tests for individual quartets displayed on the tree are
#' combined by one of 4 test correction methods suitable for non-independent tests. Which quartet
#' p-values are combined depends on one of 3 modes indicated by argument \code{qType}.
#'
#' The input tree should be computed by ASTRAL, or another heuristic known to
#' produce a resolution of the tree of blobs in ideal circumstances.
#'
#' Plot levels allow for several graphical depictions of the edge contract process,
#' from the starting tree to a final star tree.
#'
#' For testing with simulated data, a reference tree (e.g., the true Tree of Blobs) 
#' may be provided and the RF distance between it and each tree will be shown.
#'
#' @references
#' \insertRef{ECTo2026}{MSCquartets}
#'
#' \insertRef{MAR19}{MSCquartets}
#'
#' \insertRef{LiuXie2020}{MSCquartets}
#'
#' \insertRef{Chen2022}{MSCquartets}
#' 
#' @param genedata one of:
#' \enumerate{
#' \item a character string giving the name of a file containing a list of newick 
#' gene trees (see argument \code{omit}),
#' \item a multiphylo object containing a collection of gene trees (see argument \code{omit}),
#' \item a table of resolved quartet counts  from a collection of
#' gene trees, or that table with additional columns of quartet p-values. If p_star
#' or both p_T1-D and qindex-D are present (such as from a previous call to this function) 
#' they are not recomputed.}
#' @param tree a tree as a phylo object, or a character string giving the name 
#' of a file with a newick tree, assumed to be a resolution of the true
#' tree of blobs
#' @param qType a choice of which quartet p-values are combined to determine one for a
#' tree edge, "bi" for all initial bipartition quartets, "quad" for all initial quadripartition
#' quartets, or "mul" for all multipartition quartets at current contracting step
#' @param testCorrection a method for combining possibly dependent p-values to 
#' get a single p-value, one of "Bon", "Cauchy", "BBC", "CBC"
#' @param alpha test level for choosing inferred tree of blobs
#' @param beta test level for collapsing edges showing no resolution
#' @param colorCutoff level for changing color of edge p-values for T1 test
#' @param plot 0 for no plot;
#'             1 to plot all trees as edges are contracted, along with plot of tree p-values;
#'             2 to add -log_10 of p-values < \code{colorCutoff} for T1 test, or > beta for star test, to edges in plots;
#'             3 to add red dots at multifurcations;
#'             4 to suppress red dots but show all edge p-values
#' @param refTree a reference tree, as a phylo object, to compute RF distance to 
#' all Tk (for simulation studies); will be unrooted   
#' @param omit \code{FALSE} to treat unresolved quartets on gene trees as 1/3 of each resolution;
#' \code{TRUE} to discard unresolved quartet data; ignored if genedata given as quartet table
#' @param lambda parameter for power-divergence statistic (e.g., 0 for likelihood
#' ratio statistic, 1 for Chi-squared statistic)
#' @param method "MLest", "conservative", or "bootstrap"
#' @param smallsample	"precomputed" or "bootstrap", method of obtaining p-value
#' when sample is small (<30)
#' @param smallcounts	"precomputed" or "bootstrap", method of obtaining p-value
#' when some (but not all) counts are small
#' @param bootstraps	number of samples for bootstrapping
#'
#' @returns An invisible list with 8 elements:
#' \describe{
#' \item{$treeList}{A list corresponding to edge contraction steps.}
#' \item{$indexEarly}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 is above \code{alpha}.}
#' \item{$indexLate}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 of it and all subsequent ones is above \code{alpha}.}
#' \item{$alpha}{The value of \code{alpha} determining the previous 2 entries.}
#' \item{$beta}{The value of \code{beta} for contracting edges using the 
#' combined star test.}
#' \item{$qType}{The quartet type mode used.}
#' \item{$testCorrection}{The p-value combination method used.}
#' \item{$pTable}{A table of quartet p-values for relevant
#' quartet tests.}
#' }
#' 
#' In \code{$treeList}, the
#' first is the input tree,
#' the second the result of contracting edges judged unresolved
#' by the combined star test,
#' and subsequent ones the result of collapsing edges with the current
#' smallest combined T1 test p-value, with the final one a star tree.
#' Each element in this list itself a
#' list of \describe{
#' \item{$tree}{A tree, with all edge lengths 1 and a node labels
#' of -log of the parent edge's combined p (star for the input tree, T1 for others).}
#' \item{$logpTree}{-log base 10 of the combined T1 p-value of the tree using \code{testCorrection}.}
#' \item{$logpT1Edge}{-log base 10 of the combined T1 p-value of the edge(s)
#'  that were contracted from previous tree (\code{NA}
#' for input tree and second tree).}
#' \item{$numEdgeCon}{The number of edges that have been contracted from the input tree}
#' \item{$RF}{The RF distance between this tree and \code{refTree}, if \code{refTree} is not NULL.}
#' }
#'
#' @examples
#' data(ASTRALtreeLeopardusLescroart)
#' Atree=read.tree(text=ASTRALtreeLeopardusLescroart)
#' data(tableLeopardusLescroart)
#' ECToBlob(tableLeopardusLescroart,Atree,"bi","Bon")
#'
#'
#' @export
#'
ECToBlob = function(genedata,
                    tree,
                    qType = "mul",
                    testCorrection = "Bon",
                    alpha = NA,
                    beta = .8,
                    colorCutoff = .05,
                    plot = 3,
                    refTree = NULL,
                    omit=FALSE,
                    lambda = 0,
                    method = "MLest",
                    smallsample = "precomputed",
                    smallcounts = "precomputed",
                    bootstraps = 10 ^ 4
                    )
{
  if (qType == "bi") {
    out = ECToBlob_bi(
      genedata = genedata,
      tree = tree,
      testCorrection = testCorrection,
      alpha = alpha,
      beta = beta,
      colorCutoff = colorCutoff,
      plot = plot,
      refTree = refTree,
      omit=omit,
      lambda = lambda,
      method = method,
      smallsample = smallsample,
      smallcounts = smallcounts,
      bootstraps = bootstraps
    )
    return(out)
  }
  else {
    if (qType == "quad") {
      out = ECToBlob_quad(
        genedata = genedata,
        tree = tree,
        testCorrection = testCorrection,
        alpha = alpha,
        beta = beta,
        colorCutoff = colorCutoff,
        plot = plot,
        refTree = refTree,
        omit=omit,
        lambda = lambda,
        method = method,
        smallsample = smallsample,
        smallcounts = smallcounts,
        bootstraps = bootstraps
      )
      return(out)
    }
    else {
      if (qType == "mul") {
        out = ECToBlob_mul(
          genedata = genedata,
          tree = tree,
          testCorrection = testCorrection,
          alpha = alpha,
          beta = beta,
          colorCutoff = colorCutoff,
          plot = plot,
          refTree = refTree,
          omit=omit,
          lambda = lambda,
          method = method,
          smallsample = smallsample,
          smallcounts = smallcounts,
          bootstraps = bootstraps
        )
        return(out)
      }
      else
        stop("Invalid argument: qType")
    }
  }
}

####################################################################
#' Edge Contraction to infer a Tree of Blobs - bipartition mode
#'
#' Infer the tree of blobs using edge contraction and bipartitions
#'
#' Given quartet gene tree count data from a network and a starting tree that is
#' a resolution of its tree of blobs, contract edges for which there is poor support
#' due to, first, star-like quartet signals and then successively the most extreme
#' non-tree-like signals.
#'
#' p-values for star and T1 tests for individual quartets displayed on the tree are
#' combined by one of 4 test correction methods suitable for non-independent tests. For star test,
#' only p-values for quartets defining each edge are combined. For T1 test,
#' p-values for quartets defining paths containing the edge (i.e., from the edge bipartitions)
#' are combined.
#'
#' The input tree should be computed by ASTRAL, or another heuristic known to
#' produce a resolution of the tree of blobs in ideal circumstances.
#'
#' Plot levels allow for several graphical depictions of the edge contract process,
#' from the starting tree to a final star tree.
#'
#' For testing with simulated data, a reference tree (e.g., the true Tree of Blobs) 
#' may be provided and the RF distance between it and each tree will be shown.
#'
#' @references
#' \insertRef{ECTo2026}{MSCquartets}
#'
#' \insertRef{MAR19}{MSCquartets}
#'
#' \insertRef{LiuXie2020}{MSCquartets}
#'
#' \insertRef{Chen2022}{MSCquartets}
#' 
#' @param genedata one of:
#' \enumerate{
#' \item a character string giving the name of a file containing a list of newick 
#' gene trees (see argument \code{omit}),
#' \item a multiphylo object containing a collection of gene trees (see argument \code{omit}),
#' \item a table of resolved quartet counts  from a collection of
#' gene trees, or that table with additional columns of quartet p-values. If p_star
#' or both p_T1-D and qindex-D are present (such as from a previous call to this function) 
#' they are not recomputed.}
#' @param tree a tree as a phylo object, or a character string giving the name of 
#' a file with a newick tree, assumed to be a resolution of the true
#' tree of blobs
#' @param testCorrection a method for combining possibly dependent p-values to 
#' get a single p-value, one of "Cauchy", "Bon", "BBC", "CBC"
#' @param alpha test level for choosing inferred tree of blobs
#' @param beta test level for collapsing edges showing no resolution
#' @param colorCutoff level for changing color of edge p-values for T1 test
#' @param plot 0 for no plot;
#'             1 to plot all trees as edges are contracted, along with plot of tree p-values;
#'             2 to add -log_10 of p-values < \code{colorCutoff} for T1 test, or > beta for star test, to edges in plots;
#'             3 to add red dots at multifurcations;
#'             4 to suppress red dots but show all edge p-values.
#' @param refTree a reference tree, as a phylo object, to compute RF distance to 
#' all Tk (for simulation studies); will be unrooted
#' @param omit \code{FALSE} to treat unresolved quartets on gene trees as 1/3 of each resolution;
#' \code{TRUE} to discard unresolved quartet data; ignored if genedata given as quartet table
#' @param method "MLest", "conservative", or "bootstrap"
#' @param lambda parameter for power-divergence statistic (e.g., 0 for likelihood
#' ratio statistic, 1 for Chi-squared statistic)
#'
#' @param smallsample	"precomputed" or "bootstrap", method of obtaining p-value
#' when sample is small (<30)
#' @param smallcounts	"precomputed" or "bootstrap", method of obtaining p-value
#' when some (but not all) counts are small
#' @param bootstraps	number of samples for bootstrapping
#'
#' @returns An invisible list with 8 elements:
#' \describe{
#' \item{$treeList}{A list corresponding to edge contraction steps.}
#' \item{$indexEarly}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 is above \code{alpha}.}
#' \item{$indexLate}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 of it and all subsequent ones is above \code{alpha}.}
#' \item{$alpha}{The value of \code{alpha} determining the previous 2 entries.}
#' \item{$beta}{The value of \code{beta} for contracting edges using the 
#' combined star test.}
#' \item{$qType}{The quartet type mode used.}
#' \item{$testCorrection}{The p-value combination method used.}
#' \item{$pTable}{A table of quartet p-values for relevant
#' quartet tests.}
#' }
#' 
#' In \code{$treeList}, the
#' first is the input tree,
#' the second the result of contracting edges judged unresolved
#' by the combined star test,
#' and subsequent ones the result of collapsing edges with the current
#' smallest combined T1 test p-value, with the final one a star tree.
#' Each element in this list itself a
#' list of \describe{
#' \item{$tree}{A tree, with all edge lengths 1 and a node labels
#' of -log of the parent edge's combined p (star for the input tree, T1 for others).}
#' \item{$logpTree}{-log base 10 of the combined T1 p-value of the tree using \code{testCorrection}.}
#' \item{$logpT1Edge}{-log base 10 of the combined T1 p-value of the edge(s)
#'  that were contracted from previous tree (\code{NA}
#' for input tree and second tree).}
#' \item{$numEdgeCon}{The number of edges that have been contracted from the input tree}
#' \item{$RF}{The RF distance between this tree and \code{refTree}, if \code{refTree} is not NULL.}
#' }
#'
#' @examples
#' data(ASTRALtreeLeopardusLescroart)
#' Atree=read.tree(text=ASTRALtreeLeopardusLescroart)
#' data(tableLeopardusLescroart)
#' ECToBlob_bi(tableLeopardusLescroart,Atree,"Bon")
#'
#' @export
#'
ECToBlob_bi = function(genedata,
                       tree,
                       testCorrection = "Bon",
                       alpha = NA,
                       beta = .8,
                       colorCutoff = .05,
                       plot = 3,
                       refTree = NULL,
                       omit=FALSE,
                       lambda = 0,
                       method = "MLest",
                       smallsample = "precomputed",
                       smallcounts = "precomputed",
                       bootstraps = 10 ^ 4
                       )
{
  if (!(testCorrection %in% c("Cauchy", "Bon", "BBC", "CBC")))
    stop("Invalid argument: testCorrection")
  if (!(plot %in% c(0,1,2,3,4)))
    stop("Invalid argument: plot")
  
  #get data on quartets
  if (!(any(c("matrix","character","multiPhylo") %in% class(genedata)))) 
            stop("Data must be supplied as an object of type multiPhylo, character, or matrix.")
  if (!("matrix" %in% class(genedata))) {
  if ("character" %in% class(genedata)) {
      genetrees <- read.tree(file=genedata) #read gene trees
      message(paste("Read", length(genetrees), "gene trees from file."))
  } else { genetrees = genedata} 
    taxanames = genetrees[[1]]$tip.label   # ... get taxa names from first tree
    taxanames = sort(taxanames)
    if (length(taxanames) <= 25) {
      namelist = paste0(taxanames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxanames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxanames), " taxa: ", namelist)
    genedata = quartetTable(genetrees)   # tally quartets on gene trees
    genedata = quartetTableResolved(genedata, omit)   # treat unresolved quartets
  }

  # prepare tree
  if ("character" %in% class(tree)) { tree <- read.tree(file=tree)}
  tree = unroot(tree)
  tree = di2multi(tree) #makes sure tree is unrooted and no zero length branches
  ntaxa = length(tree$tip.label) #number of taxa
  tree$edge.length = rep(1, dim(tree$edge)[1]) #make tree metric, with all edges length 1
  internalEdges = which(tree$edge[, 2] > length(tree$tip.label)) #numbers of internal edges
  if (length(internalEdges) == 0)
    stop("Tree has no internal edges.")
  treeTaxa=sort(tree$tip.label)
  dataTaxa=sort(colnames(genedata)[1:ntaxa])
  
  if (!all.equal(treeTaxa,dataTaxa))
    stop("Taxa on tree do not match those in genedata")
  numT0edges = dim(tree$edge)[[1]]

  out = c()# create empty output
  out$qType = "bi"
  out$testCorrection = testCorrection
  
  # apply T1-D and star test for all sets of 4 taxa
  pTable = quartetECtestInd(
    genedata,
    tree,
    lambda = lambda,
    method = method,
    smallsample = smallsample,
    smallcounts = smallcounts,
    bootstraps = bootstraps
  )
  out$pTable = pTable

  D = cophenetic(tree) #wipe out p_T1-D for quartets not resolved
  D = D[order(rownames(D)), order(colnames(D))]
  for (m in 1:dim(pTable)[1]) {
    qnames = which(pTable[m, 1:ntaxa] == 1)
    a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
    b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
    c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
    if (min(c(a, b, c)) == max(c(a, b, c)))
      pTable[m, "p_T1-D"] = NA
  }

  #prepare for indexing table entries
  {
    taxanames = colnames(pTable)[1:ntaxa] #names of taxa, as ordered in table
    np1 = ntaxa + 1 #number of taxa + 1
    #store binomial coefficients for efficiency in computing indices in table to access sets of 4 taxa
    C = matrix(nrow = ntaxa, ncol = 4) #store binomial coefficients for efficiency in computing indices in table to access sets of 4 taxa
    for (i in 0:(ntaxa - 1)) {
      for (j in 1:4) {
        C[i + 1, j] = choose(i, j)
      }
    }
    Cn4 = choose(ntaxa, 4) # need one extra value
  }

  #Compute combined p-values for star test for all edges
  {
    edgep = matrix(0, length(internalEdges), 2) #create array for combined p-values
    colnames(edgep) = c("edge", "pcomb")

    for (j in 1:length(internalEdges)) {
      #Combine p_star for all quartets defining this edge on original tree.
      pNode = tree$edge[internalEdges[j], 1] #find taxon groups around parent and child node of edge
      cNode = tree$edge[internalEdges[j], 2]
      pGroups = nodeGroups(tree, pNode)
      cGroups = nodeGroups(tree, cNode)

      #remove group above child (depends on nodeGroups returning non-descendants last in list)
      cGroups = cGroups[-length(cGroups)]
      cUnion = unlist(cGroups)

      for (i in 1:length(pGroups)) {
        #remove group below parent
        if (all(cUnion %in% pGroups[[i]]))
          remove = i
      }
      pGroups = pGroups[-remove]

      # compute combined p values for star
      pvecstar = c() #storage for p_star values

      #run through all choices of 2 groups at child and 2 at parent
      for (group1 in 2:length(pGroups)) {
        for (group2 in 1:(group1 - 1)) {
          for (group3 in 2:length(cGroups)) {
            for (group4 in 1:(group3 - 1)) {
              #run through all choices of 1 taxon per group
              for (at in pGroups[[group1]]) {
                for (bt in pGroups[[group2]]) {
                  for (ct in cGroups[[group3]]) {
                    for (dt in cGroups[[group4]]) {
                      #find a,b,c,d entry in table and get p_=star
                      positions = sort(match(tree$tip.label[c(at, bt, ct, dt)], taxanames)) # find table columns for these taxa
                      index = Cn4 - C[np1 - positions[1], 4] - C[np1 - positions[2], 3] - C[np1 - positions[3], 2] - C[np1 - positions[4], 1] #row index for this quartet
                      pstar = unname(pTable[index, "p_star"])
                      pvecstar = c(pvecstar, pstar)
                    }
                  }
                }
              }
            }
          }
        }
      }
      edgep[j, ] = c(internalEdges[j], combineP(pvecstar, testCorrection))
    }
    edgep = rbind(edgep) #make sure result is a matrix if only 1 row
  }

  #Put edge star p-value -logs in as node labels on T_0
  tree$node.label = rep("", tree$Nnode)
  tree$node.label[tree$edge[edgep[, "edge"], 2] - ntaxa] = -log(edgep[, "pcomb"], 10)

  #Put initial tree T_0 in list, with special treatment if reference Tree supplied
  pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute combined values for tree T_0
  pTreeList = c(pTree) # save in list
  indexTreeList = c(0)
  if (is.null(refTree))
    trees = list(list(
      "tree" = ape::write.tree(tree),
      "logpTree" = -log(pTree, 10),
      "logpT1Edge" = NA,
      "numEdgeCon" = 0
    ))
  else{
    refTree=unroot(refTree)
    RF = as.numeric(ape::dist.topo(refTree, tree))
    trees = list(
      list(
        "tree" = ape::write.tree(tree),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = 0,
        "RF" = RF
      )
    )# put it in list
    RFlist = c(RF)# and save RF for plotting
  }

  if (plot > 0) {
    #Plot initial tree
    line1 = ""
    if (!is.null(refTree))
      line1 = paste0("RF to reference Tree = ", RF, "; \n")

    plot(
      tree,
      type = "unrooted",
      cex.sub = .7,
      main = paste0("T0 (Input tree): ", testCorrection, ", bi"),
      sub = paste0(
        line1,
        "-log_10(p) for combined T1 over tree = ",
        formatC(-log(pTree, 10), format = 'e', digits = 3),
        "; \n Edge labels are -log_10(p) for combined star tests for that edge."
      )
    )
    if (plot %in% c(2, 3)) {
      badedgeindex = which(edgep[, "pcomb"] > beta)
      if (length(badedgeindex) > 0) {
        badedges = edgep[badedgeindex, "edge"]
        bgColor = rep("tomato1", length(badedges))
        ape::edgelabels(
          formatC(
            -log(edgep[badedgeindex, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = badedges,
          bg = bgColor,
          cex = .5
        )
      }
    }
    if (plot == 3) {
      polyList = c() #find multifurcations, starting at "top" in unrooted tree
      if (length(Descendants(tree, ntaxa + 1, 'children')) > 3)
        polyList = c(polyList, ntaxa + 1)
      if (tree$Nnode >= 2) {
        #loop over other internal nodes
        for (i in (ntaxa + 2):(ntaxa + tree$Nnode)) {
          if (length(Descendants(tree, i, 'children')) > 2)
            polyList = c(polyList, i)
        }
      }
      #add colored dots for these nodes
      lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
      internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
      XX <- lastPP$xx[polyList]
      YY <- lastPP$yy[polyList]
      points(XX,
             YY,
             pch = 16,
             col = "red",
             cex = 1)
    }
    if (plot == 4) {
      edgeind = which(edgep[, "pcomb"] != Inf)
      if (length(edgeind) > 0) {
        bgColor = rep("lightblue", dim(edgep)[[1]])
        bgColor[which(edgep[edgeind, "pcomb"] > beta)] = "tomato1"
        ape::edgelabels(
          formatC(
            -log(edgep[edgeind, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = edgep[edgeind, "edge"],
          bg = bgColor,
          cex = .5
        )
      }
    }
  }

  #Contract edges based on star test
  message('Contracting edges with star test at level beta = ',beta)
  edgepRow = which(edgep[, "pcomb"] > beta) #determine entries for edges to contract for star test
  edges2contract = edgep[edgepRow, "edge"] # and the edge numbers
  num2contract = length(edges2contract)
  tree$edge.length[edges2contract] = 0 #contract those edges
  
 
  D = cophenetic(tree) #wipe out p_T1-D for quartets no longer resolved
  D = D[order(rownames(D)), order(colnames(D))]
  for (m in 1:dim(pTable)[1]) {
    qnames = which(pTable[m, 1:ntaxa] == 1)
    a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
    b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
    c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
    if (min(c(a, b, c)) == max(c(a, b, c)))
      pTable[m, "p_T1-D"] = NA
  }
  
  # Computation of combined edge p for T1 test, once and for all, using T0
  internalEdges = which(tree$edge[, 2] > ntaxa) #numbers of internal edges
  edgep = matrix(0, length(internalEdges), 2) #create array for combined p-values
  colnames(edgep) = c("edge", "pcomb")
  for (ie in 1:length(internalEdges)) {
    if (ie %in% edges2contract) pvecT1=c(1) #ignore edges just contracted
    else { 
    pvecT1=c()
    cNode = tree$edge[internalEdges[ie], 2] # child node of edge
    cUnion = Descendants(tree, cNode)[[1]] # descendant taxa
    pUnion = setdiff(1:ntaxa, cUnion) # and other taxa
    for (i in 2:length(pUnion)) {
      for (j in 1:(i - 1)) {
        for (k in 2:(length(cUnion))) {
          for (l in 1:(k - 1)) {
            at = pUnion[[i]]
            bt = pUnion[[j]]
            ct = cUnion[[k]]
            dt = cUnion[[l]]
            positions = sort(match(tree$tip.label[c(at, bt, ct, dt)], taxanames)) # find table columns for these taxa
            index = Cn4 - C[np1 - positions[1], 4] - C[np1 - positions[2], 3] - C[np1 - positions[3], 2] - C[np1 - positions[4], 1] #row index for this quartet
            pvecT1 = c(pvecT1, unname(pTable[index, "p_T1-D"]))
          }
        }
      }
    }
    }
    edgep[ie, ] = c(internalEdges[ie], combineP(pvecT1, testCorrection))
  }
  edgep = rbind(edgep) #make sure result is a matrix if only 1 row

  #Put edge T1 p-value -logs in as node labels on Tk
  tree$node.label = rep("", tree$Nnode)
  tree$node.label[tree$edge[edgep[, "edge"], 2] - ntaxa] = -log(edgep[, "pcomb"], 10)
  treem = di2multi(tree) # tree for putting in output, with multifurcations
  numEdges = dim(treem$edge)[[1]]

  #Put star-contracted tree Tk in list, with special treatment if reference Tree supplied
  pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute combined values for tree T_0
  pTreeList = c(pTreeList, pTree) # save in list
  indexTreeList = c(indexTreeList, num2contract)
  if (is.null(refTree))
    trees = append(trees, list(
      list(
        "tree" = ape::write.tree(treem),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = num2contract
      )
    ))
  else{
    RF = as.numeric(ape::dist.topo(refTree, treem))
    trees = append(trees, list(
      list(
        "tree" = ape::write.tree(treem),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = num2contract,
        "RF" = RF
      )
    ))# put it in list
    RFlist = c(RFlist,RF)# and save RF for plotting
  }

  if (plot > 0) {
    #Plot this tree
    line1 = ""
    if (!is.null(refTree))
      line1 = paste0("RF to reference Tree = ", RF, "; \n")

    plot(
      tree,
      type = "unrooted",
      cex.sub = .7,
      main = paste0("T", num2contract, ": ", testCorrection, ", bi"),
      sub = paste0(
        line1,
        "-log_10(p) for combined T1 over tree = ",
        formatC(-log(pTree, 10), format = 'e', digits = 3),
        "; \n Edge labels are -log_10(p) for combined T1 test for that edge;\n",
        num2contract,
        " edges contracted by combined star test."
      )
    )
    if (plot %in% c(2, 3)) {
      badedgeindex = which(edgep[, "pcomb"] < colorCutoff)
      if (length(badedgeindex) > 0) {
        badedges = edgep[badedgeindex, "edge"]
        bgColor = rep("tomato1", length(badedges))
        ape::edgelabels(
          formatC(
            -log(edgep[badedgeindex, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = badedges,
          bg = bgColor,
          cex = .5
        )
      }
    }
    if (plot == 3) {
      polyList = c() #find multifurcations, starting at "top" in unrooted tree
      if (length(Descendants(treem, ntaxa + 1, 'children')) > 3)
        polyList = c(polyList, ntaxa + 1)
      if (tree$Nnode >= 2) {
        #loop over other internal nodes
        for (i in (ntaxa + 2):(ntaxa + tree$Nnode)) {
          if (length(Descendants(treem, i, 'children')) > 2)
            polyList = c(polyList, i)
        }
      }
      #add colored dots for these nodes
      lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
      internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
      XX <- lastPP$xx[polyList]
      YY <- lastPP$yy[polyList]
      points(XX,
             YY,
             pch = 16,
             col = "red",
             cex = 1)
    }
    if (plot == 4) {
      edgeind = which(edgep[, "pcomb"] != Inf)
      if (length(edgeind) > 0) {
        bgColor = rep("lightblue", dim(edgep)[[1]])
        bgColor[which(edgep[edgeind, "pcomb"] < colorCutoff)] = "tomato1"
        ape::edgelabels(
          formatC(
            -log(edgep[edgeind, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = edgep[edgeind, "edge"],
          bg = bgColor,
          cex = .5
        )
      }
    }
  }

   while (numEdges > ntaxa)
    #loop with successive contracting steps
  {
    minp = min(edgep[, "pcomb"],na.rm = TRUE) # p for edges to contract
    contract = which(edgep[, "pcomb"] == minp) # find all minimal edge p
    tree$edge.length[edgep[contract, 1]] = 0 # and set their lengths to 0
    edgep[contract, 2] = Inf # and never consider them again
    treem = di2multi(tree) #note this retains correct node label for edge p
    numEdges = dim(treem$edge)[[1]]
    
    D = cophenetic(treem) #wipe out p_T1-D for quartets no longer resolved
    D = D[order(rownames(D)), order(colnames(D))]
    for (m in 1:dim(pTable)[1]) {
      qnames = which(pTable[m, 1:ntaxa] == 1)
      a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
      b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
      c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
      if (min(c(a, b, c)) == max(c(a, b, c)))
        pTable[m, "p_T1-D"] = NA
    }

    countContracted = numT0edges - numEdges #total number of edges contracted
    pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute new combined values for tree
    pTreeList = c(pTreeList, pTree) # save new tree p
    indexTreeList = c(indexTreeList, countContracted)


    if (is.null(refTree))
      trees = append(trees, list(
        list(
          "tree" = ape::write.tree(treem),
          "logpTree" = -log(pTree, 10),
          "logpT1Edge" = -log(minp, 10),
          "numEdgeCon" = countContracted
        )
      ))

    else{
      RF = as.numeric(ape::dist.topo(refTree, treem))
      trees = append(trees, list(
        list(
          "tree" = ape::write.tree(treem),
          "logpTree" = -log(pTree, 10),
          "logpT1Edge" = -log(minp, 10),
          "numEdgeCon" = countContracted,
          "RF" = RF
        )
      ))
      RFlist = c(RFlist, RF)# and save RF for plotting
    }

    if (plot > 0) {
      line1 = ""
      if (!is.null(refTree))
        line1 = paste0("RF to reference Tree = ", RF, "; \n")
      plot(
        tree,
        type = "unrooted",
        cex.sub = .7,
        main = paste0("T", countContracted, ": ", testCorrection, ", bi"),
        sub = paste0(
          line1,
          "-log_10(p) for combined T1 over tree = ",
          formatC(-log(pTree, 10), format = 'e', digits = 3),
          "; \n Edge labels are -log_10(p) from combined edge T1;\n",
          length(contract),
          " more edge(s) contracted by combined T1 test."
        )
      )

      if (plot > 1)
      {
        if (plot %in% c(2, 3)) {
          badedgeindex = which(edgep[, "pcomb"] < colorCutoff)
          if (length(badedgeindex > 0))
          {
            badedges = edgep[badedgeindex, "edge"]
            bgColor = rep("tomato1", length(badedges))
            ape::edgelabels(
              formatC(
                -log(edgep[badedgeindex, "pcomb"], 10),
                format = 'e',
                digits = 0
              ),
              edge = badedges,
              bg = bgColor,
              cex = .5
            )
          }
        }
        if (plot == 3)
        {
          contractedEdges = which(tree$edge.length == 0)
          polyList = tree$edge[contractedEdges, 1]
          #colored dots for these nodes
          lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
          internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
          XX <- lastPP$xx[polyList]
          YY <- lastPP$yy[polyList]
          points(XX,
                 YY,
                 pch = 16,
                 col = "red",
                 cex = 1)
        }
        if (plot == 4) {
          edgeind = which(edgep[, "pcomb"] != Inf)
          if (length(edgeind) > 0) {
            bgColor = rep("lightblue", dim(edgep)[[1]])
            bgColor[which(edgep[edgeind, "pcomb"] < colorCutoff)] = "tomato1"
            ape::edgelabels(
              formatC(
                -log(edgep[edgeind, "pcomb"], 10),
                format = 'e',
                digits = 0
              ),
              edge = edgep[edgeind, "edge"],
              bg = bgColor,
              cex = .5
            )
          }
        }
      }
    }
    }
  
  out$treeList = trees

  if (!is.na(alpha)) {
    small = which(pTreeList < alpha)
    if (length(small) == 0)
      small = 0
    out$indexLate = max(small) + 1
    out$indexEarly = which(pTreeList > alpha)[1]
  } else {
    out$indexLate = NA
    out$indexEarly = NA
  }

  #final plot, of p-values for tree, and RF
  if (plot > 0) {
    y = -log(pTreeList, 10)
    yfinpos = which(is.finite(y))
    yinfpos = setdiff(1:length(y), yfinpos)
    yinf = max(y[yfinpos]) * 1.5
    plot(
      indexTreeList[yinfpos],
      rep(yinf, length(yinfpos)),
      ylim = c(0, yinf),
      xlim = c(-.5, max(indexTreeList) + .5),
      main = paste0("Tree p-values; ", testCorrection, ", bi"),
      xlab = "Number of contracted edges",
      ylab = "-log_10(p) (solid red = Inf)",
      col = "red",
      pch = 19
    )
    points(indexTreeList[yfinpos], y[yfinpos], col = "blue")


    if (!is.null(refTree))
      plot(
        indexTreeList,
        RFlist,
        main = paste0("RFdist to supplied ToB; ", testCorrection, ", bi"),
        xlab = "Tree",
        ylab = "RFdist",
        col = "blue"
      )
  }

  out$alpha=alpha
  out$beta=beta
  
  out = out[c("treeList",
              "indexEarly",
              "indexLate",
              "alpha",
              "beta",
              "qType",
              "testCorrection",
              "pTable")]

  invisible(out)
}

####################################################################
#' Edge Contraction to infer a Tree of Blobs - quadripartition mode
#'
#' Infer the tree of blobs using edge contraction and quadripartitions
#'
#' Given quartet gene tree count data from a network and a starting tree that is
#' a resolution of its tree of blobs, contract edges for which there is poor support
#' due to, first, star-like quartet signals and then successively the most extreme
#' non-tree-like signals.
#'
#' p-values for star and T1 tests for individual quartets displayed on the tree are
#' combined by one of 4 test correction methods suitable for non-independent tests. For both
#' star and T1 test, only p-values for quartets defining each edge are combined.
#' (For T1, this means defining the edge after any star edge contractions have been done.)
#'
#' The input tree should be computed by ASTRAL, or another heuristic known to
#' produce a resolution of the tree of blobs in ideal circumstances.
#'
#' Plot levels allow for several graphical depictions of the edge contract process,
#' from the starting tree to a final star tree.
#'
#' For testing with simulated data, a reference tree (e.g., the true Tree of Blobs) 
#' may be provided and the RF distance between it and each tree will be shown.
#'
#' @references
#' \insertRef{ECTo2026}{MSCquartets}
#'
#' \insertRef{MAR19}{MSCquartets}
#'
#' \insertRef{LiuXie2020}{MSCquartets}
#'
#' \insertRef{Chen2022}{MSCquartets}
#' 
#' @param genedata one of:
#' \enumerate{
#' \item a character string giving the name of a file containing a list of newick 
#' gene trees (see argument \code{omit}),
#' \item a multiphylo object containing a collection of gene trees (see argument \code{omit}),
#' \item a table of resolved quartet counts  from a collection of
#' gene trees, or that table with additional columns of quartet p-values. If p_star
#' or both p_T1-D and qindex-D are present (such as from a previous call to this function) 
#' they are not recomputed.}
#' @param tree a tree as a phylo object, or a character string giving the name of 
#' a file with a newick tree, assumed to be a resolution of the true
#' tree of blobs
#' @param testCorrection a method for combining possibly dependent p-values to 
#' get a single p-value, one of "Cauchy", "Bon", "BBC", "CBC"
#' @param alpha test level for choosing inferred tree of blobs
#' @param beta test level for collapsing edges showing no resolution
#' @param colorCutoff level for changing color of edge p-values for T1 test
#' @param plot 0 for no plot;
#'             1 to plot all trees as edges are contracted, along with plot of tree p-values;
#'             2 to add -log_10 of p-values < \code{colorCutoff} for T1 test, or > beta for star test, to edges in plots;
#'             3 to add red dots at multifurcations;
#'             4 to suppress red dots but show all edge p-values
#' @param refTree a reference tree, as a phylo object, to compute RF distance to 
#' all Tk (for simulation studies); will be unrooted
#' @param omit \code{FALSE} to treat unresolved quartets on gene trees as 1/3 of each resolution;
#' \code{TRUE} to discard unresolved quartet data; ignored if genedata given as quartet table
#' @param lambda parameter for power-divergence statistic (e.g., 0 for likelihood
#' ratio statistic, 1 for Chi-squared statistic)
#' @param method "MLest", "conservative", or "bootstrap"
#' @param smallsample	"precomputed" or "bootstrap", method of obtaining p-value
#' when sample is small (<30)
#' @param smallcounts	"precomputed" or "bootstrap", method of obtaining p-value
#' when some (but not all) counts are small
#' @param bootstraps	number of samples for bootstrapping
#'
#' @returns An invisible list with 8 elements:
#' \describe{
#' \item{$treeList}{A list corresponding to edge contraction steps.}
#' \item{$indexEarly}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 is above \code{alpha}.}
#' \item{$indexLate}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 of it and all subsequent ones is above \code{alpha}.}
#' \item{$alpha}{The value of \code{alpha} determining the previous 2 entries.}
#' \item{$beta}{The value of \code{beta} for contracting edges using the 
#' combined star test.}
#' \item{$qType}{The quartet type mode used.}
#' \item{$testCorrection}{The p-value combination method used.}
#' \item{$pTable}{A table of quartet p-values for relevant
#' quartet tests.}
#' }
#' 
#' In \code{$treeList}, the
#' first is the input tree,
#' the second the result of contracting edges judged unresolved
#' by the combined star test,
#' and subsequent ones the result of collapsing edges with the current
#' smallest combined T1 test p-value, with the final one a star tree.
#' Each element in this list itself a
#' list of \describe{
#' \item{$tree}{A tree, with all edge lengths 1 and a node labels
#' of -log of the parent edge's combined p (star for the input tree, T1 for others).}
#' \item{$logpTree}{-log base 10 of the combined T1 p-value of the tree using \code{testCorrection}.}
#' \item{$logpT1Edge}{-log base 10 of the combined T1 p-value of the edge(s)
#'  that were contracted from previous tree (\code{NA}
#' for input tree and second tree).}
#' \item{$numEdgeCon}{The number of edges that have been contracted from the input tree}
#' \item{$RF}{The RF distance between this tree and \code{refTree}, if \code{refTree} is not NULL.}
#' }
#'
#' @examples
#' data(ASTRALtreeLeopardusLescroart)
#' Atree=read.tree(text=ASTRALtreeLeopardusLescroart)
#' data(tableLeopardusLescroart)
#' ECToBlob_quad(tableLeopardusLescroart,Atree,"Bon")
#'
#' @export
#'
ECToBlob_quad = function(genedata,
                         tree,
                         testCorrection = "Bon",
                         alpha = NA,
                         beta = .8,
                         colorCutoff = .05,
                         plot = 3,
                         refTree = NULL, 
                         omit=FALSE,
                         lambda = 0,
                         method = "MLest",
                         smallsample = "precomputed",
                         smallcounts = "precomputed",
                         bootstraps = 10 ^ 4
                         )
{
  if (!(testCorrection %in% c("Cauchy", "Bon", "BBC", "CBC")))
    stop("Invalid argument: testCorrection")
  if (!(plot %in% c(0,1,2,3,4)))
    stop("Invalid argument: plot")
  
  #get data on quartets
  if (!(any(c("matrix","character","multiPhylo") %in% class(genedata)))) 
    stop("Data must be supplied as an object of type multiPhylo, character, or matrix.")
  if (!("matrix" %in% class(genedata))) {
    if ("character" %in% class(genedata)) {
      genetrees <- read.tree(file=genedata) #read gene trees
      message(paste("Read", length(genetrees), "gene trees from file."))
    } else { genetrees = genedata} 
    taxanames = genetrees[[1]]$tip.label   # ... get taxa names from first tree
    taxanames = sort(taxanames)
    if (length(taxanames) <= 25) {
      namelist = paste0(taxanames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxanames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxanames), " taxa: ", namelist)
    genedata = quartetTable(genetrees)   # tally quartets on gene trees
    genedata = quartetTableResolved(genedata, omit)   # treat unresolved quartets
  }
  
  # prepare tree
  if ("character" %in% class(tree)) { tree <- read.tree(file=tree) } #read input tree 
  tree = unroot(tree)
  tree = di2multi(tree) #makes sure tree is unrooted and no zero length branches
  ntaxa = length(tree$tip.label) #number of taxa
  tree$edge.length = rep(1, dim(tree$edge)[1]) #make tree metric, with all edges length 1
  internalEdges = which(tree$edge[, 2] > length(tree$tip.label)) #numbers of internal edges
  if (length(internalEdges) == 0)
    stop("Tree has no internal edges.")
  treeTaxa=sort(tree$tip.label)
  dataTaxa=sort(colnames(genedata)[1:ntaxa])
  if (!all.equal(treeTaxa,dataTaxa))
    stop("Taxa on tree do not match those in genedata")
  numT0edges = dim(tree$edge)[[1]]
  
  out = c()# create empty output
  out$qType = "quad"
  out$testCorrection = testCorrection

  # apply T1-D and star test for all sets of 4 taxa
  pTable = quartetECtestInd(
    genedata,
    tree,
    lambda = lambda,
    method = method,
    smallsample = smallsample,
    smallcounts = smallcounts,
    bootstraps = bootstraps
  )
  out$pTable = pTable

  D = cophenetic(tree) #wipe out p_T1-D for any quartets that are not resolved
  D = D[order(rownames(D)), order(colnames(D))]
  for (m in 1:dim(pTable)[1]) {
    qnames = which(pTable[m, 1:ntaxa] == 1)
    a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
    b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
    c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
    if (min(c(a, b, c)) == max(c(a, b, c)))
      pTable[m, "p_T1-D"] = NA
  }

  #prepare for indexing table entries
  {
    taxanames = colnames(pTable)[1:ntaxa] #names of taxa, as ordered in table
    np1 = ntaxa + 1 #number of taxa + 1
    #store binomial coefficients for efficiency in computing indices in table to access sets of 4 taxa
    C = matrix(nrow = ntaxa, ncol = 4) #store binomial coefficients for efficiency in computing indices in table to access sets of 4 taxa
    for (i in 0:(ntaxa - 1)) {
      for (j in 1:4) {
        C[i + 1, j] = choose(i, j)
      }
    }
    Cn4 = choose(ntaxa, 4) # need one extra value
  }

  #Compute combined p-values for star and T1 test for all edges
  {
    edgep = matrix(0, length(internalEdges), 3) #create array for combined p-values
    colnames(edgep) = c("edge", "pcombstar", "pcomb")

    for (j in 1:length(internalEdges)) {
      #Combine p_star for all quartets defining this edge on original tree.
      pNode = tree$edge[internalEdges[j], 1] #find taxon groups around parent and child node of edge
      cNode = tree$edge[internalEdges[j], 2]
      pGroups = nodeGroups(tree, pNode)
      cGroups = nodeGroups(tree, cNode)

      #remove group above child (depends on nodeGroups returning non-descendants last in list)
      cGroups = cGroups[-length(cGroups)]
      cUnion = unlist(cGroups)

      for (i in 1:length(pGroups)) {
        #remove group below parent
        if (all(cUnion %in% pGroups[[i]]))
          remove = i
      }
      pGroups = pGroups[-remove]

      # compute combined p values for star and T1
      pvecstar = c() #storage for p_star values
      pvecT1 = c() #storage for p_T1-D values

      #run through all choices of 2 groups at child and 2 at parent
      for (group1 in 2:length(pGroups)) {
        for (group2 in 1:(group1 - 1)) {
          for (group3 in 2:length(cGroups)) {
            for (group4 in 1:(group3 - 1)) {
              #run through all choices of 1 taxon per group
              for (at in pGroups[[group1]]) {
                for (bt in pGroups[[group2]]) {
                  for (ct in cGroups[[group3]]) {
                    for (dt in cGroups[[group4]]) {
                      #find a,b,c,d entry in table and get p_=star
                      positions = sort(match(tree$tip.label[c(at, bt, ct, dt)], taxanames)) # find table columns for these taxa
                      index = Cn4 - C[np1 - positions[1], 4] - C[np1 - positions[2], 3] - C[np1 - positions[3], 2] - C[np1 - positions[4], 1] #row index for this quartet
                      pstar = unname(pTable[index, "p_star"])
                      pvecstar = c(pvecstar, pstar)
                      pT1 = unname(pTable[index, "p_T1-D"])
                      pvecT1 = c(pvecT1, pT1)
                    }
                  }
                }
              }
            }
          }
        }
      }
      edgep[j, ] = c(
        internalEdges[j],
        combineP(pvecstar, testCorrection),
        combineP(pvecT1, testCorrection)
      )
    }
    edgep = rbind(edgep) #make sure result is a matrix if only 1 row
  }



  #Put edge star p-value -logs in as node labels on T_0
  tree$node.label = rep("", tree$Nnode)
  tree$node.label[tree$edge[edgep[, "edge"], 2] - ntaxa] = -log(edgep[, "pcombstar"], 10)

  #Put initial tree T_0 in list, with special treatment if reference Tree supplied
  pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute combined values for tree T_0
  pTreeList = c(pTree) # save in list
  indexTreeList = c(0)
  if (is.null(refTree))
    trees = list(list(
      "tree" = ape::write.tree(tree),
      "logpTree" = -log(pTree, 10),
      "logpT1Edge" = NA,
      "numEdgeCon" = 0
    ))
  else{
    refTree=unroot(refTree)
    RF = as.numeric(ape::dist.topo(refTree, tree))
    trees = list(
      list(
        "tree" = ape::write.tree(tree),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = 0,
        "RF" = RF
      )
    )# put it in list
    RFlist = c(RF)# and save RF for plotting
  }


  if (plot > 0) {
    #Plot initial tree
    line1 = ""
    if (!is.null(refTree))
      line1 = paste0("RF to reference Tree = ", RF, "; \n")

    plot(
      tree,
      type = "unrooted",
      cex.sub = .7,
      main = paste0("T0 (Input tree): ", testCorrection, ", quad"),
      sub = paste0(
        line1,
        "-log_10(p) for combined T1 over tree = ",
        formatC(-log(pTree, 10), format = 'e', digits = 3),
        "; \n Edge labels are -log_10(p) for combined star tests for that edge."
      )
    )
    if (plot %in% c(2, 3)) {
      badedgeindex = which(edgep[, "pcombstar"] > beta)
      if (length(badedgeindex) > 0) {
        badedges = edgep[badedgeindex, "edge"]
        bgColor = rep("tomato1", length(badedges))
        ape::edgelabels(
          formatC(
            -log(edgep[badedgeindex, "pcombstar"], 10),
            format = 'e',
            digits = 0
          ),
          edge = badedges,
          bg = bgColor,
          cex = .5
        )
      }
    }
    if (plot == 3) {
      polyList = c() #find multifurcations, starting at "top" in unrooted tree
      if (length(Descendants(tree, ntaxa + 1, 'children')) > 3)
        polyList = c(polyList, ntaxa + 1)
      if (tree$Nnode >= 2) {
        #loop over other internal nodes
        for (i in (ntaxa + 2):(ntaxa + tree$Nnode)) {
          if (length(Descendants(tree, i, 'children')) > 2)
            polyList = c(polyList, i)
        }
      }
      #add colored dots for these nodes
      lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
      internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
      XX <- lastPP$xx[polyList]
      YY <- lastPP$yy[polyList]
      points(XX,
             YY,
             pch = 16,
             col = "red",
             cex = 1)
    }
    if (plot == 4) {
      edgeind = which(edgep[, "pcombstar"] != Inf)
      if (length(edgeind) > 0) {
        bgColor = rep("lightblue", dim(edgep)[[1]])
        bgColor[which(edgep[edgeind, "pcombstar"] > beta)] = "tomato1"
        ape::edgelabels(
          formatC(
            -log(edgep[edgeind, "pcombstar"], 10),
            format = 'e',
            digits = 0
          ),
          edge = edgep[edgeind, "edge"],
          bg = bgColor,
          cex = .5
        )
      }
    }
  }

  #Contract edges based on star test
  message('Contracting edges with star test at level beta = ',beta)
  edgepRow = which(edgep[, "pcombstar"] > beta) #determine entries for edges to contract for star test
  edges2contract = edgep[edgepRow, "edge"] # and the edge numbers
  num2contract = length(edges2contract)
  tree$edge.length[edges2contract] = 0 #contract those edges

  D = cophenetic(tree) #wipe out p_T1-D for quartets no longer resolved
  D = D[order(rownames(D)), order(colnames(D))]
  for (m in 1:dim(pTable)[1]) {
    qnames = which(pTable[m, 1:ntaxa] == 1)
    a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
    b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
    c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
    if (min(c(a, b, c)) == max(c(a, b, c)))
      pTable[m, "p_T1-D"] = NA
  }


  #Put edge T1 p-value -logs in as node labels on Tk
  tree$node.label = rep("", tree$Nnode)
  tree$node.label[tree$edge[edgep[, "edge"], 2] - ntaxa] = -log(edgep[, "pcomb"], 10)


  #Put star-contracted tree Tk in list, with special treatment if reference Tree supplied
  pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute combined values for tree T_0
  pTreeList = c(pTreeList, pTree) # save in list
  indexTreeList = c(indexTreeList, num2contract)
  treem = di2multi(tree)
  if (is.null(refTree))
    trees = append(trees, list(
      list(
        "tree" = ape::write.tree(treem),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = num2contract
      )
    ))
  else{
    RF = as.numeric(ape::dist.topo(refTree, tree))
    trees = append(trees, list(
      list(
        "tree" = ape::write.tree(treem),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = num2contract,
        "RF" = RF
      )
    ))# put it in list
    RFlist = c(RFlist,RF)# and save RF for plotting
  }

  if (plot > 0) {
    #Plot this tree
    line1 = ""
    if (!is.null(refTree))
      line1 = paste0("RF to reference Tree = ", RF, "; \n")

    plot(
      treem,
      type = "unrooted",
      cex.sub = .7,
      main = paste0("T", num2contract, ": ", testCorrection, ", quad"),
      sub = paste0(
        line1,
        "-log_10(p) for combined T1 over tree = ",
        formatC(-log(pTree, 10), format = 'e', digits = 3),
        "; \n Edge labels are -log_10(p) for combined T1 test for that edge;\n",
        num2contract,
        " edges contracted from by combined star test."
      )
    )
    if (plot %in% c(2, 3)) {
      badedgeindex = which(edgep[, "pcomb"] < colorCutoff)
      if (length(badedgeindex) > 0) {
        badedges = edgep[badedgeindex, "edge"]
        bgColor = rep("tomato1", length(badedges))
        ape::edgelabels(
          formatC(
            -log(edgep[badedgeindex, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = badedges,
          bg = bgColor,
          cex = .5
        )
      }
    }
    if (plot == 3) {
      polyList = c() #find multifurcations, starting at "top" in unrooted tree
      if (length(Descendants(treem, ntaxa + 1, 'children')) > 3)
        polyList = c(polyList, ntaxa + 1)
      if (treem$Nnode >= 2) {
        #loop over other internal nodes
        for (i in (ntaxa + 2):(ntaxa + tree$Nnode)) {
          if (length(Descendants(treem, i, 'children')) > 2)
            polyList = c(polyList, i)
        }
      }
      #add colored dots for these nodes
      lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
      internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
      XX <- lastPP$xx[polyList]
      YY <- lastPP$yy[polyList]
      points(XX,
             YY,
             pch = 16,
             col = "red",
             cex = 1)
    }
    if (plot == 4) {
      edgeind = which(edgep[, "pcomb"] != Inf)
      if (length(edgeind) > 0) {
        bgColor = rep("lightblue", dim(edgep)[[1]])
        bgColor[which(edgep[edgeind, "pcomb"] < colorCutoff)] = "tomato1"
        ape::edgelabels(
          formatC(
            -log(edgep[edgeind, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = edgep[edgeind, "edge"],
          bg = bgColor,
          cex = .5
        )
      }
    }
  }

  numEdges = dim(treem$edge)[[1]]

  while (numEdges > ntaxa)
    #loop with successive contracting steps
  {
    minp = min(edgep[, "pcomb"]) # p for edges to contract
    contract = which(edgep[, "pcomb"] == minp) # find all minimal edge p
    tree$edge.length[edgep[contract, 1]] = 0 # and set their lengths to 0
    edgep[contract, "pcomb"] = Inf # and never consider them again
    treem = di2multi(tree) #note this retains correct node label for edge p
    numEdges = dim(treem$edge)[[1]] #how many edges? 
    
    D = cophenetic(tree) #wipe out p_T1-D for quartets no longer resolved
    D = D[order(rownames(D)), order(colnames(D))]
    for (m in 1:dim(pTable)[1]) {
      qnames = which(pTable[m, 1:ntaxa] == 1)
      a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
      b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
      c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
      if (min(c(a, b, c)) == max(c(a, b, c)))
        pTable[m, "p_T1-D"] = NA
    }
    
    countContracted = numT0edges - numEdges #total number of edges contracted
    pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute new combined values for tree
    pTreeList = c(pTreeList, pTree) # save new tree p
    indexTreeList = c(indexTreeList, countContracted)

    if (is.null(refTree))
      trees = append(trees, list(
        list(
          "tree" = ape::write.tree(treem),
          "logpTree" = -log(pTree, 10),
          "logpT1Edge" = -log(minp, 10),
          "numEdgeCon" = countContracted
        )
      ))

    else{
      RF = as.numeric(ape::dist.topo(refTree, treem))
      trees = append(trees, list(
        list(
          "tree" = ape::write.tree(treem),
          "logpTree" = -log(pTree, 10),
          "logpT1Edge" = -log(minp, 10),
          "numEdgeCon" = countContracted,
          "RF" = RF
        )
      ))
      RFlist = c(RFlist, RF)# and save RF for plotting
    }

    if (plot > 0) {
      line1 = ""
      if (!is.null(refTree))
        line1 = paste0("RF to reference Tree = ", RF, "; \n")
      plot(
        tree,
        type = "unrooted",
        cex.sub = .7,
        main = paste0("T", countContracted, ": ", testCorrection, ", quad"),
        sub = paste0(
          line1,
          "-log_10(p) for combined T1 over tree = ",
          formatC(-log(pTree, 10), format = 'e', digits = 3),
          "; \n Edge labels are -log_10(p) from combined edge T1;\n",
          length(contract),
          " more edge(s) contracted by combined T1 test."
        )
      )

      if (plot > 1)
      {
        if (plot %in% c(2, 3)) {
          badedgeindex = which(edgep[, "pcomb"] < colorCutoff)
          if (length(badedgeindex > 0))
          {
            badedges = edgep[badedgeindex, "edge"]
            bgColor = rep("tomato1", length(badedges))
            ape::edgelabels(
              formatC(
                -log(edgep[badedgeindex, "pcomb"], 10),
                format = 'e',
                digits = 0
              ),
              edge = badedges,
              bg = bgColor,
              cex = .5
            )
          }
        }
        if (plot == 3)
        {
          contractedEdges = which(tree$edge.length == 0)
          polyList = tree$edge[contractedEdges, 1]
          #colored dots for these nodes
          lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
          internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
          XX <- lastPP$xx[polyList]
          YY <- lastPP$yy[polyList]
          points(XX,
                 YY,
                 pch = 16,
                 col = "red",
                 cex = 1)
        }
        if (plot == 4) {
          edgeind = which(edgep[, "pcomb"] != Inf)
          if (length(edgeind) > 0) {
            bgColor = rep("lightblue", dim(edgep)[[1]])
            bgColor[which(edgep[edgeind, "pcomb"] < colorCutoff)] = "tomato1"
            ape::edgelabels(
              formatC(
                -log(edgep[edgeind, "pcomb"], 10),
                format = 'e',
                digits = 0
              ),
              edge = edgep[edgeind, "edge"],
              bg = bgColor,
              cex = .5
            )
          }
        }
      }
    }

  }
  out$treeList = trees

  if (!is.na(alpha)) {
    small = which(pTreeList < alpha)
    if (length(small) == 0)
      small = 0
    out$indexLate = max(small) + 1
    out$indexEarly = which(pTreeList > alpha)[1]
  } else {
    out$indexLate = NA
    out$indexEarly = NA
  }

  #final plots, of p-values for tree, and RF
  if (plot > 0) {
    y = -log(pTreeList, 10)
    yfinpos = which(is.finite(y))
    yinfpos = setdiff(1:length(y), yfinpos)
    yinf = max(y[yfinpos]) * 1.5
    plot(
      indexTreeList[yinfpos],
      rep(yinf, length(yinfpos)),
      ylim = c(0, yinf),
      xlim = c(-.5, max(indexTreeList) + .5),
      main = paste0("Tree p-values; ", testCorrection, ", quad"),
      xlab = "Number of contracted edges",
      ylab = "-log_10(p) (solid red = Inf)",
      col = "red",
      pch = 19
    )
    points(indexTreeList[yfinpos], y[yfinpos], col = "blue")

    if (!is.null(refTree))
      plot(
        indexTreeList,
        RFlist,
        main = paste0("RFdist to supplied ToB; ", testCorrection, ", quad"),
        xlab = "Tree",
        ylab = "RFdist",
        col = "blue"
      )
  }

  out$alpha=alpha
  out$beta=beta
  
  out = out[c("treeList",
              "indexEarly",
              "indexLate",
              "alpha",
              "beta",
              "qType",
              "testCorrection",
              "pTable")]
  
  invisible(out)
}


####################################################################
#' Edge Contraction to infer a Tree of Blobs - multipartition mode
#'
#' Infer the tree of blobs using edge contraction and multipartitions
#'
#' Given quartet gene tree count data from a network and a starting tree that is
#' a resolution of its tree of blobs, contract edges for which there is poor support
#' due to, first, star-like quartet signals and then successively the the most extreme
#' non-tree-like signals.
#'
#' p-values for star and T1 tests for individual quartets displayed on the tree are
#' combined by one of 4 test correction methods suitable for non-independent tests. For both
#' star test, only p-values for quartets defining each edge are combined. For
#' T1 test, quartets from the current multipartition defining an edge are used.
#'
#' The input tree should be computed by ASTRAL, or another heuristic known to
#' produce a resolution of the tree of blobs in ideal circumstances.
#'
#' Plot levels allow for several graphical depictions of the edge contract process,
#' from the starting tree to a final star tree.
#'
#' For testing with simulated data, a reference tree (e.g., the true Tree of Blobs) 
#' may be provided and the RF distance between it and each tree will be shown.
#'
#' @references
#' \insertRef{ECTo2026}{MSCquartets}
#'
#' \insertRef{MAR19}{MSCquartets}
#'
#' \insertRef{LiuXie2020}{MSCquartets}
#'
#' \insertRef{Chen2022}{MSCquartets}
#' 
#' @param genedata one of:
#' \enumerate{
#' \item a character string giving the name of a file containing a list of newick 
#' gene trees (see argument \code{omit}),
#' \item a multiphylo object containing a collection of gene trees (see argument \code{omit}),
#' \item a table of resolved quartet counts  from a collection of
#' gene trees, or that table with additional columns of quartet p-values. If p_star
#' or both p_T1-D and qindex-D are present (such as from a previous call to this function) 
#' they are not recomputed.}
#' @param tree a tree as a phylo object, or a character string giving the name of 
#' a file with a newick tree, assumed to be a resolution of the true
#' tree of blobs
#' @param testCorrection a method for combining possibly dependent p-values to 
#' get a single p-value, one of "Cauchy", "Bon", "BBC", "CBC"
#' @param alpha test level for choosing inferred tree of blobs
#' @param beta test level for collapsing edges showing no resolution
#' @param colorCutoff level for changing color of edge p-values for T1 test
#' @param plot 0 for no plot;
#'             1 to plot all trees as edges are contracted, along with plot of tree p-values;
#'             2 to add -log_10 of p-values < \code{colorCutoff} for T1 test, or > beta for star test, to edges in plots;
#'             3 to add red dots at multifurcations;
#'             4 to suppress red dots but show all edge p-values
#' @param refTree a reference tree, as a phylo object, to compute the RF distance to 
#' all Tk (for simulation studies); will be unrooted
#' @param omit \code{FALSE} to treat unresolved quartets on gene trees as 1/3 of each resolution;
#' \code{TRUE} to discard unresolved quartet data; ignored if genedata given as quartet table
#' @param lambda parameter for power-divergence statistic (e.g., 0 for likelihood
#' ratio statistic, 1 for Chi-squared statistic)
#' @param method "MLest", "conservative", or "bootstrap"
#' @param smallsample	"precomputed" or "bootstrap", method of obtaining p-value
#' when sample is small (<30)
#' @param smallcounts	"precomputed" or "bootstrap", method of obtaining p-value
#' when some (but not all) counts are small
#' @param bootstraps	number of samples for bootstrapping
#'
#' @returns An invisible list with 8 elements:
#' \describe{
#' \item{$treeList}{A list corresponding to edge contraction steps.}
#' \item{$indexEarly}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 is above \code{alpha}.}
#' \item{$indexLate}{The index in \code{$treeList} of the first tree such that
#' the combined tree T1 of it and all subsequent ones is above \code{alpha}.}
#' \item{$alpha}{The value of \code{alpha} determining the previous 2 entries.}
#' \item{$beta}{The value of \code{beta} for contracting edges using the 
#' combined star test.}
#' \item{$qType}{The quartet type mode used.}
#' \item{$testCorrection}{The p-value combination method used.}
#' \item{$pTable}{A table of quartet p-values for relevant
#' quartet tests.}
#' }
#' 
#' In \code{$treeList}, the
#' first is the input tree,
#' the second the result of contracting edges judged unresolved
#' by the combined star test,
#' and subsequent ones the result of collapsing edges with the current
#' smallest combined T1 test p-value, with the final one a star tree.
#' Each element in this list itself a
#' list of \describe{
#' \item{$tree}{A tree, with all edge lengths 1 and a node labels
#' of -log of the parent edge's combined p (star for the input tree, T1 for others).}
#' \item{$logpTree}{-log base 10 of the combined T1 p-value of the tree using \code{testCorrection}.}
#' \item{$logpT1Edge}{-log base 10 of the combined T1 p-value of the edge(s)
#'  that were contracted from previous tree (\code{NA}
#' for input tree and second tree).}
#' \item{$numEdgeCon}{The number of edges that have been contracted from the input tree}
#' \item{$RF}{The RF distance between this tree and \code{refTree}, if \code{refTree} is not NULL.}
#' }
#'
#' @examples
#' data(ASTRALtreeLeopardusLescroart)
#' Atree=read.tree(text=ASTRALtreeLeopardusLescroart)
#' data(tableLeopardusLescroart)
#' ECToBlob_mul(tableLeopardusLescroart,Atree,"Bon")
#'
#' @export
#'
ECToBlob_mul = function(genedata,
                        tree,
                        testCorrection = "Bon",
                        alpha = NA,
                        beta = .8,
                        colorCutoff = .05,
                        plot = 3,
                        refTree = NULL,
                        omit=FALSE,
                        lambda = 0,
                        method = "MLest",
                        smallsample = "precomputed",
                        smallcounts = "precomputed",
                        bootstraps = 10 ^ 4
                        )
{
  if (!(testCorrection %in% c("Cauchy", "Bon", "BBC", "CBC")))
    stop("Invalid argument: testCorrection")
  if (!(plot %in% c(0,1,2,3,4)))
    stop("Invalid argument: plot")
  
  #get data on quartets
  if (!(any(c("matrix","character","multiPhylo") %in% class(genedata)))) 
    stop("Data must be supplied as an object of type multiPhylo, character, or matrix.")
  if (!("matrix" %in% class(genedata))) {
    if ("character" %in% class(genedata)) {
      genetrees <- read.tree(file=genedata) #read gene trees
      message(paste("Read", length(genetrees), "gene trees from file."))
    } else { genetrees = genedata} 
    taxanames = genetrees[[1]]$tip.label   # ... get taxa names from first tree
    taxanames = sort(taxanames)
    if (length(taxanames) <= 25) {
      namelist = paste0(taxanames, collapse = ", ")
    } else {
      namelist = paste0(paste0(taxanames[1:25], collapse = ", "),
                        ",...(see output table for full list)")
    }
    message("Analyzing ", length(taxanames), " taxa: ", namelist)
    genedata = quartetTable(genetrees)   # tally quartets on gene trees
    genedata = quartetTableResolved(genedata, omit)   # treat unresolved quartets
  }

  # prepare tree
  if ("character" %in% class(tree)) { tree <- read.tree(file=tree) } #read input tree 
  tree = unroot(tree)
  tree = di2multi(tree) #makes sure tree is unrooted and no zero length branches
  ntaxa = length(tree$tip.label) #number of taxa
  tree$edge.length = rep(1, dim(tree$edge)[1]) #make tree metric, with all edges length 1
  internalEdges = which(tree$edge[, 2] > length(tree$tip.label)) #numbers of internal edges
  if (length(internalEdges) == 0)
    stop("Tree has no internal edges.")
  treeTaxa=sort(tree$tip.label)
  dataTaxa=sort(colnames(genedata)[1:ntaxa])
  if (!all.equal(treeTaxa,dataTaxa))
    stop("Taxa on tree do not match those in genedata")
  numT0edges = dim(tree$edge)[[1]]
  

  out = c()# create empty output
  out$qType = "mul"
  out$testCorrection = testCorrection

  # apply T1-D and star test for all sets of 4 taxa
  pTable = quartetECtestInd(
    genedata,
    tree,
    lambda = lambda,
    method = method,
    smallsample = smallsample,
    smallcounts = smallcounts,
    bootstraps = bootstraps
  )
  out$pTable = pTable

  D = cophenetic(tree) #wipe out p_T1-D for quartets that are resolved
  D = D[order(rownames(D)), order(colnames(D))]
  for (m in 1:dim(pTable)[1]) {
    qnames = which(pTable[m, 1:ntaxa] == 1)
    a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
    b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
    c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
    if (min(c(a, b, c)) == max(c(a, b, c)))
      pTable[m, "p_T1-D"] = NA
  }
  #prepare for indexing table entries
  {
    taxanames = colnames(pTable)[1:ntaxa] #names of taxa, as ordered in table
    np1 = ntaxa + 1 #number of taxa + 1
    #store binomial coefficients for efficiency in computing indices in table to access sets of 4 taxa
    C = matrix(nrow = ntaxa, ncol = 4) #store binomial coefficients for efficiency in computing indices in table to access sets of 4 taxa
    for (i in 0:(ntaxa - 1)) {
      for (j in 1:4) {
        C[i + 1, j] = choose(i, j)
      }
    }
    Cn4 = choose(ntaxa, 4) # need one extra value
  }

  #Compute combined p-values for star and T1 test for all edges
  {
    edgep = matrix(0, length(internalEdges), 2) #create array for combined p-values
    colnames(edgep) = c("edge", "pcomb")

    for (j in 1:length(internalEdges)) {
      #Combine p_star for all quartets defining this edge on original tree.
      pNode = tree$edge[internalEdges[j], 1] #find taxon groups around parent and child node of edge
      cNode = tree$edge[internalEdges[j], 2]
      pGroups = nodeGroups(tree, pNode)
      cGroups = nodeGroups(tree, cNode)

      #remove group above child (depends on nodeGroups returning non-descendants last in list)
      cGroups = cGroups[-length(cGroups)]
      cUnion = unlist(cGroups)

      for (i in 1:length(pGroups)) {
        #remove group below parent
        if (all(cUnion %in% pGroups[[i]]))
          remove = i
      }
      pGroups = pGroups[-remove]

      # compute combined p values for star
      pvecstar = c() #storage for p_star values

      #run through all choices of 2 groups at child and 2 at parent
      for (group1 in 2:length(pGroups)) {
        for (group2 in 1:(group1 - 1)) {
          for (group3 in 2:length(cGroups)) {
            for (group4 in 1:(group3 - 1)) {
              #run through all choices of 1 taxon per group
              for (at in pGroups[[group1]]) {
                for (bt in pGroups[[group2]]) {
                  for (ct in cGroups[[group3]]) {
                    for (dt in cGroups[[group4]]) {
                      #find a,b,c,d entry in table and get p_=star
                      positions = sort(match(tree$tip.label[c(at, bt, ct, dt)], taxanames)) # find table columns for these taxa
                      index = Cn4 - C[np1 - positions[1], 4] - C[np1 - positions[2], 3] - C[np1 - positions[3], 2] - C[np1 - positions[4], 1] #row index for this quartet
                      pstar = unname(pTable[index, "p_star"])
                      pvecstar = c(pvecstar, pstar)
                    }
                  }
                }
              }
            }
          }
        }
      }
      edgep[j, ] = c(internalEdges[j], combineP(pvecstar, testCorrection))
    }
    edgep = rbind(edgep) #make sure result is a matrix if only 1 row
  }

  #Put edge star p-value -logs in as node labels on T_0
  tree$node.label = rep("", tree$Nnode)
  tree$node.label[tree$edge[edgep[, "edge"], 2] - ntaxa] = -log(edgep[, "pcomb"], 10)

  #Put initial tree T_0 in list, with special treatment if reference Tree supplied
  pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute combined values for tree T_0
  pTreeList = c(pTree) # save in list
  indexTreeList = c(0)
  if (is.null(refTree))
    trees = list(list(
      "tree" = ape::write.tree(tree),
      "logpTree" = -log(pTree, 10),
      "logpT1Edge" = NA,
      "numEdgeCon" = 0
    ))
  else{
    refTree=unroot(refTree)
    RF = as.numeric(ape::dist.topo(refTree, tree))
    trees = list(
      list(
        "tree" = ape::write.tree(tree),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = 0,
        "RF" = RF
      )
    )# put it in list
    RFlist = c(RF)# and save RF for plotting
  }

  if (plot > 0) {
    #Plot initial tree
    line1 = ""
    if (!is.null(refTree))
      line1 = paste0("RF to reference Tree = ", RF, "; \n")

    plot(
      tree,
      type = "unrooted",
      cex.sub = .7,
      main = paste0("T0 (Input tree): ", testCorrection, ", mul"),
      sub = paste0(
        line1,
        "-log_10(p) for combined T1 over tree = ",
        formatC(-log(pTree, 10), format = 'e', digits = 3),
        "; \n Edge labels are -log_10(p) for combined star tests for that edge."
      )
    )
    if (plot %in% c(2, 3)) {
      badedgeindex = which(edgep[, "pcomb"] > beta)
      if (length(badedgeindex) > 0) {
        badedges = edgep[badedgeindex, "edge"]
        bgColor = rep("tomato1", length(badedges))
        ape::edgelabels(
          formatC(
            -log(edgep[badedgeindex, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = badedges,
          bg = bgColor,
          cex = .5
        )
      }
    }
    if (plot == 3) {
      polyList = c() #find multifurcations, starting at "top" in unrooted tree
      if (length(Descendants(tree, ntaxa + 1, 'children')) > 3)
        polyList = c(polyList, ntaxa + 1)
      if (tree$Nnode >= 2) {
        #loop over other internal nodes
        for (i in (ntaxa + 2):(ntaxa + tree$Nnode)) {
          if (length(Descendants(tree, i, 'children')) > 2)
            polyList = c(polyList, i)
        }
      }
      #add colored dots for these nodes
      lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
      internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
      XX <- lastPP$xx[polyList]
      YY <- lastPP$yy[polyList]
      points(XX,
             YY,
             pch = 16,
             col = "red",
             cex = 1)
    }
    if (plot == 4) {
      edgeind = which(edgep[, "pcomb"] != Inf)
      if (length(edgeind) > 0) {
        bgColor = rep("lightblue", dim(edgep)[[1]])
        bgColor[which(edgep[edgeind, "pcomb"] > beta)] = "tomato1"
        ape::edgelabels(
          formatC(
            -log(edgep[edgeind, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = edgep[edgeind, "edge"],
          bg = bgColor,
          cex = .5
        )
      }
    }
  }

  #Contract edges based on star test
  message('Contracting edges with star test at level beta = ',beta)
  edgepRow = which(edgep[, "pcomb"] > beta) #determine entries for edges to contract for star test
  edges2contract = edgep[edgepRow, "edge"] # and the edge numbers
  num2contract = length(edges2contract)
  tree$edge.length[edges2contract] = 0 #contract those edges
  tree = di2multi(tree) #replace tree


  D = cophenetic(tree) #wipe out p_T1-D for quartets no longer resolved
  D = D[order(rownames(D)), order(colnames(D))]
  for (m in 1:dim(pTable)[1]) {
    qnames = which(pTable[m, 1:ntaxa] == 1)
    a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
    b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
    c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
    if (min(c(a, b, c)) == max(c(a, b, c)))
      pTable[m, "p_T1-D"] = NA
  }

  # Computation of initial combined edge p for T1 test, using tree after star test contraction
  internalEdges = which(tree$edge[, 2] > ntaxa) #numbers of internal edges
  edgep = matrix(0, length(internalEdges), 2) #create array for combined p-values
  colnames(edgep) = c("edge", "pcomb")

  for (j in 1:length(internalEdges)) {
    #Combine p_T1-D for all quartets defining this edge on original tree.
    pNode = tree$edge[internalEdges[j], 1] #find taxon groups around parent and child node of edge
    cNode = tree$edge[internalEdges[j], 2]
    pGroups = nodeGroups(tree, pNode)
    cGroups = nodeGroups(tree, cNode)

    #remove group above child (depends on nodeGroups returning non-descendants last in list)
    cGroups = cGroups[-length(cGroups)]
    cUnion = unlist(cGroups)

    for (i in 1:length(pGroups)) {
      #remove group below parent
      if (all(cUnion %in% pGroups[[i]]))
        remove = i
    }
    pGroups = pGroups[-remove]

    # compute combined p values for T1
    pvecT1 = c() #storage for p_T1-D values

    #run through all choices of 2 groups at child and 2 at parent
    for (group1 in 2:length(pGroups)) {
      for (group2 in 1:(group1 - 1)) {
        for (group3 in 2:length(cGroups)) {
          for (group4 in 1:(group3 - 1)) {
            #run through all choices of 1 taxon per group
            for (at in pGroups[[group1]]) {
              for (bt in pGroups[[group2]]) {
                for (ct in cGroups[[group3]]) {
                  for (dt in cGroups[[group4]]) {
                    #find a,b,c,d entry in table and get p_=star
                    positions = sort(match(tree$tip.label[c(at, bt, ct, dt)], taxanames)) # find table columns for these taxa
                    index = Cn4 - C[np1 - positions[1], 4] - C[np1 - positions[2], 3] - C[np1 - positions[3], 2] - C[np1 - positions[4], 1] #row index for this quartet
                    pT1 = unname(pTable[index, "p_T1-D"])
                    pvecT1 = c(pvecT1, pT1)
                  }
                }
              }
            }
          }
        }
      }
    }
    edgep[j, ] = c(internalEdges[j], combineP(pvecT1, testCorrection))
  }
  edgep = rbind(edgep) #make sure result is a matrix if only 1 row


  #Put edge T1 p-value -logs in as node labels on Tk
  tree$node.label = rep("", tree$Nnode)
  tree$node.label[tree$edge[edgep[, "edge"], 2] - ntaxa] = -log(edgep[, "pcomb"], 10)


  #Put star-contracted tree Tk in list, with special treatment if reference Tree supplied
  pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute combined values for tree T_0
  pTreeList = c(pTreeList, pTree) # save in list
  indexTreeList = c(indexTreeList, num2contract)
  if (is.null(refTree))
    trees = append(trees, list(
      list(
        "tree" = ape::write.tree(tree),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = num2contract
      )
    ))
  else{
    RF = as.numeric(ape::dist.topo(refTree, tree))
    trees = append(trees, list(
      list(
        "tree" = ape::write.tree(tree),
        "logpTree" = -log(pTree, 10),
        "logpT1Edge" = NA,
        "numEdgeCon" = num2contract,
        "RF" = RF
      )
    ))# put it in list
    RFlist = c(RFlist,RF)# and save RF for plotting
  }

  if (plot > 0) {
    #Plot this tree
    line1 = ""
    if (!is.null(refTree))
      line1 = paste0("RF to reference Tree = ", RF, "; \n")

    plot(
      tree,
      type = "unrooted",
      cex.sub = .7,
      main = paste0("T", num2contract, ": ", testCorrection, ", mul"),
      sub = paste0(
        line1,
        "-log_10(p) for combined T1 over tree = ",
        formatC(-log(pTree, 10), format = 'e', digits = 3),
        "; \n Edge labels are -log_10(p) for combined T1 test for that edge;\n",
        num2contract,
        " edges contracted from by combined star test."
      )
    )
    if (plot %in% c(2, 3)) {
      badedgeindex = which(edgep[, "pcomb"] < colorCutoff)
      if (length(badedgeindex) > 0) {
        badedges = edgep[badedgeindex, "edge"]
        bgColor = rep("tomato1", length(badedges))
        ape::edgelabels(
          formatC(
            -log(edgep[badedgeindex, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = badedges,
          bg = bgColor,
          cex = .5
        )
      }
    }
    if (plot == 3) {
      polyList = c() #find multifurcations, starting at "top" in unrooted tree
      if (length(Descendants(tree, ntaxa + 1, 'children')) > 3)
        polyList = c(polyList, ntaxa + 1)
      if (tree$Nnode >= 2) {
        #loop over other internal nodes
        for (i in (ntaxa + 2):(ntaxa + tree$Nnode)) {
          if (length(Descendants(tree, i, 'children')) > 2)
            polyList = c(polyList, i)
        }
      }
      #add colored dots for these nodes
      lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
      internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
      XX <- lastPP$xx[polyList]
      YY <- lastPP$yy[polyList]
      points(XX,
             YY,
             pch = 16,
             col = "red",
             cex = 1)
    }
    if (plot == 4) {
      edgeind = which(edgep[, "pcomb"] != Inf)
      if (length(edgeind) > 0) {
        bgColor = rep("lightblue", dim(edgep)[[1]])
        bgColor[which(edgep[edgeind, "pcomb"] < colorCutoff)] = "tomato1"
        ape::edgelabels(
          formatC(
            -log(edgep[edgeind, "pcomb"], 10),
            format = 'e',
            digits = 0
          ),
          edge = edgep[edgeind, "edge"],
          bg = bgColor,
          cex = .5
        )
      }
    }
  }

  numEdges = length(internalEdges)

  while (numEdges > 0)
    #loop with successive contracting steps
  {
    minp = min(edgep[, "pcomb"]) # p for edges to contract
    contract = which(edgep[, "pcomb"] == minp) # find all minimal edge p
    tree$edge.length[edgep[contract, 1]] = 0 # and set their lengths to 0
    edgep[contract, 2] = Inf # and never consider them again
    tree = di2multi(tree)

    # Computation of combined edge p for T1 test, using tree after last contraction
    internalEdges = which(tree$edge[, 2] > ntaxa) #numbers of internal edges
    if (length(internalEdges) > 0) {
      edgep = matrix(0, length(internalEdges), 2) #create array for combined p-values
      colnames(edgep) = c("edge", "pcomb")

      for (j in 1:length(internalEdges)) {
        pNode = tree$edge[internalEdges[j], 1] #find taxon groups around parent and child node of edge
        cNode = tree$edge[internalEdges[j], 2]
        pGroups = nodeGroups(tree, pNode)
        cGroups = nodeGroups(tree, cNode)

        #remove group above child (depends on nodeGroups returning non-descendants last in list)
        cGroups = cGroups[-length(cGroups)]
        cUnion = unlist(cGroups)

        for (i in 1:length(pGroups)) {
          #remove group below parent
          if (all(cUnion %in% pGroups[[i]]))
            remove = i
        }
        pGroups = pGroups[-remove]

        # compute combined p values for T1
        pvecT1 = c() #storage for p_T1-D values

        #run through all choices of 2 groups at child and 2 at parent
        for (group1 in 2:length(pGroups)) {
          for (group2 in 1:(group1 - 1)) {
            for (group3 in 2:length(cGroups)) {
              for (group4 in 1:(group3 - 1)) {
                #run through all choices of 1 taxon per group
                for (at in pGroups[[group1]]) {
                  for (bt in pGroups[[group2]]) {
                    for (ct in cGroups[[group3]]) {
                      for (dt in cGroups[[group4]]) {
                        #find a,b,c,d entry in table and get p_=star
                        positions = sort(match(tree$tip.label[c(at, bt, ct, dt)], taxanames)) # find table columns for these taxa
                        index = Cn4 - C[np1 - positions[1], 4] - C[np1 - positions[2], 3] - C[np1 - positions[3], 2] - C[np1 - positions[4], 1] #row index for this quartet
                        pT1 = unname(pTable[index, "p_T1-D"])
                        pvecT1 = c(pvecT1, pT1)
                      }
                    }
                  }
                }
              }
            }
          }
        }
        edgep[j, ] = c(internalEdges[j], combineP(pvecT1, testCorrection))
      }
      edgep = rbind(edgep) #make sure result is a matrix if only 1 row

      #Put edge T1 p-value -logs in as node labels on Tk
      tree$node.label = rep("", tree$Nnode)
      tree$node.label[tree$edge[edgep[, "edge"], 2] - ntaxa] = -log(edgep[, "pcomb"], 10)
    }

    D = cophenetic(tree) #wipe out p_T1-D for quartets no longer resolved
    D = D[order(rownames(D)), order(colnames(D))]
    for (m in 1:dim(pTable)[1]) {
      qnames = which(pTable[m, 1:ntaxa] == 1)
      a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
      b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
      c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
      if (min(c(a, b, c)) == max(c(a, b, c)))
        pTable[m, "p_T1-D"] = NA
    }

    countContracted = numT0edges - dim(tree$edge)[[1]] #total number of edges contracted
    pTree = combineP(na.omit(pTable[, "p_T1-D"]), testCorrection) # compute new combined values for tree
    pTreeList = c(pTreeList, pTree) # save new tree p
    indexTreeList = c(indexTreeList, countContracted)

    if (is.null(refTree))
      trees = append(trees, list(
        list(
          "tree" = ape::write.tree(tree),
          "logpTree" = -log(pTree, 10),
          "logpT1Edge" = -log(minp, 10),
          "numEdgeCon" = countContracted
        )
      ))

    else{
      RF = as.numeric(ape::dist.topo(refTree, tree))
      trees = append(trees, list(
        list(
          "tree" = ape::write.tree(tree),
          "logpTree" = -log(pTree, 10),
          "logpT1Edge" = -log(minp, 10),
          "numEdgeCon" = countContracted,
          "RF" = RF
        )
      ))
      RFlist = c(RFlist, RF)# and save RF for plotting
    }

    numEdges = dim(tree$edge)[[1]] - ntaxa  #how many edges left to collapse?

    if (plot > 0) {
      line1 = ""
      if (!is.null(refTree))
        line1 = paste0("RF to reference Tree = ", RF, "; \n")
      plot(
        tree,
        type = "unrooted",
        cex.sub = .7,
        main = paste0("T", countContracted, ": ", testCorrection, ", mul"),
        sub = paste0(
          line1,
          "-log_10(p) for combined T1 over tree = ",
          formatC(-log(pTree, 10), format = 'e', digits = 3),
          "; \n Edge labels are -log_10(p) from combined edge T1;\n",
          length(contract),
          " more edge(s) contracted by combined T1 test."
        )
      )

      if (plot > 1)
      {
        if (plot %in% c(2, 3)) {
          badedgeindex = which(edgep[, "pcomb"] < colorCutoff)
          if (length(badedgeindex > 0))
          {
            badedges = edgep[badedgeindex, "edge"]
            bgColor = rep("tomato1", length(badedges))
            ape::edgelabels(
              formatC(
                -log(edgep[badedgeindex, "pcomb"], 10),
                format = 'e',
                digits = 0
              ),
              edge = badedges,
              bg = bgColor,
              cex = .5
            )
          }
        }
        if (plot == 3)
        {
          polyList = c() # find multifurcations
          if (length(Descendants(tree, ntaxa + 1, 'children')) > 3) {
            polyList = c(polyList, ntaxa + 1)
          }
          if (tree$Nnode >= 2) {
            for (i in (ntaxa + 2):(ntaxa + tree$Nnode)) {
              #loop over internal nodes
              if (length(Descendants(tree, i, 'children')) > 2) {
                polyList = c(polyList, i)
              }
            }
          }
          
          #colored dots for these nodes
          lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
          internal_nodes <- (lastPP$Ntip + 1):length(lastPP$xx)
          XX <- lastPP$xx[polyList]
          YY <- lastPP$yy[polyList]
          points(XX,
                 YY,
                 pch = 16,
                 col = "red",
                 cex = 1)
        }
        if (plot == 4) {
          edgeind = which(edgep[, "pcomb"] != Inf)
          if (length(edgeind) > 0) {
            bgColor = rep("lightblue", dim(edgep)[[1]])
            bgColor[which(edgep[edgeind, "pcomb"] < colorCutoff)] = "tomato1"
            ape::edgelabels(
              formatC(
                -log(edgep[edgeind, "pcomb"], 10),
                format = 'e',
                digits = 0
              ),
              edge = edgep[edgeind, "edge"],
              bg = bgColor,
              cex = .5
            )
          }
        }
      }
    }

  }
  out$treeList = trees

  if (!is.na(alpha)) {
    small = which(pTreeList < alpha)
    if (length(small) == 0)
      small = 0
    out$indexLate = max(small) + 1
    out$indexEarly = which(pTreeList > alpha)[1]
  } else {
    out$indexLate = NA
    out$indexEarly = NA
  }

  #final plot, of p-values for tree, and RF
  if (plot > 0) {
    y = -log(pTreeList, 10)
    yfinpos = which(is.finite(y))
    yinfpos = setdiff(1:length(y), yfinpos)
    yinf = max(y[yfinpos]) * 1.5
    plot(
      indexTreeList[yinfpos],
      rep(yinf, length(yinfpos)),
      ylim = c(0, yinf),
      xlim = c(-.5, max(indexTreeList) + .5),
      main = paste0("Tree p-values; ", testCorrection, ", mul"),
      xlab = "Number of contracted edges",
      ylab = "-log_10(p) (solid red = Inf)",
      col = "red",
      pch = 19
    )
    points(indexTreeList[yfinpos], y[yfinpos], col = "blue")

    if (!is.null(refTree))
      plot(
        indexTreeList,
        RFlist,
        main = paste0("RFdist to supplied ToB; ", testCorrection, ", mul"),
        xlab = "Tree",
        ylab = "RFdist",
        col = "blue"
      )
   
  }

  out$alpha=alpha
  out$beta=beta
  
  out = out[c("treeList",
              "indexEarly",
              "indexLate",
              "alpha",
              "beta",
              "qType",
              "testCorrection",
              "pTable")]
  
  invisible(out)
}

#' Power divergence statistic of Cressie & Read
#'
#' Computes any of the family of power-divergence statistics for multinomial data of \insertCite{CR84;textual}{MSCquartets},
#' to compare observed and expected counts. Includes Likelihood Ratio and Chi-squared statistics as special cases.
#'
#' @references
#' \insertRef{CR84}{MSCquartets}
#'
#' @param obs  observation vector
#' @param expd  expected vector
#' @param lambda  statistic parameter (e.g., 0=Likelihood Ratio, 1=Chi-squared)
#'
#' @return value of statistic
#'
#' @examples
#' obs=c(10,20,30)
#' expd=c(20,20,20)
#' powerDivStat(obs,expd,0)
#'
#' @export
powerDivStat <- function(obs,
                         expd,
                         lambda) {
  if (is.vector(obs) == FALSE ||
      is.vector(expd) == FALSE ||
      length(obs) != length(expd) ||
      min(obs) < 0 ||
      min(expd) < 0 ||
      is.vector(lambda) == FALSE || length(lambda) != 1) {
    # Return an error and stop if arguments are misspecified. If any expectations are 0 then the function fails.
    stop(
      "Invalid arguments: obs and expd must be non-negative vectors of the same length. lambda must be real."
    )
  }
  else{
    if (all.equal(sum(obs), sum(expd)) != TRUE)
    {
      diff = sum(obs) - sum(expd)
      warning(paste0(
        "sum(obs)-sum(expd) = ",
        diff,
        ", which is not within tolerance of 0."
      ))
    }
    if (all(obs == expd) == TRUE) {
      # If observations and expectations are equal, there is no need for an algorithm for computing the power-divergence statistic since the power-divergence statistic must be 0.
      stat  <- 0
    } else {
      if (lambda %in% c(0, 1,-1 / 2,-1,-2, 2 / 3) == FALSE) {
        warning(
          "Non-standard choice of lambda. Standard choices are lambda=0,1,-1/2,-1,-2,2/3."
        )
      }
      if (lambda == 0) {
        # If lambda=0, compute the likelihood ratio statistic.
        stat <- sum(ifelse(obs == 0, 0, 2 * obs * log(obs / expd)))
      } else if (lambda == -1) {
        # If lambda=-1, compute the modified likelihood ratio statistic.
        if (min(obs) == 0) {
          # If one of the observations is 0, then the modified likelihood statistic is undefined and an error is returned, advising the user to choose a different lambda.
          stop("Some observations are 0. Use any lambda other than lambda=-1.")
        } else {
          stat <- sum(2 * expd * log(expd / obs))
        }
      } else {
        # If lambda is anything other than 0 or -1, then the function in the limit is not required.
        stat <-
          sum((2 / (lambda * (lambda + 1))) * obs * ((obs / expd) ^ lambda -
                                                       1))
      }
    }
    return(stat)
  }
}

#########################################################

# Error function
#
#@param x argument
#@return function value
#
erf <- function(x) {
  2 * pnorm(x * sqrt(2)) - 1
}

#########################################################

# Inverse error function
#
#@param x argument
#@return function value
#
erf.inv <- function(x) {
  qnorm((x + 1) / 2) / sqrt(2)
}

#########################################################

#' Modified Struve function
#'
#' This function is used in computing the probability density for
#' Model T1. The code is closely based on the \code{I0L0} function implemented
#' in Python for the package
#' RandomFieldUtils, which was available on CRAN until 12/2022.
#'
#'@param x  function argument
#'
#'@return value of negative modified Struve function
#'
#'@export
M0 <- function(x) {
  if (x<0) {
    r <- NA
  } else if (x>=0&&x<16) {
    g2 <- c(0.52468736791485599138,-0.35612460699650586196,0.20487202864009927687,-0.10418640520402693629,0.4634211095548429228*10^(-1),-0.1790587192403498630*10^(-1),0.597968695481143177*10^(-2),-0.171777547693565429*10^(-2),0.42204654469171422*10^(-3),-0.8796178522094125*10^(-4),0.1535434234869223*10^(-4),-0.219780769584743*10^(-5),0.24820683936666*10^(-6),-0.2032706035607*10^(-7),0.90984198421*10^(-9),0.2561793929*10^(-10),-0.710609790*10^(-11),0.32716960*10^(-12),0.2300215*10^(-13),-0.292109*10^(-14),-0.3566*10^(-16),0.1832*10^(-16),-0.10*10^(-18),-0.11*10^(-18))
    r <- -0.5*g2[1]
    ac <- acos((6*x-40)/(x+40))
    for (i in 2:24) {
      r <- r-g2[i]*cos((i-1)*ac)
    }
  } else {
    g3 <- c(2.00326510241160643125,0.195206851576492081*10^(-2),0.38239523569908328*10^(-3),0.7534280817054436*10^(-4),0.1495957655897078*10^(-4),0.299940531210557*10^(-5),0.60769604822459*10^(-6),0.12399495544506*10^(-6),0.2523262552649*10^(-7),0.504634857332*10^(-8),0.97913236230*10^(-9),0.18389115241*10^(-9),0.3376309278*10^(-10),0.611179703*10^(-11),0.108472972*10^(-11),0.18861271*10^(-12),0.3280345*10^(-13),0.565647*10^(-14),0.93300*10^(-15),0.15881*10^(-15),0.2791*10^(-16),0.389*10^(-17),0.70*10^(-18),0.16*10^(-18))
    r <- 0.5*g3[1]
    x2 <- x^2
    ac <- acos((800-x2)/(288+x2))
    for (i in 2:24) {
      r <- r+g3[i]*cos((i-1)*ac)
    }
    r <- -r*2/(pi*x)
  }
  return(r)
}


#########################################################

#' Probability density function for Model T1
#'
#' Value of probability density function for Model T1 of \insertCite{MAR19;textual}{MSCquartets}, Proposition 5.2.
#'
#'@references
#'\insertRef{MAR19}{MSCquartets}
#'
#'@param x  statistic value (e.g., likelihood ratio statistic, or other power divergence statistic)
#'@param mu0  parameter of density function
#'
#'@return value of density function
#'
#'@seealso \code{\link{T3density}}
#'
#'@export
T1density <-
  function(x,
           mu0) {
    (1 / 4) * exp(-(1 / 2) * x) * (sqrt(2 / (pi * x)) * (1 + erf(mu0 / sqrt(2))) -
                                     exp(-(1 / 2) * mu0 ^ 2) * Vectorize(M0)(mu0 * sqrt(x)))
  }

#########################################################

#' Probability density function for Model T3
#'
#' Value of probability density function for Model T3 of \insertCite{MAR19;textual}{MSCquartets}, Proposition 4.2.
#'
#' @references
#' \insertRef{MAR19}{MSCquartets}
#'
#' @param x  statistic value (e.g., likelihood ratio statistic, or other power divergence statistic)
#' @param mu0 parameter of density function
#' @param alpha0 parameter of density function
#' @param beta0 parameter of density function
#'
#' @return  value of density function
#'
#' @seealso \code{\link{T1density}}
#'
#' @export
T3density <- function(x,
                      mu0,
                      alpha0,
                      beta0) {
  (1 / (2 * sqrt(2 * pi * x))) * (exp(-(1 / 2) * x) * (1 - erf((1 / sqrt(
    2
  )) *
    (
      sqrt(x) * tan(beta0) - mu0
    ))) + exp(-(1 / 2) * (sqrt(x) - mu0 * cos(alpha0)) ^ 2) *
    (1 - erf((1 / sqrt(
      2
    )) * (
      sqrt(x) * tan(beta0) + mu0 * sin(alpha0)
    ))) +
    exp(-(1 / 2) * (sqrt(x) + mu0 * cos(alpha0)) ^ 2) * (1 - erf((1 / sqrt(
      2
    )) *
      (
        sqrt(x) * tan(alpha0) + mu0 * sin(alpha0)
      ))))
}

#########################################################

#' Hypothesis test for quartet counts fitting a tree under the MSC
#'
#' Test the hypothesis H_0= T1 or T3 model of \insertCite{MAR19;textual}{MSCquartets}, vs. H_1 = everything else.
#' T1 is for a specific species quartet topology, and T3 for any species quartet topology.
#'
#' @details
#' This function implements two of
#' the versions of the test given by \insertCite{MAR19;textual}{MSCquartets} as well as parametric
#' bootstrapping, with other procedures for when some expected counts are small.
#' When the topology and/or the internal
#' quartet branch length is not specified by the null hypothesis these are more accurate tests than,
#' say, a Chi-square with one degree of freedom, which is not theoretically
#' justified near the singularities and boundaries of the models.
#'
#' If \code{method="MLtest"}, this uses the test by that name described in Section 7 of \insertCite{MAR19;textual}{MSCquartets}.
#' For both the T1 and T3 models the test is slightly anticonservative over a small range of true internal edges of the quartet species tree.
#' Although the test generally performs well in practice, it lacks a uniform asymptotic guarantee over
#' the full parameter space for either T1 or T3.
#'
#' If \code{method="conservative"}, a conservative test described by \insertCite{MAR19;textual}{MSCquartets} is used. For model T3 this
#' uses the Chi-square distribution with 1 degree of freedom
#' (the "least favorable" approach), while for model T1
#' it uses the Minimum Adjusted Bonferroni, based on precomputed values from simulations with n=1e+6.
#' These conservative tests are asymptotically guaranteed to reject the null
#' hypothesis at most at a specified level, but at the expense of increased type II errors.
#'
#' If \code{method="bootstrap"}, then parametric bootstrapping is performed, based on parameter estimates of the quartet topology
#' and internal edge length. The bootstrap sample size is given by the \code{bootstrap} argument.
#'
#'
#' When some or all expected topology counts are small, the methods \code{"MLest"} and \code{"conservative"} are not appropriate.
#' The argument \code{smallsample} determines whether a precomputed bootstrap of 1e+8 samples, or actual boostrapping with the specified size,
#' is used when the total sample is small (<30).
#' The argument \code{smallcounts} determines whether bootstrapping or a faster approximate method is used when only some counts are small.
#' The approximate approach returns a precomputed p-value, found by replacing the largest observed count
#' with 1e+6 and performing 1e+8 bootstraps for the model T3. When n >30 and
#' some expected counts are small, the quartet tree error probability is small and the bootstrap p-value is
#' approximately independent of the choice of T3 or T1 and of the largest observed count.
#'
#' For model T1, the first entry of \code{obs} is treated as the count of gene quartets concordant with the species tree.
#'
#' The returned p-value should be taken with caution when there is a small sample size, e.g. less than 30 gene trees.
#' The returned value of \code{$edgelength} is a consistent estimator, but not the MLE, of the internal
#' edge length in coalescent units. Although consistent, the MLE for this length is biased.
#' Our consistent estimator is still biased, but with less bias than the MLE. See \insertCite{MAR19;textual}{MSCquartets}
#' for more discussion on dealing with the bias of parameter estimates in the
#' presence of boundaries and/or singularities of parameter spaces.
#'
#' @references
#' \insertRef{MAR19}{MSCquartets}
#'
#' @param obs  vector of 3 counts of resolved quartet frequencies
#' @param model  \code{"T1"} or \code{"T3"}, for the models of \insertCite{MAR19;textual}{MSCquartets}
#' @param lambda  parameter for power-divergence statistic (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)
#' @param method \code{"MLtest"},\code{"conservative"}, or \code{"bootstrap"}
#' @param smallsample \code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when sample is small (<30)
#' @param smallcounts \code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when some (but not all) counts are small
#' @param bootstraps  number of samples for bootstrapping
#'
#' @return \code{output} where \code{output$p.value} is the p-value and \code{output$edgelength} is a consistent estimator of the
#' internal edge length in coalescent units, possibly \code{Inf}.
#'
#' @examples
#'  quartetTreeTest(c(17,72,11),"T3")
#'  quartetTreeTest(c(17,72,11),"T1")
#'  quartetTreeTest(c(72,11,17),"T1")
#'  quartetTreeTest(c(11,17,72),"T1")
#'
#' @seealso \code{\link{quartetTreeTestInd}}
#'
#' @importFrom stats pbinom rmultinom integrate pnorm uniroot
#'
#' @export
quartetTreeTest <- function (obs, model = "T3", lambda = 0, method = "MLest", smallsample = "precomputed",
                             smallcounts = "precomputed", bootstraps = 10^4)
{
  T1bias <- function(mu02, mu0) {
    (1/sqrt(2 * pi)) * exp(-(1/2) * mu02^2) + (1/2) * mu02 *
      (1 + erf(mu02/sqrt(2))) - mu0
  }
  integrand <- function(y, mu02, alpha02, beta02) {
    (1/sqrt(2 * pi)) * y * (exp(-(1/2) * (y - mu02)^2) *
                              erf(y * (1/tan(beta02))/sqrt(2)) + exp(-(1/2) * (y +
                                                                                 mu02 * sin(alpha02))^2) * (erf((y * (1/tan(alpha02)) +
                                                                                                                   mu02 * cos(alpha02))/sqrt(2)) + erf((y * tan(alpha02 +
                                                                                                                                                                  beta02) - mu02 * cos(alpha02))/sqrt(2))))
  }
  T3bias <- function(mu02, mu0, n) {
    phi02 <- min((1/(4 * (n + mu02^2))) * (4 * n + 3 * mu02^2 -
                                             mu02 * sqrt(8 * n + 9 * mu02^2)), 1)
    alpha02 <- atan(1/sqrt(3 * (3 - 2 * phi02)))
    beta02 <- (1/2) * (pi/2 - alpha02)
    bias <- integrate(integrand, lower = 0, upper = Inf,
                      mu02 = mu02, alpha02 = alpha02, beta02 = beta02)$value -
      mu0
    return(bias)
  }
  if (is.numeric(obs) == FALSE || is.vector(obs) == FALSE ||
      length(obs) != 3 || min(obs) < 0 || sum(obs) < 1 || all.equal(sum(obs),
                                                                    round(sum(obs))) != TRUE || model %in% c("T1", "T3") ==
      FALSE || is.numeric(lambda) == FALSE || is.vector(lambda) ==
      FALSE || length(lambda) != 1 || smallsample %in% c("bootstrap",
                                                         "precomputed") == FALSE || smallcounts %in% c("bootstrap",
                                                                                                       "precomputed") == FALSE || is.numeric(bootstraps) ==
      FALSE || is.vector(bootstraps) == FALSE || length(bootstraps) !=
      1 || bootstraps <= 0 || bootstraps%%1 != 0 || method %in%
      c("MLest", "conservative", "bootstrap") == FALSE) {
    stop("Invalid arguments: obs must be a numeric non-negative vector of length 3 summing to a positive integer;\n      model must be \"T1\" or \"T3\";\n      lambda must be a real number;\n      method must be \"MLest\", \"conservative\", or \"bootstrap\";\n      smallsample must be \"bootstrap\" or \"precomputed\";\n      smallcounts must be \"bootstrap\" or \"approximate\";\n      bootstraps must be a positive integer.")
  }
  else {
    obs <- as.numeric(obs)
    n <- round(sum(obs))
    if (model == "T3") {
      obs <- sort(obs, decreasing = TRUE)
      expd <- c(obs[1], (1/2) * (n - obs[1]), (1/2) * (n -
                                                         obs[1]))
    }
    else {
      expd <- c(max(obs[1], n/3), (1/2) * (n - max(obs[1],
                                                   n/3)), (1/2) * (n - max(obs[1], n/3)))
    }
    phi0 <- 3 * (n - expd[1])/(2 * n)
    mu0 <- max(0, sqrt(2 * n) * (1 - phi0)/sqrt(phi0 * (3 -
                                                          2 * phi0)))
    if (all.equal(min(expd), 0) != TRUE) {
      if (model == "T1") {
        if (mu0 <= 1/sqrt(2 * pi)) {
          mu0unbiased <- 0
        }
        else if (sign(T1bias(0, mu0)) == sign(T1bias(mu0,
                                                     mu0))) {
          mu0unbiased <- mu0
        }
        else {
          mu0unbiased <- uniroot(T1bias, c(0, mu0), mu0 = mu0)$root
        }
        phi0unbiased <- (1/(4 * (n + mu0unbiased^2))) *
          (4 * n + 3 * mu0unbiased^2 - mu0unbiased *
             sqrt(8 * n + 9 * mu0unbiased^2))
      }
      else {
        if (mu0 <= 3 * sqrt(3)/(2 * sqrt(2 * pi))) {
          mu0unbiased <- 0
        }
        else if (sign(T3bias(0, mu0, n)) == sign(T3bias(mu0,
                                                        mu0, n))) {
          mu0unbiased <- mu0
        }
        else {
          mu0unbiased <- uniroot(T3bias, c(0, mu0), mu0 = mu0,
                                 n = n)$root
        }
        phi0unbiased <- (1/(4 * (n + mu0unbiased^2))) *
          (4 * n + 3 * mu0unbiased^2 - mu0unbiased *
             sqrt(8 * n + 9 * mu0unbiased^2))
        if (all.equal(sum(abs(obs - expd)), 0) != TRUE) {
          alpha0unbiased <- atan(1/sqrt(3 * (3 - 2 *
                                               phi0unbiased)))
          beta0unbiased <- (1/2) * (pi/2 - alpha0unbiased)
        }
      }
    }
    else {
      phi0unbiased <- phi0
    }
    t <- -log(phi0unbiased)
    if (all.equal(sum(abs(obs - expd)), 0) == TRUE) {
      p <- 1
    }
    else {
      if (n < 30) {
        if (smallsample == "precomputed" && (all.equal(sum(abs(obs -
                                                               trunc(obs))), 0) == TRUE || all.equal(sum(abs(obs -
                                                                                                             trunc(obs) - rep(1/3, 3))), 0) == TRUE || all.equal(sum(abs(obs -
                                                                                                                                                                         trunc(obs) - rep(2/3, 3))), 0) == TRUE)) {
          if (model == "T1") {
            obs <- c(obs[1], sort(obs[2:3], decreasing = TRUE))
          }
          if (all.equal(sum(abs(obs - trunc(obs))), 0) ==
              TRUE) {
            if (model == "T1") {
              pvaluetable <- precompT1int
            }
            else if (model == "T3") {
              pvaluetable <- precompT3int
            }
          }
          else if (all.equal(sum(abs(obs - trunc(obs) -
                                     rep(1/3, 3))), 0) == TRUE) {
            if (model == "T1") {
              pvaluetable <- precompT1onethird
            }
            else if (model == "T3") {
              pvaluetable <- precompT3onethird
            }
          }
          else if (all.equal(sum(abs(obs - trunc(obs) -
                                     rep(2/3, 3))), 0) == TRUE) {
            if (model == "T1") {
              pvaluetable <- precompT1twothirds
            }
            else if (model == "T3") {
              pvaluetable <- precompT3twothirds
            }
          }
          r <- which(colSums(abs(t(pvaluetable[, 1:3]) -
                                   obs) < .Machine$double.eps^0.5) == 3)
          p <- as.numeric(pvaluetable[r, 4])
        }
        else {
          if (all.equal(sum(abs(obs - trunc(obs))), 0) ==
              TRUE || all.equal(sum(abs(obs - trunc(obs) -
                                        rep(1/3, 3))), 0) == TRUE || all.equal(sum(abs(obs -
                                                                                       trunc(obs) - rep(2/3, 3))), 0) == TRUE) {
            warning(paste0("Consider choosing smallsample=",
                           "\"precomputed\"", " for accurate p-values from 10^8 bootstraps for any n<30."))
          }
          stat <- powerDivStat(obs, expd, lambda)
          if (bootstraps == 0) {
            bootstraps <- 10^4
          }
          count <- 0
          sims <- rmultinom(bootstraps, n, prob = c(1 -
                                                      (2/3) * phi0unbiased, (1/3) * phi0unbiased,
                                                    (1/3) * phi0unbiased))
          if (model == "T3") {
            sims <- apply(sims, 2, sort, decreasing = TRUE)
          }
          for (i in 1:bootstraps) {
            if (model == "T3") {
              expected <- c(sims[1, i], (1/2) * (n -
                                                   sims[1, i]), (1/2) * (n - sims[1, i]))
            }
            else {
              expected <- c(max(sims[, i], n/3), (1/2) *
                              (n - max(sims[, i], n/3)), (1/2) * (n -
                                                                    max(sims[, i], n/3)))
            }
            simstat <- powerDivStat(sims[, i], expected,
                                    lambda)
            count <- ifelse(simstat >= stat, count +
                              1, count)
          }
          p <- count/bootstraps
        }
      }
      else {
        stat <- powerDivStat(obs, expd, lambda)
        if (method == "bootstrap" || (method != "bootstrap" &&
                                      min(expd) < 5 && smallcounts == "bootstrap") ||
            (method != "bootstrap" && min(expd) < 5 &&
             smallcounts != "bootstrap" && ((all.equal(sum(abs(obs *
                                                               3 - round(obs * 3))), 0) != TRUE) || (all.equal(sum(abs(obs *
                                                                                                                       3 - round(obs * 3))), 0) == TRUE && all.equal(var(obs - trunc(obs)), 0) != TRUE)))) {
          warning("Bootstrapping has been selected. p-values are approximate. Bootstrapping is only necessary when some expectations are small and approximate bootstrap p-values not available.")
          if (method == "bootstrap" && min(expd) < 5 &&
              smallcounts == "precomputed") {
            warning("method is prioritized over smallcounts. Bootstrap has been chosen.")
          }
          if (method != "bootstrap" && min(expd) < 5 &&
              smallcounts != "bootstrap" && ((all.equal(sum(abs(obs *
                                                                3 - round(obs * 3))), 0) != TRUE) || (all.equal(sum(abs(obs *
                                                                                                                        3 - round(obs * 3))), 0) == TRUE && all.equal(var(obs - trunc(obs)), 0) != TRUE))) {
            warning("Approximate bootstrap p-values not available when two expected counts are below 5 and observed counts are not all integers, all integers + 1/3 or all integers + 2/3.")
          }
          if (bootstraps == 0) {
            bootstraps <- 10^4
          }
          count <- 0
          sims <- rmultinom(bootstraps, n, prob = c(1 -
                                                      (2/3) * phi0unbiased, (1/3) * phi0unbiased,
                                                    (1/3) * phi0unbiased))
          if (model == "T3") {
            sims <- apply(sims, 2, sort, decreasing = TRUE)
          }
          for (i in 1:bootstraps) {
            if (model == "T3") {
              expected <- c(sims[1, i], (1/2) * (n -
                                                   sims[1, i]), (1/2) * (n - sims[1, i]))
            }
            else {
              expected <- c(max(sims[, i], n/3), (1/2) *
                              (n - max(sims[, i], n/3)), (1/2) * (n -
                                                                    max(sims[, i], n/3)))
            }
            simstat <- powerDivStat(sims[, i], expected,
                                    lambda)
            count <- ifelse(simstat >= stat, count +
                              1, count)
          }
          p <- count/bootstraps
        }
        else if (method != "bootstrap" && min(expd) <
                 5 && smallcounts == "precomputed" && (all.equal(sum(abs(obs *
                                                                         3 - round(obs * 3))), 0) == TRUE) && all.equal(var(obs - trunc(obs)), 0) == TRUE) {
          sortobs23times3 = sort(round(3 * obs[2:3]))
          if (sortobs23times3[1]%%3 == 0) {
            if (all.equal(sortobs23times3, c(0, 27)) ==
                TRUE) {
              p <- 0.000919
            }
            else if (all.equal(sortobs23times3, c(3,
                                                  24)) == TRUE) {
              p <- 0.0209
            }
            else if (all.equal(sortobs23times3, c(6,
                                                  21)) == TRUE) {
              p <- 0.103
            }
            else if (all.equal(sortobs23times3, c(9,
                                                  18)) == TRUE) {
              p <- 0.359
            }
            else if (all.equal(sortobs23times3, c(12,
                                                  15)) == TRUE) {
              p <- 0.79
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  24)) == TRUE) {
              p <- 0.00191
            }
            else if (all.equal(sortobs23times3, c(3,
                                                  21)) == TRUE) {
              p <- 0.0397
            }
            else if (all.equal(sortobs23times3, c(6,
                                                  18)) == TRUE) {
              p <- 0.175
            }
            else if (all.equal(sortobs23times3, c(9,
                                                  15)) == TRUE) {
              p <- 0.523
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  21)) == TRUE) {
              p <- 0.00416
            }
            else if (all.equal(sortobs23times3, c(3,
                                                  18)) == TRUE) {
              p <- 0.0763
            }
            else if (all.equal(sortobs23times3, c(6,
                                                  15)) == TRUE) {
              p <- 0.297
            }
            else if (all.equal(sortobs23times3, c(9,
                                                  12)) == TRUE) {
              p <- 0.768
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  18)) == TRUE) {
              p <- 0.00871
            }
            else if (all.equal(sortobs23times3, c(3,
                                                  15)) == TRUE) {
              p <- 0.129
            }
            else if (all.equal(sortobs23times3, c(6,
                                                  12)) == TRUE) {
              p <- 0.476
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  15)) == TRUE) {
              p <- 0.0183
            }
            else if (all.equal(sortobs23times3, c(3,
                                                  12)) == TRUE) {
              p <- 0.24
            }
            else if (all.equal(sortobs23times3, c(6,
                                                  9)) == TRUE) {
              p <- 0.737
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  12)) == TRUE) {
              p <- 0.0393
            }
            else if (all.equal(sortobs23times3, c(3,
                                                  9)) == TRUE) {
              p <- 0.439
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  9)) == TRUE) {
              p <- 0.086
            }
            else if (all.equal(sortobs23times3, c(3,
                                                  6)) == TRUE) {
              p <- 0.681
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  6)) == TRUE) {
              p <- 0.197
            }
            else if (all.equal(sortobs23times3, c(0,
                                                  3)) == TRUE) {
              p <- 0.478
            }
          }
          else if (sortobs23times3[1]%%3 == 1) {
            if (all.equal(sortobs23times3, c(1, 28)) ==
                TRUE) {
              p <- 0.00222
            }
            else if (all.equal(sortobs23times3, c(4,
                                                  25)) == TRUE) {
              p <- 0.0237
            }
            else if (all.equal(sortobs23times3, c(7,
                                                  22)) == TRUE) {
              p <- 0.119
            }
            else if (all.equal(sortobs23times3, c(10,
                                                  19)) == TRUE) {
              p <- 0.358
            }
            else if (all.equal(sortobs23times3, c(13,
                                                  16)) == TRUE) {
              p <- 0.777
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  25)) == TRUE) {
              p <- 0.00452
            }
            else if (all.equal(sortobs23times3, c(4,
                                                  22)) == TRUE) {
              p <- 0.0436
            }
            else if (all.equal(sortobs23times3, c(7,
                                                  19)) == TRUE) {
              p <- 0.203
            }
            else if (all.equal(sortobs23times3, c(10,
                                                  16)) == TRUE) {
              p <- 0.52
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  22)) == TRUE) {
              p <- 0.0094
            }
            else if (all.equal(sortobs23times3, c(4,
                                                  19)) == TRUE) {
              p <- 0.0794
            }
            else if (all.equal(sortobs23times3, c(7,
                                                  16)) == TRUE) {
              p <- 0.295
            }
            else if (all.equal(sortobs23times3, c(10,
                                                  13)) == TRUE) {
              p <- 0.754
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  19)) == TRUE) {
              p <- 0.0194
            }
            else if (all.equal(sortobs23times3, c(4,
                                                  16)) == TRUE) {
              p <- 0.143
            }
            else if (all.equal(sortobs23times3, c(7,
                                                  13)) == TRUE) {
              p <- 0.47
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  16)) == TRUE) {
              p <- 0.0404
            }
            else if (all.equal(sortobs23times3, c(4,
                                                  13)) == TRUE) {
              p <- 0.233
            }
            else if (all.equal(sortobs23times3, c(7,
                                                  10)) == TRUE) {
              p <- 0.72
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  13)) == TRUE) {
              p <- 0.085
            }
            else if (all.equal(sortobs23times3, c(4,
                                                  10)) == TRUE) {
              p <- 0.422
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  10)) == TRUE) {
              p <- 0.113
            }
            else if (all.equal(sortobs23times3, c(4,
                                                  7)) == TRUE) {
              p <- 0.665
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  7)) == TRUE) {
              p <- 0.237
            }
            else if (all.equal(sortobs23times3, c(1,
                                                  4)) == TRUE) {
              p <- 0.532
            }
          }
          else {
            if (all.equal(sortobs23times3, c(2, 26)) ==
                TRUE) {
              p <- 0.00827
            }
            else if (all.equal(sortobs23times3, c(5,
                                                  23)) == TRUE) {
              p <- 0.0436
            }
            else if (all.equal(sortobs23times3, c(8,
                                                  20)) == TRUE) {
              p <- 0.199
            }
            else if (all.equal(sortobs23times3, c(11,
                                                  17)) == TRUE) {
              p <- 0.516
            }
            else if (all.equal(sortobs23times3, c(2,
                                                  23)) == TRUE) {
              p <- 0.0164
            }
            else if (all.equal(sortobs23times3, c(5,
                                                  20)) == TRUE) {
              p <- 0.0777
            }
            else if (all.equal(sortobs23times3, c(8,
                                                  17)) == TRUE) {
              p <- 0.298
            }
            else if (all.equal(sortobs23times3, c(11,
                                                  14)) == TRUE) {
              p <- 0.739
            }
            else if (all.equal(sortobs23times3, c(2,
                                                  20)) == TRUE) {
              p <- 0.0237
            }
            else if (all.equal(sortobs23times3, c(5,
                                                  17)) == TRUE) {
              p <- 0.146
            }
            else if (all.equal(sortobs23times3, c(8,
                                                  14)) == TRUE) {
              p <- 0.466
            }
            else if (all.equal(sortobs23times3, c(2,
                                                  17)) == TRUE) {
              p <- 0.0468
            }
            else if (all.equal(sortobs23times3, c(5,
                                                  14)) == TRUE) {
              p <- 0.238
            }
            else if (all.equal(sortobs23times3, c(8,
                                                  11)) == TRUE) {
              p <- 0.703
            }
            else if (all.equal(sortobs23times3, c(2,
                                                  14)) == TRUE) {
              p <- 0.0926
            }
            else if (all.equal(sortobs23times3, c(5,
                                                  11)) == TRUE) {
              p <- 0.409
            }
            else if (all.equal(sortobs23times3, c(2,
                                                  11)) == TRUE) {
              p <- 0.185
            }
            else if (all.equal(sortobs23times3, c(5,
                                                  8)) == TRUE) {
              p <- 0.645
            }
            else if (all.equal(sortobs23times3, c(2,
                                                  8)) == TRUE) {
              p <- 0.377
            }
            else if (all.equal(sortobs23times3, c(2,
                                                  5)) == TRUE) {
              p <- 0.525
            }
          }
        }
        else if (method != "bootstrap" && min(expd) >=
                 5) {
          if (method == "MLest") {
            if (model == "T3") {
              p <- ifelse(stat == 0, 1, min(1, max(0,
                                                   integrate(T3density, lower = stat, upper = Inf,
                                                             mu0 = mu0unbiased, alpha0 = alpha0unbiased,
                                                             beta0 = beta0unbiased)$value)))
            }
            else {
              p <- ifelse(stat == 0, 1, min(1, max(0,
                                                   integrate(T1density, lower = stat, upper = Inf,
                                                             mu0 = mu0unbiased)$value)))
            }
          }
          else if (method == "conservative") {
            if (model == "T3") {
              p <- pchisq(stat, 1, lower.tail = FALSE)
            }
            else {
              if (mu0 >= 0 && mu0 <= 0.125) {
                beta <- 0.5
                q <- pT1beta50
              }
              else if (mu0 > 0.125 && mu0 <= 1.75) {
                beta <- 0.45
                q <- pT1beta45
              }
              else if (mu0 > 1.75 && mu0 <= 2.75) {
                beta <- 0.3
                q <- pT1beta30
              }
              else if (mu0 > 2.75 && mu0 <= 3.75) {
                beta <- 0.2
                q <- pT1beta20
              }
              else {
                beta <- 0.05
                q <- pT1beta5
              }
              mu0lower <- max(0, mu0 + sqrt(2) * erf.inv(2 *
                                                           beta - 1))
              p1 <- ifelse(stat == 0, 1, min(1, max(0,
                                                    integrate(T1density, lower = stat, upper = Inf,
                                                              mu0 = mu0lower)$value)))
              p <- (length(q[q <= p1]))/length(q)
            }
          }
        }
      }
    }
    output <- list(p, t)
    names(output) <- c("p.value", "edgelength")
    return(output)
  }
}




#########################################################

#' Multiple independent hypothesis tests for quartet counts fitting a species tree under the MSC
#'
#' Perform a tree hypothesis test for all quartet counts in an input table, as if the counts for different choices of 4 taxa
#' are independent.
#'
#' @details This function assumes all quartets are resolved.  The test performed and the arguments
#' are described more fully in \code{quartetTreeTest}.
#'
#' @references
#' \insertRef{MAR19}{MSCquartets}
#'
#' @param rqt  table of resolved quartet counts, as produced by \code{quartetTableResolved}, or \code{quartetStarTestInd}
#' @param model  \code{"T1"} for a specific species tree topology, or \code{"T3"} for any species tree topology, with these
#' models explained more fully by \insertCite{MAR19;textual}{MSCquartets}
#' @param lambda power divergence statistic parameter (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)
#' @param method \code{"MLest"}, \code{"conservative"}, or \code{"bootstrap"}; see \code{quartetTreeTest} for explanation
#' @param smallsample \code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when sample is small (<30)
#' @param smallcounts \code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when some counts are small, so
#' the chosen \code{method} is inappropriate
#' @param bootstraps  number of samples for bootstrapping
#' @param speciestree  species tree, in Newick as text, to determine quartet for T1 test; required for \code{model="T1"},
#' ignored for \code{model="T3"}
#' @return
#'   if \code{model="T3"}, a copy of \code{rqt} with a new column \code{"p_T3"} appended with p-values for each quartet;
#'   if \code{model="T1"}, a copy of \code{rqt} with 2 columns appended: \code{"p_T1"} with p-values, and \code{"qindex"}
#'   giving index of quartet consistent with specified species tree,
#'   i.e., 1 if 12|34 on species tree, 2 if 13|24, 3 if 14|23
#'
#' @seealso \code{\link{quartetTreeTest}}, \code{\link{quartetTestPlot}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetTableResolved}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=c("t1","t2","t3","t4","t5","t6")
#' QT=quartetTable(gtrees,tnames)
#' RQT=quartetTableResolved(QT)
#' stree="(((t5,t6),t4),((t1,t2),t3));"
#' pTable3=quartetTreeTestInd(RQT,"T3")
#' quartetTablePrint(pTable3[1:6,])
#' stree="((((t5,t6),t4),t7),((t8,t9),((t1,t2),t3)));"
#' pTable1=quartetTreeTestInd(RQT,"T1",speciestree=stree)
#' quartetTablePrint(pTable1[1:6,])
#'
#' @importFrom ape unroot read.tree cophenetic.phylo
#' @export
quartetTreeTestInd <- function (rqt,
                                model = "T3",
                                lambda = 0,
                                method = "MLest",
                                smallsample = "precomputed",
                                smallcounts = "precomputed",
                                bootstraps = 10 ^ 4,
                                speciestree = NULL)
{
  if (model %in% c("T1", "T3") ==
      FALSE || is.numeric(lambda) == FALSE || is.vector(lambda) ==
      FALSE ||
      length(lambda) != 1 || smallsample %in% c("bootstrap", "precomputed") == FALSE ||
      smallcounts %in% c("bootstrap", "precomputed") == FALSE ||
      is.numeric(bootstraps) ==
      FALSE ||
      is.vector(bootstraps) == FALSE || length(bootstraps) !=
      1 || bootstraps <= 0 || bootstraps %% 1 != 0 || method %in%
      c("MLest", "conservative", "bootstrap") == FALSE) {
    stop(
      "Invalid arguments: lambda must be real; smallcounts must be \"bootstrap\" or \"approximate\";\n      bootstraps must be a positive integer; method must be \"MLest\",\"conservative\", or \"bootstrap\"."
    )
  }
  colnames = colnames(rqt)
  taxanames = setdiff(
    colnames,
    c(
      "12|34",
      "13|24",
      "14|23",
      "1234",
      "p_cut",
      "cutindex",
      "p_star",
      "p_T3",
      "p_T1"
    )
  )
  M = dim(rqt)[1]
  n = length(taxanames)
  if (model == "T3") {
    pTable = cbind(rqt, p_T3 = 0)
    message("Applying hypothesis test for model T3 to ", M, " quartets.")
    for (m in 1:M) {
      obs = unname(rqt[m, c("12|34", "13|24", "14|23")])
      pvec = quartetTreeTest(
        obs,
        model,
        lambda = lambda,
        smallsample = smallsample,
        smallcounts = smallcounts,
        bootstraps = bootstraps,
        method = method
      )
      pTable[m, "p_T3"] = pvec$p.value
    }
  }
  else {
    if (model != "T1") {
      stop("Invalid model: must use 'T1' or 'T3'")
    }
    else {
      if (is.null(speciestree)) {
        stop("Species tree topology must be supplied for model T1")
      }
      else {
        stree = unroot(read.tree(text = speciestree))
        nedges = dim(stree$edge)[1]
        stree$edge.length = rep(1, nedges)
        D = cophenetic.phylo(stree)
        D = D[order(rownames(D)), order(colnames(D))]
        pTable = cbind(rqt, p_T1 = 0, qindex = 0)
        message("Applying hypothesis test for model T1 to ", M, " quartets.")
        for (m in 1:M) {
          qnames = which(rqt[m, 1:n] == 1)
          a = D[qnames[1], qnames[2]] + D[qnames[3], qnames[4]]
          b = D[qnames[1], qnames[3]] + D[qnames[2], qnames[4]]
          c = D[qnames[1], qnames[4]] + D[qnames[2], qnames[3]]
          squartet = which.min(c(a, b, c))
          qcounts = rqt[m, c("12|34", "13|24", "14|23")]
          temp = qcounts[1]
          qcounts[1] = qcounts[squartet]
          qcounts[squartet] = temp
          pTable[m, "qindex"] = squartet
          pvec = quartetTreeTest(
            unname(qcounts),
            model,
            lambda = lambda,
            smallcounts = smallcounts,
            bootstraps = bootstraps
          )
          pTable[m, "p_T1"] = pvec$p.value
        }
      }
    }
  }
  return(pTable)
}


#############################################################

#' Produce simplex plot with  results of quartet hypothesis tests
#'
#' Plot a 2-d probability simplex, with points for all quartet count vectors. Colors
#' indicate rejection or failure to reject for tests at specified levels.
#'
#' @details The first argument of this function is a table of quartets and p-values. The
#' plot may show results of either the T1, T1-D, T3, or 2-cut
#' test, with or without a star tree test (depending on whether a \code{"p_star"} column is in the table and/or \code{beta =1}).
#' The p-values must be computed by previous calls to
#' \code{quartetTreeTestInd} (for \code{"T1"} or \code{"T3"} p-values), \code{quartetECtestInd} (for \code{"T1-D"} p-values)
#' and \code{quartetStarTestInd} (for \code{"star"} p-values). The \code{NANUQ}, \code{NANUQdist}, and \code{ECToBlob}
#' functions include calls to these tree test functions.
#'
#' @param pTable table of quartets and p-values, as produced by the functions mentioned above
#' @param test  model to use, for tree null hypothesis; options are \code{"T1"}, \code{"T3"}, \code{"cut"}, \code{"NANUQ"}, \code{"ECToBlob"}
#' @param alpha level for tree test with null hypothesis given by \code{test}
#' @param beta level for test with null hypothesis star tree;
#' test results plotted only if \code{beta<1} and \code{"p_star"} column present in \code{pTable}
#' @param cex scaling factor for size of plotted symbols
#' @return NULL
#'
#' @seealso \code{\link{quartetTreeTestInd}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetECtestInd}},
#' \code{\link{NANUQ}}, \code{\link{NANUQdist}}, \code{\link{ECToBlob}}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' tnames=c("t1","t2","t3","t4","t5","t6")
#' QT=quartetTable(gtrees,tnames[1:6])
#' RQT=quartetTableResolved(QT)
#' stree="(((t5,t6),t4),((t1,t2),t3));"
#' pTable=quartetTreeTestInd(RQT,"T1",speciestree=stree)
#' pTable=quartetStarTestInd(pTable)
#' quartetTestPlot(pTable, "T1", alpha=.05, beta=.95)
#'
#' @export
quartetTestPlot <- function(pTable,
                            test,
                            alpha = .05,
                            beta = 1,
                            cex = 1) {
  if (!(is.numeric(alpha) && is.numeric(beta))) {
    stop("Test levels alpha and beta must be numeric.")
  }

  lineWidth = 1.5
  yellowColor = "goldenrod3" #pick specific colors
  orangeColor = "darkgreen"
  redColor = "red"
  blueColor = "blue"

  counts = c("12|34", "13|24", "14|23")
  M = dim(pTable)[1]

  if ((test == "T1") & (beta == 1)) {
    #  T1 only
    p_tree.small = (pTable[, "p_T1"] < alpha)
    p_star.small = rep(TRUE, M)
    titletext = bquote("T1 Model," ~ alpha * "=" * .(alpha))
    legtext = c("reject tree", "fail to reject tree")
    legpch = c(2, 1)
    legcol = c(redColor, blueColor)
    model = "T1"
    #  for model 1 need to adjust point to reflect species tree
    for (m in 1:M) {
      qind = pTable[m, "qindex"]
      if (qind == 3) {
        pTable[m, counts] = pTable[m, counts[c(3, 1, 2)]]
      } else {
        if (qind == 2) {
          pTable[m, counts] = pTable[m, counts[c(2, 3, 1)]]
        }
      }
    }
  } else {
    if ((test == "T1") & (beta < 1)) {
      #  T1  and star
      p_tree.small = (pTable[, "p_T1"] < alpha)
      p_star.small = (pTable[, "p_star"] <= beta)
      titletext = bquote("T1 Model," ~ alpha * "=" * .(alpha) * "," ~ beta *
                           "=" * .(beta))
      legtext = c(
        "reject tree & star",
        "fail to reject tree/reject star",
        "fail to reject tree & star",
        "reject tree/fail to reject star"
      )
      legpch = c(2, 1, 0, 4)
      legcol = c(redColor, blueColor, yellowColor, orangeColor)
      model = "T1"
      #  for model 1 need to adjust point to reflect species tree
      for (m in 1:M) {
        qind = pTable[m, "qindex"]
        if (qind == 3) {
          pTable[m, counts] = pTable[m, counts[c(3, 1, 2)]]
        } else {
          if (qind == 2) {
            pTable[m, counts] = pTable[m, counts[c(2, 3, 1)]]
          }
        }
      }
    } else{
      if ((test == "T3") & (beta == 1)) {
        #T3 only
        p_tree.small = (pTable[, "p_T3"] < alpha)
        p_star.small = rep(TRUE, M)
        titletext = bquote("T3 Model," ~ alpha * "=" * .(alpha))
        legtext = c("reject tree", "fail to reject tree")
        legpch = c(2, 1)
        legcol = c(redColor, blueColor)
        model = "T3"
      } else {
        if ((test == "NANUQ") | ((test == "T3") & (beta < 1))) {
          #T3 and star
          p_tree.small = (pTable[, "p_T3"] < alpha)
          p_star.small = (pTable[, "p_star"] <= beta)

          if (test == "NANUQ") {
            titletext = bquote("NANUQ," ~ alpha * "=" * .(alpha) * "," ~ beta *
                                 "=" * .(beta))
          } else {
            titletext = bquote("T3 Model," ~ alpha * "=" * .(alpha) * "," ~ beta *
                                 "=" * .(beta))
          }
          legtext = c(
            "reject tree & star",
            "fail to reject tree/reject star",
            "fail to reject tree & star",
            "reject tree/fail to reject star"
          )
          legpch = c(2, 1, 0, 4)
          legcol = c(redColor, blueColor, yellowColor, orangeColor)
          model = "T3"
        } else {
          if ((test == "cut") & (beta == 1)) {
            #  cut only
            p_tree.small = (pTable[, "p_cut"] < alpha)
            p_star.small = rep(TRUE, M)
            titletext = bquote("2-cut Model," ~ alpha * "=" * .(alpha))
            legtext = c("reject cut", "fail to reject cut")
            legpch = c(2, 1)
            legcol = c(redColor, blueColor)
            model = "cut"
          } else {
            if ((test == "cut") & (beta < 1)) {
              #  cut  and star
              p_tree.small = (pTable[, "p_cut"] < alpha)
              p_star.small = (pTable[, "p_star"] <= beta)
              titletext = bquote("2-cut Model," ~ alpha * "=" * .(alpha) * "," ~ beta *
                                   "=" * .(beta))
              legtext = c(
                "reject cut & star",
                "fail to reject cut/reject star",
                "fail to reject cut & star",
                "reject cut/fail to reject star"
              )
              legpch = c(2, 1, 0, 4)
              legcol = c(redColor, blueColor, yellowColor, orangeColor)
              model = "cut"
            } else{
              if ((test == "ECToBlob") & (beta == 1)) {
              #  T1-D only
                p_tree.small = (pTable[, "p_T1-D"] < alpha)
                p_star.small = rep(TRUE, M)
                titletext = bquote("ECToBlob Tree Model," ~ alpha * "=" * .(alpha))
                legtext = c("reject tree", "fail to reject tree")
                legpch = c(2, 1)
                legcol = c(redColor, blueColor)
                model = "T1"
                for (i in 1:dim(pTable)[[1]] ) #switch order so tree index is first
                { if (pTable[i,"qindex-D"]==2) pTable[i,c("12|34","13|24")]=pTable[i,c("13|24","12|34")]
                  if (pTable[i,"qindex-D"]==3) pTable[i,c("12|34","14|23")]=pTable[i,c("14|23","12|34")]
                }
              } else {
                if ((test == "ECToBlob") & (beta < 1)) {
                  #  T1-D  and star
                  p_tree.small = (pTable[, "p_T1-D"] < alpha)
                  p_star.small = (pTable[, "p_star"] <= beta)
                  titletext = bquote("ECToBlob Tree Model," ~ alpha * "=" * .(alpha) * "," ~ beta *
                                       "=" * .(beta))
                  legtext = c(
                    "reject tree & star",
                    "fail to reject tree/reject star",
                    "fail to reject tree & star",
                    "reject tree/fail to reject star"
                  )
                  legpch = c(2, 1, 0, 4)
                  legcol = c(redColor, blueColor, yellowColor, orangeColor)
                  model = "T1"
                  for (i in 1:dim(pTable)[[1]] ) #switch order so tree index is first
                  { if (pTable[i,"qindex-D"]==2) pTable[i,c("12|34","13|24")]=pTable[i,c("13|24","12|34")]
                  if (pTable[i,"qindex-D"]==3) pTable[i,c("12|34","14|23")]=pTable[i,c("14|23","12|34")]
                  }
                } else
                  stop("Invalid test name")
              }
            }
          }
        }
      }
    }
  }


  simplexPrepare(model, maintitle = "Quartet Hypothesis Test", titletext =
                   titletext) # draw outline of simplex and lines for model

  blues = (!p_tree.small) & (p_star.small)
  reds = (p_tree.small) & (p_star.small)
  yellows = (!p_tree.small) & (!p_star.small)
  oranges = (p_tree.small) & (!p_star.small)

  bluepoints = pTable[which(blues), counts, drop = FALSE] # points to plot blue
  nblue = dim(bluepoints)[1]
  redpoints = pTable[which(reds), counts, drop = FALSE] #      red
  nred = dim(redpoints)[1]
  yellowpoints = pTable[which(yellows), counts, drop = FALSE] #      yellow
  nyellow = dim(yellowpoints)[1]
  orangepoints = pTable[which(oranges), counts, drop = FALSE] #        orange
  norange = dim(orangepoints)[1]

  if (nblue > 0) {
    for (i in 1:nblue) {
      simplexPoint(
        bluepoints[i,],
        type = "o",
        pch = 1,
        col = blueColor,
        lwd = lineWidth,
        cex = cex
      )
    }
  }
  if (nred > 0) {
    for (i in 1:dim(redpoints)[1]) {
      simplexPoint(
        redpoints[i,],
        type = "o",
        pch = 2,
        col = redColor,
        lwd = lineWidth,
        cex = cex
      )
    }
  }
  if (nyellow > 0) {
    for (i in 1:dim(yellowpoints)[1]) {
      simplexPoint(
        yellowpoints[i,],
        type = "o",
        pch = 0,
        col = yellowColor,
        lwd = lineWidth,
        cex = cex
      )
    }
  }
  if (norange > 0) {
    message("Some points (green) not rejected as star, but rejected as tree.")
    for (i in 1:dim(orangepoints)[1]) {
      simplexPoint(
        orangepoints[i,],
        type = "o",
        pch = 4,
        col = orangeColor,
        lwd = lineWidth,
        cex = cex
      )
    }
  }
  legend(
    'topleft',
    legend = legtext,
    pch = legpch,
    col = legcol,
    cex = 0.8,
    inset = 0.01,
    pt.cex = 1.2,
    lty = 0,
    lwd = 2
  )
}


#########################################

#' Bayesian posterior probability of error in 4-taxon unrooted species tree topology estimate
#'
#' From a gene quartet count concordance factor (qcCF), computes Bayesian posterior probabilities
#' of the three 4-taxon species tree topologies and the Bayesian posterior probability that the
#' assumed topology is incorrect, under the assumption that the counts arise from the MSC on some species tree.
#'
#' @details The Jeffreys prior is used for internal branch length, along with the uniform prior
#' on the resolved topology.
#'
#' @references
#' \insertRef{MAR19}{MSCquartets}
#'
#'@param obs vector of counts for 3 topologies
#'@param model \code{"T3"} or \code{"T1"}, for the models of \insertCite{MAR19;textual}{MSCquartets} describing an unspecified species
#'tree topology (\code{"T3"}), or the topology whose count is the first entry of \code{obs} (\code{"T1"})
#'
#'@return \code{(error.prob, top.probs)} where \code{error.prob} is the species tree error probability
#'and \code{top.probs} is a vector of the three species tree topology probabilities in the order of \code{obs};
#'for model \code{"T1"} the species tree used is the one
#'corresponding to the first count; for model \code{"T3"} the species
#'tree is the one corresponding to the largest count
#'
#'
#'@examples
#' obs <- c(28,32,30)
#' quartetTreeErrorProb(obs,model="T1")
#' quartetTreeErrorProb(obs,model="T3")
#'
#'@importFrom zipfR Ibeta
#'
#'@export
quartetTreeErrorProb <- function(obs,
                                 model = "T3") {
  if (is.vector(obs) == FALSE ||
      length(obs) != 3 ||
      min(obs) < 0 ||
      model %in% c("T1", "T3") == FALSE) {
    stop(
      'Invalid arguments: obs must be an non-negative vector of length 3, model must be "T1" or "T3".'
    )
  } else {
    # If all observations are equal, then all tree topologies have equal probabilities and quartet tree
    # error probability is 2/3.
    if (all.equal(obs[1], obs[2]) == TRUE &&
        all.equal(obs[1], obs[3]) == TRUE &&
        all.equal(obs[2], obs[3]) == TRUE) {
      treeerrorprob <- 2 / 3
      probs <- c(1 / 3, 1 / 3, 1 / 3)
    } else {
      # Compute posterior probabilities for each topology. Prior probabilities for each tree topology
      # are 1/3 and the Jeffreys prior is used for phi0.
      logp1 <-
        obs[1] * log(2) + zipfR::Ibeta(2 / 3, obs[2] + obs[3] + 1 / 2, obs[1] +
                                         1 / 2, log = TRUE)
      logp2 <-
        obs[2] * log(2) + zipfR::Ibeta(2 / 3, obs[1] + obs[3] + 1 / 2, obs[2] +
                                         1 / 2, log = TRUE)
      logp3 <-
        obs[3] * log(2) + zipfR::Ibeta(2 / 3, obs[1] + obs[2] + 1 / 2, obs[3] +
                                         1 / 2, log = TRUE)

      b1 <- exp(logp2 - logp1)
      b2 <- exp(logp3 - logp1)

      p1 <- 1 / (1 + b1 + b2)
      p2 <- b1 / (1 + b1 + b2)
      p3 <- b2 / (1 + b1 + b2)

      probs <- c(p1, p2, p3)

      if (model == "T1") {
        treeerrorprob <- sum(probs[2:3])
      } else {
        treeerrorprob <- sum(probs[setdiff(1:3, which.max(obs))])
      }
    }

    output <- list(treeerrorprob, probs)
    names(output) <-
      c("tree.error.probability", "tree.probabilities")
    return(output)
  }
}

#############################################################################

#' Apply Holm-Bonferroni method to adjust for multiple tests
#'
#' Apply the Holm-Bonferroni method to
#' adjust for multiple hypothesis tests performed on quartets from a data set of gene trees.
#'
#' @param pTable a table of quartets with p-values, as computed by
#' \code{quartetTreeTestInd} or \code{quartetStarTestInd}
#' @param model one of \code{"T1"}, \code{"T3"}, or \code{"star"}, where \code{pTable} contains a column \code{p_model} of p-values
#' @param alpha test level, for rejection of adjusted p-values less than or equal to \code{alpha}
#' @return the same table, with rows reordered, and 2 new columns of 1) adjusted p-values,
#' and 2) "Y" or "N" for indicating "reject" or "fail to reject"
#'
#' @seealso \code{\link{quartetTreeTestInd}}, \code{\link{quartetStarTestInd}}
#'
#'
#' @details When p-values are computed for each quartet using
#' \code{quartetTreeTestInd} or \code{quartetStarTestInd},
#' multiple comparisons are being done for one dataset. The
#' Holm-Bonferroni method \insertCite{HolmBonf79}{MSCquartets} adjusts these p-values upward,
#' controlling the familywise error rate. The probability
#' of at least one false discovery (rejection of the null hypothesis)
#' is no more than the significance level.
#'
#' The Holm-Bonferroni method does not require that test hypotheses are independent, which
#' is important for its application to quartet counts presumed to have arisen on a single
#' species tree.
#'
#' This can be a low power test (often failing to reject when the null hypothesis is false).
#' In particular for the T1 and T3 tests, it does not consider the relationships between edge
#' lengths for different sets of four taxa.
#'
#' Warning: Output of this function should not be used as input for other
#' MSCquartets functions, as it reorders the quartets in the table.
#'
#' @references \insertRef{HolmBonf79}{MSCquartets}
#'
#' @examples
#' gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
#' taxanames=taxonNames(gtrees)
#' QT=quartetTable(gtrees,taxanames[1:6])
#' RQT=quartetTableResolved(QT)
#' pTable=quartetTreeTestInd(RQT,"T3")
#' pTable[1:10,]
#' HBpTable=HolmBonferroni(pTable,"T3",.05)
#' HBpTable[1:10,]
#'
#' @export
HolmBonferroni <- function(pTable,
                           model,
                           alpha = .05) {
  if (!(model %in% c("T1", "T3", "star")))  {
    stop('Argument model must be one of "T1", "T3", or "star".')
  }
  columnname = paste0("p_", model)
  if (!(columnname %in% colnames(pTable))) {
    stop(c('Argument pTable has no column ', columnname, '.'))
  }
  m = nrow(pTable)
  pTable <-
    pTable[order(pTable[, columnname]),] #sort rows of table by columname

  multipliers <- m:1 # descending integers
  HBps <-
    pmin(1, pTable[, columnname] * multipliers) #adjust p-values
  FirstNonFail = min(c(which(HBps > alpha), m + 1))
  Reject = c(rep("Y", FirstNonFail - 1), rep("N", m - FirstNonFail + 1))
  newcols = cbind(HBps, Reject)
  pname = paste0("HB", columnname)
  colnames(newcols) = c(pname, paste0("Reject(", pname, ")"))
  pTable = cbind.data.frame(pTable, newcols)
  message("Holm-Bonferroni method leads to rejection for ",
          (FirstNonFail - 1),
          " tests.")
  return(pTable)
}

#' Draw 2-d probability simplex, with model lines for T3 or T1 model
#'
#' Outline the 2-d probability simplex, and draw the T1 or T3 model points for quartet frequencies.
#' The models "T1" and "T3" are described more fully by \insertCite{MAR19;textual}{MSCquartets}.
#'
#' @references
#' \insertRef{MAR19}{MSCquartets}
#' 
#' @param model \code{"T1"} or \code{"T3"}, for 1-tree or 3-tree model
#' @param maintitle main title for plot
#' @param titletext additional text for title
#' @return NULL
#'
#' @examples
#'    simplexPrepare("T3",maintitle="Main title",titletext="further text")
#'
#' @seealso \code{\link{simplexLabels}},
#'          \code{\link{simplexPoint}},
#'          \code{\link{simplexSegment}},
#'          \code{\link{simplexText}},
#'          \code{\link{simplexCoords}}
#'
#' @export
simplexPrepare <- function(model = "T3", 
                           maintitle = NULL, 
                           titletext = NULL) {
  if (!(model %in% c("T1", "T3", "cut")))
    stop("Invalid model name; use 'T1','T3', or 'cut'.") #check parameters
  lineWidth = 2
  
  oldpar=par(mar = c(0, 0, 4, 0) + 0.1)# set margin
  on.exit(par(oldpar))# and restore when done         
  
  top = c(1, 0, 0) # Simplex vertices
  left = c(0, 1, 0)
  right = c(0, 0, 1)
  mid = rep(1, 3) #Simplex barycenter
  lim <- c(-1, 1)
  yylim <- c(-.1, .6)
  
  plot(
    0,
    0,
    xlim = lim,
    ylim = yylim,
    type = 'n',
    asp = 1,
    axes = FALSE,
    xlab = '',
    ylab = '',
    main = maintitle
  ) # create plot
  mtext(eval(bquote(.(titletext))),
        side = 3,
        line = 0,
        cex = 1.2) # add rest of title text
  simplexSegment(top, left, lty = 'dotted', lwd = lineWidth) # outline Simplex
  simplexSegment(top, right, lty = 'dotted', lwd = lineWidth)
  simplexSegment(left, right, lty = 'dotted', lwd = lineWidth)
  simplexSegment(top, mid, lwd = lineWidth) # Plot lines giving model for quartet probabilities under the coalsecent on a species tree
  if (model == "T3") {
    # for 3-tree model use 3 lines stopping at centroid
    simplexSegment(left, mid, lwd = lineWidth)
    simplexSegment(right, mid, lwd = lineWidth)
  }
  if (model =="cut") {
    # for cut model use 3 lines through centroid
    botmid=c(0,.5,.5)
    rightmid=c(.5,0,.5)
    leftmid=c(.5,.5,0)
    simplexSegment(mid, botmid, lwd = lineWidth)
    simplexSegment(left, rightmid, lwd = lineWidth)
    simplexSegment(right, leftmid, lwd = lineWidth)
  }
}

###############################################

#' Plot point in 2-d probability simplex
#'
#' Normalizes a point given in 3-d non-normalized coordinates, then plots it in 
#' the 2-d probability simplex.
#' 
#' @param v a 3-d point in non-negative orthant, coordinates not summing to 0
#' 
#' @param ... other options to pass to graphics::points function
#' 
#' @return NULL
#'
#' @examples
#'    simplexPrepare("T3","Example Plot")
#'    simplexPoint(c(15,65,20),pch=3,col="blue")
#'
#' @seealso \code{\link{simplexLabels}},
#'          \code{\link{simplexPrepare}},
#'          \code{\link{simplexSegment}},
#'          \code{\link{simplexText}},
#'          \code{\link{simplexCoords}}
#' @export
simplexPoint <- function(v, 
                         ...) {
  coords <- simplexCoords(v)
  points(coords$x, coords$y, ...)
}

##################################################

#' Plot line segment in 2-d probability simplex
#'
#' Normalizes two points in 3-d, and draws line segment between them in 2-d probability simplex.
#' 
#' @param v,w  3-d endpoints of line segment in non-negative orthant, coords not summing to 0
#' 
#' @param ... other options to pass to graphics::segments function
#' 
#' @return NULL
#'
#' @examples
#'    simplexPrepare("T3","Example Plot")
#'    simplexSegment(c(15,65,20),c(15,70, 15),col="green")
#'
#' @seealso \code{\link{simplexLabels}},
#'          \code{\link{simplexPoint}},
#'          \code{\link{simplexPrepare}},
#'          \code{\link{simplexText}},
#'          \code{\link{simplexCoords}}
#' @export
simplexSegment <- function(v, 
                           w, 
                           ...) {
  # Draw line segment in planar simplex
  # Args:
  #      v,w = vectors giving endpoints in R^3
  #
  coords0 <- simplexCoords(v)
  coords1 <- simplexCoords(w)
  segments(coords0$x, coords0$y, coords1$x, coords1$y, ...)
}

########################################################

#' Add text at a point in 2-d probability simplex
#' 
#' Add text to a 2-d probability simplex plot, at specified location.
#'
#' @param v a 3-d point in non-negative orthant, coordinates not summing to 0
#' 
#' @param  label text to add to plot
#' 
#' @param ... other options to pass to graphics::text function
#' 
#' @return NULL
#'
#' @examples
#'    simplexPrepare("T3","Example Plot")
#'    simplexText(c(15,65,20),"tree ac|bd")
#'
#' @seealso \code{\link{simplexLabels}},
#'          \code{\link{simplexPoint}},
#'          \code{\link{simplexPrepare}},
#'          \code{\link{simplexSegment}},
#'          \code{\link{simplexCoords}}
#' @export
simplexText <- function(v, 
                        label = '', 
                        ...) {
  coords <- simplexCoords(v)
  text(coords$x, coords$y, label = label, ...)
}

#######################################################

#' Label vertices of 2-d probability simplex  
#' 
#' Add labels to vertices of the probability simplex.
#'
#' @param top label for top
#' @param left label for left bottom
#' @param right label for right bottom
#' @return NULL
#'
#' @examples
#'    simplexPrepare("T3","Example Plot")
#'    simplexLabels("ab|cd","ac|bd","ad|bc")
#'
#' @seealso
#'          \code{\link{simplexPoint}},
#'          \code{\link{simplexPrepare}},
#'          \code{\link{simplexSegment}},
#'          \code{\link{simplexText}},
#'          \code{\link{simplexCoords}}
#' @export
simplexLabels <- function(top = '',
                    left = '',
                    right = '') {
  topCoord   <- simplexCoords(c(1, 0, 0))
  leftCoord  <- simplexCoords(c(0, 1, 0))
  rightCoord <- simplexCoords(c(0, 0, 1))
  text(topCoord$x, topCoord$y, top, pos = 3)
  text(leftCoord$x, leftCoord$y, left, pos = 1)
  text(rightCoord$x, rightCoord$y, right, pos = 1)
}

###############################################

#' Convert 3-d coordinates to 2-d probability simplex coordinates
#' 
#' Convert from 3-d Cartesian coordinates to 2-d coordinates suitable for plotting in the probability simplex.
#'
#' @details Applies an affine coordinate trandformation that maps the centroid (1/3,1/3,1/3) to the origin (0,0), and 
#' rescales so that the line segments between (1,0,0), (0,1,0), and (0,0,1) are mapped to segments of length 1.
#' 
#' An input vector \code{v} is first normalized so its component sum to 1 before the map is applied.
#' 
#' @param v vector of 3 non-negative numbers, not summing to 0
#' 
#' @return 2-d coordinates to plot normalized point in simplex
#'
#' @examples
#'      simplexCoords(c(15,65,20))
#'
#' @seealso \code{\link{simplexLabels}},
#'          \code{\link{simplexPoint}},
#'          \code{\link{simplexPrepare}},
#'          \code{\link{simplexSegment}},
#'          \code{\link{simplexText}}
#' @export
simplexCoords <- function(v) {
  sum <- sum(v)
  
  if ((length(v) != 3) ||
      (sum(v < 0) > 0) || (sum == 0))
    stop("Invalid coordinates")
  
  v <- v / sum  #normalize so ternary coord sum to 1
  x <-
    (.5 * v[1] + v[3] - .5)  #map to planar triangle, with sides 1, barycenter at origin
  y <- (.8660254 * v[1] - 0.2886751)
  xyCoords <- data.frame(cbind(x = x, y = y))
  return(xyCoords)
}

---
title: "Inferring a Tree of Blobs with TINNIK"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Inferring a Tree of Blobs with TINNIK}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: vigREFS.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>")
  
knitr::opts_chunk$set(fig.align = "center", 
                      fig.show = "hold",
                      out.width = "55%",
                      fig.width = 7,  
                      fig.height = 6)


```

```{r setup}
library(MSCquartets)
```


## Why use TINNIK?

`TINNIK` infers a "tree of blobs" under the network 
multispecies coalescent model. The resulting unrooted topological tree partially depicts the species 
relationships that led to a collection of gene trees, showing only the cut edges of the network 
(ones which join otherwise unconnected pieces of the network). Any cycles 
or more complicated blobs formed by reticulations in the network are contracted to multifurcations (polytomies)
in this tree.

The tree of blobs thus isolates those parts of the network where reticulations have made
relationships non-tree-like. A researcher might then apply other methods to investigate the structure of 
each blob, perhaps by reducing to consideration of a smaller number of taxa. However, with current methods inferring complicated blob structure accurately may be difficult or impossible. Since the `TINNIK` algorithm is statistically consistent under the NMSC model regardless of the unknown blob structure, it may provide the strongest network inference possible without making assumptions on the unknown network structure.

Many current methods of network inference are statistically consistent only
if the unknown network is level-1 (each blob is as simple as possible, containing only a single reticulation). 
They can only return a level-1 network as output, and may give no indication as to whether a level-1 network is an adequate model for the data. When a level-1 method 
is used for network inference, checking that `TINNIK`'s results are consistent with 
the level-1 output can provide a researcher some justification for that assumption.

## Preparing the input data

`TINNIK` requires input in the form of a collection of gene trees on a common collection of taxa. 
Thus from multilocus sequence data it is necessary to first align each gene's sequences and
use standard phylogenetic methods and software (e.g., IQtree, RAxML) to obtain an unrooted topological tree for each gene. While 
these trees are not strictly data, as they are themselves inferred, `TINNIK` treats them as such. If the gene trees are rooted, or have edge lengths, that information is simply ignored.

We work with an example data set of 1730 gene trees of 7 Papionini species, extracted from @Vanderpool2020 and analyzed by `TINNIK` in @ABMR24. Gene trees in Newick format are read from a text file to a multiphylo object using an _ape_ function:

```{r}
# read text file of gene trees supplied with MSCquartets package

gts=read.tree(file = system.file("extdata","dataPapioniniVanderpool",package="MSCquartets"))

```


#### Notes:

While some missing taxa on gene trees can be handled by `TINNIK`, it is necessary that each subset of 4 taxa appears on at least 1 gene tree and desirable that it appears on many. Statistical tests are performed for each such set, and the amount of data for these is determined by the count of trees displaying the set.

All understanding of `TINNIK`'s statistical behavior is established by assuming that the input gene trees are a true sample under the NMSC model. In practice, inferred gene trees are used, which likely contain some error. 
In particular, as with other network inference methods using inferred gene trees as input, widespread lack of resolution in the gene trees is unlikely to lead to a good analysis, 



## An initial analysis

The first steps of `TINNIK` are to count occurrences of each quartet tree topology across all gene trees, for each set of 4 taxa, and then to apply two hypothesis tests to these counts to judge their fit to 4-taxon star and resolved species trees, producing two p-values, "p_star" and "p_T3"

The easiest way to do this is to run the commands below, where we save the table of quartet counts and associated p-values as "pT". The `TINNIK` command here does a full analysis, which we generally should not take as our final one, as we are using default nominal levels for the hypothesis tests.

```{r}

# perform initial TINNIK analysis for gene trees, using defaults

output=TINNIK(gts)

# save table of quartet information and p-values

pT=output$pTable

```

The `TINNIK` command produced three plots. The first is a simplex plot of the hypothesis test results, at the specified levels. Each set of 4 taxa is represented by one plotted symbol, with those near the 3 lines indicating a tree-like relationship, and those farther from the lines a putative non-tree-like one. Those near the centroid of the simplex indicate a star-like relationship.  (See @AMR2020 for a more complete explanation of these plots.)

The second simplex plot shows the outcome of the `TINNIK` inference rule, where some of the quartets initially viewed as tree-like (T-quartets) from the hypothesis test are subsequently flagged as coming off a blob (B-quartets). (See @ABMR22 or @ABMR24 for a more complete explanation.)

The final plot shows `TINNIK`'s inferred tree of blobs, with 5- and 4- multifurcations joined by an edge, indicating 4- and 5-blobs.
Note this result is dependent on the levels of the hypothesis tests shown in the subtitle of the plot. The default values are $\alpha=.05$ for the test with null hypothesis "the quartet has a tree-like relationship", and $\beta=.95$ for the test with null hypothesis "the quartet has a star-like relationship".

#### Notes: 

Since this data set is relatively small, it does not take long to produce the table of quartet counts and p-values, and we could simply recalculate it for additional runs. For a large data set (many taxa, many gene trees) producing this table is the most significant factor in run-time, so saving it for reuse is wise.

The table "pT" can be printed directly in R, or printed in a nicer format with `quartetTablePrint`. However, its rows should not be reordered, as the `TINNIK` algorithm uses an indexing function to access them quickly.

Instead of the T3 hypothesis test for tree-likeness, the analysis can be run using the cut test (see the `TINNIK` documentation). Doing so may produce fewer initial B-quartets, and thus a more resolved tree of blobs, but is more appropriate only for what might be considered extreme network structures (see @ABMR24).

## Varying the test levels

An initial run of `TINNIK` with default test levels is rarely a sufficient analysis. One should always vary the test levels $\alpha$ and $\beta$ to judge robustness of the inferred tree of blobs to their choice. 

We generally have little understanding of the error that might be in the gene trees, and making different choices of the levels can be used to address this somewhat. Setting $\alpha=0$ will lead to all quartets being judged as tree-like, and increasing $\alpha$ will potentially increase the number considered to have putative 4-blobs. In a noisy data set, the default $\alpha=.05$ may result in many putative 4-blobs, and a smaller value of $\alpha$ may be needed for a useful inference. Since hypothesis testing is done on each set of 4 taxa independently, even with no gene tree error merely having more taxa present makes the erroneous judgment of some putative 4-blobs more likely.

Similarly, a value of $\beta=1$ will result in no quartet relationship being judged as star-like, but decreasing $\beta$ may result in more star-like quartets.

Smaller $\alpha$s and/or larger $\beta$s tend to produce more resolution in the inferred tree of blobs. 

For our example data set, we see no symbols plotted near the centroid in the first simplex plot, so only values of $\beta$ very close to 0 are likely to result in any judgment of star trees. An examination of "pT" confirms this, as the maximum in the "p_star" column is $\approx.112e-32$. To see the effect of a tiny $\beta$, we reuse the tabulated quartet information in "pT" and enter:

```{r}

TINNIK(pT, alpha=.05, beta=1e-40)

```

Here the first simplex plot shows a few symbols where the star-tree is now not rejected. However, as these are
treated by the `TINNIK` algorithm as B-quartets, we see no change in the next simplex plot or the inferred tree of blobs from our initial `TINNIK` run.


Since the original simplex plot shows rejection of tree-likeness for some quartets plotted near the line segments, varying  $\alpha$ is likely to be more interesting. Using a smaller $\alpha$, to potentially increase the number of quartets which are judged as tree-like, we enter:

```{r}

TINNIK(pT, alpha=.01, beta=.95)

```

The first simplex plot indeed shows fewer red triangles (initial B-quartets) and more blue circles, while the second plot has fewer gold squares (all B-quartets) and more green "x"s, producing an inferred tree of blobs with more resolution --- in fact, a fully-resolved tree.

This leads us to try a choice of $\alpha$ intermediate to the previous ones, so we enter:

```{r}

TINNIK(pT, alpha=.02, beta=.95)

```

Here the simplex plots and the tree of blobs vary from either of the earlier ones, showing one of the multifurcations in our initial analysis, but resolving the other. Further varying $\alpha$ can give intervals over which these analyses are stable. 


#### Note:

In reporting any `TINNIK` analysis it is essential to report the test levels used, and we strongly recommend exploring with values beyond the defaults.


## References

---
title: "Inferring a Tree of Blobs with ECToBlob"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Inferring a Tree of Blobs with ECToBlob}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
 
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>")
 
knitr::opts_chunk$set(fig.align = "center",
                      fig.show = "hold",
                      out.width = "55%",
                      fig.width = 7,  
                      fig.height = 6)


```

```{r setup}
library(MSCquartets)
```
 
## Why use ECToBlob?

`ECToBlob` infers a tree of blobs under the network multispecies coalescent (NMSC) model. The resulting unrooted tree represents only the cut edges of the underlying species network, that is, edges whose removal disconnects the network.

More complex structures, such as “blobs,” are collapsed into multifurcations (polytomies). In this way, the tree highlights where evolutionary relationships are tree-like and where they are not.

This makes the tree of blobs useful for network inference. It isolates regions where reticulation has occurred. Researchers can then investigate each blob separately, possibly focusing on smaller subsets of taxa. While current methods may not be able to resolve complex blobs, `ECToBlob` is statistically consistent under the NMSC model, regardless of the unknown internal blob structure.

## Preparing the input data

`ECToBlob` requires two inputs:

### (1) Gene trees (`genedata`)

A collection of gene trees on a common set of taxa. These are typically obtained by:

i) Aligning sequences for each locus  
ii) Inferring gene trees using phylogenetic software (e.g., IQ-TREE, RAxML)  

Although gene trees are themselves inferred, `ECToBlob` treats them as observed data. The root and branch lengths (if present) are ignored.

Example:

```{r eval=FALSE}
# Read gene trees and compute quartet counts
gts <- read.tree(file = "genetreefile")
tableLeopardusLescroart <- quartetTable(gts)
```

### Notes on gene trees
- The data can be provided in the function `ECToblob` either as a collection of gene trees in a phylo object or as the path and filename of a document containing gene trees in Newick format, with each gene tree on a separate row. Alternatively, the data may be supplied as a table obtained by tallying quartets and conducting statistical tests. Such a table can be generated from the output of several functions, including a previous run of `ECToBlob` (or  `NANUQ` or  `TINNIK`). Providing the latter is usually faster, as it avoids the need to again tally quartets and perform some statistical tests.
- Missing taxa are allowed, but every set of 4 taxa must appear in at least one gene tree, and ideally many.
- The initial statistical tests are applied per quartet, so the number of gene trees containing each quartet directly affects reliability.
- Theoretical guarantees assume gene trees are sampled under the NMSC model. In practice, inferred trees may introduce error.


### (2) A  tree (`tree`)

A tree assumed to be a resolution of the true tree of blobs. Such a tree can be produced by  ASTRAL or TREE-QMC since, for sufficiently large datasets, trees inferred by these methods have been shown to be  resolutions of the true tree of blobs.

`MSCquartets` contains the ASTRAL tree from the gene trees of Lescroart et al. (2023):

```{r}
AstralLeopard_tree <-   read.tree(text = ASTRALtreeLeopardusLescroart)
```

### Notes on the input tree

If the user wants to explore the package but has not yet computed a suitable reference tree (for e.g., from ASTRAL or TOB-QMC), one can construct a tree for exploratory purposes using a quartet distance implemented in the MSCquartets package:

```{r eval=FALSE}
gts <- read.tree(file = "genetreefile")
alternative_tree <- quartetDistTree(gtrees)
```
 
Note that,  while there is no guarantee that the tree obtained from the function `quartetDistTree` is a resolution of the tree of blobs, in practice, it often gives the same output as ASTRAL. We therefore suggest that the  `quartetDistTree` tree should only be used for exploratory purposes, and not for final analyses.
 
## Initial analysis

`ECToBlob` begins by counting quartet topologies across gene trees and applying hypothesis tests to each quartet. These tests evaluate fit to:

- a star tree → `p_star`  
- a resolved tree (as displayed on `tree`) → `p_T1`  

A basic analysis is run by:

```{r}
out <- ECToBlob(tableLeopardusLescroart, AstralLeopard_tree, alpha = 0.05)

pTable <- out$pTable
```

### Output plots

The function produces several plots:

1. The input tree → A plot of the input tree.
2. The tree after collapsing internal edges deemed unresolved by the combined star test (`p_star`) → A plot of the input tree after collapsing those edges for which the data does not support a resolution. (This tree may be the same as the input tree.)
3. Trees with progressively collapsed internal edges based on combined T1 test edge p-values → These trees are obtained by contracting edges in order from smallest to largest combined p-value. In case of ties, multiple edges will be contracted simultaneously.
4. A final star tree → This tree always results from collapsing all internal edges.
5. A plot of the number of edges contracted vs. \(-\log_{10}(p\text{-value})\), where the p-value is for the combined T1 test values for all edges remaining in the tree → This plot shows the relationship between the p-value for a null hypothesis of the tree's resolution (in the same order they appeared in the previous plots) and the number of edges contracted to obtain that tree. The y-axis is plotted on a logarithmic scale, and y-values typically decrease as more edges are contracted. The user may want to focus on trees where the p-value shows a sharp drop.

### Note on computation

For large datasets (many taxa or gene trees), computing the quartet table is the most time-consuming step. Saving `pTable` for reuse is recommended.
 
## Combining quartet p-values

Each edge in the input tree is assigned a p-value by combining the p-values of quartets that support it. The set of quartet p-values combined for a given edge can be determined in three ways:

1) `"bi"`: bipartition quartets  
2) `"quad"`: quadripartition quartets  
3) `"mul"`: multipartition quartets (updated during contraction)  

### Multiple testing correction

Once the quartet p-values defining an edge have been determined, they can be combined in four different ways to account for multiple testing:

1) **Bon (`Bonferroni`)**
2) **Cauchy (`Cauchy`)**
3) **BBC** (Bonferroni + Cauchy, combined via Bonferroni)
4) **CBC** (Bonferroni + Cauchy, combined via Cauchy)
 
### Examples

Here we show examples of different choices of both multiple testing correction and quartet p-value selection.

```{r}
out_1 <- ECToBlob(pTable, AstralLeopard_tree, alpha=0.05, qType="mul",  testCorrection="Bon",    plot=0)
out_2 <- ECToBlob(pTable, AstralLeopard_tree, alpha=0.05, qType="quad", testCorrection="Cauchy", plot=0)
out_3 <- ECToBlob(pTable, AstralLeopard_tree, alpha=0.05, qType="bi",   testCorrection="BBC",    plot=0)
out_4 <- ECToBlob(pTable, AstralLeopard_tree, alpha=0.05, qType="mul",  testCorrection="CBC",    plot=0)
```

These choices may yield different results and should be explored.

 
## Selecting a tree using the $\alpha$ threshold

The parameter $\alpha$ (`alpha`) determines which tree in the sequence will be selected. In contrast, the choice of $\beta$ (`beta`, test level for collapsing edges showing no resolution) influences which edges will initially be contracted as not consistent with any resolution. Succinctly,

- $\alpha$ affects **which tree is selected** in the sequence
- $\beta$ affects **which edges are collapsed initially**, influencing all downstream trees.

To select a tree based on $\alpha$, one has two options, though often they will agree:

1) **`indexEarly`**: first tree where the tree p-value exceeds $\alpha$  
2) **`indexLate`**: first tree where the tree p-value and all later tree p-values exceeds $\alpha$

One can extract the trees associated with these indices from the output as follows:

```{r}
early_tree <- out_2$treeList[[out_2$indexEarly]]$tree
plot(read.tree(text = early_tree))
```

```{r eval=FALSE}
late_tree <- out_2$treeList[[out_2$indexLate]]$tree
plot(read.tree(text = late_tree))
```
 
### Reporting recommendations

Any reported `ECToBlob` analysis should include:

- $\alpha$ and $\beta$ values  
- Quartet type (`qType`)  
- Test correction method (`testCorrection`)  
- Selected index: early (`$indexEarly`) or late (`$indexLate`).

Exploring multiple parameter settings is strongly recommended.
 
## References

---
title: "Inferring a Level-1 network with the NANUQ+ routines"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Inferring a Level-1 network with the NANUQ+ routines}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
bibliography:   vigREFS.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>")
  
knitr::opts_chunk$set(fig.align = "center", 
                      fig.show = "hold",
                      out.width = "55%",
                      fig.width = 7,  
                      fig.height = 6)


```

```{r setup}
library(MSCquartets)
```


## Why use NANUQ$^+$ routines?

NANUQ$^+$ is a collection of routines for inference of level-1 network features under the network multispecies coalescent (NMSC) model. Notably, they do not assume that the entire network is level-1. Taking as input a collection of unrooted topological gene trees, output is an unrooted (semidirected) topological level-1 network that captures species relationships, displaying cycles and cut edges (tree-like network features). Complex blobs (those that are not cycles) may be left as multifurcations of degree greater than 3 (polytomies) in this structure.

The initial step infers the unknown network's tree of blobs, using the function `TINNIK` (described in another vignette). This tree partially represents species relationships, showing only cut edges and contracting all reticulation cycles to multifurcations. Multifurcations suspected to arise from cycles can then be resolved into optimal cycle structures using `resolveCycle`. This resolution relies on a least-squares approach, comparing an empirical NANUQ distance among taxon groups around a multifurcation to expected distances for possible cycle configurations. For cycles under a default size of 10, a full search is fast, while larger cycles are processed using a quartet-based heuristic. 

The user may combine cycle resolutions for some or all of the individual multifurcations with `combineCycle`. Blobs that are not suspected to arise from cycles can be left unresolved. When all multifurcations are believed to be cycle-derived, `resolveLevel1` both resolves all multifurcations and combines the resolutions into fully-resolved level-1 networks.

We recommend using the  `NANUQ` function before resolving any multifurcations, to assess which 
ones may have resulted from network cycles. For a level-1 network, `NANUQ` generates a splits graph with distinctive structural features called 'darts'. If the level-1 assumption is violated, this structural pattern is disrupted, providing insight into whether resolving blobs as cycles is warranted.

For blobs that are cycles, all procedures here provide statistically consistent estimates of network structure under the NMSC model.

## Preparing the input data

The NANUQ$^+$ routines take as input a collection of gene trees for a set of taxa. From multilocus sequence data, standard phylogenetic tools must first be used to infer unrooted topological trees for each gene. Although these trees are not raw data, they are treated as such. If the gene trees are rooted or contain edge lengths, that information is simply ignored in these routines.

We work with an example data set of 16,338 gene trees of 16 Leopardus species, supplied by @Lescroart2023 and analyzed with NANUQ$^+$ by @ABRW24. Gene trees in Newick format can be read from a text file and their displayed quartet trees counted with commands such as:

```{r, eval=FALSE}
# read text file of gene trees and count quartets on them

gts<-read.tree(file = 'genetreefile')
tableLeopardusLescroart=quartetTable(gts)
```

We will use a table of this data supplied within *MSCquartets*, pre-computed by the above commands. Both `TINNIK` and `NANUQ` can take as input either the gene tree file or the quartet table.

```{r}
# load data file containing quartet counts for Leopardus data set supplied with MSCquartets package
# These counts are will be accessed as `tableLeopardusLescroart`.

data(tableLeopardusLescroart)
```

#### Notes:

All functions used here allow gene trees with missing taxa, provided each subset of four taxa appears on at least one tree (ideally, on many). Statistical tests are conducted for each of these subsets, with the sample size being the number of trees displaying the subset.

The good statistical behavior of the analysis below is established under the assumption that the input gene trees are a true sample from the NMSC model. In practice, inferred gene trees, containing some error, are used instead. 
Poorly inferred gene trees or those lacking sufficient resolution can compromise results.

## Step 1: Inferring the tree of blobs

The function `TINNIK` can be used to infer the tree of blobs for an unknown  network. This first applies two hypothesis tests for each choice of 4 taxa, testing whether the gene tree counts allow for the rejection of a resolved quartet tree or a star tree for those species. (See `TINNIK`'s vignette or [@ABMR24] for more information.)

```{r}
# perform TINNIK analysis for gene trees, using defaults

output<-TINNIK(tableLeopardusLescroart)

# save table of quartet information with p-values

pT<-output$pTable
```

The default `TINNIK` test levels are $\alpha=.05$ for the null hypothesis "the quartet has a tree-like relationship", and $\beta=.95$ for the null hypothesis "the quartet has a star-like relationship". An initial run of `TINNIK` with default test levels is rarely a sufficient analysis. One should always vary the $\alpha$ and $\beta$ to judge robustness of the inferred tree of blobs to their choice. 

As detailed by @ABRW24, for this data we choose $\alpha=5e-29$, imposing a stringent test for judging non-tree-likeness. Note in the first plot below this gives a clear separation between red (quartets interpreted as 4-blobs) and blue (quartets interpreted as tree-like) symbols, allowing for noise in the gene trees without rejecting tree-like-ness too strongly.

For this dataset and test levels we obtained a tree of blobs with two multifurcations, both of degree five.

```{r}
# perform improved TINNIK analysis to infer the tree of blobs

output<-TINNIK(pT, alpha=5e-29,beta = 0.95)
```

Output produced by `TINNIK` includes the table of quartet counts and associated test p-values and the tree of blobs. We rename these to use as input in further functions.

```{r}
# run TINNIK to infer the tree of blobs

output<-TINNIK(pT, alpha=5e-29,beta = 0.95)
```


```{r}
# rename output 

pT<-output$pTable #quartet count data with p-values for tests
ToB<-output$ToB   #the TINNIK tree of blobs
```

## Using `NANUQ` to explore whether multifurcations should be resolved into cycles

The `NANUQ` function can help detect whether a blob might be a cycle. On it's own, `NANUQ` is a statistically consistent level-1 network inference algorithm under the NMSC model. It takes the same input as TINNIK (a set of gene trees or a quartet table, along with two test levels) and produces, among other outputs, a distance measure that can be used as input to the `neighborNet` algorithm (as implemented in the `phangorn` package) to generate a splits network with specific structural properties. When the model is violated, this structure is disrupted, providing insight into potential violations of the level-1 assumption.

Further details on NANUQ and its applications are given by @ABR19 and @RBMA2020. We recommend using the same test levels but encourage testing alternative levels as well.

```{r}
# perform NANUQ analysis for table of quartet information and p-values

D<-NANUQ(pT, alpha = 5e-29,beta = 0.95) # Run the NANUQ routine
NN<-neighborNet(D$dist) # Run the NeighborNet algorithm on the NANUQ distance
plot(NN) # plot the splits-graph with neighborNet
```

As in [@ABRW24], we identified one multifurcation that aligns well with the model hypothesis, displaying a characteristic dart-like structure, and another that does not. We cannot determine whether the latter multifurcation results from a model violation (presumably level-1-ness) or simply noise. Nevertheless, this indicates caution is needed in interpreting findings associated with this multifurcation.

#### Note:

We used the NeighborNet implementation `neighborNet` in the `phangorn` package here, to do the full analysis within R. However, we recommend using SplitsTree [@SplitsTree] for the splits graph, as it has been more thoroughly tested. For more details, see [@RBMA2020].

## Step 2: Resolving multifurcations

When resolving the multifurcations, the first step is to numerically label all internal nodes in the tree of blobs, so that we and the code can refer to individual multifurcations. 

```{r}
  #Label internal nodes of the tree of blobs, and plot

  ToB<-labelIntNodes(ToB)
```

Here the multifurcations are nodes 18 and 20. We will resolve each one independently using the function `resolveCycle`.  While the same test levels as in TINNIK can be used, we suggest experimenting with different levels.

```{r}
  # resolve node 18

  resC18<-resolveCycle(ToB,18,pT,alpha=5e-29,beta=0.95)
```
The output `resC18` is a list that includes a network displaying the best resolution of node 18, with the other multifurcation remaining unresolved. Other information in this list is needed for combining resolutions of different multifurcations, or producing the histogram of residuals for all possible resolutions.

```{r}
  # resolve node 2O

  resC20<-resolveCycle(ToB,20,pT,alpha=5e-29,beta=0.95)
```

Note multifurcation 20 produced five optimal resolutions (that is, those 5 resolutions have residual sums of squares that are within a small tolerance). This suggests that this multifurcation may not arise from a cycle, or that the data has too much noise to be sure.

Alternatively, if one suspects that all multifurcations arise from a cycle, one could attempt to resolve all multifurcations to obtain a level-1 network.

```{r}
#Fully resolve the tree of blobs to a level-1 network

resN<-resolveLevel1(ToB=output$ToB, pTable=output$pTable, alpha=5e-29, beta=0.95, 
                    distance="NANUQ")
```

The output `resN` includes, as its first element, a list of the 5 level-1 networks that accommodate all the optimal resolutions that can co-occur.

#### Note:

In reporting any NANUQ$^+$ analysis, it is essential to report the test levels used, and, as mentioned throughout, we strongly recommend exploring with values beyond the defaults.

## Estimating numerical parameters and further comparing multiple optimal resolutions

NANUQ$^+$ yielded 5 fully-resolved topological networks for this data analysis. 

One can use the Julia package *PhyloNetworks* [@PhyloNetworks] to optimize numerical parameters (edge lengths and hybridization parameters) on these under a quartet pseudo-likelihood criterion, and also use the pseudo-likelihood score to further compare these networks. Note, however, that comparing networks by psuedo-likelihood is a compromise due to the difficulty of computing full likelihoods.


The commands below (based on 2024 *PhyloNetworks*) provide an example of this. Note that this is Julia code and not R code, and it requires additional installation of software.


````{verbatim}
using PhyloNetworks #Load package
gts = readMultiTopology("genetreefile") #load gene trees to Julia
q, t = countquartetsintrees(gts; showprogressbar = true) #compute the quartet counts
global df = writeTableCF(q, t) # to get a DataFrame that can be saved to a file later
global empCFs = readTableCF(df)
ntwk = readMultiTopology("Network.nwk", false) # read the topology of the newtwok to be optimized
net = topologyMaxQPseudolik!(ntwk,empCFs) # optimize network parameters to obtain the pseudo-likelihood score
````

#### Note:

Inferring numerical parameters is only suggested for networks that are fully resolved, such as those from the function `resolveLevel1`. Attempted optimization on networks that are partially resolved, where some multifurcations represent unknown blob structures, may lead to erroneous inference.

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{cutDensity}
\alias{cutDensity}
\title{Probability density function for Cut Model}
\usage{
cutDensity(lambda, mu0, alpha0, beta0)
}
\arguments{
\item{lambda}{statistic value (e.g., likelihood ratio statistic, or other power divergence statistic)}

\item{mu0}{parameter of density function}

\item{alpha0}{parameter of density function}

\item{beta0}{parameter of density function}
}
\value{
value of density function
}
\description{
Value of probability density function for Cut Model of \insertCite{ABMR24;textual}{MSCquartets}, Section 3.
}
\references{
\insertRef{ABMR24}{MSCquartets}
}
\seealso{
\code{\link{T1density},\link{T3density}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{expmodNANUQCycleDist}
\alias{expmodNANUQCycleDist}
\title{Expected modNANUQ cycle distance}
\usage{
expmodNANUQCycleDist(n)
}
\arguments{
\item{n}{number of edges in cycle (at least 4)}
}
\value{
an nxn distance matrix with rows/columns ordered from hybrid following circular order
}
\description{
Compute expected modNANUQ distance for a sunlet network with 4 or more taxa.
This is used in a hueristic method for resolving multifurcation in a tree
of blobs to a cycle by NANUQ+ commands.
}
\examples{
expmodNANUQCycleDist(5)

#'@seealso \code{\link{resolveCycle}} \code{\link{ordersHeuristicmodNANUQ}}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetDist.R
\name{quartetWeightedDist}
\alias{quartetWeightedDist}
\title{Compute the Weighted Quartet Distance between taxa}
\usage{
quartetWeightedDist(dqt)
}
\arguments{
\item{dqt}{an (\code{n} choose 4) x (\code{n+1}) matrix of the form output by \code{quartetTableDominant}}
}
\value{
a pairwise distance matrix on \code{n} taxa
}
\description{
Compute the Weighted Quartet Distance between taxa of \insertCite{YR19;textual}{MSCquartets} from a table specifying a collection of quartets on
\code{n} taxa and the quartets' internal branch lengths.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
DQT=quartetTableDominant(RQT,bigweights="finite")
D=quartetWeightedDist(DQT)
tree=NJ(D)
stree=WQDSAdjustLengths(tree)
write.tree(stree)

}
\references{
\insertRef{YR19}{MSCquartets}
}
\seealso{
\code{\link{quartetTableDominant}},
          \code{\link{WQDSAdjustLengths}},
          \code{\link{WQDS}},
          \code{\link{WQDC}}, 
          \code{\link{WQDCrecursive}},
          \code{\link{quartetWeightedDist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WQDCrecursive.R
\name{estimateEdgeLengths}
\alias{estimateEdgeLengths}
\title{Estimate edge lengths on a species tree from gene tree quartet counts}
\usage{
estimateEdgeLengths(tree, rqt, terminal = 1, method = "simpleML", lambda = 1/2)
}
\arguments{
\item{tree}{a phylo object, giving a resolved tree on which to estimate edge lengths}

\item{rqt}{a resolved quartet table, as from \code{quartetTableResolved},
in which all taxa on \code{tree} appear}

\item{terminal}{an edge length to assign to terminal edges, whose lengths cannot be estimated}

\item{method}{\code{"simpleML"} or \code{"simpleBayes"}}

\item{lambda}{a positive parameter for the \code{"simpleBayes"} method}
}
\value{
an unrooted metric tree with the same topology as \code{tree}, of type phylo
}
\description{
Estimate edge lengths, in coalescent units, on an unrooted species tree 
from a table of resolved quartet counts from a collection of gene trees.
}
\details{
While the argument \code{tree} may be supplied as rooted or unrooted, metric or topological,
only its unrooted topology will be used for determining new metric estimates.

Counts of quartets for all those quartets which define a single edge
on the tree (i.e., whose internal edge is the
single edge on the unrooted input tree) are summed, and from this an
estimate of the branch length is computed. If \code{method= "simpleML"} this is the maximum likelihood estimate.
If \code{method="simpleBayes"} this is the Bayesian estimate of Theorem 2
of \insertCite{SayMir16;textual}{MSCquartets}, using parameter \code{lambda}.
Using \code{lambda=1/2}
gives a flat prior on [1/3,1] for the probability of the quartet displayed on the species tree.

These methods are referred to as `simple' since they use only the quartets defining a single edge of the species tree.
Quartets with central edges composed of several edges in the species tree are ignored.

Note that branch length estimates may be 0 (if the count for the quartet
displayed on the input tree is not dominant),
positive, or \code{Inf} (if
the counts for quartet topologies not displayed on the input tree are all 0, and \code{method="simpleML"}).
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
taxanames=taxonNames(gtrees)
QT=quartetTable(gtrees,taxanames[1:6])
RQT=quartetTableResolved(QT)
DQT=quartetTableDominant(RQT)
tree=QDS(DQT)
write.tree(tree)
plot(tree)
metricMTree=estimateEdgeLengths(tree,RQT,method="simpleML")
write.tree(metricMTree)
plot(metricMTree)
metricBTree=estimateEdgeLengths(tree,RQT,method="simpleBayes")
write.tree(metricBTree)
plot(metricBTree)

}
\references{
\insertRef{SayMir16}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{compatibleSplits}
\alias{compatibleSplits}
\title{Extract compatible splits}
\usage{
compatibleSplits(sp, tol = 0, plot = FALSE)
}
\arguments{
\item{sp}{an object of class splits}

\item{tol}{splits with weights below tol are dropped}

\item{plot}{a logical; if TRUE plots tree displaying remaining spilts}
}
\value{
splits objects containing only those that are compatible and high weight
}
\description{
Given an object of class splits, first discards any with weight less than a tolerance, and
then further removes all remaining splits that are incompatible with any other remaining one.
}
\examples{
data(pTableYeastRokas)
dist=NANUQdist(pTableYeastRokas, alpha=.05, beta=.80,outfile=NULL)
nn=neighborNet(dist)
plot(nn,"2D")
tob=treeFromSplits(compatibleSplits(nn$splits),plot=TRUE) #produce tree of blobs of splits graph

}
\seealso{
\code{\link{treeFromSplits}, \link{TINNIK}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimplexPlot.R
\name{simplexPoint}
\alias{simplexPoint}
\title{Plot point in 2-d probability simplex}
\usage{
simplexPoint(v, ...)
}
\arguments{
\item{v}{a 3-d point in non-negative orthant, coordinates not summing to 0}

\item{...}{other options to pass to graphics::points function}
}
\description{
Normalizes a point given in 3-d non-normalized coordinates, then plots it in 
the 2-d probability simplex.
}
\examples{
   simplexPrepare("T3","Example Plot")
   simplexPoint(c(15,65,20),pch=3,col="blue")

}
\seealso{
\code{\link{simplexLabels}},
         \code{\link{simplexPrepare}},
         \code{\link{simplexSegment}},
         \code{\link{simplexText}},
         \code{\link{simplexCoords}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ExpectedCFs.R
\name{expectedCFs}
\alias{expectedCFs}
\title{Produce table of expected quartet concordance factors for a species tree}
\usage{
expectedCFs(speciestree, plot = TRUE, model = "T1", cex = 1)
}
\arguments{
\item{speciestree}{phylo or character object specifying un/rooted metric species tree}

\item{plot}{\code{TRUE} (default) to produce simplex plot of CFs, or \code{FALSE} to omit plot}

\item{model}{"T1" or "T3" specifying model for plot}

\item{cex}{scaling factor for size of plotted symbols}
}
\value{
an (\code{n} choose 4)x(\code{n}+3) matrix encoding
    4 taxon subsets of taxa and expectation of each of the
    quartets 12|34, 13|24, 14|23 across gene trees
}
\description{
Compiles table of expected quartet concordance factors (CFs) for gene trees under the MSC model 
on a metric species tree.
}
\details{
The species tree may be rooted or unrooted, but must have edge lengths given in coalescent units. Note that while the MSC
requires a rooted metric species tree parameter, as shown in \insertCite{Allman2011}{MSCquartets} the quartet CFs are independent 
of the root.

In the returned table, columns are labeled by taxon names and quartet names ("12|34", etc.).
1s and 0s in taxon columns indicate the taxa in a quartet. Quartet 12|34 means the first and second 
indicated taxa form a cherry, 13|24 means the first and third form a cherry, and 14|23 means
the first and fourth form a cherry. Unresolved quartets always have expectation 0 under the MSC.

If a simplex plot is produced, for the \code{T1} model all CFs will lie on the vertical model line, 
and many choices of 4 taxa will give the same CFs. For model \code{T3} the model lines the CFs are 
plotted on depend on taxon names and are essentially arbitrary.
}
\examples{
stree="((((t5:5000,t6:5000):5000,t4:10000):2500,t7:12500):7500,((t8:3000,t9:3000):5000,
((t1:4000,t2:4000):2500,t3:6500):1500):12000);"
st=read.tree(text=stree)
st$edge.length=st$edge.length/10000
expectedCFs(st)

}
\references{
\insertRef{Allman2011}{MSCquartets}
}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableResolved}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetDist.R
\name{WQDSAdjustLengths}
\alias{WQDSAdjustLengths}
\title{Adjust edge lengths on tree built from Weighted Quartet distance 
to estimate metric tree}
\usage{
WQDSAdjustLengths(tree)
}
\arguments{
\item{tree}{an unrooted metric tree, of type phylo}
}
\value{
an unrooted metric tree, of type phylo
}
\description{
Modify edge lengths of a tree built from a distance table produced by \code{quartetWeightedDist},
to remove scaling factors related to the size of the split associated to the edge.
}
\details{
As explained by \insertCite{YR19;textual}{MSCquartets}, a metric tree produced from
the weighted quartet distance has edge lengths
inflated by a factor dependent on the associated split size. Removing these
factors 
yields a consistent estimate of the metric species tree displaying the weighted
quartets, if such a tree exists.

This function should not be used on trees output from \code{WQDS}, \code{WQDC},
or \code{WQDCrecursive}, as 
their edges are already adjusted. It can be used on trees built from the distance
computed by \code{quartetWeightedDist}.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
DQT=quartetTableDominant(RQT,bigweights="finite")
D=quartetWeightedDist(DQT)
tree=NJ(D)
write.tree(tree)
plot(tree)
stree=WQDSAdjustLengths(tree)
write.tree(stree)
plot(stree)

}
\references{
\insertRef{YR19}{MSCquartets}
}
\seealso{
\code{\link{WQDS}},
\code{\link{WQDC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ParallelTableCpp.R
\name{quartetTableParallel}
\alias{quartetTableParallel}
\title{Produce table of counts of quartets displayed on trees, in parallel for large data sets}
\usage{
quartetTableParallel(trees, taxonnames = NULL, epsilon = 0, numCores)
}
\arguments{
\item{trees}{multiphylo object containing un/rooted metric/topological trees}

\item{taxonnames}{vector of \code{n} names of taxa of interest; if \code{NULL} then taken from taxa on \code{trees[[1]]}}

\item{epsilon}{minimum for branch lengths to be treated as non-zero}

\item{numCores}{number of cores to use for parallel calls}
}
\value{
an (\code{n} choose 4)x(\code{n}+4) matrix (or (\code{random})x(\code{n}+4) matrix) encoding
    4 taxon subsets of \code{taxonnames} and counts of each of the
    quartets 12|34, 13|24, 14|23, 1234 across the trees
}
\description{
Compiles table of quartet count concordance factors (qcCFs) for topological quartets displayed on a
collection of trees. Gives the same output as \code{quartetTable}, but operates in parallel.
}
\details{
The number of available cores can be determined by \code{parallel::detectCores()}.
With overhead, tabulating quartets for a large data set (many taxa and/or many gene trees) on a 4-core
computer using \code{numCores=4} may require less than half the elapsed time of the sequential \code{quartetTable}.

The names in \code{taxonnames} may be any subset of those on the trees.
Branch lengths of non-negative size less than or equal to \code{epsilon}
are treated as zero, giving polytomies.

In the returned table, columns are labeled by taxon names and quartet names ("12|34", etc.).
1s and 0s in taxon columns indicate the taxa in a quartet. Quartet 12|34
means the first and second indicated taxa form a cherry, 13|24 means the first and third form a cherry, 14|23 means
the first and fourth form a cherry, and 1234 means the quartet is unresolved.

An error occurs if any branch length is negative.
Warnings are given if some of \code{taxonnames} are missing on some trees, or
if some 4-taxon set is not on any tree.

If \code{random}>0, then for efficiency \code{random} should be much smaller then
the number of possible 4 taxon subsets.

If the quartet counts are to be used for NANUQ, or any other routines requiring resolved quartet counts,
\code{\link{quartetTableResolved}} must be run following \code{quartetTableParallel}. See example below.
}
\examples{
\donttest{
gtrees=read.tree(file=system.file("extdata","dataHeliconiusMartin",package="MSCquartets"))
QT=quartetTableParallel(gtrees,numCores=2)
RQT=quartetTableResolved(QT)
pTable=NANUQ(RQT,alpha=1e-40, beta=1e-30, outfile = file.path(tempdir(), "NANUQdist"))}

}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableResolved}}, \code{\link{quartetTableDominant}}, \code{\link{taxonNames}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{quartetBTinferencePlot}
\alias{quartetBTinferencePlot}
\title{Produce simplex plot with  results of B/T-quartet inference}
\usage{
quartetBTinferencePlot(pTable, Bquartets, test, alpha, beta, cex = 1)
}
\arguments{
\item{pTable}{table of quartets and p-values}

\item{Bquartets}{indicator vector for B-quartets (1=B, 0=T), ordered as in pTable}

\item{test}{test model used for tree null hypothesis; options are \code{"cut"}, \code{"T3"}}

\item{alpha}{significance level used by TINNIK for test \code{test}}

\item{beta}{significance level used by TINNIK for star tree test}

\item{cex}{scaling factor for size of plotted symbols}
}
\description{
Plot a 2-d probability simplex, with points for all normalized quartet count
vectors. Colors indicate B- or T-quartets from TINNIK algorithm, at specified
test levels.
}
\details{
The first argument of this function is a table of quartets and p-values. The
plot may show results using either the T3, or 2-cut
test, and a star tree test.
The p-values must be computed by or before previous calls to
\code{TINNIK}. The second argument is the indicator vector for B/T quartets produced by \code{TINNIK}.
}
\examples{
data(pTableYeastRokas)
out=TINNIKdist(pTableYeastRokas,test="T3",alpha=.05,beta=.05)
quartetBTinferencePlot(pTableYeastRokas,out$B,test="T3",alpha=.05,beta=.05)

}
\seealso{
\code{TINNIK}, \code{\link{quartetTestPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NANUQ.R
\name{NANUQ}
\alias{NANUQ}
\title{Apply NANUQ network inference algorithm to gene tree data}
\usage{
NANUQ(
  genedata,
  outfile = "NANUQdist",
  omit = FALSE,
  epsilon = 0,
  alpha = 0.05,
  beta = 0.8,
  taxonnames = NULL,
  plot = TRUE
)
}
\arguments{
\item{genedata}{gene tree data that may be supplied in any of 3 forms:
\enumerate{
\item as a character string giving the name of a file containing Newick gene trees,
\item as a multiPhylo object containing the gene trees, or
\item as a table of quartets on the gene trees, as produced by a previous call to
\code{NANUQ} or \code{quartetTableResolved}, which has columns only for taxa, resolved quartet counts,
and possibly p_T3 and p_star
}}

\item{outfile}{a character string giving an output file name stub for
saving a \code{NANUQ} distance matrix in nexus format; to the stub \code{outfile}
will be appended an \code{alpha} and \code{beta} value and ".nex";
if \code{NULL} then then no file is written}

\item{omit}{\code{FALSE} to treat unresolved quartets as 1/3 of each resolution;
\code{TRUE} to discard unresolved quartet data; ignored if gene tree data given as quartet table}

\item{epsilon}{minimum for branch lengths to be treated as non-zero; ignored if gene tree data given as quartet table}

\item{alpha}{a value or vector of significance levels for judging p-values
testing a null hypothesis of no hybridization vs. an alternative of hybridization, for each quartet;  a smaller value applies
a less conservative test for a tree (more trees), hence a stricter requirement for desciding in favor of hybridization (fewer reticulations)}

\item{beta}{a value or vector of significance levels for judging p-values testing
a null hypothesis of a star tree (polytomy) for each quartet vs. an alternative of anything else; a smaller value applies a less conservative
test for a star tree (more polytomies), hence a stricter requirement for deciding in favor of a resolved tree or network;
if vectors, \code{alpha} and \code{beta} must have the same length}

\item{taxonnames}{if \code{genedata} is a file or a multiPhylo object, a vector of a subset
of the taxa names on the gene trees
to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genedata}
is a quartet table, this argument is ignored and all taxa in the table are used}

\item{plot}{\code{TRUE} produces simplex plots of hypothesis test results, \code{FALSE} omits plots}
}
\value{
a table \code{$pTable} of quartets and p-values for judging fit to the MSC on quartet
trees, and a distance table \code{$dist}, or list of distance tables, giving NANUQ distance (returned invisibly);
the table can be used as input to \code{NANUQ} or \code{NANUQdist} with new choices of alpha and beta, without re-tallying quartets on
gene trees; the distance table is to be used as input to NeighborNet.
}
\description{
Apply the NANUQ algorithm of \insertCite{ABR19;textual}{MSCquartets} to infer a hybridization network from a collection of gene trees,
under the level-1 network multispecies coalescent (NMSC) model.
}
\details{
This function
\enumerate{
\item counts displayed quartets across gene trees to form quartet count concordance factors (qcCFs),
\item applies appropriate hypothesis tests to judge qcCFs as representing putative hybridization,
resolved trees, or unresolved (star) trees using \code{alpha} and \code{beta} as significance levels,
\item produces a simplex plot showing results of the hypothesis tests for all qcCFs
\item computes the appropriate NANUQ distance table, writing it to a file.
}
The distance table file
can then be opened in the external software SplitsTree \insertCite{SplitsTree}{MSCquartets} (recommended) or within R using the package \code{phangorn} to
obtain a circular split system under the Neighbor-Net algorithm, which is then depicted as a splits graph.
The splits graph should be interpreted via
the theory of \insertCite{ABR19;textual}{MSCquartets} to infer the level-1 species network, or to conclude the data does
not arise from the NMSC on such a network.

If \code{alpha} and \code{beta} are vectors, they must have the same length k. Then the i-th entries are paired to
produce k plots and k output files. This is equivalent to k calls to \code{NANUQ} with scalar values of \code{alpha} and \code{beta}.

A call of \code{NANUQ} with \code{genedata} given as a table previously output from \code{NANUQ} is
equivalent to a call of \code{NANUQdist}. If \code{genedata} is a table previously output from \code{quartetTableResolved}
which lacks columns of p-values for hypothesis tests, these will be appended to the table output by \code{NANUQ}.

If plots are produced, each point represents an empirical quartet concordance factor,
color-coded to represent test results.

In general, \code{alpha} should be chosen to be small and \code{beta}
to be large so that most quartets are interpreted as resolved trees.

Usually, an initial call to \code{NANUQ} will not give a good analysis, as values
of \code{alpha} and \code{beta} are likely to need some adjustment based on inspecting the data. Saving the returned
table from \code{NANUQ} will allow for the results of the time-consuming computation of qcCFs to be
saved, along with p-values,
for input to further calls of \code{NANUQ} with new choices of \code{alpha} and \code{beta}.

See the documentation for \code{\link{quartetNetworkDist}} for an explanation of a small, rarely noticeable,
stochastic element of the algorithm.

For data sets of many gene trees, user time may be reduced by using parallel code for
counting displayed quartets. See \code{\link{quartetTableParallel}}, where example commands are given.
}
\examples{
data(pTableYeastRokas)
out=NANUQ(pTableYeastRokas, alpha=.05, beta=.80, outfile = NULL)
# Specifying an outfile would write the distance table to it for opening in SplitsTree.
# Alternately, to use the phangorn implementation of NeighborNet
# within R, enter the following additional lines:
nn=neighborNet(out$dist)
plot(nn,"2D")

}
\references{
\insertRef{ABR19}{MSCquartets}

\insertRef{SplitsTree}{MSCquartets}
}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableParallel}}, \code{\link{quartetTableDominant}}, \code{\link{quartetTreeTestInd}},
\code{\link{quartetStarTestInd}}, \code{\link{NANUQdist}}, \code{\link{quartetTestPlot}}, \code{\link{pvalHist}},
\code{\link{quartetNetworkDist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QuartetTally.R
\name{quartetTablePrint}
\alias{quartetTablePrint}
\title{Print a quartet table with nice formatting}
\usage{
quartetTablePrint(qt)
}
\arguments{
\item{qt}{a table such as returned by \code{quartetTable}, \code{quartetTableResolved},
or \code{quartetTableDominant}, possibly with extra columns added by other functions}
}
\description{
Print a quartet table with the taxa in each quartet shown by name.
}
\examples{
 gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
 tnames=taxonNames(gtrees)
 QT=quartetTable(gtrees,tnames[1:6])
 QT[1:6,]
 quartetTablePrint(QT[1:6,])
 RQT=quartetTableResolved(QT)
 RQT[1:6,]
 quartetTablePrint(RQT[1:6,])
 pTable=quartetTreeTestInd(RQT,"T3")
 pTable[1:6,]
 quartetTablePrint(pTable[1:6,])
 DQT=quartetTableDominant(RQT)
 DQT[1:6,]
 quartetTablePrint(DQT[1:6,])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{quartetCutTest}
\alias{quartetCutTest}
\title{Hypothesis test for quartet counts fitting a resolved quartet tree of blobs under NMSC}
\usage{
quartetCutTest(
  obs,
  lambda = 0,
  method = "MLest",
  smallcounts = "approximate",
  bootstraps = 10^4
)
}
\arguments{
\item{obs}{vector of 3 counts of resolved quartet frequencies}

\item{lambda}{parameter for power-divergence statistic (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)}

\item{method}{\code{"MLtest"},\code{"conservative"}, or \code{"bootstrap"}}

\item{smallcounts}{\code{"bootstrap"} or \code{"approximate"}, method of obtaining p-value when some counts are small}

\item{bootstraps}{number of samples for bootstrapping}
}
\value{
output where \code{output$p.value} is a p-value and \code{output$topology} = 1, 2, or 3
indicates the ML estimate of the topology of the quartet tree of blobs in accord with ordering of qcCF entries.
}
\description{
Test the hypothesis H_0=Cut model of \insertCite{ABMR24;textual}{MSCquartets},
Section 3., vs. H_1= everything else.
Returns p-value and estimate of tree of blobs topology.
}
\details{
The Cut model for quartet CFs is the NMSC combined with the quartet species network having
a cut edge separating two of the taxa from the other two.

This function implements the test described in \insertCite{ABMR24;textual}{MSCquartets}
as well as parametric bootstrapping,
with other procedures for when some expected counts are small.
These are more accurate tests than, say, a Chi-square with one degree of freedom,
which is not theoretically
justified near the singularity of the model, nor for small counts.

If \code{method="MLtest"}, this uses the test for the Cut model described
in Section 3 of \insertCite{ABMR24;textual}{MSCquartets}, using the ML
estimate of the generating parameter.
As shown in simulations in that paper, the test is conservative when small
levels are used for rejection.
Although the test generally performs well in practice, it lacks a uniform
asymptotic guarantee over the full parameter space.

If \code{method="conservative"}, the test uses the Chi-square distribution with 1 degree of freedom
(the "least favorable" approach). This is asymptotically guaranteed to reject the null
hypothesis at most at a specified level, but at the expense of increased type II errors.

If \code{method="bootstrap"}, then parametric bootstrapping is performed,
based on ML estimates of the CF. The bootstrap sample size is given by the \code{bootstrap} argument.

When some expected topology counts are small, the methods \code{"MLest"} and \code{"conservative"}
are not appropriate.
The argument \code{smallcounts} determines whether bootstrapping or a faster approximate method is used.
These use ML estimates of the CF under the Cut model.

If two of the three counts are small (so the estimated CF is near a vertex of the simplex),
The approximate approach
returns a precomputed p-value, found by replacing the largest observed count
with 1e+6 and performing 1e+8 bootstraps. When n is sufficiently large (at least 30) and
some expected counts are small, the probability of topological error  is small and the bootstrap p-value is
approximately independent of the largest observed count.

If one of the three counts is small (so the estimated CF is near an edge of the simplex),
a chi-squared test using the binomial model for the larger counts is used, as described
by \insertCite{ABMR24;textual}{MSCquartets}.

The returned p-value should be taken with caution when there is a small sample size, e.g. less than 30 gene trees.
}
\examples{
 quartetCutTest(c(17,72,11))
 quartetCutTest(c(48,11,41))
 quartetCutTest(c(11,48,41))
 quartetCutTest(c(48,41,11))

}
\references{
\insertRef{ABMR22}{MSCquartets}

\insertRef{ABMR24}{MSCquartets}

\insertRef{ABRW25}{MSCquartets}

\insertRef{MAR19}{MSCquartets}
}
\seealso{
\code{\link{quartetCutTestInd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{data}
\name{tableHeliconiusMartin}
\alias{tableHeliconiusMartin}
\title{Quartet table for Heliconius gene tree dataset}
\format{
an R data file
}
\source{
\doi{10.5061/dryad.dk712}
}
\usage{
data(tableHeliconiusMartin)
}
\description{
An .rda file containing a quartet table summarizing the "Heliconius" gene trees
of \insertCite{Martin2013;textual}{MSCquartets}. This table contains quartet counts
from 2909 gene trees on 7 taxa, with 4 individuals sampled for each of 3 of the taxa,
for a total of 16 leaves per gene tree.
}
\details{
This table is provided rather than the original gene trees to save storage space.
If used, please cite \insertCite{Martin2013;textual}{MSCquartets}.
}
\references{
\insertRef{Martin2013}{MSCquartets}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{quartetTreeTestInd}
\alias{quartetTreeTestInd}
\title{Multiple independent hypothesis tests for quartet counts fitting a species tree under the MSC}
\usage{
quartetTreeTestInd(
  rqt,
  model = "T3",
  lambda = 0,
  method = "MLest",
  smallsample = "precomputed",
  smallcounts = "precomputed",
  bootstraps = 10^4,
  speciestree = NULL
)
}
\arguments{
\item{rqt}{table of resolved quartet counts, as produced by \code{quartetTableResolved}, or \code{quartetStarTestInd}}

\item{model}{\code{"T1"} for a specific species tree topology, or \code{"T3"} for any species tree topology, with these
models explained more fully by \insertCite{MAR19;textual}{MSCquartets}}

\item{lambda}{power divergence statistic parameter (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)}

\item{method}{\code{"MLest"}, \code{"conservative"}, or \code{"bootstrap"}; see \code{quartetTreeTest} for explanation}

\item{smallsample}{\code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when sample is small (<30)}

\item{smallcounts}{\code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when some counts are small, so
the chosen \code{method} is inappropriate}

\item{bootstraps}{number of samples for bootstrapping}

\item{speciestree}{species tree, in Newick as text, to determine quartet for T1 test; required for \code{model="T1"},
ignored for \code{model="T3"}}
}
\value{
if \code{model="T3"}, a copy of \code{rqt} with a new column \code{"p_T3"} appended with p-values for each quartet;
  if \code{model="T1"}, a copy of \code{rqt} with 2 columns appended: \code{"p_T1"} with p-values, and \code{"qindex"}
  giving index of quartet consistent with specified species tree,
  i.e., 1 if 12|34 on species tree, 2 if 13|24, 3 if 14|23
}
\description{
Perform a tree hypothesis test for all quartet counts in an input table, as if the counts for different choices of 4 taxa
are independent.
}
\details{
This function assumes all quartets are resolved.  The test performed and the arguments
are described more fully in \code{quartetTreeTest}.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=c("t1","t2","t3","t4","t5","t6")
QT=quartetTable(gtrees,tnames)
RQT=quartetTableResolved(QT)
stree="(((t5,t6),t4),((t1,t2),t3));"
pTable3=quartetTreeTestInd(RQT,"T3")
quartetTablePrint(pTable3[1:6,])
stree="((((t5,t6),t4),t7),((t8,t9),((t1,t2),t3)));"
pTable1=quartetTreeTestInd(RQT,"T1",speciestree=stree)
quartetTablePrint(pTable1[1:6,])

}
\references{
\insertRef{MAR19}{MSCquartets}
}
\seealso{
\code{\link{quartetTreeTest}}, \code{\link{quartetTestPlot}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetTableResolved}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{resolveLevel1}
\alias{resolveLevel1}
\title{Resolve Tree of Blobs to Level-1 network}
\usage{
resolveLevel1(
  ToB,
  pTable,
  test = "T3",
  alpha,
  beta,
  distance = "NANUQ",
  hdegree = 10,
  plot = 2,
  delta = 10^-6,
  fullResMax = 10
)
}
\arguments{
\item{ToB}{an unrooted tree of blobs (phylo object) as determined by ECToBlob, TINNIK or
another method}

\item{pTable}{a table of qcCFs, with columns p_star and p_test}

\item{test}{either "T3" or "cut", indicating test to use for determining what
qcCFs indicate hybridization}

\item{alpha}{test level for p_test}

\item{beta}{test level value for p_star}

\item{distance}{cycle resolution distance to be used ("NANUQ" or "modNANUQ")}

\item{hdegree}{resolve a multifurcation of this degree or larger by a heuristic
method; must be at least 5}

\item{plot}{if 0, no plots; if 1, plot only possible root locations on ToB and
full resolution; if 2 (default), include plots of each individual blob resolution, if 3
include histograms of measure of fit for all hybrid/orders considered in choosing
best}

\item{delta}{cutoff for relative difference in squared residuals and smallest,
(RSS-minRSS)/minRSS, for determining near ties as "best" fit resolutions}

\item{fullResMax}{maximum number of full resolutions (all multifurcations at once)
to form; if the product of the number of resolutions of individual multifurcations
exceeds this, no full resolutions are produced, although \code{\link{combineCycleResolutions}}
can be applied to produce them.}
}
\value{
a list of resolutions and squared residuals:
\itemize{
\item [[1]] is a list of Newick
resolutions of entire network, with all edge lengths 1 (NULL if one cannot be produced or \code{fullResMax} is exceeded),
\item [[2]]-[[n]] are individual resolutions of each  multifurcation on \code{ToB},
each given as a list as output from \code{\link{resolveCycle}}.
}
}
\description{
Given a Tree of Blobs and qcCF information, resolve all multifurcations to
cycles. Resolution is performed by finding a least-squares best-fit of an
empirical distance to an expected distance related to the cycle, as described
in \insertCite{ABRW25;textual}{MSCquartets}.
}
\details{
Possible distances to use are the NANUQ distance and a modified NANUQ distance
that weights quartet trees differently, but from which the cycle structure is
still identifiable.

For multifucations of degree less than a designated cutoff, all possible
circular orders and choices of hybrid nodes are considered in choosing the
best. Above that cutoff, a heuristic method  is used to obtain
a small number of orders likely to be good fits, with the least-squares
fitting applied only to those.
}
\examples{
data(pTableYeastRokas)
out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)
resolveLevel1(out$ToB, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")

}
\references{
\insertRef{ABRW25}{MSCquartets}
}
\seealso{
\code{\link{ECToBlob}}, \code{\link{TINNIK}}, \code{\link{labelIntNodes}}, \code{\link{resolveCycle}},
\code{\link{combineCycleResolutions}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimplexPlot.R
\name{simplexText}
\alias{simplexText}
\title{Add text at a point in 2-d probability simplex}
\usage{
simplexText(v, label = "", ...)
}
\arguments{
\item{v}{a 3-d point in non-negative orthant, coordinates not summing to 0}

\item{label}{text to add to plot}

\item{...}{other options to pass to graphics::text function}
}
\description{
Add text to a 2-d probability simplex plot, at specified location.
}
\examples{
   simplexPrepare("T3","Example Plot")
   simplexText(c(15,65,20),"tree ac|bd")

}
\seealso{
\code{\link{simplexLabels}},
         \code{\link{simplexPoint}},
         \code{\link{simplexPrepare}},
         \code{\link{simplexSegment}},
         \code{\link{simplexCoords}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{treeOfBlobs}
\alias{treeOfBlobs}
\title{Tree of blobs for a network}
\usage{
treeOfBlobs(net, plot = FALSE)
}
\arguments{
\item{net}{A network, supplied as an extended Newick string, an evonet object, or an igraph object}

\item{plot}{if TRUE (default), plot the tree of blobs.}
}
\value{
An object of class \code{phylo} containing the unrooted topological tree
derived from the network by contracting all blobs.  All edge lengths are 1.
}
\description{
Given extended newick, an evonet object, or an igraph object for a network, return its reduced, unrooted
tree of blobs. Here 'reduced' means all nodes resulting from 2-blobs are suppressed, as are edges above
the network's LSA.
}
\examples{
network = "(((a:1,d:1):1,(b:1)#H1:1):1,(#H1,c:1):2);"
plot(read.evonet(text=network))
treeOfBlobs(network, plot=TRUE)

}
\seealso{
\code{\link{TINNIK}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimplexPlot.R
\name{simplexSegment}
\alias{simplexSegment}
\title{Plot line segment in 2-d probability simplex}
\usage{
simplexSegment(v, w, ...)
}
\arguments{
\item{v, w}{3-d endpoints of line segment in non-negative orthant, coords not summing to 0}

\item{...}{other options to pass to graphics::segments function}
}
\description{
Normalizes two points in 3-d, and draws line segment between them in 2-d probability simplex.
}
\examples{
   simplexPrepare("T3","Example Plot")
   simplexSegment(c(15,65,20),c(15,70, 15),col="green")

}
\seealso{
\code{\link{simplexLabels}},
         \code{\link{simplexPoint}},
         \code{\link{simplexPrepare}},
         \code{\link{simplexText}},
         \code{\link{simplexCoords}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{HolmBonferroni}
\alias{HolmBonferroni}
\title{Apply Holm-Bonferroni method to adjust for multiple tests}
\usage{
HolmBonferroni(pTable, model, alpha = 0.05)
}
\arguments{
\item{pTable}{a table of quartets with p-values, as computed by
\code{quartetTreeTestInd} or \code{quartetStarTestInd}}

\item{model}{one of \code{"T1"}, \code{"T3"}, or \code{"star"}, where \code{pTable} contains a column \code{p_model} of p-values}

\item{alpha}{test level, for rejection of adjusted p-values less than or equal to \code{alpha}}
}
\value{
the same table, with rows reordered, and 2 new columns of 1) adjusted p-values,
and 2) "Y" or "N" for indicating "reject" or "fail to reject"
}
\description{
Apply the Holm-Bonferroni method to
adjust for multiple hypothesis tests performed on quartets from a data set of gene trees.
}
\details{
When p-values are computed for each quartet using
\code{quartetTreeTestInd} or \code{quartetStarTestInd},
multiple comparisons are being done for one dataset. The
Holm-Bonferroni method \insertCite{HolmBonf79}{MSCquartets} adjusts these p-values upward,
controlling the familywise error rate. The probability
of at least one false discovery (rejection of the null hypothesis)
is no more than the significance level.

The Holm-Bonferroni method does not require that test hypotheses are independent, which
is important for its application to quartet counts presumed to have arisen on a single
species tree.

This can be a low power test (often failing to reject when the null hypothesis is false).
In particular for the T1 and T3 tests, it does not consider the relationships between edge
lengths for different sets of four taxa.

Warning: Output of this function should not be used as input for other
MSCquartets functions, as it reorders the quartets in the table.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
taxanames=taxonNames(gtrees)
QT=quartetTable(gtrees,taxanames[1:6])
RQT=quartetTableResolved(QT)
pTable=quartetTreeTestInd(RQT,"T3")
pTable[1:10,]
HBpTable=HolmBonferroni(pTable,"T3",.05)
HBpTable[1:10,]

}
\references{
\insertRef{HolmBonf79}{MSCquartets}
}
\seealso{
\code{\link{quartetTreeTestInd}}, \code{\link{quartetStarTestInd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimplexPlot.R
\name{simplexPrepare}
\alias{simplexPrepare}
\title{Draw 2-d probability simplex, with model lines for T3 or T1 model}
\usage{
simplexPrepare(model = "T3", maintitle = NULL, titletext = NULL)
}
\arguments{
\item{model}{\code{"T1"} or \code{"T3"}, for 1-tree or 3-tree model}

\item{maintitle}{main title for plot}

\item{titletext}{additional text for title}
}
\description{
Outline the 2-d probability simplex, and draw the T1 or T3 model points for quartet frequencies.
The models "T1" and "T3" are described more fully by \insertCite{MAR19;textual}{MSCquartets}.
}
\examples{
   simplexPrepare("T3",maintitle="Main title",titletext="further text")

}
\references{
\insertRef{MAR19}{MSCquartets}
}
\seealso{
\code{\link{simplexLabels}},
         \code{\link{simplexPoint}},
         \code{\link{simplexSegment}},
         \code{\link{simplexText}},
         \code{\link{simplexCoords}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetDist.R
\name{QDS}
\alias{QDS}
\title{Compute Quartet Distance Supertree}
\usage{
QDS(dqt, method = fastme.bal)
}
\arguments{
\item{dqt}{an (\code{n} choose 4) x \code{n} (or \code{n+1}) matrix of form output by \code{quartetTableDominant};
(Note: If present, the \code{n+1}th column of \code{dqt} is ignored)}

\item{method}{tree building function (e.g., fastme.bal, nj)}
}
\value{
an unrooted metric tree of type phylo. Edge lengths are not in interpretable units
}
\description{
Apply the Quartet Distance Supertree method of \insertCite{Rho19;textual}{MSCquartets} to a table specifying a
collection of quartets on \code{n} taxa.
}
\details{
This function is a wrapper which runs \code{quartetDist} and then builds a tree.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
DQT=quartetTableDominant(RQT)
tree=QDS(DQT)
write.tree(tree)
plot(tree)

}
\references{
\insertRef{Rho19}{MSCquartets}
}
\seealso{
\code{\link{quartetTableDominant}},
\code{\link{quartetDist}},
\code{\link{QDC}},
\code{\link{WQDS}}, 
\code{\link{WQDC}}, 
\code{\link{WQDCrecursive}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetDist.R
\name{WQDC}
\alias{WQDC}
\title{Compute Weighted Quartet Distance Consensus tree from gene tree data}
\usage{
WQDC(
  genetreedata,
  taxanames = NULL,
  method = fastme.bal,
  omit = FALSE,
  terminal = 1
)
}
\arguments{
\item{genetreedata}{gene tree data that may be supplied in any of 3 forms:
\enumerate{
\item a character string giving the name of a file containing gene trees in Newick
\item a multiPhylo object containing gene trees
\item a resolved quartet table, as produced by \code{quartetTableResolved}
}}

\item{taxanames}{if \code{genetreedata} is a file or a multiPhylo object, a vector of a subset
of the taxa names on the gene trees 
to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genetreedata} 
is a quartet table, this argument is ignored and all taxa in the table are used}

\item{method}{a distance-based tree building function, such as \code{fastme.bal} or \code{nj}}

\item{omit}{\code{TRUE} leaves out unresolved quartets, \code{FALSE} treats them as 1/3 of each resolution; ignored if 
\code{genetreedata} is given as  a resolved quartet table}

\item{terminal}{non-negative branch length to supply for terminal branches 
whose length cannot be inferred by \code{WQDC}}
}
\value{
an unrooted metric tree of type phylo
}
\description{
Compute the Weighted Quartet Distance Consensus \insertCite{YR19}{MSCquartets} estimate of a 
species tree from gene tree data. This is a consistent estimator of the unrooted 
species tree topology and all internal branch lengths.
}
\details{
This function is a wrapper which performs the steps of reading in a collection
of gene trees, tallying quartets, estimating quartet internal branch lengths, computing the weighted
quartet distance between taxa, building
a tree, and adjusting edge lengths, to give a consistent estimate of the metric species tree in coalescent units
under the MSC.

If the gene tree data indicates some quartets experienced little to no incomplete lineage 
sorting, this algorithm tends to be less topologically accurate than \code{QDC} 
(which infers no metric information) or \code{WQDCrecursive} (which gives better topologies,
and reasonably accurate lengths for short edges, though long edge lengths may still be unreliable).
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
stree=WQDC(gtrees,tnames[1:6])
write.tree(stree)
plot(stree)

}
\references{
\insertRef{YR19}{MSCquartets}
}
\seealso{
\code{\link{quartetTable}}, 
         \code{\link{quartetTableResolved}}, 
         \code{\link{quartetTableDominant}}, 
         \code{\link{quartetWeightedDist}}, 
         \code{\link{WQDCrecursive}}, 
         \code{\link{WQDS}}, 
         \code{\link{QDC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quartetDistTree.R
\name{quartetDistTree}
\alias{quartetDistTree}
\title{Build a Species Tree from Gene Trees Using the Quartet Distance}
\usage{
quartetDistTree(
  genedata,
  method = ape::fastme.bal,
  taxanames = NULL,
  omit = FALSE
)
}
\arguments{
\item{genedata}{gene tree data, supplied in any of 3 forms:
\enumerate{
\item as a character string giving the name of a file containing Newick gene trees,
\item as a \code{multiPhylo} object containing the gene trees, or
\item as a table of quartet counts on the gene trees, as produced by
\code{quartetTable} or \code{quartetTableResolved}
}}

\item{method}{a function for constructing a tree from a distance matrix, such as \code{NJ} or
\code{fastme.bal}}

\item{taxanames}{used only if \code{genedata} is a filneame or multiphylo object;
restrict gene trees to these taxa; if \code{NULL}, use taxa on first tree}

\item{omit}{used only if \code{genedata} is a filneame,\code{multiPhylo} object, 
or a quartet table with column \code{'1234'};
if TRUE omit unresolved quartets, if \code{FALSE} treat them as 1/3 of each resolution}
}
\value{
A \code{phylo} object representing the inferred tree.
}
\description{
This function takes a collection of gene trees and uses a
species tree by first computing the quartet distance of 
\insertCite{Rho19;textual}{MSCquartets}
from the gene trees and then applying a distance method.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tree<-quartetDistTree(gtrees)
write.tree(tree)
plot(tree)

}
\references{
\insertRef{Rho19}{MSCquartets}
}
\seealso{
\code{quartetDist}, \code{QDS}, \code{quartetTableDominant}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StarQuartetHypTest.R
\name{quartetStarTest}
\alias{quartetStarTest}
\title{Hypothesis test for quartet counts fitting a star tree under the MSC}
\usage{
quartetStarTest(obs)
}
\arguments{
\item{obs}{vector of 3 counts of resolved quartet frequencies}
}
\value{
p-value
}
\description{
Perform hypothesis test for star tree for a vector of  quartet counts to fit expected frequencies of (1/3,1/3,1/3).
The test performed is a standard Chi-square with 2 degrees of freedom.
}
\examples{
obs=c(16,72,12)
quartetStarTest(obs)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StarQuartetHypTest.R
\name{quartetStarTestInd}
\alias{quartetStarTestInd}
\title{Multiple independent hypothesis tests for gene quartet counts fitting 
a species quartet star tree under the MSC}
\usage{
quartetStarTestInd(rqt)
}
\arguments{
\item{rqt}{Table of resolved quartet counts, as produced by \code{quartetTableResolved}, or \code{quartetTreeTestInd}}
}
\value{
the same table as the input \code{rqt} with column \code{"p_star"} appended, containing p-values for 
judging fit to MSC on a star tree
}
\description{
Perform hypothesis tests for a species quartet star tree vs. any alternative for all quartet counts in an input table, 
as if the quartets are independent.
}
\details{
This function assumes all quartets are resolved. 
The test performed is described in \code{quartetStarTest}.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
pTable=quartetStarTestInd(RQT)
quartetTablePrint(pTable[1:6,])

}
\seealso{
\code{\link{quartetStarTest}}, \code{\link{quartetTreeTest}}, \code{\link{quartetTreeTestInd}},
\code{\link{quartetTableResolved}}, \code{\link{quartetTestPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{quartetTreeTest}
\alias{quartetTreeTest}
\title{Hypothesis test for quartet counts fitting a tree under the MSC}
\usage{
quartetTreeTest(
  obs,
  model = "T3",
  lambda = 0,
  method = "MLest",
  smallsample = "precomputed",
  smallcounts = "precomputed",
  bootstraps = 10^4
)
}
\arguments{
\item{obs}{vector of 3 counts of resolved quartet frequencies}

\item{model}{\code{"T1"} or \code{"T3"}, for the models of \insertCite{MAR19;textual}{MSCquartets}}

\item{lambda}{parameter for power-divergence statistic (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)}

\item{method}{\code{"MLtest"},\code{"conservative"}, or \code{"bootstrap"}}

\item{smallsample}{\code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when sample is small (<30)}

\item{smallcounts}{\code{"precomputed"} or \code{"bootstrap"}, method of obtaining p-value when some (but not all) counts are small}

\item{bootstraps}{number of samples for bootstrapping}
}
\value{
\code{output} where \code{output$p.value} is the p-value and \code{output$edgelength} is a consistent estimator of the
internal edge length in coalescent units, possibly \code{Inf}.
}
\description{
Test the hypothesis H_0= T1 or T3 model of \insertCite{MAR19;textual}{MSCquartets}, vs. H_1 = everything else.
T1 is for a specific species quartet topology, and T3 for any species quartet topology.
}
\details{
This function implements two of
the versions of the test given by \insertCite{MAR19;textual}{MSCquartets} as well as parametric
bootstrapping, with other procedures for when some expected counts are small.
When the topology and/or the internal
quartet branch length is not specified by the null hypothesis these are more accurate tests than,
say, a Chi-square with one degree of freedom, which is not theoretically
justified near the singularities and boundaries of the models.

If \code{method="MLtest"}, this uses the test by that name described in Section 7 of \insertCite{MAR19;textual}{MSCquartets}.
For both the T1 and T3 models the test is slightly anticonservative over a small range of true internal edges of the quartet species tree.
Although the test generally performs well in practice, it lacks a uniform asymptotic guarantee over
the full parameter space for either T1 or T3.

If \code{method="conservative"}, a conservative test described by \insertCite{MAR19;textual}{MSCquartets} is used. For model T3 this
uses the Chi-square distribution with 1 degree of freedom
(the "least favorable" approach), while for model T1
it uses the Minimum Adjusted Bonferroni, based on precomputed values from simulations with n=1e+6.
These conservative tests are asymptotically guaranteed to reject the null
hypothesis at most at a specified level, but at the expense of increased type II errors.

If \code{method="bootstrap"}, then parametric bootstrapping is performed, based on parameter estimates of the quartet topology
and internal edge length. The bootstrap sample size is given by the \code{bootstrap} argument.


When some or all expected topology counts are small, the methods \code{"MLest"} and \code{"conservative"} are not appropriate.
The argument \code{smallsample} determines whether a precomputed bootstrap of 1e+8 samples, or actual boostrapping with the specified size,
is used when the total sample is small (<30).
The argument \code{smallcounts} determines whether bootstrapping or a faster approximate method is used when only some counts are small.
The approximate approach returns a precomputed p-value, found by replacing the largest observed count
with 1e+6 and performing 1e+8 bootstraps for the model T3. When n >30 and
some expected counts are small, the quartet tree error probability is small and the bootstrap p-value is
approximately independent of the choice of T3 or T1 and of the largest observed count.

For model T1, the first entry of \code{obs} is treated as the count of gene quartets concordant with the species tree.

The returned p-value should be taken with caution when there is a small sample size, e.g. less than 30 gene trees.
The returned value of \code{$edgelength} is a consistent estimator, but not the MLE, of the internal
edge length in coalescent units. Although consistent, the MLE for this length is biased.
Our consistent estimator is still biased, but with less bias than the MLE. See \insertCite{MAR19;textual}{MSCquartets}
for more discussion on dealing with the bias of parameter estimates in the
presence of boundaries and/or singularities of parameter spaces.
}
\examples{
 quartetTreeTest(c(17,72,11),"T3")
 quartetTreeTest(c(17,72,11),"T1")
 quartetTreeTest(c(72,11,17),"T1")
 quartetTreeTest(c(11,17,72),"T1")

}
\references{
\insertRef{MAR19}{MSCquartets}
}
\seealso{
\code{\link{quartetTreeTestInd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ECToBlob.R
\name{quartetECtestInd}
\alias{quartetECtestInd}
\title{Hypothesis testing of quartet counts for ECToBlob}
\usage{
quartetECtestInd(
  rqt,
  tree,
  lambda = 0,
  method = "MLest",
  smallsample = "precomputed",
  smallcounts = "precomputed",
  bootstraps = 10^4
)
}
\arguments{
\item{rqt}{either a table of resolved quartet counts from a collection of gene trees,
or that table with additional columns of quartet p-values.}

\item{tree}{an unrooted tree determining the topologies of the displayed quartet trees to test}

\item{lambda}{parameter for power-divergence statistic (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)}

\item{method}{"MLest", "conservative", or "bootstrap"}

\item{smallsample}{"precomputed" or "bootstrap", method of obtaining p-value when sample is small (<30)}

\item{smallcounts}{"precomputed" or "bootstrap", method of obtaining p-value when some (but not all) counts are small}

\item{bootstraps}{number of samples for bootstrapping}
}
\value{
the same table as \code{rqt} with three columns appended:
\code{qindex-D} containing the indicies (1,2,or 3)
of the quartet topology displayed on \code{tree}, \code{p_T1-D}
containing the p-value for the T1 test of the displayed quartet topologies.
\code{p_star} containing p-values for the star tree test.
}
\description{
Append p-values for the star and T1 tests of displayed quartets on a given
tree to resolved quartet count table. This is used in \code{ECToBlob}
to obtain an estimated Tree of Blobs for a data set from a supplied tree that
is assumed to be a resolution of the Tree of Blobs. Three columns are appended
to the quartet count table: "p_T1-D" holds the T1 test p-values, "qindexD" holds
the index (1,2,or 3) of the quartet topology displayed on the tree, and "p_star"
holds the star tree test p-value.
}
\details{
If on the tree a quartet is unresolved, p_T1 and qindex column entry will be NA

\code{tree} must be supplied as phylo object, but may have multifurcations.

If either p_star or p_T1-D and qindex-D values are already in table, they are
not recomputed.
}
\examples{
data(ASTRALtreeLeopardusLescroart)
Atree=read.tree(text=ASTRALtreeLeopardusLescroart)
data(tableLeopardusLescroart)
pTable=quartetECtestInd(tableLeopardusLescroart,Atree)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{allPerms}
\alias{allPerms}
\title{Generate permutations}
\usage{
allPerms(n)
}
\arguments{
\item{n}{size of permutations}
}
\value{
an n!xn matrix whose rows give permutations
}
\description{
Generate all permutations of 1 to n, as rows of a matrix
}
\examples{
allPerms(4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QuartetTally.R
\name{quartetTableDominant}
\alias{quartetTableDominant}
\title{Produce table of dominant quartets, with estimates of internal edge lengths}
\usage{
quartetTableDominant(rqt, bigweights = "infinite")
}
\arguments{
\item{rqt}{a table, as produced by \code{quartetTableResolved} of size (n choose 4)x(n+3);}

\item{bigweights}{\code{"infinite"} or \code{"finite"}, to indicate whether the weight (internal edge length)
of a quartet for which only one
topology appears is given as \code{Inf} or a finite, but large, numerical value}
}
\value{
an (n choose 4)x(n+1) array with dominant quartet topology encoded by 1,1,-1,-1 in
taxon columns, with signs indicating cherries; the (n+1)th column \code{"weight"} contains the maximum likelihood estimates,
under MSC on a 4-taxon tree, of the quartets' central edge lengths, in coalescent units
}
\description{
Converts table of counts of resolved quartets on \code{n} taxa to show only dominant one, with
maximum likelihood estimate of internal edge weight under the MSC.
}
\details{
If \code{bigweights="finite"}, when for a set of 4 taxa the quartet counts are (m,0,0) then
the edge weight is computed as if the relative frequency of the dominant topology were m/(m+1).
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
RQT[1:6,]
DQT=quartetTableDominant(RQT)
DQT[1:6,]

}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableResolved}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimplexPlot.R
\name{simplexLabels}
\alias{simplexLabels}
\title{Label vertices of 2-d probability simplex}
\usage{
simplexLabels(top = "", left = "", right = "")
}
\arguments{
\item{top}{label for top}

\item{left}{label for left bottom}

\item{right}{label for right bottom}
}
\description{
Add labels to vertices of the probability simplex.
}
\examples{
   simplexPrepare("T3","Example Plot")
   simplexLabels("ab|cd","ac|bd","ad|bc")

}
\seealso{
\code{\link{simplexPoint}},
         \code{\link{simplexPrepare}},
         \code{\link{simplexSegment}},
         \code{\link{simplexText}},
         \code{\link{simplexCoords}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{treeFromSplits}
\alias{treeFromSplits}
\title{Produce tree from compatible splits}
\usage{
treeFromSplits(sp, plot = FALSE)
}
\arguments{
\item{sp}{a compatible split system, as produced by compatibleSplits}

\item{plot, }{a logical, if TRUE, plot tree}
}
\value{
a phylo object for tree displaying splits
}
\description{
Produce tree from compatible splits
}
\examples{
data(pTableYeastRokas)
dist=NANUQdist(pTableYeastRokas, alpha=.05, beta=.80,outfile=NULL)
nn=neighborNet(dist)
plot(nn,"2D")
tob=treeFromSplits(compatibleSplits(nn$splits),plot=TRUE) #produce tree of blobs of splits graph

}
\seealso{
\code{\link{compatibleSplits}, \link{TINNIK}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NANUQ.R
\name{quartetNetworkDist}
\alias{quartetNetworkDist}
\title{Compute network quartet distance between taxa}
\usage{
quartetNetworkDist(pTable, alpha, beta)
}
\arguments{
\item{pTable}{a table of quartets and p-values, as computed by NANUQ, or
\code{quartetTreeTestInd} and \code{quartetStarTestInd}}

\item{alpha}{a scalar significance level for judging p-values \code{p_T3} indicating hybridization on quartet;
smaller value gives fewer hybridization decisions}

\item{beta}{a scalar significance level for judging p-values \code{p_star} indicating quartet star tree;
smaller value gives fewer resolved tree decisions}
}
\value{
a distance table
}
\description{
Produce network quartet distance table for the NANUQ algorithm, from a table of quartets and p-values,
and specified levels of quartet hypothesis tests. The network quartet distance, which
is described more fully by \insertCite{ABR19;textual}{MSCquartets}, generalizes
the quartet distance of \insertCite{Rho19;textual}{MSCquartets}.
}
\details{
In case of a triple of quartet counts with the two largest equal and the third slighltly smaller,
along with \code{alpha} and \code{beta} leading to a star quartet being rejected and a tree not being rejected,
this function chooses a resolved quartet topology uniformly at random from the two largest counts. This is the only
stochastic element of the code, and its impact is usually negligible.
}
\examples{
data(pTableYeastRokas)
dist=quartetNetworkDist(pTableYeastRokas, alpha=.05, beta=.80)

}
\references{
\insertRef{ABR19}{MSCquartets}

\insertRef{Rho19}{MSCquartets}
}
\seealso{
\code{\link{NANUQ}}, \code{\link{NANUQdist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ECToBlob.R
\name{combineP}
\alias{combineP}
\title{Combine p-values of tests}
\usage{
combineP(pvec, method = "Bon")
}
\arguments{
\item{pvec}{a vector of p-values to be combined}

\item{method}{the combination method to be used, one of \code{"Cauchy"}, \code{"Bon"} (default), \code{"BBC"}, \code{"CBC"}, or \code{"all"}}
}
\value{
If the argument \code{method} is \code{"Cauchy"}, \code{"Bon"}, \code{"BBC"}, or \code{"CBC"} a single p-value obtained from that method is returned.
If \code{method} is "All", a vector of all 4 are returned, with entries names "p_Cauchy", "p_Bon", "p_BBC", and "p_CBC"
}
\description{
Given a vector of p-values from not-necessarily-independent tests, compute a combined p-value
using the Cauchy combination method Liu and Xie (2020), the Bonferroni method (also called MinP, the minimum times the number of tests),
or the combined Cauchy/Bon variants BBC and CBC as proposed by Chen (2022).
}
\details{
All \code{NA}a in vector are ignored.
}
\examples{
combineP(c(.01,.04,.11))
combineP(c(.01,.04,.11),"all")

}
\references{
\insertRef{LiuXie2020}{MSCquartets}

\insertRef{Chen2022}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{M0}
\alias{M0}
\title{Modified Struve function}
\usage{
M0(x)
}
\arguments{
\item{x}{function argument}
}
\value{
value of negative modified Struve function
}
\description{
This function is used in computing the probability density for
Model T1. The code is closely based on the \code{I0L0} function implemented
in Python for the package
RandomFieldUtils, which was available on CRAN until 12/2022.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WQDCrecursive.R
\name{quartetTableCollapse}
\alias{quartetTableCollapse}
\title{Reduce quartet table by combining some taxa}
\usage{
quartetTableCollapse(rqt, taxaA, taxaB)
}
\arguments{
\item{rqt}{a resolved quartet table, as from \code{quartetTableResolved}}

\item{taxaA}{a vector of taxon names in \code{rqt} to be included in the output table}

\item{taxaB}{a vector of taxon names in \code{rqt} to form new composite taxon in the output table}
}
\value{
a resolved quartet table with \code{length(taxaA)+1} taxa; the
 composite taxon is named as the concatenation of the sorted
 names in \code{taxaB}
}
\description{
Form a smaller resolved quartet table by lumping some taxa into a composite taxon.
}
\details{
This function is needed for the recursive calls in \code{WQDSrec}.
It should only be applied to a resolved quartet table which includes counts
for all possible quartets on the taxa (though counts can be zero).

The sets \code{taxaA} and \code{taxaB} must be disjoint. Their union need not be all taxa in \code{rqt}.
}
\seealso{
\code{\link{WQDCrecursive}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{TINNIK}
\alias{TINNIK}
\title{TINNIK algorithm to infer species tree of blobs}
\usage{
TINNIK(
  genedata,
  omit = FALSE,
  epsilon = 0,
  test = "T3",
  alpha = 0.05,
  beta = 0.8,
  treemethod = fastme.bal,
  delta = 2,
  taxonnames = NULL,
  plot = TRUE
)
}
\arguments{
\item{genedata}{gene tree data that may be supplied in any of 3 forms:
\enumerate{
\item as a character string giving the name of a file containing Newick gene trees,
\item as a multiPhylo object containing the gene trees, or
\item as a table of quartets on the gene trees, as produced by a previous call to
\code{TINNIK} or \code{quartetTableResolved}, which has columns only for taxa, resolved quartet counts,
and possibly p_T3, p_cut, and p_star
}}

\item{omit}{\code{FALSE} to treat unresolved quartets as 1/3 of each resolution;
\code{TRUE} to discard unresolved quartet data; ignored if gene tree data given as quartet table}

\item{epsilon}{minimum for branch lengths to be treated as non-zero; ignored if gene tree data given as quartet table}

\item{test}{a hypothesis test to perform, either "cut" or "T3" (default)}

\item{alpha}{a value or vector of significance levels for judging p-values for test specified by "test";
testing a null hypothesis of no hybridization vs. an alternative of hybridization, for each quartet;  a smaller value applies
a less conservative test for a tree (more trees), hence a stricter requirement for deciding in favor of hybridization (fewer reticulations)}

\item{beta}{a value or vector of significance levels for judging p-values testing
a null hypothesis of a star tree (polytomy) for each quartet vs. an alternative of anything else; a smaller value applies a less conservative
test for a star tree (more polytomies), hence a stricter requirement for deciding in favor of a resolved tree or network;
if vectors, \code{alpha} and \code{beta} must have the same length}

\item{treemethod}{a function implementing a method of tree inference from a distance table,
e.g. the ape package's fastme.bal or nj}

\item{delta}{a minimum edge length to retain in tree of blobs (see \insertCite{ABMR24}{MSCquartets} for related theory); shorter edges are collapsed}

\item{taxonnames}{if \code{genedata} is a file or a multiPhylo object, a vector of a subset
of the taxa names on the gene trees
to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genedata}
is a quartet table, this argument is ignored and all taxa in the table are used}

\item{plot}{\code{TRUE} produces simplex plots of hypothesis test results and plots the tree of blobs \code{FALSE} omits plots}
}
\value{
\code{output} (returned invisibly), with \code{output$ToB} the TINNIK tree of blobs, \code{output$pTable}
the table of quartets and p-values for judging fit to the MSC on quartet
trees, and \code{output$Bquartets} a TRUE/FALSE indicator vector of B-quartets; if \code{alpha, beta} are vectors, \code{output$ToB} is a vector of trees;
the table can be used as input to \code{TINNIK} or \code{TINNIKdist} with new choices of \code{alpha, beta}, without re-tallying quartets on
gene trees
}
\description{
Apply the TINNIK algorithm of \insertCite{ABMR24;textual}{MSCquartets} (see also \insertCite{ABMR22;textual}{MSCquartets})
to infer a tree of blobs for the species network from a collection of gene trees,
under the network multispecies coalescent (NMSC) model.
}
\details{
This function
\enumerate{
\item counts displayed quartets across gene trees to form quartet count concordance factors (qcCFs),
\item applies appropriate hypothesis tests to judge qcCFs as representing putative hybridization,
resolved trees, or unresolved (star) trees using \code{alpha} and \code{beta} as significance levels,
\item produces a simplex plot showing results of the hypothesis tests for all qcCFs
\item computes the appropriate TINNIK distance table, and infers the tree of blobs from the distance.
}

A call of \code{TINNIK} with \code{genedata} given as a table previously output from \code{TINNIK} is
equivalent to a call of \code{TINNIKdist} followed by tree construction from the distance table.
If \code{genedata} is a
table previously output from \code{quartetTableResolved}
which lacks columns of p-values for hypothesis tests, these will be appended to the table output by \code{TINNIK}.
This table must contain a row with quartet counts for every 4 taxon set.

If plots are produced, there are 2 simplex plots: The first shows the hypothesis test results,
and the second shows
inferred B-quartets and T-quartets. In both,
each point in the simplex plot corresponds to an empirical quartet concordance factor,
color-coded to represent test or inference results.

In general, \code{alpha} should be chosen to be small and \code{beta}
to be large so that most quartets are interpreted as resolved trees. More quartets judges to have
either blob or unresolved relationships will lead to a less resolved blob tree.

Usually, an initial call to \code{TINNIK} will not give a good analysis, as values
of \code{alpha} and \code{beta} are likely to need some adjustment based on inspecting the data. Saving the returned
table of test results from \code{TINNIK} will allow for the results of the time-consuming computation of qcCFs to be
saved, along with p-values,
for input to further calls of \code{TINNIK} with new choices of \code{alpha} and \code{beta}.

See the documentation for \code{\link{TINNIKdist}} for an explanation of a small, rarely noticeable,
stochastic element of the algorithm.

For data sets of many gene trees, user time may be reduced by using parallel code for
counting displayed quartets. See \code{\link{quartetTableParallel}}.
}
\examples{
data(pTableYeastRokas)
out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)

}
\references{
\insertRef{ABMR22}{MSCquartets}

\insertRef{ABMR24}{MSCquartets}
}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableParallel}}, \code{\link{quartetTableDominant}}, \code{\link{quartetCutTestInd}},\code{\link{quartetTreeTestInd}},
\code{\link{quartetStarTestInd}}, \code{\link{TINNIKdist}}, \code{\link{quartetTestPlot}}, \code{\link{pvalHist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{initBquartets}
\alias{initBquartets}
\title{Initialize vector of B quartets}
\usage{
initBquartets(pTable, m, alpha, beta, colptest, colpstar, Bquartets)
}
\arguments{
\item{pTable}{a quartet table with p-values}

\item{m}{number of rows in pTable}

\item{alpha}{critical value for tree test}

\item{beta}{critical value for star tree test}

\item{colptest}{column with p value for tree test}

\item{colpstar}{column with p value for star tree test}

\item{Bquartets}{0/1 vector encoding initial B quartets}
}
\description{
This is a C++ function, called from \code{TINNIKdist}, to initialize for
inference of B and T quartets.
}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableParallel}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{quartetCutTestInd}
\alias{quartetCutTestInd}
\title{Multiple independent hypothesis tests for quartet counts fitting the Cut model under the NMSC}
\usage{
quartetCutTestInd(
  rqt,
  lambda = 0,
  method = "MLest",
  smallcounts = "approximate",
  bootstraps = 10^4
)
}
\arguments{
\item{rqt}{table of resolved quartet counts, as produced by \code{quartetTableResolved}, or \code{quartetStarTestInd}}

\item{lambda}{power divergence statistic parameter (e.g., 0 for likelihood ratio statistic, 1 for Chi-squared statistic)}

\item{method}{\code{"MLest"}, \code{"conservative"}, or \code{"bootstrap"}; see \code{quartetCutTest} for explanation}

\item{smallcounts}{\code{"bootstrap"} or \code{"approximate"}, method of obtaining p-value when some counts are small, so
the chosen \code{method} is inappropriate}

\item{bootstraps}{number of samples for bootstrapping}
}
\value{
a copy of \code{rqt} with two columns appended: \code{"p_cut"} with p-values and \code{"cutindex"}
   giving index 1,2, or 3 of ML estimate of quartet tree of blobs (1 if 12|34, 2 if 13|24, 3 if 14|23)
   under Cut model.
}
\description{
Perform a hypothesis test for all quartet counts in an input table, as if the counts for different choices of 4 taxa
are independent.
}
\details{
This function assumes all quartets are resolved.  The test performed and the arguments
are described more fully in \code{quartetCutTest}.
}
\examples{
data(pTableYeastRokas)
pT=pTableYeastRokas[1:10,1:11]
pTable=quartetCutTestInd(pT)

}
\references{
\insertRef{ABMR24}{MSCquartets}
}
\seealso{
\code{\link{quartetCutTest}}, \code{\link{quartetTestPlot}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetTableResolved}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{package}
\name{MSCquartets-package}
\alias{MSCquartets}
\alias{MSCquartets-package}
\title{Multispecies Coalescent Model Quartet Package}
\description{
A package for analyzing quartets displayed on gene trees, under the
multispecies coalescent (MSC) model and network multispecies coalescet model (NMSC).
}
\details{
This package contains routines to analyze a collection of gene trees through the displayed
quartets on them.

A quartet count concordance factor (qcCF) for a set of 4 taxa is the triple of counts of the three
possible resolved quartet trees on those taxa across some set of gene trees. The major routines in this package can:
\enumerate{
\item Tabulate all qcCFs for a collection of gene trees.
\item Perform hypothesis tests of whether one or more qcCFs are consistent with the MSC model on
a species tree \insertCite{MAR19}{MSCquartets}.
\item Produce simplex plots showing all estimated CFs as well as results of hypothesis tests \insertCite{AMR2020}{MSCquartets}.
\item Infer a species tree using the qcCFs via the QDC and WQDC methods \insertCite{Rho19,YR19}{MSCquartets}.
\item Infer a level-1 species network via the NANUQ method \insertCite{ABR19}{MSCquartets}.
\item Infer the tree of blobs for a species network via the TINNiK or ECToBlob methods \insertCite{ABMR22}{MSCquartets},\insertCite{ABMR24}{MSCquartets}\insertCite{ECTo2026}{MSCquartets}.
\item Resolove multifurcations in a tree of blobs to cycles via NANUQ+ routines \insertCite{ABRW25}{MSCquartets}.
}
As discussed in the cited works, the inference methods for species trees and networks are
statistically consistent under the MSC and Network MSC respectively.

This package, and the theory on which it is based, allows gene trees to have
missing taxa (i.e., not all gene trees display all the taxa). It does require
that each subset of 4 taxa is displayed on at least one of the gene trees.


Several gene tree data sets, simulated and empirical, are included.

In publications please cite the software announcement \insertCite{RBMA2020}{MSCquartets}, as well as the
appropriate paper(s) above developing the theory behind the routines you used.
}
\references{
\insertRef{RBMA2020}{MSCquartets}

\insertRef{MAR19}{MSCquartets}

\insertRef{AMR2020}{MSCquartets}

\insertRef{Rho19}{MSCquartets}

\insertRef{YR19}{MSCquartets}

\insertRef{ABR19}{MSCquartets}

\insertRef{ABMR22}{MSCquartets}

\insertRef{ABMR24}{MSCquartets}

\insertRef{ABRW25}{MSCquartets}

\insertRef{ECTo2026}{MSCquartets}
}
\author{
\strong{Maintainer}: John Rhodes \email{j.rhodes@alaska.edu} (\href{https://orcid.org/0000-0001-9921-1091}{ORCID})

Authors:
\itemize{
  \item Elizabeth Allman \email{esallman@alaska.edu}
  \item Hector Banos \email{hbassnos@gmail.com}
  \item Jonathan Mitchell \email{jonathanmitchell88@gmail.com}
  \item Kristina Wicke \email{kristina.wicke@njit.edu}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pHist.R
\name{pvalHist}
\alias{pvalHist}
\title{Plot histogram of log p-values in table}
\usage{
pvalHist(pTable, model, pmin = 0)
}
\arguments{
\item{pTable}{a quartet table with p-values, such as from \code{NANUQ}, \code{quartetTreeTestInd},
or \code{quartetStarTestInd}}

\item{model}{one of \code{"T1"}, \code{"T3"}, or \code{"star"}, where \code{pTable}
contains a column \code{p_model} of p-values}

\item{pmin}{include only p-values above \code{pmin} in the histogram}
}
\description{
Graphical exploration of extreme p-values from quartet hypothesis tests, to aid in choosing critical values
for hypothesis tests. Log base 10 of p-values exceeding some minimum are plotted, to explore gaps in
the tail of the distribution.
}
\details{
Since logarithms are plotted, p-values close to 0 will appear as negative numbers of large magnitude, putting the tail of the distribution
to the left in the histogram.

When exploring possible critical values for the hypothesis tests in the NANUQ algorithm, use \code{model= "T3"} to
choose values for \code{alpha} which distinguish treelikeness from hybridization, and \code{model= "star"} to
choose values for \code{beta} which distinguish polytomies from resolved trees.
In general, \code{alpha} should be chosen to be small and \code{beta}
to be large so that most quartets are interpreted as resolved trees.
}
\examples{
data(pTableYeastRokas)
pvalHist(pTableYeastRokas,"T3")

}
\seealso{
\code{\link{NANUQ}}, \code{\link{NANUQdist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{quartetTestPlot}
\alias{quartetTestPlot}
\title{Produce simplex plot with  results of quartet hypothesis tests}
\usage{
quartetTestPlot(pTable, test, alpha = 0.05, beta = 1, cex = 1)
}
\arguments{
\item{pTable}{table of quartets and p-values, as produced by the functions mentioned above}

\item{test}{model to use, for tree null hypothesis; options are \code{"T1"}, \code{"T3"}, \code{"cut"}, \code{"NANUQ"}, \code{"ECToBlob"}}

\item{alpha}{level for tree test with null hypothesis given by \code{test}}

\item{beta}{level for test with null hypothesis star tree;
test results plotted only if \code{beta<1} and \code{"p_star"} column present in \code{pTable}}

\item{cex}{scaling factor for size of plotted symbols}
}
\description{
Plot a 2-d probability simplex, with points for all quartet count vectors. Colors
indicate rejection or failure to reject for tests at specified levels.
}
\details{
The first argument of this function is a table of quartets and p-values. The
plot may show results of either the T1, T1-D, T3, or 2-cut
test, with or without a star tree test (depending on whether a \code{"p_star"} column is in the table and/or \code{beta =1}).
The p-values must be computed by previous calls to
\code{quartetTreeTestInd} (for \code{"T1"} or \code{"T3"} p-values), \code{quartetECtestInd} (for \code{"T1-D"} p-values)
and \code{quartetStarTestInd} (for \code{"star"} p-values). The \code{NANUQ}, \code{NANUQdist}, and \code{ECToBlob}
functions include calls to these tree test functions.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=c("t1","t2","t3","t4","t5","t6")
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
stree="(((t5,t6),t4),((t1,t2),t3));"
pTable=quartetTreeTestInd(RQT,"T1",speciestree=stree)
pTable=quartetStarTestInd(pTable)
quartetTestPlot(pTable, "T1", alpha=.05, beta=.95)

}
\seealso{
\code{\link{quartetTreeTestInd}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetECtestInd}},
\code{\link{NANUQ}}, \code{\link{NANUQdist}}, \code{\link{ECToBlob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{expNANUQCycleDist}
\alias{expNANUQCycleDist}
\title{Expected NANUQ cycle distance}
\usage{
expNANUQCycleDist(n)
}
\arguments{
\item{n}{number of edges in cycle}
}
\value{
an nxn distance matrix with rows/columns ordered from hybrid following circular order
}
\description{
Compute expected NANUQ distance for a sunlet network with 4 or more taxa. This is used
to resolve multifurcations in a tree of blobs by NANUQ+ functions
}
\examples{
expNANUQCycleDist(5)

}
\seealso{
\code{\link{resolveCycle}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{data}
\name{pTableYeastRokas}
\alias{pTableYeastRokas}
\title{pTable for Yeast dataset}
\format{
an R data file
}
\source{
from L. Nakhleh's website at Rice U., link no longer available as of 2026
}
\usage{
data(pTableYeastRokas)
}
\description{
An .rda file dataset for the "dataYeastRokas" dataset. This is a subset of the data
of \insertCite{Rokas03;textual}{MSCquartets}.
}
\details{
This is provided primarily so that examples of other functions run more quickly. It can be reproduced by the following
example code below.
}
\examples{
\donttest{
gtrees=read.tree(file = system.file("extdata","dataYeastRokas",package="MSCquartets"))
QT=quartetTable(gtrees)
RQT=quartetTableResolved(QT)
pTable=quartetCutTestInd(RQT)
pTable=quartetTreeTestInd(pTable)
pTableYeastRokas=quartetStarTestInd(pTable)
}
}
\references{
\insertRef{Rokas03}{MSCquartets}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{data}
\name{dataPapioniniVanderpool}
\alias{dataPapioniniVanderpool}
\title{Papionini gene tree dataset}
\format{
A text file with 1703 metric Newick gene trees each with 7 leaves labelled:\cr
Cercocebus_atys, Mandrillus_leucophaeus, Papio_anubis, Theropithecus_gelada,
Macaca_fascicularis, Macaca_mulatta, Macaca_nemestrina
}
\source{
\doi{10.5061/dryad.rfj6q577d}
}
\description{
A text file dataset for Papionini containing 1730 gene trees on 7 taxa. This is a subset of the data of \insertCite{Vanderpool2020;textual}{MSCquartets}.
}
\details{
File is accessed as \code{system.file("extdata","dataPapioniniVanderpool",package="MSCquartets")}, for
example via the \code{ape} command:

\code{gts = read.tree(file=system.file("extdata","dataPapioniniVanderpool",package="MSCquartets"))}
}
\references{
\insertRef{Vanderpool2020}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QuartetTally.R
\name{quartetTableResolved}
\alias{quartetTableResolved}
\title{Modify quartet table to show only resolved quartets}
\usage{
quartetTableResolved(qt, omit = FALSE)
}
\arguments{
\item{qt}{table, as produced by \code{quartetTable} for \code{n} taxa, with \code{n+4} columns}

\item{omit}{\code{TRUE} deletes unresolved quartets column;
\code{FALSE} deletes the column but redistributes unresolved counts as (1/3,1/3,1/3) to resolved counts}
}
\value{
a table of quartet counts similar to \code{qt}, but with columns showing only resolved quartet counts
}
\description{
Converts table of all quartet counts, including unresolved ones,
by either dropping unresolved ones, or distributing them uniformly
among the three resolved counts.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
QT[1:6,]
RQT=quartetTableResolved(QT)
RQT[1:6,]

}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableDominant}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{TINNIKdist}
\alias{TINNIKdist}
\title{Compute TINNIK distance from quartets and hypothesis test p-values}
\usage{
TINNIKdist(pTable, test = "T3", alpha = 0.05, beta = 0.05)
}
\arguments{
\item{pTable}{table of resolved quartet counts, as produced by
\code{quartetTableResolved}, with extra columns from
both star hypothesis test, and either cut or T3 hypothesis tests}

\item{test}{either "cut" or "T3"}

\item{alpha}{level for cut or T3 test}

\item{beta}{level for star test}
}
\value{
a distance table \code{output$dist} and
   a vector \code{output$Bquartets} with TRUE/FALSE entries indicating B-quartets
   ordered as rows of \code{pTable}.
}
\description{
Apply the B-quartet inference algorithm of \insertCite{ABMR22;textual}{MSCquartets}, \insertCite{ABMR24;textual}{MSCquartets} to
infer all B-quartets from results of hypothesis tests, and then compute an estimate of an intertaxon distance
fitting the topological tree of blobs of the species network.
}
\details{
This function assumes \code{pTable} has columns for taxa and resolved
quartet counts as originally produced by \code{quartetTable},
and hypothesis test results as produced by
\code{quartetStarTestInd}, and either \code{quartetTreeTestInd} for the \code{T3} test or \code{quartetCutTestInd}.
Rows must be present for every 4-taxon subset.
(Note: Of functions in this package, only \code{HolmBonferroni} might modify the row order from the required one.)

This function uses the Rcpp package for significant speed up in computation time.
}
\examples{
data(pTableYeastRokas)
out=TINNIKdist(pTableYeastRokas,test="T3",alpha=.05,beta=.05)

}
\references{
\insertRef{ABMR22}{MSCquartets}

\insertRef{ABMR24}{MSCquartets}
}
\seealso{
\code{\link{quartetTable},\link{quartetTableResolved},\link{quartetStarTest}},
\code{\link{quartetCutTest}}, \code{\link{quartetStarTestInd}}, \code{\link{quartetCutTestInd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{combineCycleResolutions}
\alias{combineCycleResolutions}
\title{Combine several cycle resolutions on a tree of blobs to create a network}
\usage{
combineCycleResolutions(ToB, resolutions, plot = 1, titletext = NULL)
}
\arguments{
\item{ToB}{an unrooted tree of blobs for the network, with multifurcating nodes
labelled by \code{\link{labelIntNodes}}}

\item{resolutions}{a list of resolutions (each of which may be a list)
for different nodes with elements in format described in output of \code{\link{resolveCycle}}}

\item{plot}{if FALSE (0), no plots; if TRUE (>0) plot networks}

\item{titletext}{a string of text for plot}
}
\value{
a list of Newick strings for the networks, with all edge lengths 1
}
\description{
Given a list of resolutions of different multifurcations on a tree of blobs
(each as produced by \code{\link{resolveCycle}}), combine these with the tree
of blobs to form a network.
}
\details{
This function is useful for forming near-optimal networks when there are
several resolutions that have similar fit for some of the multifurcations.
}
\examples{
data(pTableYeastRokas)
out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)
ToB=labelIntNodes(out$ToB)
R9=resolveCycle(ToB, node=9, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")
R10=resolveCycle(ToB, node=10, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")
combineCycleResolutions(ToB, resolutions=list(R9,R10),plot=TRUE)

}
\seealso{
\code{\link{ECToBlob}}, \code{\link{TINNIK}}, \code{\link{labelIntNodes}}, \code{\link{resolveCycle}},
 \code{\link{resolveLevel1}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetDist.R
\name{WQDS}
\alias{WQDS}
\title{Compute the Weighted Quartet Distance Supertree}
\usage{
WQDS(dqt, method = fastme.bal)
}
\arguments{
\item{dqt}{an (\code{n} choose 4) x \code{n+1}) matrix of form output by \code{quartetTableDominant}}

\item{method}{a distance-based tree building function (e.g., fastme.bal, NJ, etc.)}
}
\value{
an unrooted metric tree, of type phylo
}
\description{
Apply the Weighted Quartet Distance Supertree method of \insertCite{YR19;textual}{MSCquartets} to
a collection of quartets on \code{n} taxa together with internal 
quartet branch lengths, specified by a table.
}
\details{
This function is a wrapper which runs \code{quartetWeightedDist}, builds a tree, and then adjusts edge lengths
with \code{WQDSAdjustLengths}.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
DQT=quartetTableDominant(RQT,bigweights= "finite")
tree=WQDS(DQT)
write.tree(tree)
plot(tree)

}
\references{
\insertRef{YR19}{MSCquartets}
}
\seealso{
\code{\link{quartetTableDominant}}, 
         \code{\link{quartetWeightedDist}},
         \code{\link{WQDSAdjustLengths}},
         \code{\link{WQDC}}, 
         \code{\link{WQDCrecursive}},
         \code{\link{QDS}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{nodeGroups}
\alias{nodeGroups}
\title{Groups taxa by deleting a node in a tree}
\usage{
nodeGroups(tree, nodeNum)
}
\arguments{
\item{tree}{a tree, of class "phylo"}

\item{nodeNum}{a node number, representing an internal node in the phylo representation}
}
\value{
a list of lists of tree tip numbers for each group.
The union of the groups is the set of all tips.
}
\description{
Finds groups of taxa determined by the connected components of the graph resulting from deleting an internal node in a tree.
}
\details{
When applied to a rooted tree, the last group returned is the set of tips that are
non-descendants of the node (provided any exist).
}
\examples{
 tree=read.tree(text="((a,b),((c,d,e),(f,g)));")
 nodeGroups(tree,8)
 nodeGroups(tree,10)
 nodeGroups(tree,11)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{T1density}
\alias{T1density}
\title{Probability density function for Model T1}
\usage{
T1density(x, mu0)
}
\arguments{
\item{x}{statistic value (e.g., likelihood ratio statistic, or other power divergence statistic)}

\item{mu0}{parameter of density function}
}
\value{
value of density function
}
\description{
Value of probability density function for Model T1 of \insertCite{MAR19;textual}{MSCquartets}, Proposition 5.2.
}
\references{
\insertRef{MAR19}{MSCquartets}
}
\seealso{
\code{\link{T3density}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{data}
\name{dataGeneTreeSample}
\alias{dataGeneTreeSample}
\title{Simulated gene tree dataset from species tree}
\format{
A text file with 1000 metric Newick gene trees on the taxa t1-t9
}
\description{
A text file dataset containing 1000 gene trees on 9 taxa simulated under the MSC on a species tree
}
\details{
This simulated dataset was produced by SimPhy \insertCite{simphy}{MSCquartets}, using the species tree

((((t5:5000,t6:5000):5000,t4:10000):2500,t7:12500):7500,((t8:3000,t9:3000):5000,\cr
((t1:4000,t2:4000):2500,t3:6500):1500):12000);

with a population size of 10,000 throughout the tree.

File is accessed as  \code{system.file("extdata","dataGeneTreeSample",package="MSCquartets")}, for example
via the ape command:

\code{gts=read.tree(file = system.file("extdata","dataGeneTreeSample",package="MSCquartets") )}
}
\references{
\insertRef{simphy}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{BQinference}
\alias{BQinference}
\title{Main loop of B-quartet inference}
\usage{
BQinference(pTable, C, Cn4, n, Bquartets, L1, lenL1, Nrule1, Nrule2, cuttops)
}
\arguments{
\item{pTable}{a quartet table with p-values}

\item{C}{precomputed binomial coefficients}

\item{Cn4}{precomputed binomial coefficient}

\item{n}{number of taxa}

\item{Bquartets}{0/1 vector of initial Bquartets}

\item{L1}{vector of recently inferred B quartets}

\item{lenL1}{lnegth of L1}

\item{Nrule1}{count of inference from rule 1}

\item{Nrule2}{count of inference from rule 2}

\item{cuttops}{inferred cut topologies}
}
\description{
This is a C++ function, called from \code{TINNIKdist}, to
infer B and T quartets.
}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableParallel}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{data}
\name{ASTRALtreeLeopardusLescroart}
\alias{ASTRALtreeLeopardusLescroart}
\title{ASTRAL tree for Leopardus dataset}
\format{
an R data file
}
\usage{
data(ASTRALtreeLeopardusLescroart)
}
\description{
An .rda file containing the tree produced by ASTRAL from the "Leopardus" gene trees
of \insertCite{Lescroart2023;textual}{MSCquartets}. By the theory of \insertCite{ECTo2026}{MSCquartets}, 
this is a putative resolution of the tree of blobs for the unknown network, 
and can be used by \code{ECToBlob} to infer the network's tree of blobs.
}
\details{
If used for purposes other than learning software, please cite \insertRef{Lescroart2023}{MSCquartets}.
}
\references{
\insertRef{Lescroart2023}{MSCquartets}, \insertRef{ECTo2026}{MSCquartets}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WQDCrecursive.R
\name{WQDCrecursive}
\alias{WQDCrecursive}
\title{Compute the Recursive Weighted Quartet Distance Consensus tree from gene tree data}
\usage{
WQDCrecursive(rqt, method = fastme.bal, stopAt = 2, terminal = 1)
}
\arguments{
\item{rqt}{a resolved quartet table as produced by \code{quartetTableResolved}}

\item{method}{a distance-based tree building function, such as \code{fastme.bal} or \code{nj}}

\item{stopAt}{a non-negative branch length in coalescent units; recursive calls stop when the longest 
branch in a recursively examined subtree is smaller than this value}

\item{terminal}{non-negative branch length to supply for terminal branches,
whose lengths cannot be inferred by \code{WQDCrecursive}}
}
\value{
an unrooted metric tree, of type phylo
}
\description{
Infer a metric species tree from counts
of quartets displayed on a collection of gene trees, as described by 
\insertCite{YR19;textual}{MSCquartets}. Edge lengths are in coalescent units.
}
\details{
The algorithm counts quartets displayed on the gene trees, builds a tree using \code{WQDS},
determines the split corresponding to the longest edge in that tree,
and then recursively builds trees
on the taxa in each split set together with a `composite taxon' formed by all
taxa in the other split set.
This approach is slower than non-recursive \code{WQDC}, but increases topological accuracy. Shorter branch 
lengths tend to be more accurately estimated.

This function must be called with its argument a resolved quartet
table of size (n choose 4)x(n+3). Its recursive nature
requires building smaller resolved quartet tables on split sets with an additional
composite taxon.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
stree=WQDCrecursive(RQT)
write.tree(stree)
plot(stree)

}
\references{
\insertRef{YR19}{MSCquartets}
}
\seealso{
\code{\link{quartetTableResolved}},\code{\link{quartetTable}},
\code{\link{QDC}}, \code{\link{QDS}}, \code{\link{quartetTableCollapse}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QuartetTally.R
\name{topDist}
\alias{topDist}
\title{Topological distances on a tree}
\usage{
topDist(tree, epsilon = 0)
}
\arguments{
\item{tree}{a phylo object, with or without edge lengths}

\item{epsilon}{a tolerance, so all edges shorter than epsilon are contracted}
}
\value{
a distance table, with rows and columns named by taxa
}
\description{
Compute a pairwise table of topological distances from a tree, after contracting short edges
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{resolveCycle}
\alias{resolveCycle}
\title{Resolve a node on a tree of blobs as a cycle}
\usage{
resolveCycle(
  ToB,
  node,
  pTable,
  test = "T3",
  alpha,
  beta,
  distance = "NANUQ",
  hdegree = 10,
  plot = TRUE,
  delta = 10^-6
)
}
\arguments{
\item{ToB}{an unrooted tree of blobs (phylo object), as determined by ECToBlob, TINNIK
or another method, with multifurcations labelled by \code{labelIntNodes}}

\item{node}{number of an internal node to be resolved}

\item{pTable}{a table of qcCFs, with columns p_star and p_test}

\item{test}{either "T3" or "cut", indicating the test to use for determining what
qcCFs indicate hybridization}

\item{alpha}{test level for p_test}

\item{beta}{test level for p_star}

\item{distance}{cycle resolution distance to be used; one of "NANUQ" or "modNANUQ"}

\item{hdegree}{resolve a multifurcation of this degree or larger by a heuristic
method; must be at least 5}

\item{plot}{if FALSE (0), no plots; if TRUE (>0), make plots of resolved cycle(s) considered
best and histogram of measure of fit for all hybrid/orders considered}

\item{delta}{cutoff for relative difference in squared residuals and smallest,
(RSS-minRSS)/minRSS, for determining near ties as "best" fit resolutions}
}
\value{
a list of resolution information, given as a list of:
\itemize{
\item \code{$node} node number,
\item \code{$cycleRes} list [[1]]-[[k]] of  best resolutions,
\item \code{$RSSs} RSSs from all cycle resolutions considered in choosing best.
}
Each resolution is itself a 5-element list with entries:
\itemize{
\item \code{$cycleNet} Newick network with 1 cycle (with all edge lengths 1)
\item \code{$cycleRSS} RSS for cycle,
\item \code{$taxonGroups} taxon groups for cycle,
\item \code{$order} order of groups around cycle,
\item \code{$nonRootEdges} logical vector indicating edges of \code{ToB}
where root cannot be.
}
(Items \code{$taxonGroups,$order,$nonRootEdges} are
needed to combine resolutions to form networks with multiple cycles using
\code{\link{combineCycleResolutions}}, and otherwise may not be of interest to users).
}
\description{
Given a Tree of Blobs and quartet Concordance Factor data, resolve a specific
polytomy to a cycle. Resolution is performed by finding a least-squares
best-fit of an empirical distance to an expected distance related to the cycle,
as described in \insertCite{ABRW25;textual}{MSCquartets}.
}
\details{
Possible distances to use are the NANUQ distance and a modified NANUQ distance
that weights quartet trees differently, but from which the cycle structure is
still identifiable.

For multifucations of degree less than a designated cutoff, all possible
circular orders and choices of hybrid nodes are considered in choosing the
best. Above that cutoff, a heuristic method built on the modified NANUQ
distance is used to obtain a small number of orders likely to be good fits,
with the least-squares fitting applied only to those.
}
\examples{
data(pTableYeastRokas)
out=TINNIK(pTableYeastRokas, alpha=.01, beta=.05)
ToB=labelIntNodes(out$ToB)
resolveCycle(ToB, node=9, pTable=out$pTable, alpha=.01, beta=.05, distance="NANUQ")

}
\references{
\insertRef{ABRW25}{MSCquartets}
}
\seealso{
\code{\link{ECToBlob}} \code{\link{TINNIK}}, \code{\link{labelIntNodes}}, \code{\link{combineCycleResolutions}},
\code{\link{resolveLevel1}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{labelIntNodes}
\alias{labelIntNodes}
\title{Label internal nodes on tree}
\usage{
labelIntNodes(tree, plot = TRUE, type = "unrooted")
}
\arguments{
\item{tree}{a rooted or unrooted tree (phylo object)}

\item{plot}{TRUE for plot, FALSE for no plot}

\item{type}{plot type (e.g.,"unrooted") to be passed to ape plot command}
}
\value{
a phylo object with internal node labels
}
\description{
Label all internal nodes of tree, as "Node NN" where NN is the node number, and
plot tree with labels
}
\examples{
data(pTableYeastRokas)
out=TINNIK(pTableYeastRokas,test="T3",alpha=.01, beta=.05)
labelIntNodes(out$ToB)

}
\seealso{
\code{\link{resolveCycle}}, \code{\link{combineCycleResolutions}},
  \code{\link{resolveLevel1}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{fitCycleOrders}
\alias{fitCycleOrders}
\title{Compute fit of circular orders to distance with least squares}
\usage{
fitCycleOrders(D, E, orders)
}
\arguments{
\item{D}{an empirical distance table}

\item{E}{an expected distance table, to be reordered}

\item{orders}{a vector indicating an order, or matrix whose rows give orders, to fit}
}
\value{
vector of RSSs, one for each order
}
\description{
Compute residual sum of squares (RSS) comparing empirical distance for a blob
to an expected one for a cycle with each given order/designated hybrid. This is used
in NANAUQ+ commands for resolving multifurcations in a tree of blobs to a cycle
}
\seealso{
\code{\link{resolveCycle}}, \code{\link{resolveLevel1}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetDist.R
\name{QDC}
\alias{QDC}
\title{Compute Quartet Distance Consensus tree from gene tree data}
\usage{
QDC(
  genetreedata,
  taxanames = NULL,
  method = fastme.bal,
  omit = FALSE,
  metric = FALSE
)
}
\arguments{
\item{genetreedata}{gene tree data that may be supplied in any of 3 forms: 
\enumerate{
\item a character string giving the name of a file containing gene trees in Newick,
\item a multiPhylo object containing gene trees, or
\item a resolved quartet table, such as produced by \code{quartetTableResolved}
}}

\item{taxanames}{if \code{genetreedata} is a file or a multiPhylo object, a vector of a subset
of the taxa names on the gene trees 
to be analyzed, if \code{NULL} all taxa on the first gene tree are used; if \code{genetreedata} 
is a quartet table, this argument is ignored and all taxa in the table are used}

\item{method}{a distance-based tree building function, such as \code{fastme.bal} or \code{nj}}

\item{omit}{\code{TRUE} ignores unresolved quartets; \code{FALSE} treats them as 1/3 of each resolution; 
ignored if \code{genetreedata} is supplied as a quartet table}

\item{metric}{if \code{FALSE} return topological tree; if \code{TRUE} return metric tree with
internal edge lengths estimated by \code{estimateEdgeLengths} with \code{lambda=0}, and terminal branches of length 1}
}
\value{
an unrooted tree of type phylo
}
\description{
Compute the Quartet Distance Consensus \insertCite{Rho19}{MSCquartets} estimate of an unrooted 
topological species tree from gene tree data.
}
\details{
This function is a wrapper which performs the steps of reading in a collection
of gene trees, tallying quartets, computing the quartet distance between taxa, building
a tree which consistently estimates the unrooted species tree topology under the MSC, and then possibly estimating edge
lengths using the "simpleML" method.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
stree=QDC(gtrees,tnames[1:6])
write.tree(stree)
plot(stree)
streeMetric=QDC(gtrees, tnames[1:6],metric=TRUE)
write.tree(streeMetric)
plot(streeMetric)

}
\references{
\insertRef{Rho19}{MSCquartets}
}
\seealso{
\code{\link{quartetTable}},
         \code{\link{quartetTableResolved}},
         \code{\link{quartetTableDominant}},
         \code{\link{quartetDist}},
         \code{\link{QDS}},
         \code{\link{WQDC}}, 
         \code{\link{WQDCrecursive}}
         \code{\link{estimateEdgeLengths}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{quartetTreeErrorProb}
\alias{quartetTreeErrorProb}
\title{Bayesian posterior probability of error in 4-taxon unrooted species tree topology estimate}
\usage{
quartetTreeErrorProb(obs, model = "T3")
}
\arguments{
\item{obs}{vector of counts for 3 topologies}

\item{model}{\code{"T3"} or \code{"T1"}, for the models of \insertCite{MAR19;textual}{MSCquartets} describing an unspecified species
tree topology (\code{"T3"}), or the topology whose count is the first entry of \code{obs} (\code{"T1"})}
}
\value{
\code{(error.prob, top.probs)} where \code{error.prob} is the species tree error probability
and \code{top.probs} is a vector of the three species tree topology probabilities in the order of \code{obs};
for model \code{"T1"} the species tree used is the one
corresponding to the first count; for model \code{"T3"} the species
tree is the one corresponding to the largest count
}
\description{
From a gene quartet count concordance factor (qcCF), computes Bayesian posterior probabilities
of the three 4-taxon species tree topologies and the Bayesian posterior probability that the
assumed topology is incorrect, under the assumption that the counts arise from the MSC on some species tree.
}
\details{
The Jeffreys prior is used for internal branch length, along with the uniform prior
on the resolved topology.
}
\examples{
obs <- c(28,32,30)
quartetTreeErrorProb(obs,model="T1")
quartetTreeErrorProb(obs,model="T3")

}
\references{
\insertRef{MAR19}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{data}
\name{tableLeopardusLescroart}
\alias{tableLeopardusLescroart}
\title{Quartet table for Leopardus dataset}
\format{
an R data file
}
\usage{
data(tableLeopardusLescroart)
}
\description{
An .rda file containing a quartet table summarizing the "Leopardus" gene trees
of \insertCite{Lescroart2023;textual}{MSCquartets}. This table contains quartet counts
for 16,338 "gene" trees for 16 taxa, inferred from sliding widows across
genomic sequences.
}
\details{
This table is provided rather than the original gene trees to save storage space.
If used, please cite \insertRef{Lescroart2023}{MSCquartets}.
}
\references{
\insertRef{Lescroart2023}{MSCquartets}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QuartetTally.R
\name{taxonNames}
\alias{taxonNames}
\title{Get all taxon names from a collection of trees}
\usage{
taxonNames(trees)
}
\arguments{
\item{trees}{a multiPhylo object containing a collection of trees}
}
\value{
a vector of unique names of taxa appearing on the trees
}
\description{
Create a vector of all taxon names appearing on a collection of trees, with no repeats.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NANUQ.R
\name{NANUQdist}
\alias{NANUQdist}
\title{Compute NANUQ distance and write to file}
\usage{
NANUQdist(pTable, outfile = "NANUQdist", alpha = 0.05, beta = 0.8, plot = TRUE)
}
\arguments{
\item{pTable}{a table of resolved quartets and p-values, as previously computed by \code{NANUQ}, or by both \code{quartetTreeTestInd} and
\code{quartetStarTestInd}, with columns \code{"p_T3"} and \code{"p_star"}}

\item{outfile}{a character string giving an output file name stub for
saving a \code{NANUQ} distance matrix in nexus format; to the stub \code{outfile}
will be appended an \code{alpha} and \code{beta} value and ".nex";
if \code{NULL} then not written to file}

\item{alpha}{a value or vector of significance levels for judging p-values
testing a null hypothesis of no hybridization for each quartet;  a smaller value applies
a more liberal test for a tree (more trees), hence a stricter requirement for suspecting hybridization (fewer reticulations)}

\item{beta}{a value or vector of significance levels for judging p-values testing
a null hypothesis of a star tree for each quartet; a smaller value applies a more liberal
test for a star tree (more polytomies), hence a stricter requirment for suspecting a resolved tree;
if vectors, \code{alpha} and \code{beta} must have the same length}

\item{plot}{\code{TRUE} produces simplex plots of hypothesis tests, \code{FALSE} omits plots}
}
\value{
a NANUQ distance table, or a list of such tables if \code{alpha} and \code{beta}
are vectors (returned invisibly)
}
\description{
Computes the quartet distance tables for the NANUQ algorithm of \insertCite{ABR19;textual}{MSCquartets}, using precomputed p-values for quartets,
for each of several levels specified. Distance tables are written to files, in nexus format.
}
\details{
If plots are produced, each point represents an empirical quartet concordance factor,
color-coded to represent test results giving interpretation as network, resolved tree, or star tree.

If \code{alpha} and \code{beta} are vectors, they must be of the same length k. Then the i-th entries are
paired to produce k plots and k distance tables/output files. This is equivalent to k
calls to \code{NANUQdist} with paired scalar values from the vectors of \code{alpha} and \code{beta}.

See the documentation for \code{\link{quartetNetworkDist}} for an explanation of a small, rarely noticeable,
stochastic element of the algorithm.
}
\examples{
data(pTableYeastRokas)
dist=NANUQdist(pTableYeastRokas, alpha=.05, beta=.80, outfile = NULL)

}
\references{
\insertRef{ABR19}{MSCquartets}
}
\seealso{
\code{\link{NANUQ}}, \code{\link{quartetTreeTestInd}}, \code{\link{quartetStarTestInd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{circHybOrders}
\alias{circHybOrders}
\title{Generate all circular orders with designated hybrid}
\usage{
circHybOrders(n)
}
\arguments{
\item{n}{size of order, with n>3}
}
\value{
an (n!/2)xn  (or 3xn if n=4) matrix whose rows give all circular orders.
}
\description{
Generate a matrix whose rows give all circular orders with a designated hybrid.
The order is encoded as a vector with entries from 1 to n, where the position
corresponds to a node/group of taxa.  The location in the vector of the 1
indicates the hybrid, the positions of 2, n its neighbors, etc.
}
\details{
To avoid duplication of circular orders, the entry 2 in each vector
always occurs before the entry n.

Since in using first-order quartet-based methods to infer 4-cycles the hybrid
node is not identifiable, for n=4 only 3 orders are given, with 1 as hybrid for each
}
\examples{
circHybOrders(4)
circHybOrders(5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{quartetCutMLE}
\alias{quartetCutMLE}
\title{Maximum likelihood estimate of quartet tree of blobs topology and CF under Cut model}
\usage{
quartetCutMLE(qcCF)
}
\arguments{
\item{qcCF}{a quartet count Concordance Factor (a 3-element vector)}
}
\value{
output with \code{output$topology} = 1, 2, or 3 indicating topology of
tree of blobs in accord with ordering of qcCF entries,
and \code{output$CFhat} the ML estimate for the CF
}
\description{
Computes Maximum likelihood estimate of quartet tree of blobs topology and CF under the Cut model of
\insertCite{ABMR24;textual}{MSCquartets}, Section 3. In case of ties, the topology and CF estimate are chosen randomly among those maximizing
the likelihood. In particular, a resolved tree of blobs is always returned.
}
\examples{
 quartetCutMLE(c(17,72,11))
 quartetCutMLE(c(48,11,41))
 quartetCutMLE(c(11,48,41))
 quartetCutMLE(c(48,41,11))

}
\references{
\insertRef{ABMR24}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{quartetTallyCpp}
\alias{quartetTallyCpp}
\title{Build quartet table from distances}
\usage{
quartetTallyCpp(dList, M, nt, Q, random, progressbar = FALSE)
}
\arguments{
\item{dList}{a list of distance matrices}

\item{M}{number of sets of 4 taxa}

\item{nt}{number of gene trees/distance matrices}

\item{Q}{matrix to fill out as table of quartet counts}

\item{random}{if 0 compute for all sets of 4 taxa, otherwise for M random ones}

\item{progressbar}{if TRUE, display progress bar}
}
\description{
This is a C++ function, called from quartetTable, to fill in the quartet counts.
From a list of topological distance matrices (1 for each gene tree) it determines all
gene quartets. It is not intended to be used as a stand-alone function, and hence not fully
documented. The faster looping in C++ over R gives substantial time improvements
}
\seealso{
\code{\link{quartetTable}}, \code{\link{quartetTableParallel}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeOfBlobs.R
\name{collapseEdges}
\alias{collapseEdges}
\title{Collapse short tree edges}
\usage{
collapseEdges(tree, delta)
}
\arguments{
\item{tree}{a phylo object}

\item{delta}{minimum edge length to retain}
}
\value{
a phylo object
}
\description{
Set all edges of a tree with short lengths to be zero.
}
\examples{
tree=read.tree(text="((a:1,b:1):.5,(c:.5,d:2):1);")
newtree=collapseEdges(tree,delta=1)
write.tree(newtree)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{ordersHeuristicmodNANUQ}
\alias{ordersHeuristicmodNANUQ}
\title{Choose cycle orders heuristically from  empirical modNANUQ distance}
\usage{
ordersHeuristicmodNANUQ(M, delta = 10^-6)
}
\arguments{
\item{M}{an empirical modNANUQ distance table}

\item{delta}{cutoff for relative difference in distances for determining near ties for "best" orders}
}
\value{
a list of circular orders
}
\description{
Find candidates for best hybrid node and circular order fitting the modNANUQ
distance.
}
\details{
Candidadte orders are obtained  by first picking the hybrid node (from the
minimum column sum of the distance matrix), then ordering nodes by distance
from the hybrid, and for each consecutive pair picking  nodes in the cycle
closest to the previous node. This constructs one or more orders since ties
may occur. For more details, see \insertCite{ABRW25;textual}{MSCquartets}.

This function is used by NANUQ+ commands
to resolve multifurcations in a tree of blobs of high degree.
}
\references{
\insertRef{ABMR24}{MSCquartets}
}
\seealso{
\code{\link{expmodNANUQCycleDist}} \code{\link{resolveCycle}} \code{\link{resolveLevel1}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetDist.R
\name{quartetDist}
\alias{quartetDist}
\title{Compute quartet distance between taxa}
\usage{
quartetDist(dqt)
}
\arguments{
\item{dqt}{an (\code{n} choose 4) x \code{n} (or \code{n+1}) matrix of form output by \code{quartetTableDominant};
(Note: If present, the \code{n+1}th column of \code{dqt} is ignored.)}
}
\value{
a pairwise distance matrix on \code{n} taxa
}
\description{
Compute the Quartet Distance of \insertCite{Rho19;textual}{MSCquartets} from a table specifying a collection of quartets on
\code{n} taxa.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
DQT=quartetTableDominant(RQT)
Dist=quartetDist(DQT)
tree=NJ(Dist)
write.tree(tree)
plot(tree)

}
\references{
\insertRef{Rho19}{MSCquartets}
}
\seealso{
\code{\link{quartetTableDominant}},
\code{\link{QDS}},
\code{\link{QDC}},
\code{\link{quartetWeightedDist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NANUQ.R
\name{nexusDist}
\alias{nexusDist}
\title{Write a distance table to a file in nexus format}
\usage{
nexusDist(distMatrix, outfilename)
}
\arguments{
\item{distMatrix}{a square matrix giving a distance table, with rows and columns labeled by taxon names}

\item{outfilename}{the name of an output file}
}
\description{
Write a distance table to a file in nexus format.
}
\examples{
gtrees=read.tree(file=system.file("extdata","dataGeneTreeSample",package="MSCquartets"))
tnames=taxonNames(gtrees)
QT=quartetTable(gtrees,tnames[1:6])
RQT=quartetTableResolved(QT)
DQT=quartetTableDominant(RQT)
Dist=quartetDist(DQT)
nexusDist(Dist,outfile = file.path(tempdir(), "NANUQdist"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evonetMisc.R
\name{suppressNetBinaryNodes}
\alias{suppressNetBinaryNodes}
\title{Suppress Network Binary Nodes}
\usage{
suppressNetBinaryNodes(net, addlength = TRUE)
}
\arguments{
\item{net}{a phylogenetic network, of class "evonet"}

\item{addlength}{if TRUE (default), add lengths of two incident edges for new edge.}
}
\description{
Suppress any non-root binary nodes in a phylogenetic network.
}
\details{
This function is similar to ape's \code{collapse.singles} which only works
on phylo objects that are trees.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ECToBlob.R
\name{ECToBlob_quad}
\alias{ECToBlob_quad}
\title{Edge Contraction to infer a Tree of Blobs - quadripartition mode}
\usage{
ECToBlob_quad(
  genedata,
  tree,
  testCorrection = "Bon",
  alpha = NA,
  beta = 0.8,
  colorCutoff = 0.05,
  plot = 3,
  refTree = NULL,
  omit = FALSE,
  lambda = 0,
  method = "MLest",
  smallsample = "precomputed",
  smallcounts = "precomputed",
  bootstraps = 10^4
)
}
\arguments{
\item{genedata}{one of:
\enumerate{
\item a character string giving the name of a file containing a list of newick 
gene trees (see argument \code{omit}),
\item a multiphylo object containing a collection of gene trees (see argument \code{omit}),
\item a table of resolved quartet counts  from a collection of
gene trees, or that table with additional columns of quartet p-values. If p_star
or both p_T1-D and qindex-D are present (such as from a previous call to this function) 
they are not recomputed.}}

\item{tree}{a tree as a phylo object, or a character string giving the name of 
a file with a newick tree, assumed to be a resolution of the true
tree of blobs}

\item{testCorrection}{a method for combining possibly dependent p-values to 
get a single p-value, one of "Cauchy", "Bon", "BBC", "CBC"}

\item{alpha}{test level for choosing inferred tree of blobs}

\item{beta}{test level for collapsing edges showing no resolution}

\item{colorCutoff}{level for changing color of edge p-values for T1 test}

\item{plot}{0 for no plot;
1 to plot all trees as edges are contracted, along with plot of tree p-values;
2 to add -log_10 of p-values < \code{colorCutoff} for T1 test, or > beta for star test, to edges in plots;
3 to add red dots at multifurcations;
4 to suppress red dots but show all edge p-values}

\item{refTree}{a reference tree, as a phylo object, to compute RF distance to 
all Tk (for simulation studies); will be unrooted}

\item{omit}{\code{FALSE} to treat unresolved quartets on gene trees as 1/3 of each resolution;
\code{TRUE} to discard unresolved quartet data; ignored if genedata given as quartet table}

\item{lambda}{parameter for power-divergence statistic (e.g., 0 for likelihood
ratio statistic, 1 for Chi-squared statistic)}

\item{method}{"MLest", "conservative", or "bootstrap"}

\item{smallsample}{"precomputed" or "bootstrap", method of obtaining p-value
when sample is small (<30)}

\item{smallcounts}{"precomputed" or "bootstrap", method of obtaining p-value
when some (but not all) counts are small}

\item{bootstraps}{number of samples for bootstrapping}
}
\value{
An invisible list with 8 elements:
\describe{
\item{$treeList}{A list corresponding to edge contraction steps.}
\item{$indexEarly}{The index in \code{$treeList} of the first tree such that
the combined tree T1 is above \code{alpha}.}
\item{$indexLate}{The index in \code{$treeList} of the first tree such that
the combined tree T1 of it and all subsequent ones is above \code{alpha}.}
\item{$alpha}{The value of \code{alpha} determining the previous 2 entries.}
\item{$beta}{The value of \code{beta} for contracting edges using the 
combined star test.}
\item{$qType}{The quartet type mode used.}
\item{$testCorrection}{The p-value combination method used.}
\item{$pTable}{A table of quartet p-values for relevant
quartet tests.}
}

In \code{$treeList}, the
first is the input tree,
the second the result of contracting edges judged unresolved
by the combined star test,
and subsequent ones the result of collapsing edges with the current
smallest combined T1 test p-value, with the final one a star tree.
Each element in this list itself a
list of \describe{
\item{$tree}{A tree, with all edge lengths 1 and a node labels
of -log of the parent edge's combined p (star for the input tree, T1 for others).}
\item{$logpTree}{-log base 10 of the combined T1 p-value of the tree using \code{testCorrection}.}
\item{$logpT1Edge}{-log base 10 of the combined T1 p-value of the edge(s)
 that were contracted from previous tree (\code{NA}
for input tree and second tree).}
\item{$numEdgeCon}{The number of edges that have been contracted from the input tree}
\item{$RF}{The RF distance between this tree and \code{refTree}, if \code{refTree} is not NULL.}
}
}
\description{
Infer the tree of blobs using edge contraction and quadripartitions
}
\details{
Given quartet gene tree count data from a network and a starting tree that is
a resolution of its tree of blobs, contract edges for which there is poor support
due to, first, star-like quartet signals and then successively the most extreme
non-tree-like signals.

p-values for star and T1 tests for individual quartets displayed on the tree are
combined by one of 4 test correction methods suitable for non-independent tests. For both
star and T1 test, only p-values for quartets defining each edge are combined.
(For T1, this means defining the edge after any star edge contractions have been done.)

The input tree should be computed by ASTRAL, or another heuristic known to
produce a resolution of the tree of blobs in ideal circumstances.

Plot levels allow for several graphical depictions of the edge contract process,
from the starting tree to a final star tree.

For testing with simulated data, a reference tree (e.g., the true Tree of Blobs) 
may be provided and the RF distance between it and each tree will be shown.
}
\examples{
data(ASTRALtreeLeopardusLescroart)
Atree=read.tree(text=ASTRALtreeLeopardusLescroart)
data(tableLeopardusLescroart)
ECToBlob_quad(tableLeopardusLescroart,Atree,"Bon")

}
\references{
\insertRef{ECTo2026}{MSCquartets}

\insertRef{MAR19}{MSCquartets}

\insertRef{LiuXie2020}{MSCquartets}

\insertRef{Chen2022}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ToBtoLevel1.R
\name{blobDistance}
\alias{blobDistance}
\title{Compute empirical distance between taxon groups.}
\usage{
blobDistance(pTable, taxa, groupvec, test = "T3", alpha, beta, dist = "NANUQ")
}
\arguments{
\item{pTable}{table of giving empirical gene quartet counts for the taxa on tree, with columns p_star and p_test}

\item{taxa}{a list of taxon names, who positions are used in 'groups'}

\item{groupvec}{taxon groups encoded in vector}

\item{test}{to be used for detecting hybridizations in quartete ("T3" or "cut")}

\item{alpha}{test level for p_test}

\item{beta}{test level for p_star}

\item{dist}{the distance to compute, either "NANUQ" or "modNANUQ"}
}
\value{
the distance matrix, ordered by taxon group number
}
\description{
From gene quartet counts, computes NANUQ or modNANUQ distances between groups
of taxa (which should be those around a multifurcation in a tree of blobs.
If these groups are not singletons, averaging is done over group elements.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSCquartets-package.R
\docType{data}
\name{dataYeastRokas}
\alias{dataYeastRokas}
\title{Yeast gene tree dataset}
\format{
A text file with 106 topological Newick gene trees on the taxa:
 Sbay, Scas, Scer, Sklu, Skud, Smik, Spar, and Calb (outgroup).
}
\source{
from L. Nakhleh's website at Rice U., link no longer available as of 2026
}
\description{
A text file dataset for Yeast containing 106 gene trees on 8 taxa  (7 Saccharomyces and 1 Candida outgroup). This is a subset of the data
of \insertCite{Rokas03;textual}{MSCquartets}.
}
\details{
File is accessed as \code{system.file("extdata","dataYeastRokas",package="MSCquartets")}, for example
via the \code{ape} command:

\code{ gts=read.tree(file = system.file("extdata","dataYeastRokas",package="MSCquartets"))}
}
\references{
\insertRef{Rokas03}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/QuartetTally.R
\name{sortQuartetTableRows}
\alias{sortQuartetTableRows}
\title{Sort quartet table rows by lex order}
\usage{
sortQuartetTableRows(qT)
}
\arguments{
\item{qT}{a quartet Table to be sorted}
}
\value{
sorted table
}
\description{
Sort the rows of a quartet table so they are in MSCquartet's standard lex order.
This is the order produced by the \code{quartetTable} function. The only exceptions
to this order produced in the package are when \code{quartetTable} is called with the
\code{random} argument non-zero, or when the \code{HolmBonferoni} function is called.
However, for tables made outside this package, it can be useful.
}
\seealso{
\code{\link{quartetTable}}, \code{\link{HolmBonferroni}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{powerDivStat}
\alias{powerDivStat}
\title{Power divergence statistic of Cressie & Read}
\usage{
powerDivStat(obs, expd, lambda)
}
\arguments{
\item{obs}{observation vector}

\item{expd}{expected vector}

\item{lambda}{statistic parameter (e.g., 0=Likelihood Ratio, 1=Chi-squared)}
}
\value{
value of statistic
}
\description{
Computes any of the family of power-divergence statistics for multinomial data of \insertCite{CR84;textual}{MSCquartets},
to compare observed and expected counts. Includes Likelihood Ratio and Chi-squared statistics as special cases.
}
\examples{
obs=c(10,20,30)
expd=c(20,20,20)
powerDivStat(obs,expd,0)

}
\references{
\insertRef{CR84}{MSCquartets}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimplexPlot.R
\name{simplexCoords}
\alias{simplexCoords}
\title{Convert 3-d coordinates to 2-d probability simplex coordinates}
\usage{
simplexCoords(v)
}
\arguments{
\item{v}{vector of 3 non-negative numbers, not summing to 0}
}
\value{
2-d coordinates to plot normalized point in simplex
}
\description{
Convert from 3-d Cartesian coordinates to 2-d coordinates suitable for plotting in the probability simplex.
}
\details{
Applies an affine coordinate trandformation that maps the centroid (1/3,1/3,1/3) to the origin (0,0), and 
rescales so that the line segments between (1,0,0), (0,1,0), and (0,0,1) are mapped to segments of length 1.

An input vector \code{v} is first normalized so its component sum to 1 before the map is applied.
}
\examples{
     simplexCoords(c(15,65,20))

}
\seealso{
\code{\link{simplexLabels}},
         \code{\link{simplexPoint}},
         \code{\link{simplexPrepare}},
         \code{\link{simplexSegment}},
         \code{\link{simplexText}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeQuartetHypTest.R
\name{T3density}
\alias{T3density}
\title{Probability density function for Model T3}
\usage{
T3density(x, mu0, alpha0, beta0)
}
\arguments{
\item{x}{statistic value (e.g., likelihood ratio statistic, or other power divergence statistic)}

\item{mu0}{parameter of density function}

\item{alpha0}{parameter of density function}

\item{beta0}{parameter of density function}
}
\value{
value of density function
}
\description{
Value of probability density function for Model T3 of \insertCite{MAR19;textual}{MSCquartets}, Proposition 4.2.
}
\references{
\insertRef{MAR19}{MSCquartets}
}
\seealso{
\code{\link{T1density}}
}
